<?php
	/**
	 * management of usergroups and access-authorizations
	 *
	 * @author Soeren Pieper <soeren.pieper@docemos.de> / Martin Ufer <martin.ufer@docemos.de>
	 * @copyright Copyright (C) 2009  docemos GmbH
	 * @package time-o-meter
	 *
	 * This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 3 of the License, or (at your option) any later version.
	 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
	 * You should have received a copy of the GNU General Public License along with this program; if not, see http://www.gnu.org/licenses/.
	 */

	/**
	 * management of usergroups and access-authorizations
	 * @package time-o-meter
	 * @subpackage classes
	 */
	class Access
	{
		/**
		* Legt eine Neue Benutzergruppe an.
		* Die Klasse "MySql" wird fuer die Datenbankabfragen eingebunden.
		* @param string $groupname Benutzergruppenname
		* @param array $access_array Array mit allen Zuganzrechten. $access_array = array('admin'=>'j','mangelverwaltung'=>'n'); j=zugang erlaubt, n= zugang nicht erlaubt
		* @return bool TRUE= angelegt, FALSE= nicht angelegt
		*/
		public function insert_group($groupname, $access_array)
		{
			$cdb = new MySql;

			ksort($access_array, SORT_STRING);
			$serialize_daten = @serialize($access_array);

			if ($cdb->insert("INSERT INTO hdb_usergroups (group_name, group_access) VALUES ('$groupname', '$serialize_daten')"))
			{
				if ($cdb->get_affected_rows() == 1)
				{
					return true;
				}
				else
				{
					return false;
				}
			}
			else
			{
				return false;
			}
		}

		/**
		* erstellt ein Array mir allen Zugangsrechten fuer die jeweilige Gruppe
		* Die MySQL Klasse "MySql" wird fuer die Datenbankabfragen eingebunden.
		* @param int $group_id Benutzergruppen ID
		* @return array $result = array('admin'=>'j','mangelverwaltung'=>'n');
		*/
		public function select_group($group_id)
		{
			$cdb = new MySql;

			$sql = $cdb->select("SELECT group_access FROM hdb_usergroups WHERE group_id=$group_id");
			$result = $cdb->fetch_array($sql);

			return @unserialize($result['group_access']);
		}

		/**
		* prueft die Berechtigung fuer einen bestimmten Bereich
		* Die MySQL Klasse "MySql" wird fuer die Datenbankabfragen eingebunden.
		* @param int $group_id Benutzergruppen ID
		* @param string $access Der Bereich der geprueft werden soll zb. admin
		* @return bool TRUE= zugang erlaubt, FALSE= zugang nicht erlaubt
		*/
		public function check_group($group_id, $access)
		{
			$cdb = new MySql;

			$sql = $cdb->select("SELECT group_access FROM hdb_usergroups WHERE group_id=$group_id");
			$result = $cdb->fetch_array($sql);

			$unserialize = @unserialize($result['group_access']);

			if (isset($unserialize[$access]))
			{
				// echo $unserialize[$access].'<br>';
				if ($unserialize[$access] == 'j')
				{
					return true;
				}
				else
				{
					return false;
				}
			}
			else
			{
				return false;
			}
		}

		/**
		* Bearbeitet eine Usergruppe
		* Die MySql Klasse "MySql" wird fuer die Datenbankabfragen eingebunden.
		* @param int $group_id Benutzergruppen ID
		* @param string $group_name Benutzergruppenname
		* @param array $access_array Daten die geaendert werden sollen. $access_array = array('admin'=>'j','mangelverwaltung'=>'n'); Achtung wenn nur ein Wert geaendert werden soll braucht auch nur der eine Wert uebergeben werden.
		* @return bool TRUE= aenderung erfolgreich, FALSE= aenderung nicht erfolgreich
		*/
		public function update_group($group_id, $group_name, $access_array)
		{
			$cdb = new MySql;

			$sql = $cdb->select("SELECT group_access FROM hdb_usergroups WHERE group_id=$group_id");
			$result = $cdb->fetch_array($sql);

			$unserialize = @unserialize($result['group_access']);

			foreach ($access_array as $i => $v)
			{
				$unserialize[$i] = $v;
			}

			ksort($unserialize, SORT_STRING);
			$serialize = @serialize($unserialize);

			if ($cdb->update("UPDATE hdb_usergroups SET group_name='$group_name', group_access='$serialize' WHERE group_id=$group_id"))
			{
				return true;
			}
			else
			{
				return false;
			}
		}
	}
?>