<?php
	/**
	 * create the image buttons for tom
	 *
	 * @author Soeren Pieper <soeren.pieper@docemos.de> / Martin Ufer <martin.ufer@docemos.de>
	 * @copyright Copyright (C) 2010  docemos GmbH
	 * @package time-o-meter
	 *
	 * This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 3 of the License, or (at your option) any later version.
	 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
	 * You should have received a copy of the GNU General Public License along with this program; if not, see http://www.gnu.org/licenses/.
	 */

	/**
	 * create the image buttons for tom
	 * @package time-o-meter
	 * @subpackage classes
	 */
	class Button
	{
		/**
		 * Button kann erstellt werden
		 * @var boole
		 */
		public $can_be_created = true;

		/**
		 * Allgemeine Einstellungen
		 * @var array
		 */
		private $setup = array();

		/**
		 * Prueft ob ein Button erstllt werden kann
		 * @return void
		 */
		public function __construct()
		{
			$this->setup['template_url_path'] = TOM_ABS_URL.'template/'.$GLOBALS['setup_array']['template'].'/';
			$this->setup['template_abs_path'] = TOM_ABS_PFAD.'template/'.$GLOBALS['setup_array']['template'].'/';
			$this->setup['font_abs_path'] = $this->setup['template_abs_path'].'DejaVuSansMono.ttf';
			$this->setup['font_size'] = 10;

			if (!file_exists($this->setup['font_abs_path']))
			{
				$this->can_be_created = false;
			}

			if (!function_exists('imagepng') or !function_exists('imagecopy') or !function_exists('imagettfbbox') or !function_exists('imagettftext') or !function_exists('imagecreatefrompng') or !function_exists('imagecolorallocate'))
			{
				$this->can_be_created = false;
			}

			if (function_exists('gd_info'))
			{
				$gd_info = gd_info();

				if (!isset($gd_info['FreeType Support']) or $gd_info['FreeType Support'] != true)
				{
					$this->can_be_created = false;
				}

				if (!isset($gd_info['PNG Support']) or $gd_info['PNG Support'] != true)
				{
					$this->can_be_created = false;
				}
			}
			else
			{
				$this->can_be_created = false;
			}
		}

		/**
		 * Gibt ein Button zurueck
		 * @param string $txt
		 * @param string $icon
		 * @return mixed
		 */
		public function get_button($txt, $icon)
		{
			//Button bereits vorhanden
			$button_name = $this->button_exists($txt, $icon);
			if ($button_name !== false)
			{
				return $this->button_info($button_name);
			}
			//Button noch nicht vorhanden
			else
			{
				//Image Verzeichnis ist beschreibbar und alle php functionen vorhanden
				if (is_writable($this->setup['template_abs_path'].'image/') and $this->can_be_created == true)
				{
					//button erstellen
					$button_name = $this->create_button($txt, $icon);
					if ($button_name !== false)
					{
						return $this->button_info($button_name);
					}
					else
					{
						return false;
					}
				}
				else
				{
					return false;
				}
			}
		}

		/**
		 * Erstellt einen neuen Button
		 * @param string $txt
		 * @param string $icon
		 * @return mixed
		 */
		private function create_button($txt, $icon)
		{
			//zu verwendendes Icon ist vorhanden. FAMFAMFAM Icon
			if (file_exists($this->setup['template_abs_path'].'image/'.$icon))
			{
				//groesse des textes
				$txt_size = $this->get_txt_size($txt);

				//keine Fehler bei der groessen ermittlung
				if ($txt_size['width'] > 0 and $txt_size['height'] > 0)
				{
					//hintergrundbutton auswaehlen
					$button_bg = '';
					$button_bg_size = 0;
					if ($txt_size['width'] <= 100)
					{
						$button_bg = '_button_135.png';
						$button_bg_size = 135;
					}
					elseif ($txt_size['width'] <= 120)
					{
						$button_bg = '_button_155.png';
						$button_bg_size = 155;
					}
					elseif ($txt_size['width'] <= 140)
					{
						$button_bg = '_button_175.png';
						$button_bg_size = 175;
					}
					elseif ($txt_size['width'] <= 160)
					{
						$button_bg = '_button_195.png';
						$button_bg_size = 195;
					}
					elseif ($txt_size['width'] <= 180)
					{
						$button_bg = '_button_215.png';
						$button_bg_size = 215;
					}
					elseif ($txt_size['width'] <= 210)
					{
						$button_bg = '_button_245.png';
						$button_bg_size = 245;
					}
					elseif ($txt_size['width'] <= 240)
					{
						$button_bg = '_button_275.png';
						$button_bg_size = 275;
					}
					elseif ($txt_size['width'] <= 270)
					{
						$button_bg = '_button_305.png';
						$button_bg_size = 305;
					}
					elseif ($txt_size['width'] <= 300)
					{
						$button_bg = '_button_335.png';
						$button_bg_size = 335;
					}

					//Hintergundbutton vorhanden
					if (!empty($button_bg) and file_exists($this->setup['template_abs_path'].'image/'.$button_bg))
					{
						//Hintergrundbutton laden
						$src_img = @imagecreatefrompng($this->setup['template_abs_path'].'image/'.$button_bg);
						//Icon laden
						$icon_img = @imagecreatefrompng($this->setup['template_abs_path'].'image/'.$icon);

						//Schwarze Schriftfarbe
						$black = @imagecolorallocate($src_img, 0, 0, 0);
						//text in Hintergrundbutton schreiben
						$y_txt = ceil(abs((18 - $txt_size['height']) / 2 + $txt_size['height']));
						if ($y_txt > 14)
						{
							$y_txt = 14;
						}

						@imagettftext($src_img, $this->setup['font_size'], 0, 9, $y_txt, $black, $this->setup['font_abs_path'], $txt);
						//icon in hintergrundbutton copieren
						@imagecopy($src_img, $icon_img, $button_bg_size - 21, 1, 0, 0, 16, 16);

						//buttonnamen ermitteln
						$button_name = $this->get_button_name($txt, $icon);

						//neuen Button ins imageverzeichnis kopieren
						@imagepng($src_img, $this->setup['template_abs_path'].'image/'.$button_name);

						//Speicher leeren
						@imagedestroy($src_img);
						@imagedestroy($icon_img);

						//url pfad zum neuen button zurueckgeben
						if (file_exists($this->setup['template_abs_path'].'image/'.$button_name))
						{
							return $button_name;
						}
						else
						{
							return false;
						}
					}
					else
					{
						return false;
					}
				}
				else
				{
					return false;
				}
			}
			else
			{
				return false;
			}
		}

		/**
		 * Hoehe und breite des Textes ermitteln
		 * @param string $txt
		 * @return array
		 */
		private function get_txt_size($txt)
		{
			$txt_size = array('width' => 0, 'height' => 0);

			$txt_ttf_size = @imagettfbbox($this->setup['font_size'], 0, $this->setup['font_abs_path'], $txt);

			if (isset($txt_ttf_size[2]) and $txt_ttf_size[0])
			{
				$txt_size['width'] = abs($txt_ttf_size[2] - $txt_ttf_size[0]) + 10;
			}

			if (isset($txt_ttf_size[3]) and $txt_ttf_size[5])
			{
				$txt_size['height'] = abs($txt_ttf_size[3] - $txt_ttf_size[5]);
			}

			return $txt_size;
		}

		/**
		 * Prueft ob ein Button bereits existiert
		 * @param string $txt
		 * @param string $icon
		 * @return mixed
		 */
		private function button_exists($txt, $icon)
		{
			$button_name = $this->get_button_name($txt, $icon);
			if (file_exists($this->setup['template_abs_path'].'image/'.$button_name))
			{
				return $button_name;
			}
			else
			{
				return false;
			}
		}

		/**
		 * Erstellt einen eindeutigen Buttonnamen
		 * @param string $txt
		 * @param string $icon
		 * @return string
		 */
		private function get_button_name($txt, $icon)
		{
			return '_button_'.$GLOBALS['user_sprache'].'_'.md5(md5($txt).md5($icon)).'.png';
		}

		private function button_info($button_name)
		{
			$image_array = array('width' => 0, 'height' => 0, 'url' => $this->setup['template_url_path'].'image/'.$button_name, 'abs_path' => $this->setup['template_abs_path'].'image/'.$button_name);

			$image_size = @getimagesize($this->setup['template_abs_path'].'image/'.$button_name);

			if (isset($image_size[0]) and $image_size[0] > 0)
			{
				$image_array['width'] = $image_size[0];
			}

			if (isset($image_size[1]) and $image_size[1] > 0)
			{
				$image_array['height'] = $image_size[1];
			}

			return $image_array;
		}
	}
?>