<?php
	/**
	 * checksum management for the verification of GET/POST variables that must not be changed
	 *
	 * @author Soeren Pieper <soeren.pieper@docemos.de> / Martin Ufer <martin.ufer@docemos.de>
	 * @copyright Copyright (C) 2009  docemos GmbH
	 * @package time-o-meter
	 *
	 * This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 3 of the License, or (at your option) any later version.
	 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
	 * You should have received a copy of the GNU General Public License along with this program; if not, see http://www.gnu.org/licenses/.
	 */

	/**
	 * checksum management for the verification of GET/POST variables that must not be changed
	 * @package time-o-meter
	 * @subpackage classes
	 */
	class Checksum
	{
		/**
		 * @var string Tabellenname
		 */
		private $checksum_table = 'hdb_checksum';

		/**
		* Erstellt eine pruefsumme aus einer beliebig grossen anzahl von Variablen
		* Die MySQL Klasse "MySql" wird fuer die Datenbankabfragen eingebunden.
		* @param array $DATA, Index und Value Werte der zu uebermittelnden POST bzw. GET Variablen
		* @param string $TYP, GET oder POST
		* @return string
		*/
		public function CreateChecksum($DATA, $TYP)
		{
			if (is_array($DATA))
			{
				//SQL Klasse einbinden
				$cdb = new MySql;

				//Speichert den Index des Get bzw. POST Arrays
				$index_array = array();
				//Speichert die Werte der Variablen aus denen eine pruefsumme erstellt werden soll
				$value_str = '';

				foreach ($DATA as $i => $v)
				{
					if (!_empty($v))
					{
						$index_array[] = $i;
						$value_str .= "$v";
					}
				}

				//random key fuer einen einmaligen Key
				$random = md5(uniqid(rand()));

				//userid ermitteln
				if (defined('USER_ID'))
				{
					$user = USER_ID;
				}
				else
				{
					$user = 0;
				}

				$key = md5($random.$value_str);
				$index = @serialize($index_array);
				$TYP = strtoupper($TYP);
				if ($cdb->insert("INSERT INTO ".$this->checksum_table." (cs_key, random, array, typ, user_id, add_time) VALUES ('$key', '$random', '$index', '$TYP', $user, '".date('YmdHis')."')"))
				{
					if ($cdb->get_affected_rows() == 1)
					{
						return $key;
					}
				}
			}
			return '';
		}

		/**
		* Erstellt ein hiddenfeld fuer ein Formular
		* @param array $DATA, Index und Value Werte der zu uebermittelnden POST bzw. GET Variablen
		* @param string TYP, GET oder POST
		* @return string
		*/
		public function GetHidden($DATA, $TYP = 'POST')
		{
			$key = $this->CreateChecksum($DATA, $TYP);
			if (!empty($key))
			{
				return '<input type="hidden" name="VCS" value="'.$key.'" />';
			}
			return '';
		}

		/**
		* Erstellt einen String fuer die Verwenung in einem Link
		* @param array $DATA, Index und Value Werte der zu uebermittelnden POST bzw. GET Variablen
		* @param string TYP, GET oder POST
		* @return string
		*/
		public function GetString($DATA, $TYP = 'GET')
		{
			$key = $this->CreateChecksum($DATA, $TYP);
			if (!empty($key))
			{
				return 'VCS='.$key;
			}
			return '';
		}

		/**
		* Prueft ob eine pruefsumme zu den Variablen passt
		* Die Klasse "MySql" wird fuer die Datenbankabfragen eingebunden.
		* Die Klasse "ChkVariable" wird fuer die Variablenpruefung eingebunden.
		* @param string $KEY
		* @param boole $DELKEY, Gibt an ob der Eintrag nach der pruefung geloescht werden kann
		* @return boole
		*/
		public function Chk($KEY, $DELKEY = false)
		{
			if (!empty($KEY) and strlen($KEY) == 32)
			{
				//SQL Klasse einbinden
				$cdb = new MySql;

				//Auf unerlaubt Zeichen pruefen
				$chk_var = new ChkVariable;
				$KEY = $chk_var->modifyString($KEY);

				$sql = $cdb->select("SELECT * FROM ".$this->checksum_table." WHERE cs_key='$KEY'");
				$result = $cdb->fetch_array($sql);

				if (!empty($result['cs_key']))
				{
					if ($result['typ'] == 'POST')
					{
						$str = $this->PostToString(@unserialize($result['array']));
					}
					elseif ($result['typ'] == 'GET')
					{
						$str = $this->GetToString(@unserialize($result['array']));
					}
					else
					{
						$str = '';
					}

					if (md5($result['random'].$str) == $KEY)
					{
						if ($DELKEY)
						{
							$this->DelKeyChecksum($KEY);
						}
						return true;
					}
					else
					{
						return false;
					}
				}
			}
			return false;
		}
		/**
		* Gibt aus einem POST Array die Valuewerte als string zurueck
		* @param array $INDEX
		* @return string
		*/
		private function PostToString($INDEX)
		{
			$str = '';
			foreach ($INDEX as $v)
			{
				if (isset($_POST[$v]))
				{
					$str .= (string) $_POST[$v];
				}
			}
			return $str;
		}

		/**
		* Gibt aus einem GET Array die Valuewerte als string zurueck
		* @param array $INDEX
		* @return string
		*/
		private function GetToString($INDEX)
		{
			$str = '';
			foreach ($INDEX as $v)
			{
				if (isset($_GET[$v]))
				{
					$str .= (string) $_GET[$v];
				}
			}
			return $str;
		}

		/**
		* Loescht alle pruefsummeneintraege zu einem User. Sollte bei jedem Logout durchgefuehrt werden
		* Die Klasse "MySql" wird fuer die Datenbankabfragen eingebunden.
		* @param int $USER
		* @return void
		*/
		public function DelUserChecksum($USER)
		{
			if (is_numeric($USER))
			{
				$cdb = new MySql;

				$cdb->delete('DELETE FROM '.$this->checksum_table.' WHERE user_id='.$USER);
			}
		}

		/**
		* Loescht einen pruefsummeneintrag anhand des keys
		* Die Klasse "MySql" wird fuer die Datenbankabfragen eingebunden.
		* @param string $KEY
		* @return void
		*/
		public function DelKeyChecksum($KEY)
		{
			$cdb = new MySql;

			$cdb->delete("DELETE FROM ".$this->checksum_table." WHERE cs_key='$KEY'");
		}

		/**
		* Loescht alle pruefsummeneintraege die aelter als 1,5 stunden sind. Sollte im cronjob mit durchgefuehrt werden.
		* Die Klasse "MySql" wird fuer die Datenbankabfragen eingebunden.
		* @return void
		*/
		public function DelOldChecksum()
		{
			$cdb = new MySql;

			$time = date('YmdHis', time()-5400);

			$cdb->delete("DELETE FROM ".$this->checksum_table." WHERE add_time<'$time'");
		}
	}
?>