<?php
	/**
	 * checks user-generated variables for the correct datatypes and removes unwanted characters
	 *
	 * @author Soeren Pieper <soeren.pieper@docemos.de> / Martin Ufer <martin.ufer@docemos.de>
	 * @copyright Copyright (C) 2009  docemos GmbH
	 * @TODO fuer die Verwendung wird die Funktion setError() aus der Datei error_handler.php benoetigt.
	 * @package time-o-meter
	 *
	 * This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 3 of the License, or (at your option) any later version.
	 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
	 * You should have received a copy of the GNU General Public License along with this program; if not, see http://www.gnu.org/licenses/.
	 */

	/**
	 * checks user-generated variables for the correct datatypes and removes unwanted characters
	 * @package time-o-meter
	 * @subpackage classes
	 */
	class ChkVariable
	{
		/**
		 * Bestimmt welche globalen Variablen automatisch geprueft werden.
		 * @var array
		 */
		private $verify_global_array = array('POST','GET','COOKIE');

		/**
		 * Gibt an ob eine automatische pruefung erwuenscht ist. Mit Hilfe der function switchChkGlobals() kann dieser Wert veraendert werden.
		 * @var boole
		 */
		private $chk_globals_bool = true;

		/**
		 * Gibt an ob der Datentyp entsprechend der Namensvorgabe geaendert werden soll. Achtung eine aenderung des Datentypes kann einen anderen Variablenwert bewirken.
		 * @var array
		 */
		private $change_datatype_array = array('int' => true, 'float' => true);

		/**
		 * Gibt an ob String Variablen mit addcslashes() und htmlentities() bearbeitet werden sollen.
		 * @var array
		 */
		private $change_string_array = array('slash' => true, 'html' => true);

		/**
		 * Gibt die Anzahl der Aufgetrettenen Errors zurueck
		 * @var int
		 */
		public $error_count_int = 0;

		/**
		 * Legt Fest ob eine Automatische pruefung der Globalen Variablen erfolgen soll.
		 * @param bool $chk_bool
		 * @return void
		 */
		public function switchChkGlobals($chk_bool = true)
		{
			if (is_bool($chk_bool))
			{
				$this->chk_globals_bool = $chk_bool;
			}
			else
			{
				$this->chk_globals_bool = true;
			}
		}

		/**
		 * prueft Globale Arrays nach den vorgaben von $this->verify_global_array
		 * @return void
		 */
		public function chkGlobals()
		{
			if ($this->chk_globals_bool)
			{
				//Alle Postwerte pruefen
				if (isset($_POST) and in_array('POST', $this->verify_global_array) and count($_POST) > 0)
				{
					$this->chkVars($_POST, 'POST');
				}
				//Alle Getwerte pruefen
				if (isset($_GET) and in_array('GET', $this->verify_global_array) and count($_GET) > 0)
				{
					$this->chkVars($_GET, 'GET');
				}
				//Alle Cookiewerte pruefen
				if (isset($_COOKIE) and in_array('COOKIE', $this->verify_global_array) and count($_COOKIE) > 0)
				{
					$this->chkVars($_COOKIE, 'COOKIE');
				}
			}
		}

		/**
		 * prueft eine Variable auf einen Korrekten Datentyp
		 * @param mixed $var
		 * @param string $typ
		 * @return bool
		 */
		public function chkVar($var, $typ = 'string')
		{
			$typ = strtolower($typ);
			//Variabale soll int sein
			if ($typ == 'int')
			{
				if (!$this->chkIsInt($var))
				{
					setError('Der Datentyp der Variable "'.$var.'" ist nicht int!', WARNING, __LINE__);
					return false;
				}
				else
				{
					return true;
				}
			}
			//Variabale soll float sein
			elseif ($typ == 'float')
			{
				if (!$this->chkIsFloat($var))
				{
					setError('Der Datentyp der Variable "'.$var.'" ist nicht float!', WARNING, __LINE__);
					return false;
				}
				else
				{
					return true;
				}
			}
			//Variabale soll boole sein
			elseif ($typ == 'bool')
			{
				if (!$this->chkIsBool($var))
				{
					setError('Der Datentyp der Variable "'.$var.'" ist nicht bool!', WARNING, __LINE__);
					return false;
				}
				else
				{
					return true;
				}
			}
			//Variabale soll string sein
			elseif ($typ == 'string')
			{
				if (!$this->chkIsString($var))
				{
					setError('Der Datentyp der Variable "'.$var.'" ist nicht string!', WARNING, __LINE__);
					return false;
				}
				else
				{
					return true;
				}
			}
			//Variabale soll array sein
			elseif ($typ == 'array')
			{
				if (!$this->chkIsArray($v))
				{
					setError('Der Datentyp der Variable "'.$var.'" ist nicht array!', WARNING, __LINE__);
					return false;
				}
				else
				{
					return true;
				}
			}
			else
			{
				return false;
			}
		}

		/**
		 * prueft eine Array auf Ihre Datentypen
		 * @param mixed $variable
		 * @param string $typ
		 */
		private function chkVars(&$variable, $typ)
		{
			if (is_array($variable))
			{
				foreach ($variable as $i => &$v)
				{
					//assoziatives array
					if (!is_int($i))
					{
						//Variabale soll int sein
						if (strpos($i, '_int') !== false)
						{
							//Fehler kein int
							if (!$this->chkIsInt($v))
							{
								setError('Der Datentyp der Variable "'.$v.'" ist nicht int!', WARNING, __LINE__);
								//Datentyp aendern
								if ($this->change_datatype_array['int'] === true)
								{
									setError('Der Datentyp der Variable "'.$v.'" wird in int geaendert!', NOTICE, __LINE__);
									$v = intval($v);
								}
							}
						}
						//Variabale soll float sein
						elseif (strpos($i, '_float') !== false)
						{
							if (!$this->chkIsFloat($v, ''))
							{
								if ($this->chkIsFloat($v))
								{
									if ($this->change_datatype_array['float'] === true)
									{
										setError('Der Datentyp der Variable "'.$v.'" wird in float geaendert!', NOTICE, __LINE__);
										$v = str_replace(',', '.', $v);
										$v = floatval($v);
										$this->error_count_int--;
									}
								}
								else
								{
									setError('Der Datentyp der Variable "'.$v.'" ist nicht float!', WARNING, __LINE__);
									//Datentyp aendern
									if ($this->change_datatype_array['float'] === true)
									{
										setError('Der Datentyp der Variable "'.$v.'" wird in float geaendert!', NOTICE, __LINE__);
										$v = floatval($v);
									}
								}
							}
						}
						//Variabale soll boole sein
						elseif (strpos($i, '_bool') !== false)
						{
							if (!$this->chkIsBool($v))
							{
								setError('Der Datentyp der Variable "'.$v.'" ist nicht bool!', WARNING, __LINE__);
							}
						}
						//Variabale soll string sein
						elseif (strpos($i, '_string') !== false)
						{
							if (!$this->chkIsString($v))
							{
								setError('Der Datentyp der Variable "'.$v.'" ist nicht string!', WARNING, __LINE__);
							}
							else
							{
								if ($this->change_string_array['html'] === true or $this->change_string_array['slash'] === true)
								{
									$v = $this->modifyString($v);
								}
							}
						}
						//Variabale soll array sein
						elseif (strpos($i, '_array') !== false)
						{
							if (!$this->chkIsArray($v))
							{
								setError('Der Datentyp der Variable "'.$v.'" ist nicht array!', WARNING, __LINE__);
							}
							else
							{
								//Selbstaufruf
								$this->chkVars($v, $typ);
							}
						}
						//keine datentypangabe im index vorhanden
						else
						{
							//Variabale soll string sein
							if (gettype($v) == 'string')
							{
								if ($this->change_string_array['html'] === true or $this->change_string_array['slash'] === true)
								{
									$v = $this->modifyString($v);
								}
							}
							//Variabale soll array sein
							elseif (gettype($v) == 'array')
							{
								//Selbstaufruf
								$this->chkVars($v, $typ);
							}
							//Variabale ist kein int bzw. float
							elseif (gettype($v) != 'boolean' and gettype($v) != 'integer' and gettype($v) != 'float' and gettype($v) != 'double')
							{
								setError('Der Datentyp der Variable "'.$v.'" - "'.gettype($v).'" konnte nicht ermittelt werden!', WARNING, __LINE__);
								$this->error_count_int++;
							}
						}
					}
					//Nummericher Array Index
					else
					{
						//Variabale soll string sein
						if (gettype($v) == 'string')
						{
							if ($this->change_string_array['html'] === true or $this->change_string_array['slash'] === true)
							{
								$v = $this->modifyString($v);
							}
						}
						//Variabale soll array sein
						elseif (gettype($v) == 'array')
						{
							//Selbstaufruf
							$this->chkVars($v, $typ);
						}
						//Variabale ist kein int bzw. float
						elseif (gettype($v) != 'boolean' and gettype($v) != 'integer' and gettype($v) != 'float' and gettype($v) != 'double')
						{
							setError('Der Datentyp der Variable "'.$v.'" - "'.gettype($v).'" konnte nicht ermittelt werden!', WARNING, __LINE__);
							$this->error_count_int++;
						}
					}
				}
			}
			//kein Array
			else
			{
				//Variabale soll string sein
				if (gettype($variable) == 'string')
				{
					if ($this->change_string_array['html'] === true or $this->change_string_array['slash'] === true)
					{
						$variable = $this->modifyString($variable);
					}
				}
				//Variabale ist kein int bzw. float
				elseif (gettype($v) != 'boolean' and gettype($v) != 'integer' and gettype($v) != 'float' and gettype($v) != 'double')
				{
					setError('Der Datentyp der Variable "'.$v.'" - "'.gettype($v).'" konnte nicht ermittelt werden!', WARNING, __LINE__);
					$this->error_count_int++;
				}
			}
		}

		/**
		 * Modifiziert einen string
		 * @param string $var
		 * return string
		 */
		public function modifyString($var)
		{
			//Wandelt alle geeigneten Zeichen in entsprechende HTML-Codes um
			if ($this->change_string_array['html'] === true)
			{
				if (get_magic_quotes_gpc() == 1)
				{
					$var = stripcslashes($var);
				}

				//ab 5.23. Der Parameter double_encode wurde hinzugefuegt.
				if (intval(str_replace('.', '', PHP_VERSION)) >= 523)
				{
					$var = htmlentities($var, ENT_QUOTES, 'UTF-8', false);
				}
				else
				{
					$var = html_entity_decode($var, ENT_QUOTES);
					$var = htmlentities($var, ENT_QUOTES);
				}

				//Stellt bestimmten Zeichen eines Strings ein "\" voran
				if (get_magic_quotes_gpc() == 1)
				{
					$var = addslashes($var);
				}
			}

			//Stellt bestimmten Zeichen eines Strings ein "\" voran
			if ($this->change_string_array['slash'] === true and get_magic_quotes_gpc() == 0)
			{
				$var = addslashes($var);
			}
			return $var;
		}

		/**
		 * prueft ob die Variable vom Datentyp array ist
		 * @param mixed $var
		 * @return bool
		 */
		private function chkIsArray($var)
		{
			if (is_array($var))
			{
				return true;
			}
			else
			{
				$this->error_count_int++;
				return false;
			}
		}

		/**
		 * prueft ob die Variable vom Datentyp string ist
		 * @param mixed $var
		 * @return bool
		 */
		private function chkIsString($var)
		{
			if (is_string($var))
			{
				return true;
			}
			else
			{
				$tmp_var = (string) $var;

				if ("$tmp_var" === "$var")
				{
					return true;
				}
				else
				{
					$this->error_count_int++;
					return false;
				}
			}
		}

		/**
		 * prueft ob die Variable vom Datentyp boolean ist
		 * @param mixed $var
		 * @return bool
		 */
		private function chkIsBool($var)
		{
			if (is_bool($var))
			{
				return true;
			}
			else
			{
				$this->error_count_int++;
				return false;
			}
		}

		/**
		 * prueft ob die Variable vom Datentyp float ist
		 * @param mixed $var
		 * @param string $type
		 * @return bool
		 */
		private function chkIsFloat($var, $type = 'replace')
		{
			if (is_float($var))
			{
				return true;
			}
			elseif ($type == 'replace')
			{
				//wandelt gegebenenfalls vorhandene komma in punkt
				if (substr_count($var, ',') == 1 and substr_count($var, '.') == 0)
				{
					$var = str_replace(',', '.', $var);
				}
				$tmp_var = (float) $var;

				if ("$tmp_var" === "$var")
				{
					return true;
				}
				else
				{
					$this->error_count_int++;
					return false;
				}
			}
			else
			{
				$this->error_count_int++;
				return false;
			}
		}

		/**
		 * prueft ob die Variable vom Datentyp integer ist
		 * @param mixed $var
		 * @return bool
		 */
		private function chkIsInt($var)
		{
			if (is_int($var))
			{
				return true;
			}
			else
			{
				$tmp_var = (int) $var;

				if ("$tmp_var" === "$var")
				{
					return true;
				}
				else
				{
					$this->error_count_int++;
					return false;
				}
			}
		}
	}
 ?>