<?php
	/**
	 * output of flextime and holidays
	 *
	 * @author Soeren Pieper <soeren.pieper@docemos.de> / Martin Ufer <martin.ufer@docemos.de>
	 * @copyright Copyright (C) 2010  docemos GmbH
	 * @package time-o-meter
	 *
	 * This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 3 of the License, or (at your option) any later version.
	 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
	 * You should have received a copy of the GNU General Public License along with this program; if not, see http://www.gnu.org/licenses/.
	 */

	/**
	 * output of flextime and holidays
	 * @package time-o-meter
	 * @subpackage classes
	 */
	class Holiday
	{
		private $setup_array = array();
		private $request_data = array();

		public function set_options($option_array)
		{
			//Qualifikations ID
			if (isset($option_array['quali_id']))
			{
				$this->setup_array['options']['quali_id'] = $option_array['quali_id'];
			}

			//Projekt ID
			if (isset($option_array['project_id']))
			{
				$this->setup_array['options']['project_id'] = $option_array['project_id'];
			}

			//User ID
			if (isset($option_array['user_id']))
			{
				$this->setup_array['options']['user_id'] = $option_array['user_id'];
			}

			//Antraege die in Bearbeitung sind anzueigen
			if (isset($option_array['show_open_status']))
			{
				$this->setup_array['options']['show_open_status'] = true;
			}
		}

		/**
		 * Uebergibt Abwesenheitsantraege die noch nicht in der DB sind
		 * @param int $user_id
		 * @param string $date_from
		 * @param string $date_to
		 * @param int $absence_type_id
		 * @return void
		 */
		public function set_request_data($user_id, $date_from, $date_to, $absence_type_id)
		{
			$cdb = new MySql;

			$sql = $cdb->select('SELECT user_id, vorname, nachname FROM hdb_user WHERE user_id='.$user_id);
			$result = $cdb->fetch_array($sql);

			if (isset($result['user_id']) and !empty($result['user_id']))
			{
				$this->request_data = array('user_id'			=> $user_id,
											'vorname'			=> $result['vorname'],
											'nachname'			=> $result['nachname'],
											'date_from' 		=> $date_from,
											'date_to'			=> $date_to,
											'absence_type_id'	=> $absence_type_id);

				$this->request_data['dates'] = array();

				if ($date_from != $date_to)
				{
					$kal = new Kalender();

					$diff_date_array = $kal->get_difference($date_from, $date_to);

					for ($i = 0; $i < count($diff_date_array); $i++)
					{
						$this->request_data['dates'][] = $diff_date_array[$i]['iso'];
					}
				}
				else
				{
					$this->request_data['dates'][] = $date_from;
				}
			}
		}

		/**
		* Gibt die Resturlaubstage in verschiedenen Varianten zurueck
		* Die MySQL Klasse "MySql" wird fuer die Datenbankabfragen eingebunden.
		* @param int $USERID
		* @param int $YEAR
		* @return array
		*/
		public function get_remaining_leave($USERID, $YEAR)
		{
			$cdb = new MySql;

			//gibt den Jahresurlaubsanspruch aus
			//Erster Monat es Jahres
			$sql = $cdb->select("SELECT startwert_urlaub FROM hdb_stz_monat WHERE user_id=$USERID AND LEFT(datum, 4)='$YEAR' ORDER BY datum ASC");
			$result = $cdb->fetch_array($sql);

			$jahresurlaub = $result['startwert_urlaub'];

			//gibt den resturlaub aller bereits eingetragenen Urlaubstage zurueck
			//Letzten Monat des Jahres
			$sql = $cdb->select("SELECT startwert_urlaub, uebertrag_urlaub FROM hdb_stz_monat WHERE user_id=$USERID AND LEFT(datum, 4)='$YEAR' ORDER BY datum DESC");
			$result = $cdb->fetch_array($sql);

			//prueft ob uebertrag einen zahl ist.
			if (is_numeric($result['uebertrag_urlaub']))
			{
				$taken = $result['uebertrag_urlaub'];
			}
			//Sollte nur passieren wenn der Stundenzettel leer ist
			else
			{
				$taken = $result['startwert_urlaub'];
			}

			//Speichert alle bereits eingetragenen Urlaubstage
			$sql = $cdb->select("SELECT SUM(summe_urlaub) AS days FROM hdb_stz_monat WHERE user_id=$USERID AND LEFT(datum,4)='$YEAR'");
			$result = $cdb->fetch_array($sql);

			$genommener_urlaub = $result['days'];

			//Alle genehmigten Urlaubsantraege
			$genehmigter_urlaub = 0;
			$sql = $cdb->select("SELECT SUM(day_count) AS days FROM hdb_absence_list WHERE user_id=$USERID AND absence_type_id=2 AND status='approved' AND date_from>='".date("Y-m-d")."' AND LEFT(date_from, 4)='$YEAR'");
			$result = $cdb->fetch_array($sql);

			$genehmigter_urlaub += $result['days'];

			//Beantragte Urlaubsantraege
			$beantragt_urlaub = 0;
			$sql = $cdb->select("SELECT SUM(day_count) AS days FROM hdb_absence_list WHERE user_id=$USERID AND absence_type_id=2 AND status='open' AND date_from>='".date("Y-m-d")."' AND LEFT(date_from, 4)='$YEAR'");
			$result = $cdb->fetch_array($sql);

			$beantragt_urlaub += $result['days'];

			$return = array('jahresurlaub'			=> $jahresurlaub,
							'aktueller_resturlaub'	=> $taken,
							'genommener_urlaub'		=> $genommener_urlaub,
							'genehmigter_urlaub'	=> $genehmigter_urlaub,
							'beantragter_urlaub'	=> $beantragt_urlaub);

			foreach ($return as $i => $v)
			{
				if (empty($v))
				{
					$return[$i] = 0;
				}
			}
			return $return;
		}

		/**
		 * Urlaubsinformationen fuer PDF ausgabe erstellen
		 *
		 * @param int $year
		 * @param int $month
		 * @param int $user_id
		 * @return array
		 */
		public function get_pdf_data($year, $month, $user_id = 0)
		{
			$this->get_month_setup($year, $month);

			return $this->get_user_data($user_id);
		}

		/**
		 * Erstellt eine HTML ausgabe fuer einen Monat des Jahresurlaubsplans
		 *
		 * @param int $year
		 * @param int $month
		 * @param string $type
		 * @return string
		 */
		public function get_vacation_schedule($year, $month, $type = 'holiday')
		{
			if (isset($this->setup_array['options']['user_id']))
			{
				$user_id = $this->setup_array['options']['user_id'];
			}
			else
			{
				$user_id = 0;
			}
			$this->get_month_setup($year, $month);

			$return_string = '<br />
							<table cellpadding="2" cellspacing="0" border="0" width="100%" class="border">
								<tr class="table_tr_bg_2">
									<td class="table_header_left" width="20%">'.get_text(179, 'return')/*Mitarbeiter*/.'</td>
									<td class="table_header" width="80%" align="center">'.$year.' '.$this->get_month_name().'</td>
								</tr>
								<tr>
									<td>&nbsp;</td>
									<td>
										<table cellpadding="2" cellspacing="0" border="0" width="100%">
											<tr align="center">
												<td width="7%">&nbsp;</td>';
													for ($i = 1; $i <= 31; $i++)
													{
			$return_string .=							$this->get_day_td($i);
													}
			$return_string .=					'</tr>
										</table>
									</td>
								</tr>';

			$tr_bg_int = 2;
			//Normaler Urlaubskalender
			if ($type == 'holiday')
			{
				$user_result = $this->get_user_data($user_id);
				foreach ($user_result as $user_data)
				{
					$return_string .=	'<tr class="table_tr_bg_'.$tr_bg_int.'">
											<td>'.$user_data['nachname'].', '.$user_data['vorname'].'</td>
											<td>
												<table cellpadding="2" cellspacing="0" border="0" width="100%">
												<tr align="center">
													<td width="7%">&nbsp;</td>';
														for ($i = 1; $i <= 31; $i++)
														{
					$return_string .=						$this->get_userday_td($i, $user_data);
														}
					$return_string .=			'</tr>
											</table>
											</td>
										</tr>';

					if ($tr_bg_int == 1)
					{
						$tr_bg_int++;
					}
					else
					{
						$tr_bg_int = 1;
					}
				}
			}
			elseif ($type == 'qualification')
			{
				$quali = new Qualification();

				if (isset($this->request_data) and !empty($this->request_data))
				{
					$quali->set_request_data($this->request_data);
				}

				if (isset($this->setup_array['options']['quali_id']))
				{
					$q_id = $this->setup_array['options']['quali_id'];
				}
				else
				{
					$q_id = 0;
				}

				if ($month < 10)
				{
					$quali_month = "0$month";
				}
				else
				{
					$quali_month = $month;
				}

				$qualidate = "$year-$quali_month-01";

				$q_id_array = array();
				if (is_numeric($q_id) and $q_id > 0)
				{
					$q_id_array[] = $q_id;
				}
				//komma suchen
				elseif (strpos($q_id, ',') !== false)
				{
					$q_ex = explode(',', $q_id);

					for ($i = 0; $i < count($q_ex); $i++)
					{
						if (!empty($q_ex[$i]) and is_numeric($q_ex[$i]))
						{
							$q_id_array[] = $q_ex[$i];
						}
					}
				}
				else
				{
					$quali_result_array = $quali->get_qualification(0);
					for ($i = 0; $i < count($quali_result_array); $i++)
					{
						$q_id_array[] = $quali_result_array[$i]['quali_id'];
					}
				}

				if (empty($q_id_array))
				{
					$quali_result_array = $quali->get_qualification(0);
					for ($i = 0; $i < count($quali_result_array); $i++)
					{
						$q_id_array[] = $quali_result_array[$i]['quali_id'];
					}
				}

				for ($n = 0; $n < count($q_id_array); $n++)
				{
					//noch nicht genehmigte abwesenheitsgruende anzeigen
					if (isset($this->setup_array['options']['show_open_status']) and $this->setup_array['options']['show_open_status'] == true)
					{
						$quali_array = $quali->get_quali_data($q_id_array[$n], $qualidate, 'open');
					}
					else
					{
						$quali_array = $quali->get_quali_data($q_id_array[$n], $qualidate);
					}

					if ($quali_array !== false)
					{
						//print_r_pre($quali_array);
						$return_string .=	'<tr class="table_tr_bg_3">
												<td title="'.get_text(1019, 'return')/*Mindest Mitarbeiterbestand*/.' / '.get_text(1020, 'return')/*Kritischer Mitarbeiterbestand*/.' / '.get_text(1021, 'return')/*Mitarbeiter insgesamt*/.'"><strong>'.$quali_array['quali_info']['qualifikation'].' ('.$quali_array['quali_info']['min_headcount'].'/'.$quali_array['quali_info']['critical_headcount'].'/'.$quali_array['quali_info']['user_count'].')</strong></td>
												<td>
													<table cellpadding="2" cellspacing="0" border="0" width="100%">
														<tr align="center">
															<td width="7%">&nbsp;</td>';
																for ($i = 1; $i <= 31; $i++)
																{
																	if ($i < 10)
																	{
																		$quali_day = "0$i";
																	}
																	else
																	{
																		$quali_day = $i;
																	}
																	$qualidate = "$year-$quali_month-$quali_day";

																	if (isset($quali_array['user_away_count'][$qualidate]))
																	{
						$return_string .=								$quali->get_quali_status_td($quali_array['quali_info']['user_count'], $quali_array['user_away_count'][$qualidate], $quali_array['quali_info']['min_headcount'], $quali_array['quali_info']['critical_headcount']);
																	}
																	else
																	{
						$return_string .=								$quali->get_quali_status_td($quali_array['quali_info']['user_count'], 0, $quali_array['quali_info']['min_headcount'], $quali_array['quali_info']['critical_headcount']);
																	}
																}
						$return_string .=				'</tr>
													</table>
												</td>
											</tr>';

						if (isset($quali_array['user_info']))
						{
							$tr_bg_int = 2;
							foreach ($quali_array['user_info'] as $u_id => $u_name)
							{
								$return_string .=	'<tr class="table_tr_bg_'.$tr_bg_int.'">
														<td>'.$u_name.'</td>
														<td>
															<table cellpadding="2" cellspacing="0" border="0" width="100%">
															<tr align="center">
																<td width="7%">&nbsp;</td>';
																	for ($i = 1; $i <= 31; $i++)
																	{
																		if ($i < 10)
																		{
																			$user_day = "0$i";
																		}
																		else
																		{
																			$user_day = $i;
																		}
																		$userdate = "$year-$quali_month-$user_day";

																		if (isset($quali_array['user_away_info'][$u_id][$userdate]))
																		{
																			$tmp_as_id = $quali_array['user_away_info'][$u_id][$userdate];
																		}
																		else
																		{
																			$tmp_as_id = 0;
																		}

																		if (isset($quali_array['user_away_status'][$u_id][$userdate]))
																		{
																			$tmp_as_status = $quali_array['user_away_status'][$u_id][$userdate];
																		}
																		else
																		{
																			$tmp_as_status = 'approved';
																		}

								$return_string .=						$this->get_userday_td($i, '', $tmp_as_id, $tmp_as_status);
																	}
								$return_string .=			'</tr>
														</table>
														</td>
													</tr>';

								if ($tr_bg_int == 1)
								{
									$tr_bg_int++;
								}
								else
								{
									$tr_bg_int = 1;
								}
							}
						}
					}
				}
			}
			elseif ($type == 'project')
			{
				$pro = new Project();

				if (isset($this->request_data) and !empty($this->request_data))
				{
					$pro->set_request_data($this->request_data);
				}

				if (isset($this->setup_array['options']['project_id']))
				{
					$pro_id = $this->setup_array['options']['project_id'];
				}
				else
				{
					$pro_id = 0;
				}

				if ($month < 10)
				{
					$pro_month = "0$month";
				}
				else
				{
					$pro_month = $month;
				}

				$pro_year_month = "$year-$pro_month";

				$pro_id_array = array();
				if (is_numeric($pro_id) and $pro_id > 0)
				{
					$pro_id_array[] = $pro_id;
				}
				//komma suchen
				elseif (strpos($pro_id, ',') !== false)
				{
					$pro_ex = explode(',', $pro_id);

					for ($i = 0; $i < count($pro_ex); $i++)
					{
						if (!empty($pro_ex[$i]) and is_numeric($pro_ex[$i]))
						{
							$pro_id_array[] = $pro_ex[$i];
						}
					}
				}
				else
				{
					$pro_result_array = $pro->get_project(0, $year, $pro_month);
					for ($i = 0; $i < count($pro_result_array); $i++)
					{
						$pro_id_array[] = $pro_result_array[$i]['projekt_id'];
					}
				}

				if (empty($pro_id_array))
				{
					$pro_result_array = $pro->get_project(0, $year, $pro_month);
					for ($i = 0; $i < count($pro_result_array); $i++)
					{
						$pro_id_array[] = $pro_result_array[$i]['projekt_id'];
					}
				}

				//print_r_pre($pro_id_array);
				for ($n = 0; $n < count($pro_id_array); $n++)
				{
					//noch nicht genehmigte abwesenheitsgruende anzeigen
					if (isset($this->setup_array['options']['show_open_status']) and $this->setup_array['options']['show_open_status'] == true)
					{
						$pro_array = $pro->get_project_data($pro_id_array[$n], $pro_year_month, 'open');
					}
					else
					{
						$pro_array = $pro->get_project_data($pro_id_array[$n], $pro_year_month);
					}

					//print_r_pre($pro_array);

					if ($pro_array !== false)
					{
						//print_r_pre($quali_array);
						$return_string .=	'<tr class="table_tr_bg_3">
												<td><strong>'.$pro_array['project_info']['kurzname'].' - '.$pro_array['project_info']['name'].'</strong></td>
												<td>
													<table cellpadding="2" cellspacing="0" border="0" width="100%">
														<tr align="center">
															<td width="7%">&nbsp;</td>';
																for ($i = 1; $i <= 31; $i++)
																{
																	if ($i < 10)
																	{
																		$pro_day = "0$i";
																	}
																	else
																	{
																		$pro_day = $i;
																	}
																	$prodate = "$year-$pro_month-$pro_day";

																	if (isset($pro_array['user_away_count'][$prodate]))
																	{
						$return_string .=								$pro->get_project_status_td();
																	}
																	else
																	{
						$return_string .=								$pro->get_project_status_td();
																	}
																}
						$return_string .=				'</tr>
													</table>
												</td>
											</tr>';

						if (isset($pro_array['user_info']))
						{
							$tr_bg_int = 2;
							foreach ($pro_array['user_info'] as $u_id => $u_name)
							{
								$return_string .=	'<tr class="table_tr_bg_'.$tr_bg_int.'">
														<td>'.$u_name.'</td>
														<td>
															<table cellpadding="2" cellspacing="0" border="0" width="100%">
															<tr align="center">
																<td width="7%">&nbsp;</td>';
																	for ($i = 1; $i <= 31; $i++)
																	{
																		if ($i < 10)
																		{
																			$user_day = "0$i";
																		}
																		else
																		{
																			$user_day = $i;
																		}
																		$userdate = "$year-$pro_month-$user_day";

																		if (isset($pro_array['user_away_info'][$u_id][$userdate]))
																		{
																			$tmp_as_id = $pro_array['user_away_info'][$u_id][$userdate];
																		}
																		else
																		{
																			$tmp_as_id = 0;
																		}

																		if (isset($pro_array['user_away_status'][$u_id][$userdate]))
																		{
																			$tmp_as_status = $pro_array['user_away_status'][$u_id][$userdate];
																		}
																		else
																		{
																			$tmp_as_status = 'approved';
																		}

								$return_string .=						$this->get_userday_td($i, '', $tmp_as_id, $tmp_as_status);
																	}
								$return_string .=			'</tr>
														</table>
														</td>
													</tr>';

								if ($tr_bg_int == 1)
								{
									$tr_bg_int++;
								}
								else
								{
									$tr_bg_int = 1;
								}
							}
						}
					}
				}
			}

			$return_string .= '</table>';
			return $return_string;
		}

		/**
		 * Erstellt die html ausgabe fuer einen User pro Tag
		 *
		 * @param int $day
		 * @param array $user_data
		 * @param int $as_id
		 * @param string $as_status
		 * @return string
		 */
		private function get_userday_td($day, $user_data, $as_id = 0, $as_status = '')
		{
			$return_string = '';

			if ($day <= $this->setup_array['days'])
			{
				if ($day < 10)
				{
					$day_string = '0'.$day;
				}
				else
				{
					$day_string = $day;
				}
				if ($this->setup_array['month'] < 10)
				{
					$month_string = '0'.$this->setup_array['month'];
				}
				else
				{
					$month_string = $this->setup_array['month'];
				}

				if (!empty($user_data))
				{
					//Arbeitstag
					if (!isset($user_data['holiday'][$this->setup_array['year'].'-'.$month_string.'-'.$day_string]))
					{
						$return_string .= '<td width="3%">&nbsp;</td>';
					}
					//kein Arbeitstag
					else
					{
						$color = '';
						if ($user_data['holiday'][$this->setup_array['year'].'-'.$month_string.'-'.$day_string] > 0)
						{
							$color = $this->get_absence_color($user_data['holiday'][$this->setup_array['year'].'-'.$month_string.'-'.$day_string]);

							if (!empty($color))
							{
								$color = ' style="background-color:#'.$color.';"';
							}
						}

						$status = '&times;';
						if (isset($user_data['status'][$this->setup_array['year'].'-'.$month_string.'-'.$day_string]))
						{
							if ($user_data['status'][$this->setup_array['year'].'-'.$month_string.'-'.$day_string] == 'approved')
							{
								$status = '&nbsp;';
							}
							elseif ($user_data['status'][$this->setup_array['year'].'-'.$month_string.'-'.$day_string] == 'no_submitted')
							{
								$status = '&loz;';
							}
						}

						$return_string .= '<td width="3%"'.$color.'>'.$status.'</td>';
					}
				}
				else
				{
					if ($as_id == 0)
					{
						$return_string .= '<td width="3%">&nbsp;</td>';
					}
					else
					{
						if ($as_status == 'approved')
						{
							$status = '&nbsp;';
						}
						elseif ($as_status == 'no_submitted')
						{
							$status = '&loz;';
						}
						else
						{
							$status = '&times;';
						}

						$color = '';

						$color = $this->get_absence_color($as_id);

						if (!empty($color))
						{
							$color = ' style="background-color:#'.$color.';"';
						}

						$return_string .= '<td width="3%"'.$color.'>'.$status.'</td>';
					}
				}
			}
			else
			{
				$return_string .= '<td width="3%">&nbsp;</td>';
			}
			return $return_string;
		}

		/**
		 * Gibt die Farbe zu einem Abwesenheitsgrund zurueck
		 * @param int $as_id
		 * @return string
		 */
		public function get_absence_color($as_id)
		{
			$cdb = new MySql();

			$sql = $cdb->select('SELECT color FROM hdb_absence_type WHERE absence_type_id='.$as_id);
			$result = $cdb->fetch_array($sql);

			return $result['color'];
		}

		/**
		 * Erstellt die html ausgabe fuer die Kopfzeile eines jeden Monats
		 *
		 * @param int $day
		 * @return string
		 */
		private function get_day_td($day)
		{
			$kal = new Kalender;

			$return_string = '';

			if ($day <= $this->setup_array['days'])
			{
				if ($day < 10)
				{
					$day_string = '0'.$day;
				}
				else
				{
					$day_string = $day;
				}
				if ($this->setup_array['month'] < 10)
				{
					$month_string = '0'.$this->setup_array['month'];
				}
				else
				{
					$month_string = $this->setup_array['month'];
				}

				//wochenende
				if ($kal->is_weekend($this->setup_array['year'].'-'.$month_string.'-'.$day_string) === true)
				{
					$return_string .= '<td width="3%" class="stz_freeday_bg">'.$this->get_day_name($day).'<br />'.$day.'</td>';
				}
				//Feiertag
				elseif ($kal->is_holiday($this->setup_array['year'].'-'.$month_string.'-'.$day_string) === true)
				{
					$return_string .= '<td width="3%" class="stz_holiday_bg">'.$this->get_day_name($day).'<br />'.$day.'</td>';
				}
				//Normaler Werktag
				else
				{
					$return_string .= '<td width="3%">'.$this->get_day_name($day).'<br />'.$day.'</td>';
				}
			}
			else
			{
				$return_string .= '<td width="3%">&nbsp;</td>';
			}
			return $return_string;
		}

		/**
		 * Erstellt ein Array mit Userinformationen ueber beantragbare abwesenheitsgruende
		 * Die Klasse "MySql" wird fuer die Datenbankabfragen eingebunden.
		 * Die Klasse "Kalender" wird fuer die Datumsberechnungen eingebunden.
		 * @param int $user_id
		 * @return Array
		 */
		private function get_user_data($user_id = 0)
		{
			$cdb = new MySql();
			$kal = new Kalender();

			if ($this->setup_array['month'] < 10)
			{
				$year_month_string = $this->setup_array['year'].'-0'.$this->setup_array['month'];
				$year_month_int = $this->setup_array['year'].'0'.$this->setup_array['month'];
			}
			else
			{
				$year_month_string = $this->setup_array['year'].'-'.$this->setup_array['month'];
				$year_month_int = $this->setup_array['year'].$this->setup_array['month'];
			}

			$user_array = array();

			$tmp_where = '';
			if ($user_id > 0)
			{
				$tmp_where = "user_id=$user_id AND";
			}

			$sql = $cdb->select("SELECT user_id, vorname, nachname FROM hdb_user WHERE $tmp_where anzeigen='j' ORDER BY sort_name ASC");
			while ($result = $cdb->fetch_array($sql))
			{
				$sub_sql = $cdb->select('SELECT t1.datum, t1.absence_id FROM hdb_stz t1
									LEFT JOIN hdb_absence_type t2 ON t1.absence_id=t2.absence_type_id
									WHERE t1.user_id='.$result['user_id']." AND LEFT(t1.datum, 7)='".$year_month_string."' AND t1.absence_id>0 AND t2.application=1");
				while ($sub_result = $cdb->fetch_array($sub_sql))
				{
					if (!isset($user_array[$result['user_id']]))
					{
						$user_array[$result['user_id']] = array('user_id'	=> $result['user_id'],
																'vorname'	=> $result['vorname'],
																'nachname'	=> $result['nachname'],
																'holiday'	=> array($sub_result['datum'] => $sub_result['absence_id']),
																'status'	=> array($sub_result['datum'] => 'approved'));
					}
					else
					{
						$user_array[$result['user_id']]['holiday'][$sub_result['datum']] = $sub_result['absence_id'];
						$user_array[$result['user_id']]['status'][$sub_result['datum']] = 'approved';
					}
				}

				if ($year_month_int >= date('Ym'))
				{
					if (isset($this->setup_array['options']['show_open_status']) and $this->setup_array['options']['show_open_status'] == true)
					{
						$where = "WHERE t1.user_id=".$result['user_id']." AND LEFT(t1.date_from, 7)<='$year_month_string' AND LEFT(t1.date_to, 7)>='$year_month_string' AND (t1.status='approved' OR t1.status='open') AND t2.application=1";
					}
					else
					{
						$where = "WHERE t1.user_id=".$result['user_id']." AND LEFT(t1.date_from, 7)<='$year_month_string' AND LEFT(t1.date_to, 7)>='$year_month_string' AND t1.status='approved' AND t2.application=1";
					}

					//Daten aus abwesenheitsantraegen auslesen auslesen
					$sql_date = $cdb->select("SELECT t1.absence_type_id, t1.date_from, t1.date_to, t1.status,
											(
											SELECT CASE WHEN t1.status='approved' THEN 1
											WHEN t1.status='open' THEN 2
											WHEN t1.status='rejected' THEN 3
											ELSE 4 END
											) AS tbl_sort
											FROM hdb_absence_list t1
											LEFT JOIN hdb_absence_type t2 ON t1.absence_type_id=t2.absence_type_id
											$where ORDER BY tbl_sort ASC");
					while ($result_date = $cdb->fetch_array($sql_date))
					{
						//nur ein tag abwesend
						if ($result_date['date_from'] == $result_date['date_to'])
						{
							$date_array = array();
							$date_array[0]['iso'] = $result_date['date_from'];
						}
						//mehrere tage abwesend
						else
						{
							$date_array = $kal->get_difference($result_date['date_from'], $result_date['date_to']);
						}

						for ($i = 0; $i < count($date_array); $i++)
						{
							if (substr($date_array[$i]['iso'], 0, 7) == $year_month_string)
							{
								if (!isset($user_array[$result['user_id']]))
								{
									$user_array[$result['user_id']] = array('user_id'	=> $result['user_id'],
																			'vorname'	=> $result['vorname'],
																			'nachname'	=> $result['nachname'],
																			'holiday'	=> array($date_array[$i]['iso'] => $result_date['absence_type_id']),
																			'status'	=> array($date_array[$i]['iso'] => $result_date['status']));
								}
								else
								{
									if (!isset($user_array[$result['user_id']]['holiday'][$date_array[$i]['iso']]))
									{
										$user_array[$result['user_id']]['holiday'][$date_array[$i]['iso']] = $result_date['absence_type_id'];
										$user_array[$result['user_id']]['status'][$date_array[$i]['iso']] = $result_date['status'];
									}
									elseif ($user_array[$result['user_id']]['status'][$date_array[$i]['iso']] != 'approved' and $result_date['status'] == 'approved')
									{
										$user_array[$result['user_id']]['holiday'][$date_array[$i]['iso']] = $result_date['absence_type_id'];
										$user_array[$result['user_id']]['status'][$date_array[$i]['iso']] = $result_date['status'];
									}
								}
							}
						}
					}
				}
			}

			if (isset($this->request_data) and !empty($this->request_data))
			{
				foreach ($this->request_data['dates'] as $date)
				{
					if (!isset($user_array[$this->request_data['user_id']]))
					{
						$user_array[$this->request_data['user_id']] = array('user_id'	=> $this->request_data['user_id'],
																			'vorname'	=> $this->request_data['vorname'],
																			'nachname'	=> $this->request_data['nachname'],
																			'holiday'	=> array($date => $this->request_data['absence_type_id']),
																			'status'	=> array($date => 'no_submitted'));
					}
					elseif (!isset($user_array[$this->request_data['user_id']]['holiday'][$date]))
					{
						$user_array[$this->request_data['user_id']]['holiday'][$date] = $this->request_data['absence_type_id'];
						$user_array[$this->request_data['user_id']]['status'][$date] = 'no_submitted';
					}
				}
			}

			//print_r_pre($user_array);
			return $user_array;
		}

		/**
		 * Gibt die kurzform vom Tagesnamen aus der DB aus
		 *
		 * @param int $day
		 * @return string
		 */
		private function get_day_name($day)
		{
			$day_array = array('so_kurz', 'mo_kurz', 'di_kurz', 'mi_kurz', 'do_kurz', 'fr_kurz', 'sa_kurz');
			$day_number = date('w', mktime(0, 0, 0, $this->setup_array['month'], $day, $this->setup_array['year']));

			if (isset($day_array[$day_number]))
			{
				return get_text('', 'return', 'decode_on', $day_array[$day_number]);
			}
			else
			{
				return '';
			}
		}

		/**
		 * Gibt den Monatsnamen zurueck
		 *
		 * @return string
		 */
		private function get_month_name()
		{
			$month_array = array(1 => 'jan',
								2 => 'feb',
								3 => 'mar',
								4 => 'apr',
								5 => 'mai',
								6 => 'jun',
								7 => 'jul',
								8 => 'aug',
								9 => 'sep',
								10 => 'okt',
								11 => 'nov',
								12 => 'dez');

			if (isset($month_array[$this->setup_array['month']]))
			{
				return get_text('', 'return', 'decode_on', $month_array[$this->setup_array['month']]);
			}
			else
			{
				return '';
			}
		}

		/**
		 * Erstellt ein Aray mit allgemeinen Einstellungsinformationen
		 *
		 * @param int $year
		 * @param int $month
		 */
		private function get_month_setup($year, $month)
		{
			$this->setup_array['month'] = $month;
			$this->setup_array['year'] = $year;
			$this->setup_array['days'] = date('t', mktime(0, 0, 0, $month, 1, $year));
		}
	}
?>