<?php
	/**
	 * main class for all date- and calendar-classes, other classes should inherit from this parent class using the extends keyword
	 *
	 * @author Soeren Pieper <soeren.pieper@docemos.de> / Martin Ufer <martin.ufer@docemos.de>
	 * @copyright Copyright (C) 2009  docemos GmbH
	 * @package time-o-meter
	 *
	 * This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 3 of the License, or (at your option) any later version.
	 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
	 * You should have received a copy of the GNU General Public License along with this program; if not, see http://www.gnu.org/licenses/.
	 */

	/**
	 * main class for all date- and calendar-classes, other classes should inherit from this parent class using the extends keyword
	 * @package time-o-meter
	 * @subpackage classes
	 */
	class Kalender
	{
		/**
		 * Speichert allgemeine Einstellungen
		 */
		public $setup_array = array();

		/**
		 * Speichert Temporaere Daten		 *
		 */
		private $tmp_array = array();

		public function __construct()
		{
			//Zeitzone definieren
			//FIXME: die folgende Zeile muss einkommentiert werden, wenn sie NICHT in den Grundeinstellungen des Projekts vorkommt
			//date_default_timezone_set('Europe/Berlin');

			//FIXME: Das Sollte aus den Grundeinstellungen kommen
			//Ansonsten koennen nachfolgende Datumsformate uebergeben werden
			//siehe http://de3.php.net/manual/de/function.date.php
			// d
			// m
			// Y
			if (isset($GLOBALS['setup_array']['date_format']) and !empty($GLOBALS['setup_array']['date_format']))
			{
				$this->setup_array['date_format']					= $GLOBALS['setup_array']['date_format'];
			}
			else
			{
				$this->setup_array['date_format']					= 'FREE';
			}

			//Allgemeine Datumsausgabe
			if (!empty($this->setup_array['date_format']))
			{
				//FREE ist ein pseudonym fuer das Deutsche Datumsformat z.B. 15.03.2008
				if (strtoupper($this->setup_array['date_format']) == 'FREE')
				{
					$this->setup_array['date_format'] 				= 'd.m.Y';
				}
				//ISO ist ein pseudonym fuer das ISO Datumsformat z.B. 2008-03-15
				elseif (strtoupper($this->setup_array['date_format']) == 'ISO')
				{
					$this->setup_array['date_format']				= 'Y-m-d';
				}
			}
			//sollte nichts definiert sein wird standardmaessig ISO geladen
			else
			{
				$this->setup_array['date_format'] 					= 'Y-m-d';
			}

			$this->setup_array['current_timestamp']					= time();
			$this->setup_array['current_date_iso']					= date('Y-m-d', $this->setup_array['current_timestamp']);
			$this->setup_array['current_date']						= date($this->setup_array['date_format'], $this->setup_array['current_timestamp']);
			$this->setup_array['current_day']						= date('d', $this->setup_array['current_timestamp']);
			$this->setup_array['current_day_number']				= date('w', $this->setup_array['current_timestamp']);
			$this->setup_array['days_in_month']						= date('t', $this->setup_array['current_timestamp']);
			$this->setup_array['current_week_number']				= date('W', $this->setup_array['current_timestamp']);
			$this->setup_array['current_month']						= date('m', $this->setup_array['current_timestamp']);
			$this->setup_array['current_year']						= date('Y', $this->setup_array['current_timestamp']);
			$this->setup_array['current_win_timestamp']				= date('YmdHis', $this->setup_array['current_timestamp']);

			//Wochentage an denen Wochenende ist
			//ACHTUNG: Sonntag ist 7 NICHT 0
			$this->setup_array['weekend_days']						= array(
																			//1/*Montag*/,
																			//2/*Dienstag*/,
																			//3/*Mittwoch*/,
																			//4/*Donnerstag*/,
																			//5/*Freitag*/,
																			6/*Samstag*/,
																			7/*Sonntag*/
																			);

			//Kurzbezeichnungen der Monate (dienen als Key fuer die Texttabelle)
			$this->setup_array['month_array'] 						= array(
																			1 => 'jan',
																			2 => 'feb',
																			3 => 'mar',
																			4 => 'apr',
																			5 => 'mai',
																			6 => 'jun',
																			7 => 'jul',
																			8 => 'aug',
																			9 => 'sep',
																			10 => 'okt',
																			11 => 'nov',
																			12 => 'dez'
																			);

			//Speichert Feiertage
			//Der Index ist das ISO Datum als Value dient die Bezeichnuns z.B. Weihnachtstag
			$this->setup_array['holiday_date_name']					= array();
			//Speichert Feiertage
			//Der index ist nummerisch fortlaufend Value ist das ISO Datum des Feiertags
			$this->setup_array['holiday_date']						= array();

			//Speichert Halbe Arbeitstage
			//Der Index ist das ISO Datum als Value dient die Bezeichnuns z.B. Sylvester
			$this->setup_array['half_holiday_date_name']			= array();
			//Speichert Halbe Arbeitstage
			//Der index ist nummerisch fortlaufend Value ist das ISO Datum des Feiertags
			$this->setup_array['half_holiday_date']					= array();

			//Gibt an ob die Vordefinierten Feiertage bereits ausgelesen wurden
			//Die Vraiable verhindert ein staendiges Neuberechnen der Feiertage bei mehrfachem Aufruf einzelner Funktionen
			$this->setup_array['is_checked_predefined_holidays']	= false;
			//Speichert vordefinierte Feiertage
			//Der Index ist das ISO Datum als Value dient die Textkey der Sprachtabelle
			$this->setup_array['predefined_holiday_date_name']		= array();
			//Speichert vordefinierte Feiertage
			//Der index ist nummerisch fortlaufend Value ist das ISO Datum des Feiertags
			$this->setup_array['predefined_holiday_date']			= array();
		}

		/**
		 * Konvertiert ein Datum vom Aktuell eingestellten Datumsformat ins ISO Format
		 *
		 * @param string $date
		 * @return string
		 */
		public function free_to_iso($date)
		{
			$result = $this->explode_date($date, 'd.m.Y');

			if ($result !== false)
			{
				return $result[0].'-'.$result[1].'-'.$result[2];
			}
			else
			{
				return '0000-00-00';
			}
		}

		/**
		 * Konvertiert ein Datum von d.m.Y zu Y-m-d
		 *
		 * @param string $date
		 * @return string
		 */
		public function iso_to_free($date)
		{
			$result = $this->explode_date($date, 'Y-m-d');

			if ($result !== false)
			{
				//Datumsformat
				$tmp_date = $this->setup_array['date_format'];
				//Jahr
				$tmp_date = str_replace('Y', $result[0], $tmp_date);
				//Monat
				$tmp_date = str_replace('m', $result[1], $tmp_date);
				//Tag
				$tmp_date = str_replace('d', $result[2], $tmp_date);
				return $tmp_date;
			}
			else
			{
				return '';
			}
		}

		/**
		 * Wandelt einen Windowstimestamp in ein beliebig anderes Fromat
		 *
		 * @param string $win_time
		 * @param string $format
		 * @param int $sub_start
		 * @param int $sub_len
		 * @return string
		 */
		public function win_to_time($win_time, $format = 'all', $sub_start = 0, $sub_len = 0)
		{
			$return = '';
			if (!empty($win_time))
			{
				$Y = substr($win_time, 0, 4);
				$m = substr($win_time, 4, 2);
				$d = substr($win_time, 6, 2);
				$H = substr($win_time, 8, 2);
				$i = substr($win_time, 10, 2);
				$s = substr($win_time, 12, 2);

				//Datum und Uhrzeit
				if ($format == 'all')
				{
					$return = $this->iso_to_free($Y.'-'.$m.'-'.$d);
					$return .= ' '.$H.':'.$i.':'.$s;
				}
				//Datum
				elseif ($format == 'date')
				{
					$return = $this->iso_to_free($Y.'-'.$m.'-'.$d);
				}
				//Uhrzeit
				elseif ($format == 'time')
				{
					$return = $H.':'.$i.':'.$s;
				}
				//Unixtimestamp
				elseif ($format == 'unix')
				{
					$return = mktime($H, $i, $s, $m, $d, $Y);
				}

				if ($sub_start != 0 and $sub_len != 0)
				{
					return substr($return, $sub_start, $sub_len);
				}
				else
				{
					return $return;
				}
			}
			else
			{
				return $return;
			}
		}

		/**
		 * Erstellt aus dem eingestellten Datumsformat einen bezeichnenden Textstring
		 * z.B. Y-m-d -> JJJJ-MM-DD
		 *
		 * @return string
		 */
		public function get_date_format()
		{
			if (!empty($this->setup_array['date_format']))
			{
				$search_array = array('d', 'm', 'Y');
				//						DD				MM			JJJJ
				$replace_array = array(get_text(591, 'return'), get_text(592, 'return'), get_text(593, 'return'));

				return str_replace($search_array, $replace_array, $this->setup_array['date_format']);
			}
			else
			{
				return '';
			}
		}

		/**
		 * Gibt diverse Informationen zum uebergebenen Datum zurueck
		 *
		 * @param string $date
		 * @param string $date_format, uebergebenes Datumsformat. Ist $date_format empty wird standardmaessig $setup_array['date_format'] verwendet
		 * @return array
		 */
		public function read_date_info($date, $date_format = '')
		{
			$return_array = array();

			//uebergebenes Datumsformat festlegen
			if (strtoupper($date_format) == 'FREE')
			{
				$date_format = 'd.m.Y';
			}
			elseif (strtoupper($date_format) == 'ISO')
			{
				$date_format = 'Y-m-d';
			}
			elseif (empty($date_format))
			{
				$date_format = $this->setup_array['date_format'];
			}

			$date_result = $this->explode_date($date, $date_format);

			if ($date_result !== false)
			{

				$day = $date_result[2];
				$month = $date_result[1];
				$year = $date_result[0];

				$date_timestamp = mktime(0, 0, 0, $month, $day, $year);
				$date_new = date($date_format, $date_timestamp);

				//Wenn keine Fehler aufgetretten sind sollte hier genau das selbe raus kommen
				if ($date == $date_new)
				{
					$return_array['date']			= $date_new;
					$return_array['timestamp']		= $date_timestamp;
					$return_array['iso']			= date('Y-m-d', $date_timestamp);
					$return_array['day']			= date('d', $date_timestamp);
					$return_array['day_number']		= date('w', $date_timestamp);
					$return_array['days_in_month']	= date('t', $date_timestamp);
					$return_array['week_number']	= date('W', $date_timestamp);
					$return_array['month']			= date('m', $date_timestamp);
					$return_array['year']			= date('Y', $date_timestamp);
					$return_array['win_timestamp']	= date('YmdHis', $date_timestamp);

					return $return_array;
				}
				else
				{
					return $return_array;
				}
			}
			else
			{
				return $return_array;
			}
		}

		/**
		 * Zerlegt ein Datum in seine Bestandteile
		 *
		 * @param string $date
		 * @param string $date_format, Formatstring des uebergebenen Datums
		 * @return mixed uebergibt false im Fehlerfall und array($year, $month, $day) bei erfolg
		 */
		public function explode_date($date, $date_format)
		{
			$day = '';
			$month = '';
			$year = '';
			$tmp_date = $date;
			//Tag, Monat, Jahr suchen
			for ($i = 0; $i < strlen($date_format); $i++)
			{
				if ($date_format[$i] == 'd')
				{
					$day = substr($tmp_date, 0, 2);
					$tmp_date = substr($tmp_date, 2);
				}
				elseif ($date_format[$i] == 'm')
				{
					$month = substr($tmp_date, 0, 2);
					$tmp_date = substr($tmp_date, 2);
				}
				elseif ($date_format[$i] == 'Y')
				{
					$year = substr($tmp_date, 0, 4);
					$tmp_date = substr($tmp_date, 4);
				}
				else
				{
					$tmp_date = substr($tmp_date, 1);
				}
			}

			//pruefen ob Tag, Monats und Jahresangaben vorhanden
			if (!empty($day) and !empty($month) and !empty($year))
			{
				//Datumswerte pruefen
				if (@checkdate($month, $day, $year))
				{
					return array($year, $month, $day);
				}
			}
			return false;
		}

		/**
		 * prueft ob an dem uebergebenen Datum ein Feiertag ist
		 *
		 * @param string $date, ISO Datum
		 * @return boole
		 */
		public function is_holiday($date)
		{
			//Es existiert bereits ein Feiertag im Array
			if (isset($this->setup_array['holiday_date_name'][$date]))
			{
				return true;
			}
			//Suche nach einem Feiertag aus der DB
			elseif ($this->get_db_holidays($date))
			{
				return true;
			}

			return false;
		}

		/**
		 * prueft ob an dem uebergeben Datum ein Halber Arbeitstag ist
		 *
		 * @param string $date, ISO Datum
		 * @return boole
		 */
		public function is_half_holiday($date)
		{
			//Es existiert bereits ein Feiertag im Array
			if (isset($this->setup_array['half_holiday_date_name'][$date]))
			{
				return true;
			}
			//Suche nach einem Feiertag aus der DB
			elseif ($this->get_db_half_holidays($date))
			{
				return true;
			}

			return false;
		}

		/**
		 * pruefen ob Wochenende ist
		 *
		 * @param string $date, ISO Datum
		 * @return boole
		 */
		public function is_weekend($date, $day_number = 'NULL')
		{
			if ($day_number != 'NULL')
			{
				//Sonntag in 7 aendern
				if ($day_number == 0)
				{
					$day_number = 7;
				}

				if (in_array($day_number, $this->setup_array['weekend_days']))
				{
					return true;
				}
			}
			elseif (!empty($date))
			{
				$date_info = $this->read_date_info($date, 'ISO');
				//Sonntag in 7 aendern
				if ($date_info['day_number'] == 0)
				{
					$date_info['day_number'] = 7;
				}

				if (in_array($date_info['day_number'], $this->setup_array['weekend_days']))
				{
					return true;
				}
			}
			return false;
		}

		/**
		 * prueft ob das uebergebene Datum ein ISO Datum ist. Liefert bei erfolg das Original ISO Datum zurueck ansonsten false
		 *
		 * @param string $date
		 * @return mixed
		 */
		public function is_iso_date($date)
		{
			if (strlen($date) == 10)
			{
				$year = intval(substr($date, 0, 4));
				$month = substr($date, 5, 2);
				$day = substr($date, 8, 2);
				if (@checkdate($month, $day, $year))
				{
					return $date;
				}
				else
				{
					return false;
				}
			}
			else
			{
				return false;
			}
		}

		/**
		 * Addiert oder Subtraiert Tage, Monate oder Jahre von einem Datum
		 *
		 * @param string $date
		 * @param int $year
		 * @param int $month
		 * @param int $day
		 * @return array
		 */
		public function date_calculator($date, $year, $month, $day)
		{
			//pruefen ob ein datum uebergeben wurde wenn nicht aktuelles verwenden
			if (empty($date))
			{
				$date = $this->setup_array['current_date_iso'];
			}
			//pruefen ob das uebergebene Datum ein ISO Datum ist
			if ($this->is_iso_date($date) !== false)
			{
				//Allgemeine Informationen zum uebergebenen Datum auslesen
				$date_info_array = $this->read_date_info($date, 'ISO');

				//pruefen ob ein Jahr angegeben wurde
				if (!empty($year))
				{
					$new_year = $date_info_array['year'] + intval($year);
				}
				else
				{
					$new_year = $date_info_array['year'];
				}

				//pruefen ob ein Monat angegeben wurde
				if (!empty($month))
				{
					$new_month = $date_info_array['month'] + intval($month);
				}
				else
				{
					$new_month = $date_info_array['month'];
				}

				//pruefen ob ein Tag angegeben wurde
				if (!empty($day))
				{
					$new_day = $date_info_array['day'] + intval($day);
				}
				else
				{
					$new_day = $date_info_array['day'];
				}
				//aus den geaenderten datumsangaben einen neuen timestamp erstellen
				$new_timestamp = mktime(0, 0, 0, $new_month, $new_day, $new_year);

				//neues datum erstellen
				$new_date_array = array();
				$new_date_array[0] = date('Y-m-d', $new_timestamp);
				$new_date_array[1] = $new_timestamp;
				$new_date_array[2] = $this->read_date_info($new_date_array[0], 'ISO');

				return $new_date_array;
			}
			return array('0000-00-00', '', '');
		}

		/**
		 * Erstellt ein Array mit Datumswerten die zwischen den zwei uebergebenen Datumswerten liegen
		 *
		 * @param string $start
		 * @param string $end
		 * @return array
		 */
		public function get_difference($start, $end)
		{
			$return_array = array();

			$tmp_date = $start;
			$while = true;

			while ($while === true)
			{
				$date_info_array = $this->read_date_info($tmp_date, 'ISO');

				$return_array[] = array('iso'	=> $date_info_array['iso'],
										'free'	=> $date_info_array['date'],
										'stamp'	=> $date_info_array['timestamp']);

				$next_date = $this->date_calculator($tmp_date, '', '', 1);

				$tmp_date = $next_date[0];

				if (str_replace('-', '', $tmp_date) > str_replace('-', '', $end))
				{
					$while = false;
				}
			}
			return $return_array;
		}

		/**
		 * Gibt zu einem Key einen Text zurueck
		 *
		 * @param mixed $key
		 * @return string
		 */
		public function get_kalender_text($key, $text_typ = '')
		{
			if (empty($text_typ))
			{
				if (is_numeric($key))
				{
					return get_text($key, 'return');
				}
				else
				{
					return get_text('', 'return', 'decode_on', $key);
				}
			}
			elseif ($text_typ == 'month_name')
			{
				$month = intval($key);

				if ($month > 0 and $month <= 12)
				{
					return get_text('', 'return', 'decode_on', $this->setup_array['month_array'][$month]);
				}
			}
			return '';
		}

		/**
		 * Sucht zu einem Datum einen Halben Feiertag aus der DB
		 *
		 * @param string $date
		 * @param boole $return_boole
		 * @return mixed
		 */
		private function get_db_half_holidays($date, $return_boole = true)
		{
			$cdb = new MySql;

			//Archive pruefen
			$sql = $cdb->select("SELECT ft_id, datum, bezeichnung, default_array_key, type, halbe_sollstunden FROM hdb_feiertage_archiv WHERE datum='$date' AND halbe_sollstunden='j'");
			$result = $cdb->fetch_array($sql);
			//Archive hat einen eintrag
			if (isset($result['datum']) and !empty($result['datum']))
			{
				if (!empty($result['default_array_key']))
				{
					$tmp_holiday_name = $this->get_kalender_text($result['default_array_key']);
				}
				else
				{
					$tmp_holiday_name = $result['bezeichnung'];
				}

				//Datum in Array aufnehmen, eine nochmalige DB Abfrage entfaellt damit
				$this->setup_array['half_holiday_date_name'][$result['datum']] = $tmp_holiday_name;
				$this->setup_array['half_holiday_date'][] = $result['datum'];

				if ($return_boole == true)
				{
					return true;
				}
				else
				{
					return array('ft_id'			=> $result['ft_id'],
								'bezeichnung'		=> $result['bezeichnung'],
								'default_array_key'	=> $result['default_array_key'],
								'type'				=> $result['type'],
								'halbe_sollstunden'	=> $result['halbe_sollstunden']);
				}
			}
			//Feiertags DB abfragen
			else
			{
				$tmp_date_array = $this->explode_date($date, 'Y-m-d');
				$year = $tmp_date_array[0];
				$month = $tmp_date_array[1];
				$day = $tmp_date_array[2];

				//Einen Feiertag aus der DB auslesen der ein Festes Datum hat
				$sql = $cdb->select("SELECT ft_id, bezeichnung, default_array_key, type, halbe_sollstunden FROM hdb_feiertage WHERE jahr='$year' AND monat='$month' AND tag='$day' AND halbe_sollstunden='j'");
				$result = $cdb->fetch_array($sql);

				if (isset($result['ft_id']) and !empty($result['ft_id']))
				{
					if (!empty($result['default_array_key']))
					{
						$tmp_holiday_name = $this->get_kalender_text($result['default_array_key']);
					}
					else
					{
						$tmp_holiday_name = $result['bezeichnung'];
					}

					//Datum in Array aufnehmen, eine nochmalige DB Abfrage entfaellt damit
					$this->setup_array['half_holiday_date_name'][$date] = $tmp_holiday_name;
					$this->setup_array['half_holiday_date'][] = $date;

					if ($return_boole == true)
					{
						return true;
					}
					else
					{
						return array('ft_id'			=> $result['ft_id'],
									'bezeichnung'		=> $result['bezeichnung'],
									'default_array_key'	=> $result['default_array_key'],
									'type'				=> $result['type'],
									'halbe_sollstunden'	=> $result['halbe_sollstunden']);
					}
				}

				//Einen Feiertag aus der DB auslesen der immer zu einem bestimmten Tag im Jahr stattfindet
				$sql = $cdb->select("SELECT ft_id, bezeichnung, default_array_key, type, halbe_sollstunden FROM hdb_feiertage WHERE monat='$month' AND tag='$day' AND halbe_sollstunden='j'");
				$result = $cdb->fetch_array($sql);

				if (isset($result['ft_id']) and !empty($result['ft_id']))
				{
					if (!empty($result['default_array_key']))
					{
						$tmp_holiday_name = $this->get_kalender_text($result['default_array_key']);
					}
					else
					{
						$tmp_holiday_name = $result['bezeichnung'];
					}

					//Datum in Array aufnehmen, eine nochmalige DB Abfrage entfaellt damit
					$this->setup_array['half_holiday_date_name'][$date] = $tmp_holiday_name;
					$this->setup_array['half_holiday_date'][] = $date;

					if ($return_boole == true)
					{
						return true;
					}
					else
					{
						return array('ft_id'			=> $result['ft_id'],
									'bezeichnung'		=> $result['bezeichnung'],
									'default_array_key'	=> $result['default_array_key'],
									'type'				=> $result['type'],
									'halbe_sollstunden'	=> $result['halbe_sollstunden']);
					}
				}

				//Vordefinierte Feiertage in Array Laden
				$this->get_predefined_holidays($year);

				//Alle Feiertage pruefen die con den Vordefinierten Feiertagen kommen
				$sql = $cdb->select("SELECT ft_id, bezeichnung, default_array_key, type, halbe_sollstunden FROM hdb_feiertage WHERE default_array_key IS NOT NULL AND halbe_sollstunden='j'");
				while ($result = $cdb->fetch_array($sql))
				{
					if (isset($this->setup_array['predefined_holiday_date_name'][$date]) and in_array($result['default_array_key'], $this->setup_array['predefined_holiday_date_name']))
					{
						$this->setup_array['half_holiday_date_name'][$date] = $this->get_kalender_text($result['default_array_key']);
						$this->setup_array['half_holiday_date'][] = $date;
						if ($return_boole == true)
						{
							return true;
						}
						else
						{
							return array('ft_id'			=> $result['ft_id'],
										'bezeichnung'		=> $result['bezeichnung'],
										'default_array_key'	=> $result['default_array_key'],
										'type'				=> $result['type'],
										'halbe_sollstunden'	=> $result['halbe_sollstunden']);
						}
					}
				}
			}
			if ($return_boole == false)
			{
				return true;
			}
			else
			{
				return array();
			}
		}

		/**
		 * Sucht zu einem Datum einen Feiertag aus der DB
		 *
		 * @param string $date
		 * @param boole $return_boole
		 * @return mixed
		 */
		private function get_db_holidays($date, $return_boole = true)
		{
			$cdb = new MySql;

			//Archive pruefen
			$sql = $cdb->select("SELECT ft_id, datum, bezeichnung, default_array_key, type, halbe_sollstunden FROM hdb_feiertage_archiv WHERE datum='$date' AND halbe_sollstunden='n'");
			$result = $cdb->fetch_array($sql);
			//Archive hat einen eintrag
			if (isset($result['datum']) and !empty($result['datum']))
			{
				if (!empty($result['default_array_key']))
				{
					$tmp_holiday_name = $this->get_kalender_text($result['default_array_key']);
				}
				else
				{
					$tmp_holiday_name = $result['bezeichnung'];
				}

				//Datum in Array aufnehmen, eine nochmalige DB Abfrage entfaellt damit
				$this->setup_array['holiday_date_name'][$result['datum']] = $tmp_holiday_name;
				$this->setup_array['holiday_date'][] = $result['datum'];

				if ($return_boole == true)
				{
					return true;
				}
				else
				{
					return array('ft_id'			=> $result['ft_id'],
								'bezeichnung'		=> $result['bezeichnung'],
								'default_array_key'	=> $result['default_array_key'],
								'type'				=> $result['type'],
								'halbe_sollstunden'	=> $result['halbe_sollstunden']);
				}
			}
			//Feiertags DB abfragen
			else
			{
				$tmp_date_array = $this->explode_date($date, 'Y-m-d');
				$year = $tmp_date_array[0];
				$month = $tmp_date_array[1];
				$day = $tmp_date_array[2];

				//Einen Feiertag aus der DB auslesen der ein Festes Datum hat
				$sql = $cdb->select("SELECT ft_id, bezeichnung, default_array_key, type, halbe_sollstunden FROM hdb_feiertage WHERE jahr='$year' AND monat='$month' AND tag='$day' AND halbe_sollstunden='n'");
				$result = $cdb->fetch_array($sql);

				if (isset($result['ft_id']) and !empty($result['ft_id']))
				{
					if (!empty($result['default_array_key']))
					{
						$tmp_holiday_name = $this->get_kalender_text($result['default_array_key']);
					}
					else
					{
						$tmp_holiday_name = $result['bezeichnung'];
					}

					//Datum in Array aufnehmen, eine nochmalige DB Abfrage entfaellt damit
					$this->setup_array['holiday_date_name'][$date] = $tmp_holiday_name;
					$this->setup_array['holiday_date'][] = $date;

					if ($return_boole == true)
					{
						return true;
					}
					else
					{
						return array('ft_id'			=> $result['ft_id'],
									'bezeichnung'		=> $result['bezeichnung'],
									'default_array_key'	=> $result['default_array_key'],
									'type'				=> $result['type'],
									'halbe_sollstunden'	=> $result['halbe_sollstunden']);
					}
				}

				//Einen Feiertag aus der DB auslesen der immer zu einem bestimmten Tag im Jahr stattfindet
				$sql = $cdb->select("SELECT ft_id, bezeichnung, default_array_key, type, halbe_sollstunden FROM hdb_feiertage WHERE monat='$month' AND tag='$day' AND halbe_sollstunden='n'");
				$result = $cdb->fetch_array($sql);

				if (isset($result['ft_id']) and !empty($result['ft_id']))
				{
					if (!empty($result['default_array_key']))
					{
						$tmp_holiday_name = $this->get_kalender_text($result['default_array_key']);
					}
					else
					{
						$tmp_holiday_name = $result['bezeichnung'];
					}

					//Datum in Array aufnehmen, eine nochmalige DB Abfrage entfaellt damit
					$this->setup_array['holiday_date_name'][$date] = $tmp_holiday_name;
					$this->setup_array['holiday_date'][] = $date;

					if ($return_boole == true)
					{
						return true;
					}
					else
					{
						return array('ft_id'			=> $result['ft_id'],
									'bezeichnung'		=> $result['bezeichnung'],
									'default_array_key'	=> $result['default_array_key'],
									'type'				=> $result['type'],
									'halbe_sollstunden'	=> $result['halbe_sollstunden']);
					}
				}

				//Vordefinierte Feiertage in Array Laden
				$this->get_predefined_holidays($year);

				//Alle Feiertage pruefen die von den Vordefinierten Feiertagen kommen
				$sql = $cdb->select("SELECT ft_id, bezeichnung, default_array_key, type, halbe_sollstunden FROM hdb_feiertage WHERE default_array_key IS NOT NULL AND halbe_sollstunden='n'");
				while ($result = $cdb->fetch_array($sql))
				{
					if (isset($this->setup_array['predefined_holiday_date_name'][$date]) and $result['default_array_key'] == $this->setup_array['predefined_holiday_date_name'][$date])
					{
						$this->setup_array['holiday_date_name'][$date] = $this->get_kalender_text($result['default_array_key']);
						$this->setup_array['holiday_date'][] = $date;
						if ($return_boole == true)
						{
							return true;
						}
						else
						{
							return array('ft_id'			=> $result['ft_id'],
										'bezeichnung'		=> $result['bezeichnung'],
										'default_array_key'	=> $result['default_array_key'],
										'type'				=> $result['type'],
										'halbe_sollstunden'	=> $result['halbe_sollstunden']);
						}
					}
				}
			}
			if ($return_boole == true)
			{
				return false;
			}
			else
			{
				return array();
			}
		}

		/**
		 * Erstellt ein Array mit allen Feritagen zu einem uebergebenen Jahr
		 *
		 * @param int $year
		 * @return array
		 */
		public function get_db_holidays_year($year)
		{
			// instanz der benoetigten mysql-klasse erzeugen
			$cdb = new MySql;

			$array_feiertage = array();

			// einmalige feiertage
			$sql = $cdb->select("SELECT * FROM hdb_feiertage WHERE type='unique' AND jahr='$year'");
			while ($result = $cdb->fetch_array($sql))
			{
				$temp_datum = $result['jahr'].'-'.$result['monat'].'-'.$result['tag'];
				$array_feiertage[$temp_datum] = array('datum' =>				$temp_datum,
														'bezeichnung' =>		$result['bezeichnung'],
														'ft_id' =>				$result['ft_id'],
														'type' =>				$result['type'],
														'halbe_sollstunden' =>	$result['halbe_sollstunden']);
			}

			// unbewegliche, jaehrlich wiederkehrende feiertage
			$sql = $cdb->select("SELECT * FROM hdb_feiertage WHERE type='annual'");
			while ($result = $cdb->fetch_array($sql))
			{
				$temp_datum = $year.'-'.$result['monat'].'-'.$result['tag'];
				$array_feiertage[$temp_datum] = array('datum' =>				$temp_datum,
														'bezeichnung' =>		$result['bezeichnung'],
														'ft_id' =>				$result['ft_id'],
														'type' =>				$result['type'],
														'halbe_sollstunden' =>	$result['halbe_sollstunden']);
			}

			//Vordefinierte Feiertage in Array Laden
			$this->get_predefined_holidays($year);

			$sql = $cdb->select("SELECT * FROM hdb_feiertage WHERE type='default'");
			while ($result = $cdb->fetch_array($sql))
			{
				if (in_array($result['default_array_key'], $this->setup_array['predefined_holiday_date_name']))
				{
					$temp_datum = array_search($result['default_array_key'], $this->setup_array['predefined_holiday_date_name']);
					$array_feiertage[$temp_datum] = array('datum' => $temp_datum, 'bezeichnung' => $this->get_kalender_text($result['default_array_key']), 'ft_id' => $result['ft_id'], 'type' => $result['type'], 'halbe_sollstunden' => $result['halbe_sollstunden']);
				}
			}

			// feiertage nach datum sortieren
			ksort($array_feiertage);

			return $array_feiertage;
		}

		/**
		 * prueft ob zum uebergebenen Jahr bereits ein Feiertag in der DB existiert
		 *
		 * @param int $year
		 * @return boole
		 */
		public function is_insert_db_holidays($year)
		{
			$cdb = new MySql;

			//Einen Feiertag aus der DB auslesen der ein Festes Datum hat
			$sql = $cdb->select("SELECT ft_id FROM hdb_feiertage WHERE jahr='$year' AND halbe_sollstunden='n'");
			$result = $cdb->fetch_array($sql);

			if (isset($result['ft_id']) and !empty($result['ft_id']))
			{
				return true;
			}

			//Einen Feiertag aus der DB auslesen der immer zu einem bestimmten Tag im Jahr stattfindet
			$sql = $cdb->select("SELECT ft_id FROM hdb_feiertage WHERE type='annual' AND halbe_sollstunden='n'");
			$result = $cdb->fetch_array($sql);

			if (isset($result['ft_id']) and !empty($result['ft_id']))
			{
				return true;
			}

			//Vordefinierte Feiertage in Array Laden
			$this->get_predefined_holidays($year);

			//Alle Feiertage pruefen die von den Vordefinierten Feiertagen kommen
			$sql = $cdb->select("SELECT ft_id FROM hdb_feiertage WHERE default_array_key IS NOT NULL AND halbe_sollstunden='n'");
			$result = $cdb->fetch_array($sql);

			if (isset($result['ft_id']) and !empty($result['ft_id']))
			{
				return true;
			}

			return false;
		}

		/**
		 * Erstellt ein Array mit allen Feiertagen die nicht extra in der Datenbank vorhanden sind
		 * z.B. Festefeiertage wie Weihnachtstag
		 *
		 * @param int $year
		 * @param string $country
		 * @return void
		 */
		public function get_predefined_holidays($year, $country = 'de')
		{
			if ($year >= 1970 and $year <= 2037)
			{
				//Die Vordefinierten Feiertage wurden noch nicht ausgelesen
				if ($this->setup_array['is_checked_predefined_holidays'] == false)
				{
					$this->setup_array['is_checked_predefined_holidays'] = true;

					// deutsche feiertage
					if (strtolower($country) == 'de')
					{
						// feste/unbewegliche Feiertage finden immer am gleichen Tag des Jahres statt
						$this->setup_array['predefined_holiday_date_name'][$year.'-01-01'] = 'neujahr';					//Neujahrstag
						$this->setup_array['predefined_holiday_date_name'][$year.'-01-06'] = 'holy3kings';				//Heilige Drei Koenige
						$this->setup_array['predefined_holiday_date_name'][$year.'-05-01'] = 'tag_der_arbeit';			//Tag der Arbeit
						$this->setup_array['predefined_holiday_date_name'][$year.'-08-15'] = 'maria_himmelfahrt';		//Maria Himmelfahrt
						$this->setup_array['predefined_holiday_date_name'][$year.'-10-03'] = 'tag_der_einheit';			//Tag der deutschen Einheit
						$this->setup_array['predefined_holiday_date_name'][$year.'-10-31'] = 'reformationstag';			//Reformationstag
						$this->setup_array['predefined_holiday_date_name'][$year.'-11-01'] = 'allerheiligen';			//Allerheiligen
						$this->setup_array['predefined_holiday_date_name'][$year.'-12-25'] = 'erster_weihnachtstag';	//1. Weihnachtsfeiertag
						$this->setup_array['predefined_holiday_date_name'][$year.'-12-26'] = 'zweiter_weihnachtstag';	//2. Weihnachtsfeiertag

						//Datum von Ostersonntag
						$easter_date = date('Y-m-d', $this->get_easter_date($year));
						// ostermontag = ostersonntag + 1 tag
						$temp_date_array = $this->date_calculator($easter_date, '', '', '1');
						$this->setup_array['predefined_holiday_date_name'][$temp_date_array[2]['iso']] = 'ostermontag';	//Ostermontag

						// karfreitag = Ostersonntag -2 Tage
						$temp_date_array = $this->date_calculator($easter_date, '', '', '-2');
						$this->setup_array['predefined_holiday_date_name'][$temp_date_array[2]['iso']] = 'karfreitag'; //Karfreitag

						// Christi Himmelfahrt = ostersonntag + 39 tage
						$temp_date_array = $this->date_calculator($easter_date, '', '', '39');
						$this->setup_array['predefined_holiday_date_name'][$temp_date_array[2]['iso']] = 'christi_himmelfahrt'; //Christi Himmelfahrt

						// Pfingstmontag = ostersonntag + 50 tage
						$temp_date_array = $this->date_calculator($easter_date, '', '', '50');
						$this->setup_array['predefined_holiday_date_name'][$temp_date_array[2]['iso']] = 'pfingstmontag'; //Pfingstmontag
					}

					//ArraySortieren
					ksort($this->setup_array['predefined_holiday_date_name']);

					//Array erzeugen welches nur das ISO Datum enthaelt
					foreach ($this->setup_array['predefined_holiday_date_name'] as $date => $holidayname)
					{
						$this->setup_array['predefined_holiday_date'][] = $date;
					}
				}
			}
		}

		/**
		 * Gibt den Unixtimestamp 0 Uhr fuer Ostern zurueck
		 * @param int $year
		 * @return int
		 */
		public function get_easter_date($year)
		{
			if (function_exists('easter_date'))
			{
				return easter_date($year);
			}
			else
			{
				$easter_array = array();
				$easter_array[1970] = 7513200;
				$easter_array[1971] = 40172400;
				$easter_array[1972] = 71017200;
				$easter_array[1973] = 104281200;
				$easter_array[1974] = 135126000;
				$easter_array[1975] = 165366000;
				$easter_array[1976] = 198630000;
				$easter_array[1977] = 229474800;
				$easter_array[1978] = 259714800;
				$easter_array[1979] = 292978800;
				$easter_array[1980] = 323823600;
				$easter_array[1981] = 356479200;
				$easter_array[1982] = 387324000;
				$easter_array[1983] = 418168800;
				$easter_array[1984] = 451432800;
				$easter_array[1985] = 481672800;
				$easter_array[1986] = 512521200;
				$easter_array[1987] = 545781600;
				$easter_array[1988] = 576021600;
				$easter_array[1989] = 606870000;
				$easter_array[1990] = 640130400;
				$easter_array[1991] = 670374000;
				$easter_array[1992] = 703634400;
				$easter_array[1993] = 734479200;
				$easter_array[1994] = 765324000;
				$easter_array[1995] = 797983200;
				$easter_array[1996] = 828828000;
				$easter_array[1997] = 859676400;
				$easter_array[1998] = 892332000;
				$easter_array[1999] = 923176800;
				$easter_array[2000] = 956440800;
				$easter_array[2001] = 987285600;
				$easter_array[2002] = 1017529200;
				$easter_array[2003] = 1050789600;
				$easter_array[2004] = 1081634400;
				$easter_array[2005] = 1111878000;
				$easter_array[2006] = 1145138400;
				$easter_array[2007] = 1175983200;
				$easter_array[2008] = 1206226800;
				$easter_array[2009] = 1239487200;
				$easter_array[2010] = 1270332000;
				$easter_array[2011] = 1303596000;
				$easter_array[2012] = 1333836000;
				$easter_array[2013] = 1364684400;
				$easter_array[2014] = 1397944800;
				$easter_array[2015] = 1428184800;
				$easter_array[2016] = 1459033200;
				$easter_array[2017] = 1492293600;
				$easter_array[2018] = 1522533600;
				$easter_array[2019] = 1555797600;
				$easter_array[2020] = 1586642400;
				$easter_array[2021] = 1617487200;
				$easter_array[2022] = 1650146400;
				$easter_array[2023] = 1680991200;
				$easter_array[2024] = 1711839600;
				$easter_array[2025] = 1745100000;
				$easter_array[2026] = 1775340000;
				$easter_array[2027] = 1806188400;
				$easter_array[2028] = 1839448800;
				$easter_array[2029] = 1869688800;
				$easter_array[2030] = 1902952800;
				$easter_array[2031] = 1933797600;
				$easter_array[2032] = 1964041200;
				$easter_array[2033] = 1997301600;
				$easter_array[2034] = 2028146400;
				$easter_array[2035] = 2058390000;
				$easter_array[2036] = 2091650400;
				$easter_array[2037] = 2122495200;

				if (isset($easter_array[$year]))
				{
					return $easter_array[$year];
				}
				else
				{
					return 0;
				}
			}
		}

		/**
		 * Traegt Feiertage ins Archive ein
		 * @return void
		 */
		public function holiday_to_archiv()
		{
			//pruefen ob heute ein Halber bzw. ein ganzer Feiertag ist
			if ($this->is_holiday($this->setup_array['current_date_iso']) or $this->is_half_holiday($this->setup_array['current_date_iso']))
			{
				$cdb = new MySql;

				//pruefen ob der Feiertag noch nicht im Archiv ist
				$sql = $cdb->select("SELECT ft_id FROM hdb_feiertage_archiv WHERE datum='".$this->setup_array['current_date_iso']."'");
				$result = $cdb->fetch_array($sql);

				if (!isset($result['ft_id']) or empty($result['ft_id']))
				{
					//Ganzer Feiertag
					if ($this->is_holiday($this->setup_array['current_date_iso']))
					{
						$result_ft = $this->get_db_holidays($this->setup_array['current_date_iso'], false);

						if (is_array($result_ft) and $result_ft['type'] == 'default' and empty($result_ft['bezeichnung']) and !empty($result_ft['default_array_key']))
						{
							$result_ft['bezeichnung'] = get_text('', 'return', 'decode_on', $result_ft['default_array_key']);
						}
					}
					//Halber Feiertag
					elseif ($this->is_half_holiday($this->setup_array['current_date_iso']))
					{
						$result_ft = $this->get_db_half_holidays($this->setup_array['current_date_iso'], false);

						if (is_array($result_ft) and $result_ft['type'] == 'default' and empty($result_ft['bezeichnung']) and !empty($result_ft['default_array_key']))
						{
							$result_ft['bezeichnung'] = get_text('', 'return', 'decode_on', $result_ft['default_array_key']);
						}
					}

					if (isset($result_ft) and count($result_ft) > 0)
					{
						$cdb->insert("INSERT INTO hdb_feiertage_archiv (datum, bezeichnung, default_array_key, type, halbe_sollstunden) VALUES ('".$this->setup_array['current_date_iso']."', '".$result_ft['bezeichnung']."', '".$result_ft['default_array_key']."', '".$result_ft['type']."', '".$result_ft['halbe_sollstunden']."')");
					}
				}
			}
		}
	}
?>