<?php
	/**
	 * HTML calendar with miscellaneous javascript functions
	 *
	 * @author Soeren Pieper <soeren.pieper@docemos.de> / Martin Ufer <martin.ufer@docemos.de>
	 * @copyright Copyright (C) 2009  docemos GmbH
	 * @package time-o-meter
	 *
	 * This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 3 of the License, or (at your option) any later version.
	 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
	 * You should have received a copy of the GNU General Public License along with this program; if not, see http://www.gnu.org/licenses/.
	 */

	/**
	 * HTML calendar with miscellaneous javascript functions
	 * @package time-o-meter
	 * @subpackage classes
	 */
	class KalenderWindow extends Kalender
	{
		/**
		 * Gibt einen HTML Kalender aus
		 *
		 * @param string $date Datum im Format von $date_format
		 * @param string $month
		 * @param string $year
		 * @param string $date_format
		 * @return string
		 */
		public function get_kalender($date, $month = '', $year = '', $date_format = '')
		{
			//Informationen zu einem Datum auslesen
			if (!empty($date))
			{
				$date_info_array = $this->read_date_info($date, $date_format);
			}
			//Informationen zum ersten Tag eines Monats auslesen
			elseif (!empty($month) and !empty($year))
			{
				$date_info_array = $this->read_date_info($year.'-'.$month.'-01', 'ISO');
			}
			else
			{
				$date_info_array = array();
			}

			$month_array = array();
			$week_array = array();
			if (count($date_info_array) > 0)
			{
				for ($i = 1; $i <= $date_info_array['days_in_month']; $i++)
				{
					if ($i < 10)
					{
						$tmp_day = "0$i";
					}
					else
					{
						$tmp_day = $i;
					}

					$tmp_iso_date = $date_info_array['year'].'-'.$date_info_array['month'].'-'.$tmp_day;

					//Ist eigendlich nur um einen doppelaufruf von read_date_info() zu vermeiden
					if ($tmp_iso_date == $date_info_array['iso'])
					{
						$month_array[$i] = $date_info_array;
					}
					else
					{
						$month_array[$i] = $this->read_date_info($tmp_iso_date, 'ISO');
					}

					//Das Datum ist heute
					if ($month_array[$i]['iso'] == $this->setup_array['current_date_iso'])
					{
						$month_array[$i]['current_day'] = true;
					}
					else
					{
						$month_array[$i]['current_day'] = false;
					}

					//Wochenende
					if ($this->is_weekend($month_array[$i]['iso']))
					{
						$month_array[$i]['holiday'] 		= false;
						$month_array[$i]['half_holiday']	= false;
						$month_array[$i]['weekend'] 		= true;
						$month_array[$i]['workday'] 		= false;
					}
					//Feiertag pruefen
					elseif ($this->is_holiday($month_array[$i]['iso']))
					{
						$month_array[$i]['holiday'] 		= true;
						$month_array[$i]['half_holiday'] 	= false;
						$month_array[$i]['weekend'] 		= false;
						$month_array[$i]['workday'] 		= false;
					}
					//Halber arbeitstag
					elseif ($this->is_half_holiday($month_array[$i]['iso']))
					{
						$month_array[$i]['holiday'] 		= false;
						$month_array[$i]['half_holiday'] 	= true;
						$month_array[$i]['weekend'] 		= false;
						$month_array[$i]['workday'] 		= false;
					}
					//Normaler Arbeitstag
					else
					{
						$month_array[$i]['holiday'] 		= false;
						$month_array[$i]['half_holiday'] 	= false;
						$month_array[$i]['weekend'] 		= false;
						$month_array[$i]['workday'] 		= true;
					}

					//Array nach Wochen Sortieren
					$week_array[$month_array[$i]['week_number']][$month_array[$i]['day_number']] = $month_array[$i];
				}
			}

			return $this->get_html_kalender($week_array);
		}

		/**
		 * Erzeugt eine HTML Ausgabe des Kalenders
		 *
		 * @param array $week_array
		 * @return string
		 */
		private function get_html_kalender($week_array)
		{
			if (!isset($_GET['ipt_id_string']))
			{
				$_GET['ipt_id_string'] = '';
			}

			$return_string = '<form method="post" action="kalender.php'.$GLOBALS['gv']->create_get_string('?ipt_id_string='.$_GET['ipt_id_string']).'" accept-charset="UTF-8">';
			$return_string .= '<table cellpadding="2" cellspacing="0" border="0" width="256" class="kalender_main_table">
								<colgroup>
									<col width="32" />
									<col width="32" />
									<col width="32" />
									<col width="32" />
									<col width="32" />
									<col width="32" />
									<col width="32" />
									<col width="32" />
								</colgroup>'."\n";

			//Aktuelles Datum ausgeben
			$return_string .= '<tr align="center"><td colspan="8" class="kalender_current_date">'.$this->setup_array['current_date'].'</td></tr>'."\n";

			//Selectbox fuer Monat und Jahr
			$return_string .= '<tr>';
			$return_string .= '<td colspan="4" align="right">'.$this->get_html_select_month().'</td>';
			$return_string .= '<td colspan="4" align="left">'.$this->get_html_select_year().'</td>';
			$return_string .= '</tr>'."\n";

			//Wochentagzzeile
			$return_string .= '<tr align="center" class="kalender_day_name_tr">';
			$return_string .= '<td title="'.$this->get_kalender_window_text('kw').'">'.$this->get_kalender_window_text('kw_kurz').'</td>';
			$return_string .= '<td title="'.$this->get_kalender_window_text('mo').'">'.$this->get_kalender_window_text('mo_kurz').'</td>';
			$return_string .= '<td title="'.$this->get_kalender_window_text('di').'">'.$this->get_kalender_window_text('di_kurz').'</td>';
			$return_string .= '<td title="'.$this->get_kalender_window_text('mi').'h">'.$this->get_kalender_window_text('mi_kurz').'</td>';
			$return_string .= '<td title="'.$this->get_kalender_window_text('do').'">'.$this->get_kalender_window_text('do_kurz').'</td>';
			$return_string .= '<td title="'.$this->get_kalender_window_text('fr').'">'.$this->get_kalender_window_text('fr_kurz').'</td>';
			$return_string .= '<td title="'.$this->get_kalender_window_text('sa').'">'.$this->get_kalender_window_text('sa_kurz').'</td>';
			$return_string .= '<td title="'.$this->get_kalender_window_text('so').'">'.$this->get_kalender_window_text('so_kurz').'</td>';
			$return_string .= '</tr>'."\n";

			foreach ($week_array as $kw => $kw_data)
			{
				$return_string .= '<tr align="center" class="kalender_day_tr">';
				//KW ausgeben
				$return_string .= '<td class="kalender_kw_td">'.$kw.'</td>';

				for ($i = 1; $i <= 7; $i++)
				{
					if ($i == 7)
					{
						$day_number = 0;
					}
					else
					{
						$day_number = $i;
					}

					//Tag liegt in einem anderen Monat
					if (!isset($kw_data[$day_number]))
					{
						$return_string .= '<td>&nbsp;</td>';
					}
					else
					{
						if ($kw_data[$day_number]['current_day'] == true)
						{
							$day_style = ' class="kalender_current_date"';
						}
						elseif ($kw_data[$day_number]['holiday'] == true)
						{
							$day_style = ' class="kalender_holiday_td"';
						}
						elseif ($kw_data[$day_number]['half_holiday'] == true)
						{
							$day_style = ' class="kalender_halfholiday_td"';
						}
						elseif ($kw_data[$day_number]['weekend'] == true)
						{
							$day_style = ' class="kalender_weekend"';
						}
						else
						{
							$day_style = '';
						}

						$tmp_free_date = $this->iso_to_free($kw_data[$day_number]['date']);

						$return_string .= '<td'.$day_style.' id="day_'.$kw_data[$day_number]['day'].'" onclick="kalender_close(\''.$tmp_free_date.'\', \''.$_GET['ipt_id_string'].'\')" onmouseover="kalender_onmouseover(\''.$tmp_free_date.'\', \'day_'.$kw_data[$day_number]['day'].'\')">'.$kw_data[$day_number]['day'].'</td>';
					}
				}
				$return_string .= '</tr>'."\n";
			}
			//Zeile fuer die Ausgabe des OnMouseOver Datums
			if (isset($_GET['kalender_date_string']) and !empty($_GET['kalender_date_string']))
			{
				$return_string .= '<tr align="center" class="kalender_onmouseover_date_tr"><td colspan="8" id="get_onmouseover_date">'.$_GET['kalender_date_string'].'</td></tr>'."\n";
			}
			else
			{
				$return_string .= '<tr align="center" class="kalender_onmouseover_date_tr"><td colspan="8" id="get_onmouseover_date">&nbsp;</td></tr>'."\n";
			}
			$return_string .= '</table>';
			$return_string .= '</form>';
			return $return_string;
		}

		/**
		 * Erstellt eine html selectbox mit Jahren
		 * @return string
		 */
		private function get_html_select_year()
		{
			if (isset($_POST['kalender_year_string']))
			{
				$year = $_POST['kalender_year_string'];
			}
			else
			{
				$year = $this->setup_array['current_year'];
			}

			$return_string = '<select name="kalender_year_string" onchange="submit();" class="kalender_ipt">';

			for ($i = $year - 5; $i <= $year + 5; $i++)
			{
				if ($i <= 2037 and $i >= 1979)
				{
					if (isset($_POST['kalender_year_string']) and $i == $_POST['kalender_year_string'])
					{
						$selected = ' selected="selected"';
					}
					else
					{
						$selected = '';
					}
					$return_string .= '<option value="'.$i.'"'.$selected.'>'.$i.'</option>';
				}
			}

			$return_string .= '</select>';
			return $return_string;
		}

		/**
		 * Erstellt eine html selectbox mit Monaten
		 * @return string
		 */
		private function get_html_select_month()
		{
			$monthname_array = array('01' => 'jan',
									'02' => 'feb',
									'03' => 'mar',
									'04' => 'apr',
									'05' => 'mai',
									'06' => 'jun',
									'07' => 'jul',
									'08' => 'aug',
									'09' => 'sep',
									'10' => 'okt',
									'11' => 'nov',
									'12' => 'dez');

			$return_string = '<select name="kalender_month_string" onchange="submit();" class="kalender_ipt">';
			foreach ($monthname_array as $month_number => $month_key)
			{
				if (isset($_POST['kalender_month_string']) and $month_number == $_POST['kalender_month_string'])
				{
					$selected = ' selected="selected"';
				}
				else
				{
					$selected = '';
				}
				$return_string .= '<option value="'.$month_number.'"'.$selected.'>'.$this->get_kalender_window_text($month_key).'</option>';
			}
			$return_string .= '</select>';
			return $return_string;
		}

		/**
		 * Gibt zu einem Key einen Text zurueck
		 *
		 * @param mixed $key
		 */
		private function get_kalender_window_text($key)
		{
			if (is_numeric($key))
			{
				return get_text($key, 'return');
			}
			else
			{
				return get_text('', 'return', 'decode_on', $key);
			}
		}
	}
?>
