<?php
	class LDAP
	{
		private $ldap_options = array();
		private $ldap_host;

		public function __construct()
		{
			if (isset($GLOBALS['setup_array']['ldap']))
			{
				$this->ldap_options = $GLOBALS['setup_array']['ldap'];
			}
			else
			{
				$this->ldap_options['active'] = false;
			}
		}

		/**
		 * bind an ldap Server durchführen
		 * @param string $username
		 * @param string $pw
		 * @return mixed
		 */
		public function authenticate($username, $pw)
		{
			if ($this->connect())
			{
				$username_decode = html_entity_decode($username, ENT_QUOTES, 'UTF-8');
				$pw_decode = html_entity_decode($pw, ENT_QUOTES, 'UTF-8');

				//ldap praefix anhaengen
				if (!empty($this->ldap_options['loginname']['praefix']))
				{
					if (substr($username_decode, 0, strlen($this->ldap_options['loginname']['praefix'])) != $this->ldap_options['loginname']['praefix'])
					{
						$username_decode = $this->ldap_options['loginname']['praefix'].$username_decode;
					}
				}

				//ldap suffix anhaengen
				if (!empty($this->ldap_options['loginname']['suffix']))
				{
					$strlen = strlen($this->ldap_options['loginname']['suffix']) * -1;
					if (substr($username_decode, $strlen) != $this->ldap_options['loginname']['suffix'])
					{
						$username_decode .= $this->ldap_options['loginname']['suffix'];
					}
				}

				$ldap_bind = @ldap_bind($this->ldap_host, $username_decode, $pw_decode);

				//login erfolgreich
				if ($ldap_bind)
				{
					return true;
				}
				//keine Serververbindung
				elseif (ldap_errno($this->ldap_host) == -1)
				{
					return -1;
				}
				//login fehlgeschlagen
				else
				{
					return false;
				}
			}
			//keine Serververbindung
			else
			{
				return -1;
			}
		}

		/**
		 * Verbindung zum LDAP Server herstellen
		 * @return boole
		 */
		private function connect()
		{
			if ($this->ldap_options['active'] == true)
			{
				if ($this->ldap_options['ssl'] == true)
				{
					if (strpos($this->ldap_options['domain'], 'ldap://') !== false)
					{
						$ssl_domain = str_replace('ldap://', 'ldaps://', $this->ldap_options['domain']);
					}
					elseif (strpos($this->ldap_options['domain'], 'ldaps://') === false)
					{
						$ssl_domain = 'ldaps://'.$this->ldap_options['domain'];
					}
					else
					{
						$ssl_domain = $this->ldap_options['domain'];
					}

					$this->ldap_host = @ldap_connect($ssl_domain, 636);
				}
				else
				{
					$this->ldap_host = @ldap_connect($this->ldap_options['domain']);
				}

				if ($this->ldap_host)
				{
					$this->set_connection_options();

					return true;
				}
				else
				{
					return false;
				}
			}
			else
			{
				return false;
			}
		}

		/**
		 * Verbindungseinstellungen
		 * @return void
		 */
		private function set_connection_options()
		{
			@ldap_set_option($this->ldap_host, LDAP_OPT_PROTOCOL_VERSION, 3);
        	@ldap_set_option($this->ldap_host, LDAP_OPT_REFERRALS, 0);

        	if ($this->ldap_options['tls'] == true)
        	{
        		@ldap_start_tls($this->ldap_host);
        	}
		}

		/**
		 * Verbindung beenden
		 * @return void
		 */
		private function close()
		{
			@ldap_close($this->ldap_host);
		}

		/**
		 * Close connection
		 * @return void
		 */
		public  function __destruct()
		{
			$this->close();
		}

		/**
		 * Prueft ob PHP mit LDAP Support laeuft
		 * @return boole
		 */
		public function ldap_exists()
		{
			if (function_exists('ldap_connect') and function_exists('ldap_bind') and function_exists('ldap_set_option') and function_exists('ldap_start_tls') and function_exists('ldap_close'))
			{
				return true;
			}
			else
			{
				return false;
			}
		}
	}
?>