<?php
	/**
	 * recording of miscellaneous user-activities (logbook)
	 *
	 * @author Soeren Pieper <soeren.pieper@docemos.de> / Martin Ufer <martin.ufer@docemos.de>
	 * @copyright Copyright (C) 2009  docemos GmbH
	 * @package time-o-meter
	 *
	 * This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 3 of the License, or (at your option) any later version.
	 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
	 * You should have received a copy of the GNU General Public License along with this program; if not, see http://www.gnu.org/licenses/.
	 */

	/**
	 * recording of miscellaneous user-activities (logbook)
	 * @package time-o-meter
	 * @subpackage classes
	 */
	class Logbook
	{
		public $setup_array = array();

		public function __construct()
		{
			$this->setup_array['log_pfad'] = TOM_ABS_PFAD.'files/log/';
			//Pruefen ob Lobbuch aktiv ist
			if (isset($GLOBALS['setup_array']['logbook']))
			{
				if ($GLOBALS['setup_array']['logbook'] == 'j' or $GLOBALS['setup_array']['logbook'] == 1)
				{
					$this->setup_array['log_aktiv'] = true;
				}
				else
				{
					$this->setup_array['log_aktiv'] = false;
				}
			}
			else
			{
				$this->setup_array['log_aktiv'] = true;
			}
		}

		/**
		* Diese Funktion traegt Login bzw. Logout vorgaenge in die DB
		* @param int $user_id
		* @param int $action Gibt an ob es sich um ein Login = 1 oder Logout handelt = 0
		* @param string $login_session md5 schluessel der beim Login vergeben wird
		* @return void
		*/
		public function login_insert($user_id, $action, $login_session)
		{
			if ($this->setup_array['log_aktiv'] == true)
			{
				//SQL Klasse einbinden
				$cdb = new MySql;

				//Login, einfaches insert ohne pruefung auf bereits existierende Daten
				if ($action == 1)
				{
					$cdb->insert("INSERT INTO hdb_log_login (user_id, login_time, ip, login_session) VALUES ($user_id, '".date("YmdHis")."', '".$_SERVER['REMOTE_ADDR']."', '$login_session')");
				}
				//Logout, mit der pruefung auf einen Logindatensatz, Theoretisch muss es immer einen geben, es sei den er wurde waehrend der User Online ist geloescht
				else
				{
					if (!empty($login_session))
					{
						//prueft ob ein Logindatensatz vorhanden ist. Sollte eigendlich immer der Fall sein.
						$sql = $cdb->select("SELECT COUNT(log_id) AS count_id FROM hdb_log_login WHERE login_session='$login_session'");
						$result = $cdb->fetch_array($sql);

						//Es Sollte natuerlich nur ein Eintrag vorhanden sein
						if ($result['count_id'] == 1)
						{
							$cdb->update("UPDATE hdb_log_login SET logout_time='".date("YmdHis")."', login_session=NULL WHERE login_session='$login_session'");
						}
					}
				}
			}
		}

		/**
		 * Enterfernt Loginsessioneintraege aus der Logbuchtabelle die aelter als ein Monat sind
		 * @return void
		 */
		public function clear_login_session()
		{
			//SQL Klasse einbinden
			$cdb = new MySql;
			//Kalenderklasse
			$kal = new Kalender;

			//Datum vor einem Monat finden
			$last_month = $kal->date_calculator('', '', -1, '');
			$last_month = $kal->is_iso_date($last_month[0]);

			if ($last_month !== false)
			{
				$last_month_string = str_replace('-', '', $last_month);

				$sql = $cdb->update("UPDATE hdb_log_login SET login_session=NULL WHERE LEFT(login_time, 8) < '$last_month_string' AND login_session IS NOT NULL");
			}
		}

		/**
		* Speichert alle aenderungen in den Stundenzetteln
		* @param array $stz_setup, Enthaelt alle Setupeinstellungen des Stundenzettels
		* @param array $stz_data, Enthaelt alle Daten fuer eine Ausgabe
		* @return void
		*/
		public function insert_log_stz($stz_setup, $stz_data)
		{
			if ($this->setup_array['log_aktiv'] == true)
			{
				//SQL Klasse einbinden
				$cdb = new MySql;

				if (isset($stz_setup['user_id']) and $stz_setup['user_id'] > 0)
				{
					$month = $stz_setup['year'].'-'.$stz_setup['month'].'-'.'01';
					$user_id = $stz_setup['user_id'];
					$log_time = date('YmdHis');
					$file_name = $log_time.'_'.$user_id.'.log';
					$user_edit_id = USER_ID;

					if ($h = @fopen($this->setup_array['log_pfad'].'stz/'.$file_name, 'w'))
					{
						fwrite($h, '<?xml version="1.0" encoding="utf-8" ?>'."\n");
						fwrite($h, '<stz_log>'."\n");
						fwrite($h, "\t".'<month>'.utf8_encode($month).'</month>'."\n");
						fwrite($h, "\t".'<user_id>'.utf8_encode($user_id).'</user_id>'."\n");
						fwrite($h, "\t".'<log_time>'.utf8_encode($log_time).'</log_time>'."\n");
						fwrite($h, "\t".'<user_edit_id>'.utf8_encode($user_edit_id).'</user_edit_id>'."\n");
						fwrite($h, "\t".'<stz_setup_array>'.utf8_encode(@serialize($stz_setup)).'</stz_setup_array>'."\n");
						fwrite($h, "\t".'<stz_data_array>'.utf8_encode(@serialize($stz_data)).'</stz_data_array>'."\n");
						fwrite($h, '</stz_log>'."\n");
						fclose($h);
						$cdb->insert("INSERT INTO hdb_log_stz (month, user_id, log_time, file_name, edit_user_id) VALUES ('$month', $user_id, '$log_time', '$file_name', $user_edit_id)");
					}
				}
			}
		}

		/**
		 * Erstellt einen Logeintrag zu einer versendeten Mail
		 * @param string $type
		 * @param string $subject
		 * @param string $body
		 * @param mixed $to, array('mustermann@domain.de', 'Frank Musterman') oder 'mustermann@domain.de'
		 * @param array $attachment array(array('filepfad', 'filename'), array('filepfad')) filename muss nicht angegeben werden
		 * @param mixed $bcc, array('mustermann@domain.de', 'Frank Musterman') oder 'mustermann@domain.de'
		 * @param mixed $cc, array('mustermann@domain.de', 'Frank Musterman') oder 'mustermann@domain.de'
		 * @return void
		 */
		public function insert_log_mail($type, $subject, $body, $to, $attachment, $bcc, $cc)
		{
			if ($this->setup_array['log_aktiv'] == true)
			{
				$log_file = date('Ymd').'.log';

				$open_file = false;
				//Datei existiert bereits. Datei oeffnen und Dateizeiger ans ende positionieren
				if (file_exists($this->setup_array['log_pfad'].'mail/'.$log_file))
				{
					if ($h = @fopen($this->setup_array['log_pfad'].'mail/'.$log_file, 'a'))
					{
						$open_file = true;
					}
				}
				//Datei existiert noch nicht. Datei anlegen und erste Zeile Schreiben
				else
				{
					if ($h = @fopen($this->setup_array['log_pfad'].'mail/'.$log_file, 'w'))
					{
						$open_file = true;
						fwrite($h, '<?xml version="1.0" encoding="utf-8" ?>'."\n");
					}
				}

				if ($open_file == true)
				{
					//Blockelement
					fwrite($h, '<mail_log>'."\n");

					//Zeit
					fwrite($h, "\t<time>".utf8_encode(date('YmdHis'))."</time>\n");

					//Userid wenn vorhanden. Wenn keine da sollte es immer der CJ sein
					if (defined('USER_ID'))
					{
						fwrite($h, "\t<user>".utf8_encode(USER_ID)."</user>\n");
					}
					else
					{
						fwrite($h, "\t<user />\n");
					}
					//Mailtyp. HTML oder Text
					fwrite($h, "\t<type>".utf8_encode($type)."</type>\n");

					//Betreff
					if (!empty($subject))
					{
						fwrite($h, "\t<subject>".utf8_encode($subject)."</subject>\n");
					}
					else
					{
						fwrite($h, "\t<subject />\n");
					}
					//Mailbody
					if (!empty($body))
					{
						fwrite($h, "\t<body>".utf8_encode(str_replace('<', '[', str_replace('>', ']',$body)))."</body>\n");
					}
					else
					{
						fwrite($h, "\t<body />\n");
					}

					//Empfaenger
					if (!empty($to))
					{
						if (is_array($to))
						{
							$tmp_to = '';
							foreach ($to as $to_mail)
							{
								if (empty($tmp_to))
								{
									$tmp_to = $to_mail;
								}
								else
								{
									$tmp_to .= ', '.$to_mail;
								}
							}
							$to = $tmp_to;
						}
						fwrite($h, "\t<to>".utf8_encode($to)."</to>\n");
					}
					else
					{
						fwrite($h, "\t<to />\n");
					}

					//Empfaenger
					if (!empty($bcc))
					{
						fwrite($h, "\t<bcc>".utf8_encode($bcc)."</bcc>\n");
					}
					else
					{
						fwrite($h, "\t<bcc />\n");
					}

					//Empfaenger
					if (!empty($cc))
					{
						fwrite($h, "\t<cc>".utf8_encode($cc)."</cc>\n");
					}
					else
					{
						fwrite($h, "\t<cc />\n");
					}

					//Anhhaenge
					if (is_array($attachment) and count($attachment) > 0)
					{
						fwrite($h, "\t<attachment>".utf8_encode(@serialize($attachment))."</attachment>\n");
					}
					else
					{
						fwrite($h, "\t<attachment />\n");
					}

					//Blockelement
					fwrite($h, "</mail_log>\n");
					fclose($h);
				}
			}
		}

		/**
		 * Liest alle Verzeichnisse aus die Logdateien enthalten
		 * @return void
		 *
		 */
		public function pack_all_log_files()
		{
			//Log verzeichnis oeffnen
			if ($h = opendir($this->setup_array['log_pfad']))
			{
				//logverzeichnis lesen
				while (false !== ($dir = readdir($h)))
				{
					//Nur Verzeichnisse
					if (is_dir($this->setup_array['log_pfad'].$dir) and $dir != '.' and $dir != '..')
					{
						$this->pack_log_files($dir);
					}
				}
			}
		}

		/**
		 * Packt in einem Logverzeichnis alle logs des vergangenen monats
		 *
		 * @param string $type
		 * @return void
		 */
		private function pack_log_files($type)
		{
			$os = strtoupper(PHP_OS);

			//nur ausfuehren wenn die zipklasse bzw. das system linux ist
			if (class_exists('PclZip') or $os == 'LINUX')
			{
				$kal = new Kalender;
				$cdb = new MySql;

				if (!empty($type))
				{
					$log_dir = $this->setup_array['log_pfad'].$type.'/';

					if (!file_exists($log_dir))
					{
						$log_dir = '';
					}
				}
				else
				{
					$log_dir = '';
				}

				//Speichert alle zu Packenden Dateien
				$filename_array = array();
				//LogVerzeichnis auslesen
				if (!empty($log_dir))
				{
					//monatserster -1 monate
					$temp_date = $kal->date_calculator(date("Y-m").'-01', '', -1, '');
					//Sting im format Ym
					$year_month = substr(str_replace('-', '', $temp_date[0]), 0, 6);

					if ($h = opendir($log_dir))
					{
						while (false !== ($file = readdir($h)))
						{
							//Nur Dateien
							if (is_file($log_dir.$file) and $file != '.' and $file != '..')
							{
								//Nur Dateien des Monats nur nur log Dateien
								if (substr($file, 0, 6) == $year_month and substr($file, -3) == 'log')
								{
									$filename_array[] = $log_dir.$file;
								}
							}
						}
					}
				}

				//min. eine Datei gefunden
				if (count($filename_array) > 0)
				{
					$zip_error = true;
					//Dateien per php in winzip packen
					if (class_exists('PclZip'))
					{
						//Archviename
						$zip_name = $year_month.'.zip';

						//Zip Datei erstellen
						$zip = new PclZip($log_dir.$zip_name);
						//Zu zippende Dateien uebergeben
						$result = $zip->create($filename_array, PCLZIP_OPT_REMOVE_PATH, $log_dir);

						//Keine Zipfehler
						if ($result != 0)
						{
							$zip_error = false;
							for ($i = 0; $i < count($filename_array); $i++)
							{
								@unlink($filename_array[$i]);
							}
						}
					}
					//Dateien per gz und tar archivieren
					elseif ($os == 'LINUX')
					{
						//Array to string
						$file_name_string = '';
						$file_name_gz_string = '';
						for ($i = 0; $i < count($filename_array); $i++)
						{
							if (!empty($file_name_string))
							{
								$file_name_string .= ' '.$filename_array[$i];
								$file_name_gz_string .= ' '.$filename_array[$i].'.gz';
							}
							else
							{
								$file_name_string = $filename_array[$i];
								$file_name_gz_string = $filename_array[$i].'.gz';
							}
						}
						//Dateien in gz komprimieren
						system('gzip -9q '.$file_name_string, $fp);
						if ($fp == 0)
						{
							//tar erstellen
							system("tar cvf $log_dir$year_month.tar ".$file_name_gz_string, $fp);
							if ($fp == 0)
							{
								//gz dateien loeschen
								for ($i = 0; $i < count($filename_array); $i++)
								{
									@unlink($filename_array[$i].'.gz');
								}
								$zip_error = false;
							}
						}
					}

					//Bei Stundenzettel zusaetzlich alle Eintraege aus der DB entfernen
					if ($type == 'stz' and $zip_error == false)
					{
						$cdb->delete("DELETE FROM hdb_log_stz WHERE LEFT(log_time, 6)='$year_month'");
					}
				}
			}
		}
	}
?>