<?php
	/**
	 * provides several login/logout functions and compares logindata with userdata from database
	 *
	 * Diese Klasse Prueft ob mit den angegebenen Zugangsdaten ein Zugang gewaehrt werden darf.
	 * @author Soeren Pieper <soeren.pieper@docemos.de> / Martin Ufer <martin.ufer@docemos.de>
	 * @copyright Copyright (C) 2010  docemos GmbH
	 * @package time-o-meter
	 */

	/**
	 * provides several login/logout functions and compares logindata with userdata from database
	 * @package file-o-meter
	 * @subpackage class
	 */
	class Login
	{
		/**
		* Gibt die Loginsperrzeit in Sekunden an.
		* @var int
		*/
		private $timeout = 0;

		/**
		* Gibt die Anzahl der Loginversuche an.
		* @var int
		*/
		private $login_trials = 0;

		/**
		 * Gibt an ob ein LDAP Server abgefragt werden soll
		 * @var boole
		 */
		private $ldap_login = false;

		public function __construct($options = array())
		{
			if (isset($options['timeout']))
			{
				$this->timeout = $options['timeout'];
			}
			else
			{
				$this->timeout = 600;
			}

			if (isset($options['login_trials']))
			{
				$this->timeout = $options['login_trials'];
			}
			else
			{
				$this->timeout = 3;
			}

			if (!isset($GLOBALS['setup_array']['ldap']['active']))
			{
				$this->ldap_login = false;
			}
			else
			{
				$this->ldap_login = $GLOBALS['setup_array']['ldap']['active'];
			}
		}

		/**
		* Diese Funktion Prueft die angegebenen Userdaten. Bei erfolgreicher Pruefung wird ein define mit der User_Id erstellt.
		* @param string $pw Das unverschluesselte PW vom User.
		* @param string $username Loginname des Users.
		* @return array $return['error'] = array('1=Kein PW vorhanden.','2=Keine Benutzername vorhanden.','4=Zu den Angegebenen Benutzerdaten konnte keine User gefunden werden.','5=Account ist deaktiviert','6=Account ist zur Zeit gesperrt. Siehe $return['timeout']','7=Fehler beim aendern der Userdaten.');
		*/
		public function chk_login($pw, $username)
		{
			//SQL Klasse
			$cdb = new MySql();
			//Passwort
			$cp = new CryptPw();
			//Logbook
			$lb = new Logbook();

			//array fuer return meldungen
			$return = array();
			//Pruefen ob Passwort vorhanden
			if (!empty($pw))
			{
				//Pruefen ob Benutzername vorhanden
				if (!empty($username))
				{
					$new_pw = $cp->encode_pw($pw);

					//normales TOM Login
					$tom_login = true;

					//LDAP Login
					if ($this->ldap_login == true)
					{
						$tom_login = false;
						$set_new_pw = false;

						$ldap = new LDAP();
						$ldap_result = $ldap->authenticate($username, $pw);

						//Login erfolgreich
						if ($ldap_result === true)
						{
							//TOM ist zweit loginsystem das pw in hdb_user aktualisieren
							if (isset($GLOBALS['setup_array']['ldap']['tom_login']) and $GLOBALS['setup_array']['ldap']['tom_login'] == true)
							{
								$set_new_pw = true;
							}

							//user suchen
							$user_result = $this->get_user_data($username, $new_pw, 'ldap', $set_new_pw);
						}
						//kein login am LDAP
						elseif ($ldap_result === false)
						{
							$return['error'][] = 8;
							$return['login'] = false;
						}
						//Keine Verbindung zum LDAP Server
						elseif ($ldap_result == -1)
						{
							//alternativ TOM Login benutzen
							if (isset($GLOBALS['setup_array']['ldap']['tom_login']) and $GLOBALS['setup_array']['ldap']['tom_login'] == true)
							{
								$tom_login = true;
							}
							//kein alternatives login
							else
							{
								$return['error'][] = 8;
								$return['login'] = false;
							}
						}
						//sollte nie der fall sein
						else
						{
							$return['error'][] = 8;
							$return['login'] = false;
						}
					}

					//normales Login
					if ($tom_login)
					{
						//fuer den fall das der ldap server nicht erreichbar ist
						if ($this->ldap_login == true)
						{
							$user_result = $this->get_user_data($username, $new_pw, 'ldap', false);
						}
						else
						{
							$user_result = $this->get_user_data($username, $new_pw);
						}
					}

					if (isset($user_result) and $user_result !== false)
					{
						//Pruefen ob der User existiert
						if ($user_result['user_id'] > 0)
						{
							//Ptuefen ob Account aktiv ist
							if ($user_result['login_aktiv'] == 'j')
							{
								//Pruefen ob timeout vorhanden
								if ($user_result['timeout'] <= time())
								{
									$global_key = $GLOBALS['gv']->create_glob_key();

									//Loginkey und IP Adresse eintragen. Loginversuche und Timeout Wert Nullen.
									if ($cdb->update("UPDATE hdb_user SET
													session_key='$global_key',
													login_ip='".$_SERVER['REMOTE_ADDR']."',
													login_trials='0',
													timeout='0'
													WHERE user_id=".$user_result['user_id']))
									{
										//USER ID in define Schreiben
										define('USER_ID', $user_result['user_id']);
										$lb->login_insert($user_result['user_id'], 1, $global_key);
										$return['login'] = true;
										$return['global_var'] = $global_key;
									}
									else
									{
										//Fehler beim aendern der Userdaten
										$return['error'][] = 7;
										$return['login'] = false;
									}
								}
								else
								{
									//Timeout ist aktiv
									$return['timeout'] = $user_result['timeout'];
									$return['error'][] = 6;
									$return['login'] = false;
								}
							}
							else
							{
								//Account ist deaktiviert
								$return['error'][] = 5;
								$return['login'] = false;
							}
						}
						else
						{
							//kein Benutzeraccount vorhanden
							$return['error'][] = 4;
							$return['login'] = false;

							//timeout aktualisieren
							$this->update_timeout($username);
						}
					}
					else
					{
						//kein Benutzeraccount vorhanden
						$return['error'][] = 4;
						$return['login'] = false;
					}
				}
				else
				{
					//kein benutzername vorhanden
					$return['error'][] = 2;
					$return['login'] = false;
				}
			}
			else
			{
				//kein pw vorhanden
				$return['error'][] = 1;
				$return['login'] = false;
			}
			return $return;
		}

		/**
		 * Gibt die Userdaten zu den Logindaten
		 * @param string $username
		 * @param string $pw
		 * @param string $type
		 * @param boole $set_new_pw
		 * @return mixed
		 */
		private function get_user_data($username, $pw, $type = 'tom', $set_new_pw = false)
		{
			//SQL Klasse
			$cdb = new MySql();

			//TOM Login
			if ($type == 'tom')
			{
				$sql = $cdb->select("SELECT user_id, timeout, login_aktiv FROM hdb_user WHERE username='$username' AND pw='$pw'");
				$result = $cdb->fetch_array($sql);

				if (isset($result['user_id']) and $result['user_id'] > 0)
				{
					return $result;
				}
				else
				{
					return false;
				}
			}
			//LDAP Logindaten
			elseif ($type == 'ldap' and isset($GLOBALS['setup_array']['ldap']))
			{
				//eventuell vorhandenen Praefix entfernen
				if (!empty($GLOBALS['setup_array']['ldap']['loginname']['praefix']))
				{
					if (substr($username, 0, strlen($GLOBALS['setup_array']['ldap']['loginname']['praefix'])) == $GLOBALS['setup_array']['ldap']['loginname']['praefix'])
					{
						$username = substr($username, strlen($GLOBALS['setup_array']['ldap']['loginname']['praefix']));
					}
				}

				//eventuell vorhandenen Suffix entfernen
				if (!empty($GLOBALS['setup_array']['ldap']['loginname']['suffix']))
				{
					$strlen = strlen($GLOBALS['setup_array']['ldap']['loginname']['suffix']) * -1;
					if (substr($username, $strlen) == $GLOBALS['setup_array']['ldap']['loginname']['suffix'])
					{
						$username = substr($username, 0, $strlen);
					}
				}

				//ein Tabellenspalte in hdb_user
				if (!empty($GLOBALS['setup_array']['ldap']['loginname']['user_data_1']) and empty($GLOBALS['setup_array']['ldap']['loginname']['user_data_2']) and empty($GLOBALS['setup_array']['ldap']['loginname']['user_data_3']))
				{
					//eventuell vorhandenen Trenner entfernen
					if (!empty($GLOBALS['setup_array']['ldap']['loginname']['separator_1']))
					{
						$strlen = strlen($GLOBALS['setup_array']['ldap']['loginname']['separator_1']) * -1;
						if (substr($username, $strlen) == $GLOBALS['setup_array']['ldap']['loginname']['separator_1'])
						{
							$username = substr($username, 0, $strlen);
						}
					}

					$first_arg = $username;
				}
				//zwei Tabellenspalten in hdb_user
				elseif (!empty($GLOBALS['setup_array']['ldap']['loginname']['user_data_1']) and !empty($GLOBALS['setup_array']['ldap']['loginname']['user_data_2']) and empty($GLOBALS['setup_array']['ldap']['loginname']['user_data_3']))
				{
					//trenner zwischen den datenfeldern
					if (!empty($GLOBALS['setup_array']['ldap']['loginname']['separator_1']))
					{
						if (substr_count($username, $GLOBALS['setup_array']['ldap']['loginname']['separator_1']) == 1)
						{
							$ex = explode($GLOBALS['setup_array']['ldap']['loginname']['separator_1'], $username);

							if (is_array($ex) and count($ex) == 2)
							{
								$first_arg = $ex[0];
								$second_arg = $ex[1];
							}
							else
							{
								return false;
							}
						}
						else
						{
							return false;
						}
					}
					else
					{
						return false;
					}
				}
				//drei Tabellenspalten in hdb_user
				elseif (!empty($GLOBALS['setup_array']['ldap']['loginname']['user_data_1']) and !empty($GLOBALS['setup_array']['ldap']['loginname']['user_data_2']) and !empty($GLOBALS['setup_array']['ldap']['loginname']['user_data_3']))
				{
					//trenner zwischen den datenfeldern
					if (!empty($GLOBALS['setup_array']['ldap']['loginname']['separator_1']))
					{
						if (substr_count($username, $GLOBALS['setup_array']['ldap']['loginname']['separator_1']) == 1)
						{
							$ex = explode($GLOBALS['setup_array']['ldap']['loginname']['separator_1'], $username);

							if (is_array($ex) and count($ex) == 2)
							{
								$first_arg = $ex[0];

								if (!empty($GLOBALS['setup_array']['ldap']['loginname']['separator_2']))
								{
									if (substr_count($ex[1], $GLOBALS['setup_array']['ldap']['loginname']['separator_2']) == 1)
									{
										$ex = explode($GLOBALS['setup_array']['ldap']['loginname']['separator_2'], $ex[1]);

										if (is_array($ex) and count($ex) == 2)
										{
											$second_arg = $ex[0];
											$third_arg = $ex[1];
										}
										else
										{
											return false;
										}
									}
									else
									{
										return false;
									}
								}
								else
								{
									return false;
								}
							}
							else
							{
								return false;
							}
						}
						else
						{
							return false;
						}
					}
					else
					{
						return false;
					}
				}
				//solte nie der fall sein
				else
				{
					return false;
				}

				if ($GLOBALS['setup_array']['ldap']['loginname']['user_data_1'] == 'loginname')
				{
					$where = "WHERE username='$first_arg'";
				}
				elseif ($GLOBALS['setup_array']['ldap']['loginname']['user_data_1'] == 'firstname')
				{
					$where = "WHERE vorname='$first_arg'";
				}
				elseif ($GLOBALS['setup_array']['ldap']['loginname']['user_data_1'] == 'lastname')
				{
					$where = "WHERE nachname='$first_arg'";
				}
				elseif ($GLOBALS['setup_array']['ldap']['loginname']['user_data_1'] == 'lastname')
				{
					$where = "WHERE email='$first_arg'";
				}
				else
				{
					return false;
				}

				if (isset($second_arg))
				{
					if ($GLOBALS['setup_array']['ldap']['loginname']['user_data_2'] == 'loginname')
					{
						$where .= " AND username='$second_arg'";
					}
					elseif ($GLOBALS['setup_array']['ldap']['loginname']['user_data_2'] == 'firstname')
					{
						$where .= " AND vorname='$second_arg'";
					}
					elseif ($GLOBALS['setup_array']['ldap']['loginname']['user_data_2'] == 'lastname')
					{
						$where .= " AND nachname='$second_arg'";
					}
					elseif ($GLOBALS['setup_array']['ldap']['loginname']['user_data_2'] == 'lastname')
					{
						$where .= " AND email='$second_arg'";
					}
					else
					{
						return false;
					}
				}

				if (isset($third_arg))
				{
					if ($GLOBALS['setup_array']['ldap']['loginname']['user_data_3'] == 'loginname')
					{
						$where .= " AND username='$third_arg'";
					}
					elseif ($GLOBALS['setup_array']['ldap']['loginname']['user_data_3'] == 'firstname')
					{
						$where .= " AND vorname='$third_arg'";
					}
					elseif ($GLOBALS['setup_array']['ldap']['loginname']['user_data_3'] == 'lastname')
					{
						$where .= " AND nachname='$third_arg'";
					}
					elseif ($GLOBALS['setup_array']['ldap']['loginname']['user_data_3'] == 'lastname')
					{
						$where .= " AND email='$third_arg'";
					}
					else
					{
						return false;
					}
				}

				$sql = $cdb->select('SELECT COUNT(user_id) AS count_id, user_id, timeout, login_aktiv FROM hdb_user '.$where);
				$result = $cdb->fetch_array($sql);

				//count_id sollte immer 1 sein, andernfalls lassen sich die Logindaten nicht eindeutig zuordnen
				if (isset($result['user_id']) and $result['user_id'] > 0 and $result['count_id'] == 1)
				{
					//PW aktualisieren
					if ($set_new_pw == true)
					{
						$cdb->update("UPDATE hdb_user SET pw='$pw' WHERE user_id=".$result['user_id']);
					}
					return $result;
				}
				else
				{
					return false;
				}
			}
			else
			{
				return false;
			}
		}

		/**
		* Diese Funktion prueft bei jedem Seitenaufruf ob das Login aktiv ist.
		* @param string $global_key
		* @return bool
		*/
		public function chk_login_key($global_key)
		{
			$cdb = new MySql;
			//Prueft ob Loginkey vorhanden
			if (!empty($global_key))
			{
				//Benutzersuche
				$sql = $cdb->select("SELECT user_id FROM hdb_user WHERE session_key='$global_key' AND login_ip='".$_SERVER['REMOTE_ADDR']."' AND login_aktiv='j'");
				$result = $cdb->fetch_array($sql);

				//Pruefen ob der User existiert
				if ($result['user_id'] > 0)
				{
					//zusaetzlich cookiepruefung durchfuehren
					if (TOM_LOGIN_COOKIE == true and isset($GLOBALS['TOM_VAR']['TOM_COOKIE_EXISTS']) and $GLOBALS['TOM_VAR']['TOM_COOKIE_EXISTS'] == 1)
					{
						if (isset($_COOKIE[TOM_SESSION_NAME]) and isset($GLOBALS['TOM_VAR']['TOM_SESSION_COOKIE']) and $_COOKIE[TOM_SESSION_NAME] == $GLOBALS['TOM_VAR']['TOM_SESSION_COOKIE'])
						{
							//USER ID in define Schreiben
							define('USER_ID', $result['user_id']);

							return true;
						}
						else
						{
							//alle Sessions zu diesem Benutzer loeschen
							$this->del_session($global_key);
							return false;
						}

					}
					//einfach Pruefung ohne cookie
					else
					{
						//USER ID in define Schreiben
						define('USER_ID', $result['user_id']);

						return true;
					}
				}
				else
				{
					//alle Sessions zu diesem Benutzer loeschen
					$this->del_session($global_key);
					return false;
				}
			}
			else
			{
				return false;
			}
		}

		/**
		* Zaehlt die Loginversuche.
		* @param string $username
		* @return void
		*/
		private function update_timeout($username)
		{
			$cdb = new MySql;

			//Prueft ob zum Angegebenen Beutzernamen ein Account existiert.
			$sql = $cdb->select("SELECT user_id, login_trials, timeout FROM hdb_user WHERE username='$username';");
			$result = $cdb->fetch_array($sql);

			//Pruefen ob der User existiert
			if ($result['user_id'] > 0)
			{
				//counter zuruecksetzten wenn der letzte fehlgeschlagene loginversuch laenger her ist als der timeout wert vorgibt
				if ($result['timeout'] + $this->timeout <= time())
				{
					$cdb->update("UPDATE hdb_user SET
								login_trials='0',
								timeout='0'
								WHERE user_id=".$result['user_id']);
				}

				//Erster fehlversuch.
				if ($result['login_trials'] == 0)
				{
					$cdb->update("UPDATE hdb_user SET
								login_trials='1',
								timeout='".time()."'
								WHERE user_id=".$result['user_id']);
				}
				//letzter Fehlversuch
				elseif ($result['login_trials'] == $this->login_trials)
				{
					$time = time() + $this->timeout;
					$cdb->update("UPDATE hdb_user SET
								login_trials=login_trials+1,
								timeout='$time'
								WHERE user_id=".$result['user_id']);
				}
				else
				{
					$cdb->update("UPDATE hdb_user SET
								login_trials=login_trials+1
								WHERE user_id=".$result['user_id']);
				}
			}
		}

		/**
		* Logout Funktion.
		* @param string $global_key Loginkey
		* @return void
		*/
		public function logout($global_key = '')
		{
			$this->del_session($global_key);
		}

		/**
		* Loescht alle Sessions.
		* @param string $global_key Loginkey
		* @return void
		*/
		private function del_session($global_key)
		{
			$cdb = new MySql();
			//Logbook
			$lb = new Logbook();

			//Pruefen ob Userid vorhanden ist
			if (defined('USER_ID'))
			{
				$user_id = USER_ID;
			}
			else
			{
				$user_id = '';
			}

			if ($user_id > 0)
			{
				$cdb->update('UPDATE hdb_user SET session_key=NULL WHERE user_id='.$user_id);
			}
			elseif (!empty($global_key))
			{
				$cdb->update("UPDATE hdb_user SET session_key=NULL WHERE session_key='$global_key'");
			}

			if (!empty($global_key))
			{
				//Logbook
				$lb->login_insert(0, 0, $global_key);
				//einen speziellen Key loeschen
				$cdb->delete("DELETE FROM hdb_session WHERE sess_key='$global_key'");
			}
			//alle die abgelaufen sind loeschen
			$cdb->delete("DELETE FROM hdb_session WHERE sess_expiry<'".date('YmdHis', time())."'");
		}

		/**
		* Diese Funktion Prueft die uebergebenen Logindaten fuer den Zugriff auf einen Webservice.
		* @param string $pw Das vom User angegebene Passwort.
		* @param string $username Das vom User angegebene Passwort.
		* @return string false oder md5 hash
		*/
		public function webservice_login($pw, $username)
		{
			//SQL Klasse
			$cdb = new MySql;
			//Passwort
			$cp = new CryptPw();

			//Pruefen ob Passwort vorhanden
			if (!empty($pw))
			{
				//Pruefen ob Benutzername vorhanden
				if (!empty($username))
				{
					$tom_login = true;
					$new_pw = $cp->encode_pw($pw);

					//LDAP Login
					if ($this->ldap_login == true)
					{
						$tom_login = false;
						$set_new_pw = false;

						$ldap = new LDAP();
						$ldap_result = $ldap->authenticate($username, $pw);

						//Login erfolgreich
						if ($ldap_result === true)
						{
							//TOM ist zweit loginsystem das pw in hdb_user aktualisieren
							if (isset($GLOBALS['setup_array']['ldap']['tom_login']) and $GLOBALS['setup_array']['ldap']['tom_login'] == true)
							{
								$set_new_pw = true;
							}

							//user suchen
							$user_result = $this->get_user_data($username, $new_pw, 'ldap', $set_new_pw);
						}
						//kein login am LDAP
						elseif ($ldap_result === false)
						{
							return 'false';
						}
						//Keine Verbindung zum LDAP Server
						elseif ($ldap_result == -1)
						{
							//alternativ TOM Login benutzen
							if (isset($GLOBALS['setup_array']['ldap']['tom_login']) and $GLOBALS['setup_array']['ldap']['tom_login'] == true)
							{
								$tom_login = true;
							}
							//kein alternatives login
							else
							{
								return 'false';
							}
						}
						//sollte nie der fall sein
						else
						{
							return 'false';
						}
					}

					//normales Login
					if ($tom_login)
					{
						//fuer den fall das der ldap server nicht erreichbar ist
						if ($this->ldap_login == true)
						{
							$user_result = $this->get_user_data($username, $new_pw, 'ldap', false);
						}
						else
						{
							$user_result = $this->get_user_data($username, $new_pw);
						}
					}
					if (isset($user_result) and $user_result !== false)
					{
						//Pruefen ob der User existiert
						if ($user_result['user_id'] > 0)
						{
							//Ptuefen ob Account aktiv ist
							if ($user_result['login_aktiv'] == 'j')
							{
								//Pruefen ob timeout vorhanden
								if ($user_result['timeout'] <= time())
								{
									$key = md5(uniqid(rand(), true));
									$expire = date('YmdHis', time() + 3600);

									if ($cdb->insert("INSERT INTO hdb_webservice_access (ws_key, user_id, expire) VALUES ('$key', ".$user_result['user_id'].", '$expire')"))
									{
										return $key;
									}
									else
									{
										return 'false1';
									}
								}
								else
								{
									return 'false2';
								}
							}
							else
							{
								return 'false3';
							}
						}
						else
						{
							return 'false4';
						}
					}
					else
					{
						return 'false5';
					}
				}
				else
				{
					return 'false6';
				}
			}
			else
			{
				return 'false7';
			}
		}

		/**
		 * Prueft den uebergebenen Loginkey fuer den Zugriff auf Webservice Funktionen
		 * @param string $key
		 * @return boole
		 */
		public function webservice_key($key)
		{
			//SQL Klasse
			$cdb = new MySql;

			$sql = $cdb->select("SELECT t1.ws_key, t1.user_id, t2.group_id FROM hdb_webservice_access t1
								LEFT JOIN hdb_user t2 ON t1.user_id=t2.user_id
								WHERE t1.ws_key='$key' AND t1.expire>'".date('YmdHis')."'");
			$result = $cdb->fetch_array($sql);

			if (isset($result['ws_key']) and !empty($result['ws_key']) and !empty($result['user_id']) and !empty($result['group_id']))
			{
				if (!defined('USER_ID'))
				{
					define('USER_ID', $result['user_id']);
				}

				if (!defined('USER_GROUP_ID'))
				{
					define('USER_GROUP_ID', $result['group_id']);
				}

				$cdb->update("UPDATE hdb_webservice_access SET expire='".date('YmdHis', time() + 3600)."' WHERE ws_key='$key'");

				return 'true';
			}
			else
			{
				return 'false';
			}
		}
	}
?>