<?php
	/**
	 * connects to a MySQL-DB and provides a variety of SQL-Statements
	 *
	 * @author Soeren Pieper <soeren.pieper@docemos.de> / Martin Ufer <martin.ufer@docemos.de>
	 * @copyright Copyright (C) 2009  docemos GmbH
	 * @TODO fuer die Verwendung wird die Funktion setError() aus der Datei error_handler.php benoetigt.
	 * @package time-o-meter
	 *
	 * This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 3 of the License, or (at your option) any later version.
	 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
	 * You should have received a copy of the GNU General Public License along with this program; if not, see http://www.gnu.org/licenses/.
	 */

	/**
	 * Enthaelt die Zugangsdaten fuer eine oder mehrere MySql Datenbanken.
	 * Wenn nur eine Mysqldatenbank verwendet wird braucht kein Mehrdimensionales Array erstellt werden z.B.
	 * $MySql_db_setup = array('db_server'=>'localhost', 'db_name'=>'example_db', 'db_user'=>'db_user', 'db_pw'=>'db_pw');
	 * Bei der verwednung mehrerer Datenbank ist ein Mehrdimensionales Array zu erstellen z.B.
	 * $MySql_db_setup = array('db_identifier_1' => array('db_server'=>'localhost', 'db_name'=>'example_db', 'db_user'=>'db_user', 'db_pw'=>'db_pw'),
	 * 							'db_identifier_2' => array('db_server'=>'localhost', 'db_name'=>'example_db', 'db_user'=>'db_user', 'db_pw'=>'db_pw'));
	 * In diesem Falle ist die DB-Bezeichnung "db_identifier_1" bzw. "db_identifier_2" beim Aufruf der Klasse mit zu uebergeben z.B. $db = new MySql('db_identifier_1'); oder $db = new MySql('db_identifier_2');
	 * Die Angabe des Verbindungsports muss nicht erfolgen kann aber in dem index 'db_port'=>'3306' erfolgen
	 * Die Angabe des Verbindungssocket muss nicht erfolgen kann aber in dem index 'db_socket'=>'/mysql/mysql5.sock' erfolgen. ACHTUNG Es kann nur der port oder der Socket verwendet werden. Socket hat vorrang.
	 * @var array
	 */
	$GLOBALS['MySql_db_setup'] = array('db_server'	=> TOM_DB_SERVER,
										'db_name'	=> TOM_DB_NAME,
										'db_user'	=> TOM_DB_USER,
										'db_pw'		=> TOM_DB_PW,
										'db_port'	=> TOM_DB_PORT,
										'db_socket'	=> TOM_DB_SOCKET);

	/*
	$MySql_db_setup = array('erste'	=> array('db_server'=>'localhost', 'db_name'=>'test_mysql_1', 'db_user'=>'root', 'db_pw'=>'123456'),
							'zweite'=> array('db_server'=>'localhost', 'db_name'=>'test_mysql_2', 'db_user'=>'root', 'db_pw'=>'123456'));
	*/


	/**
	 * connects to a MySQL-DB and provides a variety of SQL-Statements
	 * @package time-o-meter
	 * @subpackage classes
	 */
	class MySql
	{
		/**
		 * Speichert die Zugangsdaten zu einer Datenbank
		 * @var array
		 */
		private $db_connect_setup_array = array();

		/**
		 * Legt Fest zu welcher Datenbank eine Verbindung Hergestellt werden soll.
		 *
		 * @param string $db_identifier
		 * @return MySql
		 */
		public function __construct($db_identifier = '')
		{
			//pruefen ob Datenbankverbindungsdaten vorhanden sind
			if (isset($GLOBALS['MySql_db_setup']))
			{
				//nur eine DB wird verwendet
				if (empty($db_identifier))
				{
					//MySQL Socket verwenden
					if (isset($GLOBALS['MySql_db_setup']['db_socket']) and !empty($GLOBALS['MySql_db_setup']['db_socket']))
					{
						$db_server = $GLOBALS['MySql_db_setup']['db_server'].$GLOBALS['MySql_db_setup']['db_socket'];
					}
					//MySQL Port verwenden
					elseif (isset($GLOBALS['MySql_db_setup']['db_port']) and !empty($GLOBALS['MySql_db_setup']['db_port']))
					{
						$db_server = $GLOBALS['MySql_db_setup']['db_server'].':'.$GLOBALS['MySql_db_setup']['db_port'];
					}
					else
					{
						$db_server = $GLOBALS['MySql_db_setup']['db_server'];
					}
					$this->db_connect_setup_array = array('db_server' 	=> $db_server,
															'db_name'	=> $GLOBALS['MySql_db_setup']['db_name'],
															'db_user'	=> $GLOBALS['MySql_db_setup']['db_user'],
															'db_pw'		=> $GLOBALS['MySql_db_setup']['db_pw']);
				}
				else
				{
					if (isset($GLOBALS['MySql_db_setup'][$db_identifier]))
					{
						//pruefen ob kein Standardport verwendet werden soll
						if (isset($GLOBALS['MySql_db_setup'][$db_identifier]['db_socket']) and !empty($GLOBALS['MySql_db_setup'][$db_identifier]['db_socket']))
						{
							$db_server = $GLOBALS['MySql_db_setup'][$db_identifier]['db_server'].$GLOBALS['MySql_db_setup'][$db_identifier]['db_port'];
						}
						elseif (isset($GLOBALS['MySql_db_setup'][$db_identifier]['db_port']) and !empty($GLOBALS['MySql_db_setup'][$db_identifier]['db_port']))
						{
							$db_server = $GLOBALS['MySql_db_setup'][$db_identifier]['db_server'].':'.$GLOBALS['MySql_db_setup'][$db_identifier]['db_port'];
						}
						else
						{
							$db_server = $GLOBALS['MySql_db_setup'][$db_identifier]['db_server'];
						}

						$this->db_connect_setup_array = array('db_server' 	=> $db_server,
																'db_name'	=> $GLOBALS['MySql_db_setup'][$db_identifier]['db_name'],
																'db_user'	=> $GLOBALS['MySql_db_setup'][$db_identifier]['db_user'],
																'db_pw'		=> $GLOBALS['MySql_db_setup'][$db_identifier]['db_pw']);
					}
					else
					{
						//Fehlermeldung
						setError('Es wurden keine Verbindungsdaten zum Datenbankserver gefunden.', WARNING, __LINE__);
						exit('Es konnte keine Verbindung zur Datenbank hergestellt werden.');
					}
				}
			}
			else
			{
				//Fehlermeldung
				setError('Es wurden keine Verbindungsdaten zum Datenbankserver gefunden.', WARNING, __LINE__);
				exit('Es konnte keine Verbindung zur Datenbank hergestellt werden.');
			}
		}

		/**
		 * Stellt eine Verbindung zu einer MySql DB her.
		 * @return resource
		 */
		private function db_connect()
		{
			//Verbindung zum Datenbankserver
			if ($link = @mysql_connect($this->db_connect_setup_array['db_server'], $this->db_connect_setup_array['db_user'], $this->db_connect_setup_array['db_pw']))
			{
				@mysql_set_charset('utf8', $link);
				//Datenbank auswaehlen
				if (!@mysql_select_db($this->db_connect_setup_array['db_name'], $link))
				{
					//Fehlermeldung
					setError('Die angegebene Datenbank "'.$this->db_connect_setup_array['db_name'].'" wurde nicht gefunden. '.mysql_error(), WARNING, __LINE__);
					exit('Es konnte keine Verbindung zur Datenbank hergestellt werden.');
				}
				else
				{
					return $link;
				}
			}
			else
			{
				//Fehlermeldung
				setError('Es konnte keine Verbindung zum Datenbankserver "'.$this->db_connect_setup_array['db_server'].'" hergestellt werden. '.mysql_error(), WARNING, __LINE__);
				exit('Es konnte keine Verbindung zur Datenbank hergestellt werden.');
			}
		}

		/**
		 * Entfernt unerwuenschte Zeichen aus einem SQL Statment. Sollten sich Zeichen in einem SQL Statemant befinden die da nicht hingehoeren wird angebrochen.
		 *
		 * @param string $sql
		 * @return mixed
		 */
		public function clear_sql_string($sql)
		{
			$sql = trim($sql);
			//SQL Kommentare entfernen
			$search_array = array('/*', '--');
			$replace_array = array(' ', ' ');
			str_replace($search_array, $replace_array, $sql);

			//SQL string in kleine Buchstaben wandeln
			$tmp_sql = strtolower($sql);
			//alle SQL Kommentare entfernen
			$tmp_sql = preg_replace('~\/\*(.*?)\*\/~s', '', $tmp_sql);

			//Speichern ob ein unerlaubter Befehl vorhanden ist
			$forbidden_command_exists_bool = false;

			//nicht erlaubte Befehle welche zu einem Abbruch fuehren
			$forbidden_command_array = array('union', 'sleep', 'benchmark');

			foreach ($forbidden_command_array as $command)
			{
				if (strpos($tmp_sql, $command) !== false && preg_match('~(^|[^a-z])'.$command.'($|[^[a-z])~s', $tmp_sql) != 0)
				{
					$forbidden_command_exists_bool = true;
				}
			}

			//Subabfragen unterbinden
			$forbidden_command_array = array('update', 'insert', 'delete', 'load data', 'truncate', 'drop', 'alter', 'show');
			foreach ($forbidden_command_array as $command)
			{
				//if (preg_match('~\([^)]*?'.$command.'~s', $tmp_sql) != 0)
				// nach Klammer und 0 oder mehr Leerzeichen gefolgt vom SQL Befehl suchen
				if (preg_match('~\(\s*'.$command.'~s', $tmp_sql) != 0)
				{
					$forbidden_command_exists_bool = true;
				}
			}

			if ($forbidden_command_exists_bool)
			{
				//Fehlermeldung
				return false;
			}
			else
			{
				return $sql;
			}
		}

		/**
		 * Function fuer eine SELECT Abfrage
		 * @param string $sql
		 * @return mixed
		 */
		public function select($sql)
		{
			return $this->query($sql);
		}

		/**
		 * Function fuer eine INSERT Abfrage
		 * @param string $sql
		 * @return bool
		 */
		public function insert($sql)
		{
			$result = $this->query($sql);
			if ($result !== false)
			{
				return true;
			}
			else
			{
				return false;
			}
		}

		/**
		 * Function fuer eine UPDATE Abfrage
		 * @param string $sql
		 * @return bool
		 */
		public function update($sql)
		{
			$result = $this->query($sql);
			if ($result !== false)
			{
				return true;
			}
			else
			{
				return false;
			}
		}

		/**
		 * Function fuer eine DELETE Abfrage
		 * @param string $sql
		 * @return bool
		 */
		public function delete($sql)
		{
			$result = $this->query($sql);
			if ($result !== false)
			{
				return true;
			}
			else
			{
				return false;
			}
		}

		/**
		 * Liefert die Anzahl der Datensaetze im Ergebnis
		 * @param resource $sql
		 * @return int
		 */
		public function get_records($resource)
		{
			return @mysql_num_rows($resource);
		}

		/**
		 * Liefert die Anzahl betroffener Datensaetze einer vorhergehenden MySQL Operation
		 *
		 * @return int
		 */
		public function get_affected_rows()
		{
			return @mysql_affected_rows();
		}

		/**
		 * Liefert die ID einer vorherigen INSERT-Operation
		 *
		 * @return int
		 */
		public function get_last_insert_id()
		{
			return @mysql_insert_id();
		}

		/**
		 * Liefert einen Datensatz als assoziatives Array, als numerisches Array oder beides
		 *
		 * @param resource $resource
		 * @param define $array
		 * @return array
		 */
		public function fetch_array($resource, $array = MYSQL_ASSOC)
		{
			return @mysql_fetch_array($resource, $array);
		}

		/**
		 * Liefert einen Datensatz als indiziertes Array
		 *
		 * @param resource $resource
		 * @return array
		 */
		public function fetch_row($resource)
		{
			return @mysql_fetch_row($resource);
		}

		/**
		 * Liefert eine Ergebniszeile als Objekt
		 *
		 * @param resource $resource
		 * @return object
		 */
		public function fetch_object($resource)
		{
			return @mysql_fetch_object($resource);
		}

		/**
		 * Function fuer eine SQL Abfrage
		 * @param string $sql
		 * @return resource
		 * @todo exit ausgabe mit mysql_error() entfernen
		 */
		public function query($sql)
		{
			$chk_sql = $this->clear_sql_string($sql);

			if ($chk_sql !== false)
			{
				//$this->log_sql_query($chk_sql);
				if ($result = @mysql_query($chk_sql, $this->db_connect()))
				{
					return $result;
				}
				else
				{
					//$this->log_sql_query($chk_sql);
					//Fehlermeldung
					setError('Der MySql-Query ist Fehlerhaft. '.mysql_error(), WARNING, __LINE__);
					//exit('Der MySql-Query ist Fehlerhaft.');
					//Sollte nicht fuer den Produktiveinsatz verwendet werden
					exit(mysql_error().'Der MySql-Query ist Fehlerhaft.');
				}
			}
			else
			{
				//$this->log_sql_query($sql);
				//Fehlermeldung
				setError('Unerlaubter MySql-Query.', ERROR, __LINE__);
				exit('Unerlaubter MySql-Query.');
			}
		}

		/**
		 * Only for tests
		 * @param string $sql
		 * @return void
		 */
		private function log_sql_query($sql)
		{
			$h = fopen(TOM_ABS_PFAD.'files/log/sql.log', 'a');

			fwrite($h, date('YmdHis')."\t\t".$sql."\r\n\r\n");
		}
	}
?>