<?php
	/**
	 * Setup class for the management of basic settings
	 *
	 * @author Soeren Pieper <soeren.pieper@docemos.de> / Martin Ufer <martin.ufer@docemos.de>
	 * @copyright Copyright (C) 2009  docemos GmbH
	 * @package time-o-meter
	 *
	 * This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 3 of the License, or (at your option) any later version.
	 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
	 * You should have received a copy of the GNU General Public License along with this program; if not, see http://www.gnu.org/licenses/.
	 */

	/**
	 * Setup class for the management of basic settings
	 * @package time-o-meter
	 * @subpackage classes
	 */
	class Setup
	{
		/**
		* Legt einen neuen Setup-Datensatz an
		* Die Klasse "MySql" wird fuer die Datenbankabfragen eingebunden.
		* @param array $setup_array Array mit allen Grundeinstellungen. $setup_array=array('beschreibung'=>'wert',....);
		* @return bool TRUE= angelegt, FALSE= nicht angelegt
		*/
		public function insert_setup($setup_array)
		{
			$cdb = new MySql;

			ksort($setup_array, SORT_STRING);
			$serialize_daten = @serialize($setup_array);

			if ($cdb->insert("INSERT INTO hdb_setup (setup) VALUES ('$serialize_daten');"))
			{
				if ($cdb->get_affected_rows() == 1)
				{
					return TRUE;
				}
				else
				{
					return FALSE;
				}
			}
			else
			{
				return FALSE;
			}
		}

		/**
		* Erstellt ein Array mit allen Grundeinstellungen
		* Die MySQL Klasse "MySql" wird fuer die Datenbankabfragen eingebunden.
		* @param int $setup_id Setup ID
		* @return array $result = array('beschreibung'=>'wert'...);
		*/
		public function select_setup($setup_id = 1)
		{
			$cdb = new MySql;

			$sql = $cdb->select("SELECT setup FROM hdb_setup WHERE setup_id=$setup_id");
			$result = $cdb->fetch_array($sql);

			return @unserialize($result['setup']);
		}

		/**
		* Gibt den angeforderten Wert aus den Grundeinstellungen zurueck
		* Die MySQL Klasse "MySql" wird fuer die Datenbankabfragen eingebunden.
		* @param string $setup Setupbeschreibung
		* @param int $setup_id Setup ID
		* @return mixed Wert der jeweiligen Grundeinstellung
		*/
		public function select_once_setup($setup, $setup_id = 1)
		{
			$cdb = new MySql;

			$sql = $cdb->select("SELECT setup FROM hdb_setup WHERE setup_id=$setup_id");
			$result = $cdb->fetch_array($sql);

			$return = @unserialize($result['setup']);
			return $return[$setup];
		}

		/**
		* Bearbeitet die Grundeinstellungen
		* Die MySQL Klasse "MySql" wird fuer die Datenbankabfragen eingebunden.
		* @param int $setup_id Setup ID
		* @param array $setup_array Daten die geaendert werden sollen. $setup_array = array('beschreibung'=>'wert'...); Achtung wenn nur ein Wert geaendert werden soll braucht auch nur der eine Wert uebergeben werden.
		* @return bool TRUE= aenderung erfolgreich, FALSE= aenderung nicht erfolgreich
		*/
		public function update_setup($setup_array, $setup_id = 1)
		{
			$cdb = new MySql;

			$sql = $cdb->select("SELECT setup FROM hdb_setup WHERE setup_id=$setup_id");
			$result = $cdb->fetch_array($sql);

			$unserialize = array();
			$unserialize = @unserialize($result['setup']);

			//arbeitet das uebergebene Array ab
			foreach ($setup_array as $i => $v)
			{
				$unserialize[$i] = $v;
			}

			ksort($unserialize, SORT_STRING);
			$serialize = @serialize($unserialize);

			if ($cdb->update("UPDATE hdb_setup SET setup='$serialize' WHERE setup_id=$setup_id"))
			{
				return TRUE;
			}
			else
			{
				return FALSE;
			}
		}


		/**
		* Loescht einen Wert aus der Setuptabelle
		* @param string $INDEX
		* @return void
		*/
		public function del_setup_value($INDEX)
		{
			$cdb = new MySql;

			$sql = $cdb->select("SELECT setup FROM hdb_setup WHERE setup_id=1");
			$result = $cdb->fetch_array($sql);

			$setup = @unserialize($result['setup']);

			if (isset($setup[$INDEX]))
			{
				unset($setup[$INDEX]);
			}

			ksort($setup, SORT_STRING);
			$serialize = @serialize($setup);

			$cdb->update("UPDATE hdb_setup SET setup='$serialize' WHERE setup_id=1");
		}

		/**
		 * Erstellt ein Setuparray mit defaultwerten
		 * @return string
		 */
		public function get_default_array()
		{
			$setup = array();
			$setup['application_name']					= 'Time-O-Meter';
			$setup['auto_backup']						= array('Mo' => array('uhrzeit1' => '', 'uhrzeit2' => '', 'uhrzeit3' => ''),
																'Di' => array('uhrzeit1' => '', 'uhrzeit2' => '', 'uhrzeit3' => ''),
																'Mi' => array('uhrzeit1' => '', 'uhrzeit2' => '', 'uhrzeit3' => ''),
																'Do' => array('uhrzeit1' => '', 'uhrzeit2' => '', 'uhrzeit3' => ''),
																'Fr' => array('uhrzeit1' => '', 'uhrzeit2' => '', 'uhrzeit3' => ''),
																'Sa' => array('uhrzeit1' => '', 'uhrzeit2' => '', 'uhrzeit3' => ''),
																'So' => array('uhrzeit1' => '', 'uhrzeit2' => '', 'uhrzeit3' => ''),
																'daily' => array('uhrzeit1' => '05', 'uhrzeit2' => '', 'uhrzeit3' => ''));
			$setup['auto_backup_aktiv']					= 'j';
		    $setup['backup_mail_recipients']			= 'email@domain.com';
		    $setup['backup_mail_senden']				= 'n';
		    $setup['contact_fax']						= '';
		    $setup['contact_mail']						= 'email@domain.com';
		    $setup['contact_name']						= '';
		    $setup['contact_tel']						= '';
		    $setup['database_version']					= TOM_VERSION;
		    $setup['date_format']						= 'd/m/Y';
		    $setup['default_language']					= '1';
			$setup['dms_projects']						= array();
			$setup['dms_pw']							= '';
			$setup['dms_url']							= '';
			$setup['dms_user']							= '';
			$setup['email_altbody']						= 'Your mail client is unable to display HTML-content. Please check your mail client settings to enable HTML-mails. - Ihr Email-Programm kann derzeit keine HTML-Emails darstellen. Bitte pruefen Sie die Programm-Einstellungen, um die Anzeige von HTML-Emails zu aktivieren.';
			$setup['email_from']						= 'noreply@domain.com';
			$setup['email_fromname']					= 'noreply@domain.com';
			$setup['email_notification']				= 'j';
			$setup['email_notification_project_report']	= 'j';
    		$setup['email_recipients']					= @serialize(array(array('email' => 'email@domain.com', 'type' => 'free_input', 'user_id' => '')));
    		$setup['email_recipients_project_report']	= @serialize(array(array('email' => 'email@domain.com', 'type' => 'free_input', 'user_id' => '')));
   			$setup['email_sendmailpfad']				= '/usr/sbin/sendmail -t -i';
			$setup['email_sendtype']					= 'sendmail';
			$setup['email_smtpauth']					= '';
			$setup['email_smtpport']					= '';
			$setup['email_smtppw']						= '';
			$setup['email_smtpsecure']					= '';
			$setup['email_smtpserver']					= '';
			$setup['email_smtpuser']					= '';
			$setup['footer_text']						= 'time-o-meter &copy; 2010 Docemos GmbH';
			$setup['holiday']['overview_project']		= true;
			$setup['holiday']['overview_qualification'] = true;
			$setup['ldap']['active']					= false;
			$setup['ldap']['tom_login']					= true;
			$setup['ldap']['domain']					= '';
			$setup['ldap']['ssl']						= true;
			$setup['ldap']['tls']						= true;
			$setup['ldap']['loginname']['praefix']		= '';
			$setup['ldap']['loginname']['user_data_1']	= '';
			$setup['ldap']['loginname']['separator_1']	= '';
			$setup['ldap']['loginname']['user_data_2']	= '';
			$setup['ldap']['loginname']['separator_2']	= '';
			$setup['ldap']['loginname']['user_data_3']	= '';
			$setup['ldap']['loginname']['suffix']		= '';
			$setup['logbook'] 							= 'j';
			$setup['login_cookie']						= true;
			$setup['logo_name']							= 'logo_default.jpg';
			$setup['pdf_font']							= 'helvetica';
    		$setup['pdf_logo']							= array('logo_default.jpg', 'logo_default.jpg', 'logo_default.jpg', 'logo_default.jpg');
		    $setup['pdf_logo_template']					= 'a';
			$setup['maintenance_mode']					= 'n';
			$setup['min_pw_len']						= 8;
			$setup['min_username_len']					= 8;
			$setup['send_ablehnung']					= 'j';
			$setup['send_genehmigung']					= 'j';
			$setup['stz_editierbar_bis']				= '10';
			$setup['stz_pausenminuten']					= '45';
			$setup['stz_report_mehr_als']				= '10';
			$setup['stz_report_nach']					= '22';
			$setup['stz_report_vor']					= '06';
			$setup['stz_round_time']					= 'j';
			$setup['template']							= 'default';
			$setup['text_ablehnung']					= '';
			$setup['text_genehmigung']					= '';
			$setup['webservice_tom']					= array('aktiv' => 0, 'functions' => array(), 'ip' => array());

			return @serialize($setup);
		}
	}
?>