<?php
	/**
	 * TimeSheet class for the handling of timesheet data
	 *
	 * @author Soeren Pieper <soeren.pieper@docemos.de> / Martin Ufer <martin.ufer@docemos.de>
	 * @copyright Copyright (C) 2009  docemos GmbH
	 * @package time-o-meter
	 *
	 * This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 3 of the License, or (at your option) any later version.
	 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
	 * You should have received a copy of the GNU General Public License along with this program; if not, see http://www.gnu.org/licenses/.
	 */

	/**
	 * TimeSheet class for the handling of timesheet data
	 * @package time-o-meter
	 * @subpackage classes
	 */
	class TimeSheet extends TimeSheetOutput
	{
		/**
		 * Speichert alle Userrelevanten informationen. Wird in der function start_time_sheet() beschrieben
		 *
		 * @var array
		 */
		public $setup_array = array();

		/**
		 * Speichert Einstellungen zu einem Tag
		 *
		 * @var array
		 */
		public $setup_day_array = array();

		/**
		 * Speichert alle zugeordneten Projekte
		 *
		 * @var array
		 */
		public $project_array = array();

		/**
		 * Speichert Fehlermeldungen
		 *
		 * @var array
		 */
		protected $error_log_array = array();

		/**
		 * Speichert diverse Temporaere informationen
		 * @var array
		 */
		public $tmp_data_save_array = array();

		/**
		 * Speichert die Abwesenheitsgruende
		 *
		 * @var array
		 */
		public $absence_array = array();

		/**
		 * Gibt an ob die Zeile fuer einen Tag ausgegeben werden kann. false tritt ein bei einer Wochendarstellung wo ein Teil der Tage bereits abgeschlossen ist.
		 *
		 * @var unknown_type
		 */
		protected $show_tr_line = false;

		/**
		 * Startet die ausgabe eines Stundenzettels
		 *
		 * @param int $user_id
		 * @param string $month
		 * @param int $year
		 * @param string $show, month, week oder day. Gibt an ob der gesamt monat die aktuelle woche oder der aktuelle Tag ausgegeben werden soll
		 * @param string $return, edit oder show, Gibt an ob der Stundenzettel nur zu ansich oder zum bearbeiten ausgegeben werden soll.
		 * @return void
		 */
		public function start_time_sheet($user_id, $month, $year, $show = 'month', $return = 'edit')
		{
			$cdb = new MySql;

			/**
			 * Grundeinstellungen auslesen
			 * anfang
			 */
				$sql = $cdb->select('SELECT user_id, vorname, nachname, min_weektime, max_weektime, arbeitstage FROM hdb_user WHERE user_id='.$user_id);
				$user_result = $cdb->fetch_array($sql);

				//pruefen ob der angegebene User existiert
				if (isset($user_result['user_id']) and $user_result['user_id'] > 0)
				{
					// ACHTUNG: aenderungen des Jahresurlaubsanspruches im Laufenden Jahr werden nicht beruecksichtigt.
					// Diese koennen nur mit einer uebertragsaenderung beruecksitigt werden
					// Ansonsten wird der neue Urlaubsanspruch erst im naechsten Jahr beruecksichtigt

					$sql = $cdb->select("SELECT id, startwert_stunden, startwert_urlaub, wochenarbeitszeit, max_wochenarbeitszeit, arbeitstage, pausenzeit, abgeschlossen FROM hdb_stz_monat WHERE user_id=$user_id AND datum='".$year.'-'.$month.'-'."01'");
					$time_result = $cdb->fetch_array($sql);

					//pruefen ob Stundenzettel existiert
					if (isset($time_result['id']) and $time_result['id'] > 0)
					{
						$this->setup_array['user_id']					= $user_result['user_id'];
						$this->setup_array['vorname']					= $user_result['vorname'];
						$this->setup_array['nachname']					= $user_result['nachname'];
						$this->setup_array['startwert_urlaub']			= $time_result['startwert_urlaub'];
						$this->setup_array['startwert_stunden']			= $time_result['startwert_stunden'];
						$this->setup_array['month']						= $month;
						$this->setup_array['year']						= $year;
						$this->setup_array['month_day_count']			= date('t', mktime(0, 0, 0, $this->setup_array['month'], 1, $this->setup_array['year']));
						$this->setup_array['show']						= strtolower($show); //welcher Zeitbereich soll ausgegeben werden month, week oder day
						$this->setup_array['return']					= strtolower($return); //show oder edit
						$this->setup_array['days']						= $this->get_days($this->setup_array['show']);//Tage die ausgegeben werden sollen
						$this->setup_array['project_td_width']			= 120;//Spaltenbreite der Projektspalten
						$this->setup_array['month_sollstunden']			= 0;//Speichert die Sollstunden fuer einen Monat
						$this->setup_array['month_iststunden']			= 0;//Speichert die Iststunden fuer einen Monat
						$this->setup_array['month_ueberstunden']		= 0;//Speichert die ueberstunden fuer einen Monat
						$this->setup_array['month_nzsstunden']			= 0;//Speichert die nzsstunden fuer einen Monat
						$this->setup_array['month_urlaub_count']		= 0;//Speichert die anzahl der Urlaubstage fuer einen Monat
						$this->setup_array['month_project_iststunden']	= array();//Speichert die anzahl der Urlaubstage fuer einen Monat
						$this->setup_array['month_absence_time']		= 0;//Speichert die Stundenzahle der Bezahlen Abwesenheitsgruende
						$this->setup_array['projektkommentar_exists']	= array();//Speichert ob fuer ein Projekt ein Kommentar vorhanden ist. Dies wird fuer die PDF ausgabe benoetigt
						$this->setup_array['report_vor']				= intval($GLOBALS['setup_array']['stz_report_vor']);//Speichert die Stunde vor der nicht gearbeitet werden sollte
						$this->setup_array['report_nach']				= intval($GLOBALS['setup_array']['stz_report_nach']);//Speichert die Stunde nach der nicht gearbeitet werden sollte
						$this->setup_array['report_mehr']				= intval($GLOBALS['setup_array']['stz_report_mehr_als']);//Speichert die Stundenanzahl die pro tag nicht ueberschritten werden sollte


						//Aktueller Monat wird bearbeitet
						//fuer den aktuellen Monat koennen aenderungen in den Arbeitszeiten vorgenommen werden, diese werden bei der aktuellen berechnung sofort beruecksichtigt.
						if ($month == date('m') and $year == date('Y'))
						{
							$this->setup_array['min_weektime']	= $user_result['min_weektime'];
							$this->setup_array['max_weektime']	= $user_result['max_weektime'];
							$this->setup_array['arbeitstage']	= $user_result['arbeitstage'];
							// FIXME: Pausenzeit sollte aus der usertabelle geholt werden, nicht wie derzeit aus der setup-tabelle
							$this->setup_array['pausenzeit']	= intval($GLOBALS['setup_array']['stz_pausenminuten']);//Speichert die Voreinstellung der taegl. Pausenminuten
						}
						//es wird ein aelterer Stundenzettel bearbeitet
						//Eventuelle aenderungen an den Arbeitszeiten werden nicht beruecksichtigt
						else
						{
							$this->setup_array['min_weektime']	= $time_result['wochenarbeitszeit'];
							$this->setup_array['max_weektime']	= $time_result['max_wochenarbeitszeit'];
							$this->setup_array['arbeitstage']	= $time_result['arbeitstage'];
							$this->setup_array['pausenzeit']	= $time_result['pausenzeit'];
						}
						// Min und Max Arbeitszeit fuer einen tag
						// ACHTUNG: nur eine einfach genauigkeit. Der Wert wird auf zwei Stellen nach dem Komma gerundet
						$this->setup_array['min_daytime']		= round($this->setup_array['min_weektime'] / $this->setup_array['arbeitstage'], 2);
						$this->setup_array['max_daytime']		= round($this->setup_array['max_weektime'] / $this->setup_array['arbeitstage'], 2);
					}
					else
					{
						$this->error_log_array[] = setError(get_text(856, 'return', 'decode_on') /*Es wurde kein Stundenzettel gefunden!*/, WARNING, __LINE__);
					}
				}
				else
				{
					$this->error_log_array[] = setError(get_text(146, 'return', 'decode_on') /*Der angegebene Miterbeiter konnte nicht gefunden werden.*/, WARNING, __LINE__);
				}
			/**
			 * Grundeinstellungen auslesen
			 * ende
			 */

			//Keine Fehler bis hierhin
			if (count($this->error_log_array) == 0)
			{
				//Alle Abwesenheitsgruende suchen
				$this->get_absence_list();
				//Alle Projekte Suchen
				$this->get_user_projects();
				//Arbeitszeiten aus der DB lesen und zum grossteil in $_POST Speichern
				//Allgemeinse einstellungen zu den Tagen ermitteln
				$this->create_time_array();
			}

			//Nur ausfuehren wenn keine PDF ausgabe erfolgen soll
			if (isset($this->setup_array['return']) && $this->setup_array['return'] != 'pdf')
			{
				echo $this->get_time_sheet();
			}
		}

		/**
		 * Erstellt ein Array mit den Tagen die Ausgegeben werden sollen
		 * @param string $show, month, week oder day
		 * @return array
		 */
		protected function get_days($show)
		{
			$cdb = new MySql;
			$kal = new Kalender;

			$return_array = array();
			//nur den Aktuellen Tag ausgeben
			if ($show == 'day')
			{
				$sql = $cdb->select("SELECT abgeschlossen FROM hdb_stz_monat WHERE user_id=".$this->setup_array['user_id']." AND LEFT(datum, 7)='".date('Y-m')."'");
				$result = $cdb->fetch_array($sql);

				if ($result['abgeschlossen'] == 'n')
				{
					$return_array[] = array('day' => date('Y-m-d'), 'show' => true);
				}
				else
				{
					$return_array[] = array('day' => date('Y-m-d'), 'show' => false);
				}
			}
			//Wochenausgabe
			elseif ($show == 'week')
			{
				$day_number_int = date('w');
				//Montag
				if ($day_number_int == 1)
				{
					$for_start_int = 0;
					$for_end_int = 6;
				}
				elseif ($day_number_int == 2)
				{
					$for_start_int = -1;
					$for_end_int = 5;
				}
				elseif ($day_number_int == 3)
				{
					$for_start_int = -2;
					$for_end_int = 4;
				}
				elseif ($day_number_int == 4)
				{
					$for_start_int = -3;
					$for_end_int = 3;
				}
				elseif ($day_number_int == 5)
				{
					$for_start_int = -4;
					$for_end_int = 2;
				}
				elseif ($day_number_int == 6)
				{
					$for_start_int = -5;
					$for_end_int = 1;
				}
				elseif ($day_number_int == 0)
				{
					$for_start_int = -6;
					$for_end_int = 0;
				}

				$tmp_date_checked_array = array();
				for ($i = $for_start_int; $i <= $for_end_int; $i++)
				{
					$tmp_date = $kal->date_calculator('', '', '', $i);
					$tmp_date[2] = date('Y-m', $tmp_date[1]);

					//Die pruefung nicht fuer jeden Tag sondern immer nur fuer jeden Monat durchfuehren
					if (!array_key_exists($tmp_date[2], $tmp_date_checked_array))
					{
						$sql = $cdb->select("SELECT abgeschlossen FROM hdb_stz_monat WHERE user_id=".$this->setup_array['user_id']." AND LEFT(datum, 7)='".$tmp_date[2]."'");
						$result = $cdb->fetch_array($sql);

						if (isset($result['abgeschlossen']))
						{
							$tmp_date_checked_array[$tmp_date[2]] = $result['abgeschlossen'];
						}
						else
						{
							$tmp_date_checked_array[$tmp_date[2]] = '';
						}
					}

					if (isset($result['abgeschlossen']) and $result['abgeschlossen'] == 'n')
					{
						$return_array[] = array('day' => $tmp_date[0], 'show' => true);
					}
					else
					{
						$return_array[] = array('day' => $tmp_date[0], 'show' => false);
					}
				}
			}
			//Monatsansicht
			elseif ($show == 'month')
			{
				$sql = $cdb->select("SELECT abgeschlossen FROM hdb_stz_monat WHERE user_id=".$this->setup_array['user_id']." AND LEFT(datum, 7)='".$this->setup_array['year'].'-'.$this->setup_array['month']."'");
				$result = $cdb->fetch_array($sql);

				$day_count_int = date('t', mktime(0, 0, 0, $this->setup_array['month'], 1, $this->setup_array['year']));
				for($i = 1; $i <= $day_count_int; $i++)
				{
					if ($i < 10)
					{
						$day = "0$i";
					}
					else
					{
						$day = $i;
					}

					if ($result['abgeschlossen'] == 'n')
					{
						$return_array[] = array('day' => $this->setup_array['year'].'-'.$this->setup_array['month'].'-'.$day, 'show' => true);
					}
					else
					{
						$return_array[] = array('day' => $this->setup_array['year'].'-'.$this->setup_array['month'].'-'.$day, 'show' => false);
					}
				}
			}
			return $return_array;
		}

		/**
		 * Seichert alle Projekt in dem array $this->project_array
		 * @return void
		 */
		protected function get_user_projects()
		{
			$cdb = new MySql;

			$sql= $cdb->select("SELECT t2.projekt_id,t2.name,t2.kurzname,t2.nr_intern FROM hdb_user_projekt t1
								LEFT JOIN hdb_projekte t2 ON t1.projekt_id=t2.projekt_id
								WHERE t1.user_id=".$this->setup_array['user_id']." AND (t1.anzeigen='j' OR LEFT(t1.del_time, 6) >=".$this->setup_array['year'].$this->setup_array['month'].") AND LEFT(t2.start_datum,7)<='".$this->setup_array['year'].'-'.$this->setup_array['month']."' AND LEFT(t2.end_datum, 7)>='".$this->setup_array['year'].'-'.$this->setup_array['month']."' ORDER BY t2.name ASC");
			while ($result = $cdb->fetch_array($sql))
			{
				$get_project_boole = false;
				//Bei Anzeigen nur Projekte ausgeben zu denen ein Eintrag vorhanden ist
				if ($this->setup_array['return'] == 'show')
				{
					$sql2 = $cdb->select("SELECT SUM(stunden) AS sum_stunden FROM hdb_stz_projekt WHERE user_id=".$this->setup_array['user_id']." AND projekt_id='".$result['projekt_id']."' AND LEFT(datum,7)='".$this->setup_array['year'].'-'.$this->setup_array['month']."'");
					$sum_result = $cdb->fetch_array($sql2);

					if ($sum_result['sum_stunden'] > 0)
					{
						$get_project_boole = true;
					}
				}
				else
				{
					$get_project_boole = true;
				}

				if ($get_project_boole)
				{
					$this->project_array[] = array('name'	=> $result['name'],
												'kurzname'	=> $result['kurzname'],
												'id'		=> $result['projekt_id'],
												'nr'		=> $result['nr_intern']);
				}
			}
		}

		/**
		 * Traegt fuer einen Aktivierten User einen neuen Stundenzettel ein
		 * @param int $user_id
		 * @param float $start_hours
		 * @param float $start_holiday
		 * @return boole
		 */
		public function activate_time_sheet($user_id, $start_hours, $start_holiday)
		{
			$cdb = new MySql;

			$sql = $cdb->select('SELECT user_id, urlaubstage, min_weektime, max_weektime, arbeitstage FROM hdb_user WHERE user_id='.$user_id);
			$user_result = $cdb->fetch_array($sql);

			if (!empty($user_result['user_id']))
			{
				// FIXME: Pausenzeit sollte aus der usertabelle geholt werden, nicht wie derzeit aus der setup-tabelle
				$user_result['pausenzeit'] = intval($GLOBALS['setup_array']['stz_pausenminuten']);//Speichert die Voreinstellung der taegl. Pausenminuten

				$start_datum = date('Y-m').'-01';
				$month_timestamp = mktime(0, 0, 0, date('m'), 1, date('Y'));

				$month_stz_insert = true;
				//Pruefen ob ein Monatsstundenzettel bereits existiert
				//sollte nur der fall sein wenn ein user im Selben Monat reaktiviert wird
				$sql = $cdb->select("SELECT id FROM hdb_stz_monat WHERE user_id=$user_id AND datum='$start_datum'");
				$stz_result = $cdb->fetch_array($sql);
				if (isset($stz_result['id']) and !empty($stz_result['id']))
				{
					$month_stz_insert = false;
				}

				$insert_update_result = false;
				//Insert
				if ($month_stz_insert === true)
				{
					if ($cdb->insert("INSERT INTO hdb_stz_monat (user_id, datum, startwert_stunden, startwert_urlaub, wochenarbeitszeit, max_wochenarbeitszeit, arbeitstage, pausenzeit) VALUES ($user_id, '$start_datum', $start_hours, $start_holiday, ".$user_result['min_weektime'].", '".$user_result['max_weektime']."', '".$user_result['arbeitstage']."', '".$user_result['pausenzeit']."')"))
					{
						$insert_update_result = true;
					}
				}
				//Update
				else
				{
					if ($cdb->update("UPDATE hdb_stz_monat SET startwert_stunden='$start_hours',
																startwert_urlaub='$start_holiday',
																wochenarbeitszeit='".$user_result['min_weektime']."',
																max_wochenarbeitszeit='".$user_result['max_weektime']."',
																arbeitstage='".$user_result['arbeitstage']."',
																pausenzeit='".$user_result['pausenzeit']."'
																WHERE user_id=$user_id AND datum='$start_datum'"))
					{
						$insert_update_result = true;
					}
				}
				if ($insert_update_result === true)
				{
					$day_count_int = date('t', $month_timestamp);
					$month_year_string = date('Y-m');

					$tages_arbeitszeit = 0 - round($user_result['min_weektime'] / $user_result['arbeitstage'], 2);
					$halbtages_arbeitszeit = 0 - round($user_result['min_weektime'] / $user_result['arbeitstage'] / 2, 2);
					$error_count_int = 0;

					for($i = 1; $i <= $day_count_int; $i++)
					{
						if ($i < 10)
						{
							$day = '-0'.$i;
						}
						else
						{
							$day = '-'.$i;
						}

						//Speichert ob der Tag bereits vorhanden ist
						$insert_day = true;
						$sql = $cdb->select("SELECT stz_id FROM hdb_stz WHERE user_id=$user_id AND datum='".$month_year_string.$day."'");
						$day_result = $cdb->fetch_array($sql);

						if (isset($day_result['stz_id']) and !empty($day_result['stz_id']))
						{
							$insert_day = false;
						}

						//Neuen Tag als Feiertag eintragen nur wenn kein Wochenende ist
						if ($this->is_absence_holiday($month_year_string.$day) === true and $this->is_holiday($month_year_string.$day, 'weekend') === false)
						{
							if ($insert_day === true)
							{
								//ACHTUNG: die 3 als ID kann sich aendern
								if (!$cdb->insert("INSERT INTO hdb_stz (user_id, datum, absence_id) VALUES ($user_id, '".$month_year_string.$day."', 3)"))
								{
									setError(get_text(857, 'return', 'decode_on') /*Fehler beim Speichern des Stundenzettels!*/ .' '.get_text('', 'return', 'decode_on', 'user_id') /*Benutzer-ID*/ .': '.$user_id.', '.get_text('', 'return', 'decode_on', 'datum') /*Datum*/ .': '.$month_year_string.$day, ERROR, __LINE__);
									$error_count_int++;
								}
							}
							else
							{
								//ACHTUNG Es wird hier davon ausgegangen das wenn ein Tag bereits vorhanden ist dieser auch korrekte Daten enthaelt

								//ACHTUNG Das Update Ueberschreibt alle eventuell vorhandenen Werte. Wenn dies nicht erwuenscht ist die Nachfolgende Schleife Auskommentieren
								//if (!$cdb->update("UPDATE hdb_stz SET kommen=NULL, pause=NULL, gehen=NULL, ist_zeit=NULL, absence_id=3, ueberstunden=NULL WHERE stz_id=".$day_result['stz_id']))
								//{
								//	setError(get_text(857, 'return', 'decode_on') /*Fehler beim Speichern des Stundenzettels!*/ .' '.get_text('', 'return', 'decode_on', 'user_id') /*Benutzer-ID*/ .': '.$user_id.', '.get_text('', 'return', 'decode_on', 'datum') /*Datum*/ .': '.$month_year_string.$day, ERROR, __LINE__);
								//	$error_count_int++;
								//}

							}
						}
						//Halber Arbeitstag
						elseif ($this->is_holiday($month_year_string.$day, 'halbtags'))
						{
							if ($insert_day === true)
							{
								if (!$cdb->insert("INSERT INTO hdb_stz (user_id, datum, ueberstunden) VALUES ($user_id, '".$month_year_string.$day."', $halbtages_arbeitszeit)"))
								{
									setError(get_text(857, 'return', 'decode_on') /*Fehler beim Speichern des Stundenzettels!*/ .' '.get_text('', 'return', 'decode_on', 'user_id') /*Benutzer-ID*/ .': '.$user_id.', '.get_text('', 'return', 'decode_on', 'datum') /*Datum*/ .': '.$month_year_string.$day, ERROR, __LINE__);
									$error_count_int++;
								}
							}
							else
							{
								//ACHTUNG Es wird hier davon ausgegangen das wenn ein Tag bereits vorhanden ist dieser auch korrekte Daten enthaelt

								//ACHTUNG Das Update Ueberschreibt alle eventuell vorhandenen Werte. Wenn dies nicht erwuenscht ist die Nachfolgende Schleife Auskommentieren
								//if (!$cdb->update("UPDATE hdb_stz SET kommen=NULL, pause=NULL, gehen=NULL, ist_zeit=NULL, absence_id=0, ueberstunden=$halbtages_arbeitszeit WHERE stz_id=".$day_result['stz_id']))
								//{
								//	setError(get_text(857, 'return', 'decode_on') /*Fehler beim Speichern des Stundenzettels!*/ .' '.get_text('', 'return', 'decode_on', 'user_id') /*Benutzer-ID*/ .': '.$user_id.', '.get_text('', 'return', 'decode_on', 'datum') /*Datum*/ .': '.$month_year_string.$day, ERROR, __LINE__);
								//	$error_count_int++;
								//}
							}
						}
						//Wochenende
						elseif ($this->is_holiday($month_year_string.$day, 'weekend'))
						{
							if ($insert_day === true)
							{
								if (!$cdb->insert("INSERT INTO hdb_stz (user_id, datum) VALUES ($user_id, '".$month_year_string.$day."')"))
								{
									setError(get_text(857, 'return', 'decode_on') /*Fehler beim Speichern des Stundenzettels!*/ .' '.get_text('', 'return', 'decode_on', 'user_id') /*Benutzer-ID*/ .': '.$user_id.', '.get_text('', 'return', 'decode_on', 'datum') /*Datum*/ .': '.$month_year_string.$day, ERROR, __LINE__);
									$error_count_int++;
								}
							}
							else
							{
								//ACHTUNG Es wird hier davon ausgegangen das wenn ein Tag bereits vorhanden ist dieser auch korrekte Daten enthaelt

								//ACHTUNG Das Update Ueberschreibt alle eventuell vorhandenen Werte. Wenn dies nicht erwuenscht ist die Nachfolgende Schleife Auskommentieren
								//if (!$cdb->update("UPDATE hdb_stz SET kommen=NULL, pause=NULL, gehen=NULL, ist_zeit=NULL, absence_id=0, ueberstunden=NULL WHERE stz_id=".$day_result['stz_id']))
								//{
								//	setError(get_text(857, 'return', 'decode_on') /*Fehler beim Speichern des Stundenzettels!*/ .' '.get_text('', 'return', 'decode_on', 'user_id') /*Benutzer-ID*/ .': '.$user_id.', '.get_text('', 'return', 'decode_on', 'datum') /*Datum*/ .': '.$month_year_string.$day, ERROR, __LINE__);
								//	$error_count_int++;
								//}
							}
						}
						//normaler arbeitstag
						else
						{
							if ($insert_day === true)
							{
								if (!$cdb->insert("INSERT INTO hdb_stz (user_id, datum, ueberstunden) VALUES ($user_id, '".$month_year_string.$day."', $tages_arbeitszeit)"))
								{
									setError(get_text(857, 'return', 'decode_on') /*Fehler beim Speichern des Stundenzettels!*/ .' '.get_text('', 'return', 'decode_on', 'user_id') /*Benutzer-ID*/ .': '.$user_id.', '.get_text('', 'return', 'decode_on', 'datum') /*Datum*/ .': '.$month_year_string.$day, ERROR, __LINE__);
									$error_count_int++;
								}
							}
							else
							{
								//ACHTUNG Es wird hier davon ausgegangen das wenn ein Tag bereits vorhanden ist dieser auch korrekte Daten enthaelt

								//ACHTUNG Das Update Ueberschreibt alle eventuell vorhandenen Werte. Wenn dies nicht erwuenscht ist die Nachfolgende Schleife Auskommentieren
								//if (!$cdb->update("UPDATE hdb_stz SET kommen=NULL, pause=NULL, gehen=NULL, ist_zeit=NULL, absence_id=0, ueberstunden=$tages_arbeitszeit WHERE stz_id=".$day_result['stz_id']))
								//{
								//	setError(get_text(857, 'return', 'decode_on') /*Fehler beim Speichern des Stundenzettels!*/ .' '.get_text('', 'return', 'decode_on', 'user_id') /*Benutzer-ID*/ .': '.$user_id.', '.get_text('', 'return', 'decode_on', 'datum') /*Datum*/ .': '.$month_year_string.$day, ERROR, __LINE__);
								//	$error_count_int++;
								//}
							}
						}
					}
					if ($error_count_int > 0)
					{
						return false;
					}
					else
					{
						return true;
					}
				}
				else
				{
					return false;
				}
			}
			else
			{
				return false;
			}
		}

		/**
		 * Traegt einen neuen Leeren Stundenzettel ein. Diese Funktion wird nur beim Anlegen eines neuen Benutzers benoetigt
		 *
		 * @param int $user_id
		 * @return boole
		 */
		public function insert_new_time_sheet($user_id)
		{
			$cdb = new MySql;

			$sql = $cdb->select('SELECT user_id, urlaubstage, min_weektime, max_weektime, arbeitstage, startdatum FROM hdb_user WHERE user_id='.$user_id);
			$user_result = $cdb->fetch_array($sql);

			// FIXME: Pausenzeit sollte aus der usertabelle geholt werden, nicht wie derzeit aus der setup-tabelle
			$user_result['pausenzeit'] = intval($GLOBALS['setup_array']['stz_pausenminuten']);//Speichert die Voreinstellung der taegl. Pausenminuten

			$start_datum = substr($user_result['startdatum'], 0, 8).'01';
			$month_timestamp = mktime(0, 0, 0, substr($user_result['startdatum'], 5, 2), substr($user_result['startdatum'], 8, 2), substr($user_result['startdatum'], 0, 4));

			//Schaltjahr
			if (date('L', $month_timestamp) == 1)
			{
				$urlaubstage = round($user_result['urlaubstage'] / 366 * (366 - date('z', $month_timestamp)), 0);
			}
			else
			{
				$urlaubstage = round($user_result['urlaubstage'] / 365 * (365 - date('z', $month_timestamp)), 0);
			}

			if ($user_result['user_id'] > 0 and $cdb->insert("INSERT INTO hdb_stz_monat (user_id, datum, startwert_stunden, startwert_urlaub, wochenarbeitszeit, max_wochenarbeitszeit, arbeitstage, pausenzeit) VALUES ($user_id, '$start_datum', 0, $urlaubstage, ".$user_result['min_weektime'].", '".$user_result['max_weektime']."', '".$user_result['arbeitstage']."', '".$user_result['pausenzeit']."')"))
			{
				$day_count_int = date('t', $month_timestamp);
				$month_year_string = substr($user_result['startdatum'], 0, 7);

				$tages_arbeitszeit = 0 - round($user_result['min_weektime'] / $user_result['arbeitstage'], 2);
				$halbtages_arbeitszeit = 0 - round($user_result['min_weektime'] / $user_result['arbeitstage'] / 2, 2);
				$error_count_int = 0;

				for($i = 1; $i <= $day_count_int; $i++)
				{
					if ($i < 10)
					{
						$day = '-0'.$i;
					}
					else
					{
						$day = '-'.$i;
					}

					//Speichert ob der Tag bereits vorhanden ist
					$insert_day = true;
					$sql = $cdb->select("SELECT * FROM hdb_stz WHERE user_id=$user_id AND datum='".$month_year_string.$day."'");
					$day_result = $cdb->fetch_array($sql);

					if (isset($day_result['stz_id']) and $day_result['stz_id'] > 0)
					{
						$insert_day = false;
					}

					//Neuen Tag als Feiertag eintragen nur wenn kein Wochenende ist
					if ($this->is_absence_holiday($month_year_string.$day) === true and $this->is_holiday($month_year_string.$day, 'weekend') === false)
					{
						if ($insert_day === true)
						{
							//ACHTUNG: die 3 als ID kann sich aendern
							if (!$cdb->insert("INSERT INTO hdb_stz (user_id, datum, absence_id) VALUES ($user_id, '".$month_year_string.$day."', 3)"))
							{
								setError(get_text(857, 'return', 'decode_on') /*Fehler beim Speichern des Stundenzettels!*/ .' '.get_text('', 'return', 'decode_on', 'user_id') /*Benutzer-ID*/ .': '.$user_id.', '.get_text('', 'return', 'decode_on', 'datum') /*Datum*/ .': '.$month_year_string.$day, ERROR, __LINE__);
								$error_count_int++;
							}
						}
						else
						{
							//ACHTUNG Es wird hier davon ausgegangen das wenn ein Tag bereits vorhanden ist dieser auch korrekte Daten enthaelt

							//ACHTUNG Das Update Ueberschreibt alle eventuell vorhandenen Werte. Wenn dies nicht erwuenscht ist die Nachfolgende Schleife Auskommentieren
							//if (!$cdb->update("UPDATE hdb_stz SET kommen=NULL, pause=NULL, gehen=NULL, ist_zeit=NULL, absence_id=3, ueberstunden=NULL WHERE stz_id=".$day_result['stz_id']))
							//{
							//	setError(get_text(857, 'return', 'decode_on') /*Fehler beim Speichern des Stundenzettels!*/ .' '.get_text('', 'return', 'decode_on', 'user_id') /*Benutzer-ID*/ .': '.$user_id.', '.get_text('', 'return', 'decode_on', 'datum') /*Datum*/ .': '.$month_year_string.$day, ERROR, __LINE__);
							//	$error_count_int++;
							//}

						}
					}
					//Halber Arbeitstag
					elseif ($this->is_holiday($month_year_string.$day, 'halbtags'))
					{
						if ($insert_day === true)
						{
							if (!$cdb->insert("INSERT INTO hdb_stz (user_id, datum, ueberstunden) VALUES ($user_id, '".$month_year_string.$day."', $halbtages_arbeitszeit)"))
							{
								setError(get_text(857, 'return', 'decode_on') /*Fehler beim Speichern des Stundenzettels!*/ .' '.get_text('', 'return', 'decode_on', 'user_id') /*Benutzer-ID*/ .': '.$user_id.', '.get_text('', 'return', 'decode_on', 'datum') /*Datum*/ .': '.$month_year_string.$day, ERROR, __LINE__);
								$error_count_int++;
							}
						}
						else
						{
							//ACHTUNG Es wird hier davon ausgegangen das wenn ein Tag bereits vorhanden ist dieser auch korrekte Daten enthaelt

							//ACHTUNG Das Update Ueberschreibt alle eventuell vorhandenen Werte. Wenn dies nicht erwuenscht ist die Nachfolgende Schleife Auskommentieren
							//if (!$cdb->update("UPDATE hdb_stz SET kommen=NULL, pause=NULL, gehen=NULL, ist_zeit=NULL, absence_id=0, ueberstunden=$halbtages_arbeitszeit WHERE stz_id=".$day_result['stz_id']))
							//{
							//	setError(get_text(857, 'return', 'decode_on') /*Fehler beim Speichern des Stundenzettels!*/ .' '.get_text('', 'return', 'decode_on', 'user_id') /*Benutzer-ID*/ .': '.$user_id.', '.get_text('', 'return', 'decode_on', 'datum') /*Datum*/ .': '.$month_year_string.$day, ERROR, __LINE__);
							//	$error_count_int++;
							//}
						}
					}
					//Wochenende
					elseif ($this->is_holiday($month_year_string.$day, 'weekend'))
					{
						if ($insert_day === true)
						{
							if (!$cdb->insert("INSERT INTO hdb_stz (user_id, datum) VALUES ($user_id, '".$month_year_string.$day."')"))
							{
								setError(get_text(857, 'return', 'decode_on') /*Fehler beim Speichern des Stundenzettels!*/ .' '.get_text('', 'return', 'decode_on', 'user_id') /*Benutzer-ID*/ .': '.$user_id.', '.get_text('', 'return', 'decode_on', 'datum') /*Datum*/ .': '.$month_year_string.$day, ERROR, __LINE__);
								$error_count_int++;
							}
						}
						else
						{
							//ACHTUNG Es wird hier davon ausgegangen das wenn ein Tag bereits vorhanden ist dieser auch korrekte Daten enthaelt

							//ACHTUNG Das Update Ueberschreibt alle eventuell vorhandenen Werte. Wenn dies nicht erwuenscht ist die Nachfolgende Schleife Auskommentieren
							//if (!$cdb->update("UPDATE hdb_stz SET kommen=NULL, pause=NULL, gehen=NULL, ist_zeit=NULL, absence_id=0, ueberstunden=NULL WHERE stz_id=".$day_result['stz_id']))
							//{
							//	setError(get_text(857, 'return', 'decode_on') /*Fehler beim Speichern des Stundenzettels!*/ .' '.get_text('', 'return', 'decode_on', 'user_id') /*Benutzer-ID*/ .': '.$user_id.', '.get_text('', 'return', 'decode_on', 'datum') /*Datum*/ .': '.$month_year_string.$day, ERROR, __LINE__);
							//	$error_count_int++;
							//}
						}
					}
					//normaler arbeitstag
					else
					{
						if ($insert_day === true)
						{
							if (!$cdb->insert("INSERT INTO hdb_stz (user_id, datum, ueberstunden) VALUES ($user_id, '".$month_year_string.$day."', $tages_arbeitszeit)"))
							{
								setError(get_text(857, 'return', 'decode_on') /*Fehler beim Speichern des Stundenzettels!*/ .' '.get_text('', 'return', 'decode_on', 'user_id') /*Benutzer-ID*/ .': '.$user_id.', '.get_text('', 'return', 'decode_on', 'datum') /*Datum*/ .': '.$month_year_string.$day, ERROR, __LINE__);
								$error_count_int++;
							}
						}
						else
						{
							//ACHTUNG Es wird hier davon ausgegangen das wenn ein Tag bereits vorhanden ist dieser auch korrekte Daten enthaelt

							//ACHTUNG Das Update Ueberschreibt alle eventuell vorhandenen Werte. Wenn dies nicht erwuenscht ist die Nachfolgende Schleife Auskommentieren
							//if (!$cdb->update("UPDATE hdb_stz SET kommen=NULL, pause=NULL, gehen=NULL, ist_zeit=NULL, absence_id=0, ueberstunden=$tages_arbeitszeit WHERE stz_id=".$day_result['stz_id']))
							//{
							//	setError(get_text(857, 'return', 'decode_on') /*Fehler beim Speichern des Stundenzettels!*/ .' '.get_text('', 'return', 'decode_on', 'user_id') /*Benutzer-ID*/ .': '.$user_id.', '.get_text('', 'return', 'decode_on', 'datum') /*Datum*/ .': '.$month_year_string.$day, ERROR, __LINE__);
							//	$error_count_int++;
							//}
						}
					}
				}
				if ($error_count_int > 0)
				{
					return false;
				}
				else
				{
					return true;
				}
			}
			else
			{
				return false;
			}
		}

		/**
		 * Schliesst einen Stundenzettel ab
		 *
		 * @param int $user_id
		 * @param string $month
		 * @param int $year
		 */
		public function close_time_sheet($user_id, $month, $year)
		{
			$cdb = new MySql;
			$kal = new Kalender;

			$error_count_int = 0;

			$sql = $cdb->select("SELECT COUNT(id) AS count_id FROM hdb_stz_monat WHERE user_id=$user_id AND datum>='".$year.'-'.$month.'-'."-01'");
			$result = $cdb->fetch_array($sql);

			//Einfach den STZ Abschliessen und einen neuen Leeren erstellen
			if ($result['count_id'] == 1)
			{
				//uebertraege aus dem Monat auslesen
				$sql = $cdb->select("SELECT id, datum, uebertrag_stunden, uebertrag_urlaub FROM hdb_stz_monat WHERE user_id=$user_id AND datum='".$year.'-'.$month.'-'."-01'");
				$result = $cdb->fetch_array($sql);
				//Userdaten auslesen
				$sql = $cdb->select('SELECT urlaubstage, min_weektime, max_weektime, arbeitstage, anzeigen FROM hdb_user WHERE user_id='.$user_id);
				$user_result = $cdb->fetch_array($sql);

				// FIXME: Pausenzeit sollte aus der usertabelle geholt werden, nicht wie derzeit aus der setup-tabelle
				$user_result['pausenzeit'] = intval($GLOBALS['setup_array']['stz_pausenminuten']);//Speichert die Voreinstellung der taegl. Pausenminuten

				//Stundenzettel abschliessen
				if ($cdb->update("UPDATE hdb_stz_monat SET abgeschlossen='j' WHERE id=".$result['id']))
				{
					//pruefen ob der user aktiv ist und ob ein stundenzettel angelegt werden muss
					if ($user_result['anzeigen'] == 'j')
					{
						$next_month = $kal->date_calculator($result['datum'], '', 1, '');

						//Januar Jahresurlaubsanspruch dazurechnen
						if (substr($next_month[0], -5) == '01-01')
						{
							$new_uebertrag_urlaub = $result['uebertrag_urlaub'] + $user_result['urlaubstage'];
						}
						else
						{
							$new_uebertrag_urlaub = $result['uebertrag_urlaub'];
						}

						if ($cdb->insert("INSERT INTO hdb_stz_monat (user_id, datum, startwert_stunden, startwert_urlaub, wochenarbeitszeit, max_wochenarbeitszeit, arbeitstage, pausenzeit) VALUES ($user_id, '".$next_month[0]."', '".$result['uebertrag_stunden']."', '$new_uebertrag_urlaub', '".$user_result['min_weektime']."', '".$user_result['max_weektime']."', '".$user_result['arbeitstage']."', '".$user_result['pausenzeit']."')"))
						{
							$day_count_int = date('t', $next_month[1]);
							$month_year_string = date('Y-m', $next_month[1]);

							$tages_arbeitszeit = 0 - round($user_result['min_weektime'] / $user_result['arbeitstage'], 2);
							$halbtages_arbeitszeit = 0 - round($user_result['min_weektime'] / $user_result['arbeitstage'] / 2, 2);

							for($i = 1; $i <= $day_count_int; $i++)
							{
								if ($i < 10)
								{
									$day = '-0'.$i;
								}
								else
								{
									$day = '-'.$i;
								}

								//Speichert ob der Tag bereits vorhanden ist
								$insert_day = true;
								$sql = $cdb->select("SELECT * FROM hdb_stz WHERE user_id=$user_id AND datum='".$month_year_string.$day."'");
								$day_result = $cdb->fetch_array($sql);

								if (isset($day_result['stz_id']) and $day_result['stz_id'] > 0)
								{
									$insert_day = false;
								}

								//Neuen Tag als Feiertag eintragen nur wenn kein Wochenende ist
								if ($this->is_absence_holiday($month_year_string.$day) === true and $this->is_holiday($month_year_string.$day, 'weekend') === false)
								{
									if ($insert_day === true)
									{
										//ACHTUNG: die 3 als ID kann sich aendern
										if (!$cdb->insert("INSERT INTO hdb_stz (user_id, datum, absence_id) VALUES ($user_id, '".$month_year_string.$day."', 3)"))
										{
											setError(get_text(857, 'return', 'decode_on') /*Fehler beim Speichern des Stundenzettels!*/ .' '.get_text('', 'return', 'decode_on', 'user_id') /*Benutzer-ID*/ .': '.$user_id.', '.get_text('', 'return', 'decode_on', 'datum') /*Datum*/ .': '.$month_year_string.$day, ERROR, __LINE__);
											$error_count_int++;
										}
									}
									else
									{
										//ACHTUNG Es wird hier davon ausgegangen das wenn ein Tag bereits vorhanden ist dieser auch korrekte Daten enthaelt

										//ACHTUNG Das Update Ueberschreibt alle eventuell vorhandenen Werte. Wenn dies nicht erwuenscht ist die Nachfolgende Schleife Auskommentieren
										//if (!$cdb->update("UPDATE hdb_stz SET kommen=NULL, pause=NULL, gehen=NULL, ist_zeit=NULL, absence_id=3, ueberstunden=NULL WHERE stz_id=".$day_result['stz_id']))
										//{
										//	setError(get_text(857, 'return', 'decode_on') /*Fehler beim Speichern des Stundenzettels!*/ .' '.get_text('', 'return', 'decode_on', 'user_id') /*Benutzer-ID*/ .': '.$user_id.', '.get_text('', 'return', 'decode_on', 'datum') /*Datum*/ .': '.$month_year_string.$day, ERROR, __LINE__);
										//	$error_count_int++;
										//}
									}
								}
								//Halber Arbeitstag
								elseif ($this->is_holiday($month_year_string.$day, 'halbtags'))
								{
									if ($insert_day === true)
									{
										if (!$cdb->insert("INSERT INTO hdb_stz (user_id, datum, ueberstunden) VALUES ($user_id, '".$month_year_string.$day."', $halbtages_arbeitszeit)"))
										{
											setError(get_text(857, 'return', 'decode_on') /*Fehler beim Speichern des Stundenzettels!*/ .' '.get_text('', 'return', 'decode_on', 'user_id') /*Benutzer-ID*/ .': '.$user_id.', '.get_text('', 'return', 'decode_on', 'datum') /*Datum*/ .': '.$month_year_string.$day, ERROR, __LINE__);
											$error_count_int++;
										}
									}
									else
									{
										//ACHTUNG Es wird hier davon ausgegangen das wenn ein Tag bereits vorhanden ist dieser auch korrekte Daten enthaelt

										//ACHTUNG Das Update Ueberschreibt alle eventuell vorhandenen Werte. Wenn dies nicht erwuenscht ist die Nachfolgende Schleife Auskommentieren
										//if (!$cdb->update("UPDATE hdb_stz SET kommen=NULL, pause=NULL, gehen=NULL, ist_zeit=NULL, absence_id=0, ueberstunden=$halbtages_arbeitszeit WHERE stz_id=".$day_result['stz_id']))
										//{
										//	setError(get_text(857, 'return', 'decode_on') /*Fehler beim Speichern des Stundenzettels!*/ .' '.get_text('', 'return', 'decode_on', 'user_id') /*Benutzer-ID*/ .': '.$user_id.', '.get_text('', 'return', 'decode_on', 'datum') /*Datum*/ .': '.$month_year_string.$day, ERROR, __LINE__);
										//	$error_count_int++;
										//}
									}
								}
								//Wochenende
								elseif ($this->is_holiday($month_year_string.$day, 'weekend'))
								{
									if ($insert_day === true)
									{
										if (!$cdb->insert("INSERT INTO hdb_stz (user_id, datum) VALUES ($user_id, '".$month_year_string.$day."')"))
										{
											setError(get_text(857, 'return', 'decode_on') /*Fehler beim Speichern des Stundenzettels!*/ .' '.get_text('', 'return', 'decode_on', 'user_id') /*Benutzer-ID*/ .': '.$user_id.', '.get_text('', 'return', 'decode_on', 'datum') /*Datum*/ .': '.$month_year_string.$day, ERROR, __LINE__);
											$error_count_int++;
										}
									}
									else
									{
										//ACHTUNG Es wird hier davon ausgegangen das wenn ein Tag bereits vorhanden ist dieser auch korrekte Daten enthaelt

										//ACHTUNG Das Update Ueberschreibt alle eventuell vorhandenen Werte. Wenn dies nicht erwuenscht ist die Nachfolgende Schleife Auskommentieren
										//if (!$cdb->update("UPDATE hdb_stz SET kommen=NULL, pause=NULL, gehen=NULL, ist_zeit=NULL, absence_id=0, ueberstunden=NULL WHERE stz_id=".$day_result['stz_id']))
										//{
										//	setError(get_text(857, 'return', 'decode_on') /*Fehler beim Speichern des Stundenzettels!*/ .' '.get_text('', 'return', 'decode_on', 'user_id') /*Benutzer-ID*/ .': '.$user_id.', '.get_text('', 'return', 'decode_on', 'datum') /*Datum*/ .': '.$month_year_string.$day, ERROR, __LINE__);
										//	$error_count_int++;
										//}
									}
								}
								//normaler arbeitstag
								else
								{
									if ($insert_day === true)
									{
										$tmp_absence_id = $this->is_absence_day($user_id, $month_year_string.$day);

										//Genhemigter Abwesenheitsantrag
										if ($tmp_absence_id > 0)
										{
											if (!$cdb->insert("INSERT INTO hdb_stz (user_id, datum, absence_id) VALUES ($user_id, '".$month_year_string.$day."', $tmp_absence_id)"))
											{
												setError(get_text(857, 'return', 'decode_on') /*Fehler beim Speichern des Stundenzettels!*/ .' '.get_text('', 'return', 'decode_on', 'user_id') /*Benutzer-ID*/ .': '.$user_id.', '.get_text('', 'return', 'decode_on', 'datum') /*Datum*/ .': '.$month_year_string.$day, ERROR, __LINE__);
												$error_count_int++;
											}
										}
										//normaler Arbeitstag
										else
										{
											if (!$cdb->insert("INSERT INTO hdb_stz (user_id, datum, ueberstunden) VALUES ($user_id, '".$month_year_string.$day."', $tages_arbeitszeit)"))
											{
												setError(get_text(857, 'return', 'decode_on') /*Fehler beim Speichern des Stundenzettels!*/ .' '.get_text('', 'return', 'decode_on', 'user_id') /*Benutzer-ID*/ .': '.$user_id.', '.get_text('', 'return', 'decode_on', 'datum') /*Datum*/ .': '.$month_year_string.$day, ERROR, __LINE__);
												$error_count_int++;
											}
										}
									}
									else
									{
										//ACHTUNG Es wird hier davon ausgegangen das wenn ein Tag bereits vorhanden ist dieser auch korrekte Daten enthaelt

										//ACHTUNG Das Update Ueberschreibt alle eventuell vorhandenen Werte. Wenn dies nicht erwuenscht ist die Nachfolgende Schleife Auskommentieren
										//if (!$cdb->update("UPDATE hdb_stz SET kommen=NULL, pause=NULL, gehen=NULL, ist_zeit=NULL, absence_id=0, ueberstunden=$tages_arbeitszeit WHERE stz_id=".$day_result['stz_id']))
										//{
										//	setError(get_text(857, 'return', 'decode_on') /*Fehler beim Speichern des Stundenzettels!*/ .' '.get_text('', 'return', 'decode_on', 'user_id') /*Benutzer-ID*/ .': '.$user_id.', '.get_text('', 'return', 'decode_on', 'datum') /*Datum*/ .': '.$month_year_string.$day, ERROR, __LINE__);
										//	$error_count_int++;
										//}
									}
								}
							}
							if ($error_count_int > 0)
							{
								return false;
							}
							else
							{
								return true;
							}
						}
						else
						{
							setError(get_text(859, 'return', 'decode_on') /*Beim Abschliessen den Stundenzettels ist ein Fehler aufgetreten!*/ .' '.get_text('', 'return', 'decode_on', 'user_id') /*Benutzer-ID*/ .': '.$user_id.', '.get_text('', 'return', 'decode_on', 'datum') /*Datum*/ .': '.$next_month[0], ERROR, __LINE__);
							return false;
						}
					}
				}
				else
				{
					setError(get_text(859, 'return', 'decode_on') /*Beim Abschliessen den Stundenzettels ist ein Fehler aufgetreten!*/ .' '.get_text('', 'return', 'decode_on', 'user_id') /*Benutzer-ID*/ .': '.$user_id.', '.get_text('', 'return', 'decode_on', 'datum') /*Datum*/ .': '.$year.'-'.$month.'-01', WARNING, __LINE__);
					return false;
				}
			}
			//Es existieren bereits nachfolge Stundenzettel. Der uebertrag dieser Stundenzettel muss angepasst werden.
			else
			{
				//alle Monate finden deren uebertrag bearbeitet werden muss
				$time_sheet_edit_id_array = array();
				$sql = $cdb->select("SELECT id FROM hdb_stz_monat WHERE user_id=$user_id AND datum>'".$year.'-'.$month.'-'."-01' ORDER BY datum ASC");
				while ($result = $cdb->fetch_array($sql))
				{
					$time_sheet_edit_id_array[] = $result['id'];
				}

				//uebertraege aus dem Monat auslesen
				$sql = $cdb->select("SELECT id, uebertrag_stunden, uebertrag_urlaub FROM hdb_stz_monat WHERE user_id=$user_id AND datum='".$year.'-'.$month.'-'."-01'");
				$result = $cdb->fetch_array($sql);

				//Stundenzettel abschliessen
				if ($cdb->update("UPDATE hdb_stz_monat SET abgeschlossen='j' WHERE id=".$result['id']))
				{
					$new_start_stunden = $result['uebertrag_stunden'];
					$new_start_urlaub = $result['uebertrag_urlaub'];

					for($i = 0; $i < count($time_sheet_edit_id_array); $i++)
					{
						$sql = $cdb->select('SELECT startwert_stunden, startwert_urlaub, uebertrag_stunden, uebertrag_urlaub FROM hdb_stz_monat WHERE id='.$time_sheet_edit_id_array[$i]);
						$time_result = $cdb->fetch_array($sql);

						//pruefen ob ueberhaupt eine uebertragsaenderung erforderlich ist
						if ($i == 0 and $new_start_stunden == $time_result['startwert_stunden'] and $new_start_urlaub == $time_result['startwert_urlaub'])
						{
							break;
						}

						//Differenzstundenermitteln
						if ($time_result['startwert_stunden'] > $new_start_stunden)
						{
							$new_uebertrag_stunden = $time_result['uebertrag_stunden'] - ($time_result['startwert_stunden'] - $new_start_stunden);
						}
						elseif ($time_result['startwert_stunden'] < $new_start_stunden)
						{
							$new_uebertrag_stunden = $time_result['uebertrag_stunden'] + ($new_start_stunden - $time_result['startwert_stunden']);
						}
						else
						{
							$new_uebertrag_stunden = $time_result['uebertrag_stunden'];
						}

						//Differenzurlaub ermitteln
						if ($time_result['startwert_urlaub'] > $new_start_urlaub)
						{
							$new_uebertrag_urlaub = $time_result['uebertrag_urlaub'] - ($time_result['startwert_urlaub'] - $new_start_urlaub);
						}
						elseif ($time_result['startwert_urlaub'] < $new_start_urlaub)
						{
							$new_uebertrag_urlaub = $time_result['uebertrag_urlaub'] + ($new_start_urlaub - $time_result['startwert_urlaub']);
						}
						else
						{
							$new_uebertrag_urlaub = $time_result['uebertrag_urlaub'];
						}

						if (!$cdb->update("UPDATE hdb_stz_monat SET
											startwert_stunden='$new_start_stunden',
											startwert_urlaub='$new_start_urlaub',
											uebertrag_stunden='$new_uebertrag_stunden',
											uebertrag_urlaub='$new_uebertrag_urlaub'
											WHERE id=".$time_sheet_edit_id_array[$i]))
						{
							$error_count_int++;
							break;
						}

						$new_start_stunden = $new_uebertrag_stunden;
						$new_start_urlaub = $new_uebertrag_urlaub;
					}

					if ($error_count_int == 0)
					{
						return true;
					}
					else
					{
						setError(get_text(860, 'return', 'decode_on') /*Beim Speichern der uebertragsaenderung ist ein Fehler aufgetreten!*/ .' '.get_text('', 'return', 'decode_on', 'user_id') /*Benutzer-ID*/ .': '.$user_id, ERROR, __LINE__);
						return false;
					}
				}
				else
				{
					setError(get_text(859, 'return', 'decode_on') /*Beim Abschliessen den Stundenzettels ist ein Fehler aufgetreten!*/ .' '.get_text('', 'return', 'decode_on', 'user_id') /*Benutzer-ID*/ .': '.$user_id.', '.get_text('', 'return', 'decode_on', 'datum') /*Datum*/ .': '.$year.'-'.$month.'-01', WARNING, __LINE__);
					return false;
				}
			}
		}

		/**
		 * prueft ob fuer das uebergebene Datum der Abwesenheitsgrund Feiertag verwendet werden soll.
		 * ACHTUNG: Nur fuer das Eintragen von Neuen Leeren Stundenzetteln verwenden
		 *
		 * @param string $date, ISO Datum YYYY-MM-DD
		 */
		protected function is_absence_holiday($date)
		{
			$kal = new Kalender;

			if ($kal->is_holiday($date))
			{
				return true;
			}
			else
			{
				return false;
			}
		}

		/**
		 * pruefen ob das uebergebene Datum ein Freier Tag ist
		 *
		 * @param string $date
		 * @param string $typ
		 * @return boole
		 */
		protected function is_holiday($date, $typ)
		{
			$cdb = new MySql;
			$kal = new Kalender;

			//Feiertage finden
			if ($typ == 'holiday')
			{
				$sql = $cdb->select("SELECT absence_id FROM hdb_stz WHERE user_id=".$this->setup_array['user_id']." AND datum='$date'");
				$result = $cdb->fetch_array($sql);

				//ACHTUNG: die 3 als ID kann sich aendern
				if (isset($result['absence_id']) and $result['absence_id'] == 3)
				{
					return true;
				}
				else
				{
					return false;
				}
			}
			//Wochenende finden
			elseif ($typ == 'weekend')
			{
				if ($kal->is_weekend($date))
				{
					return true;
				}
				else
				{
					return false;
				}
			}
			//Arbeitstage mit verringerter Sollstundenzahl
			elseif ($typ == 'halbtags')
			{
				if ($kal->is_half_holiday($date))
				{
					return true;
				}
				else
				{
					return false;
				}
			}
		}

		/**
		 * Prueft ob ein genehmigter abwenheitsgrund vorliegt
		 * @param int $user_id
		 * @param string $date
		 * @return int
		 */
		protected function is_absence_day($user_id, $date)
		{
			$cdb = new MySql();

			$sql = $cdb->select("SELECT absence_type_id FROM hdb_absence_list WHERE user_id=$user_id AND date_from<='$date' AND date_to>='$date' AND status='approved'");
			$result = $cdb->fetch_array($sql);

			if (isset($result['absence_type_id']) and !empty($result['absence_type_id']))
			{
				return $result['absence_type_id'];
			}
			else
			{
				return 0;
			}
		}

		/**
		 * Gibt den Namen eines Tages zurueck
		 * @param string $date, ISO Datum
		 * @param string $ytp, L = langname (Montag) oder K = kurzname (Mo)
		 * @return string
		 */
		public function get_day_name($date, $typ = 'L')
		{
			$split_date_array = explode('-', $date);

			$index = date('w', mktime(0, 0, 0, $split_date_array[1], $split_date_array[2], $split_date_array[0]));

			if ($index == 0)
			{
				if ($typ == 'L') { return get_text('', 'return', 'decode_on', 'so');/*Sonntag*/ }
				else { return get_text('', 'return', 'decode_on', 'so_kurz');/*So*/ }
			}
			elseif ($index == 1)
			{
				if ($typ == 'L') { return get_text('', 'return', 'decode_on', 'mo');/*Montag*/ }
				else { return get_text('', 'return', 'decode_on', 'mo_kurz');/*Mo*/ }
			}
			elseif ($index == 2)
			{
				if ($typ == 'L') { return get_text('', 'return', 'decode_on', 'di');/*Dienstag*/ }
				else { return get_text('', 'return', 'decode_on', 'di_kurz');/*Di*/ }
			}
			elseif ($index == 3)
			{
				if ($typ == 'L') { return get_text('', 'return', 'decode_on', 'mi');/*Mittwoch*/ }
				else { return get_text('', 'return', 'decode_on', 'mi_kurz');/*Mi*/ }
			}
			elseif ($index == 4)
			{
				if ($typ == 'L') { return get_text('', 'return', 'decode_on', 'do');/*Donnerstag*/ }
				else { return get_text('', 'return', 'decode_on', 'do_kurz');/*Do*/ }
			}
			elseif ($index == 5)
			{
				if ($typ == 'L') { return get_text('', 'return', 'decode_on', 'fr');/*Freitag*/ }
				else { return get_text('', 'return', 'decode_on', 'fr_kurz');/*Fr*/ }
			}
			elseif ($index == 6)
			{
				if ($typ == 'L') { return get_text('', 'return', 'decode_on', 'sa');/*Samstag*/ }
				else { return get_text('', 'return', 'decode_on', 'sa_kurz');/*Sa*/ }
			}
		}

		/**
		 * Erstellt ein Array mit den Abwesenheitsgraenden
		 *
		 */
		protected function get_absence_list()
		{
			$cdb = new MySql;

			$sql = $cdb->select('SELECT * FROM hdb_absence_type ORDER BY absence_name ASC');
			while ($result = $cdb->fetch_array($sql))
			{
				$this->absence_array[$result['absence_type_id']] = array('absence_name'	=> $result['absence_name'],
																		'working_time'	=> $result['working_time'],
																		'paid'			=> $result['paid'],
																		'total_day'		=> $result['total_day'],
																		'display'		=> $result['display']);
			}
		}

		/**
		 * Gibt den Namen eines Monats zurueck
		 * @param string $date, ISO Datum oder Monatsnummer
		 * @return string
		 */
		protected function get_month_name($date)
		{
			if (strlen($date) == 10)
			{
				$split_date_array = explode('-', $date);
				$index = intval($split_date_array[1]);
			}
			else
			{
				$index = intval($date);
			}

			if ($index == 1) { return get_text('', 'return', 'decode_on', 'jan');/*Januar*/ }
			elseif ($index == 2) { return get_text('', 'return', 'decode_on', 'feb');/*Februar*/ }
			elseif ($index == 3) { return get_text('', 'return', 'decode_on', 'mar');/*Maerz*/ }
			elseif ($index == 4) { return get_text('', 'return', 'decode_on', 'apr');/*April*/ }
			elseif ($index == 5) { return get_text('', 'return', 'decode_on', 'mai');/*Mai*/ }
			elseif ($index == 6) { return get_text('', 'return', 'decode_on', 'jun');/*Juni*/ }
			elseif ($index == 7) { return get_text('', 'return', 'decode_on', 'jul');/*Juli*/ }
			elseif ($index == 8) { return get_text('', 'return', 'decode_on', 'aug');/*August*/ }
			elseif ($index == 9) { return get_text('', 'return', 'decode_on', 'sep');/*September*/ }
			elseif ($index == 10) { return get_text('', 'return', 'decode_on', 'okt');/*Oktober*/ }
			elseif ($index == 11) { return get_text('', 'return', 'decode_on', 'nov');/*November*/ }
			elseif ($index == 12) { return get_text('', 'return', 'decode_on', 'dez');/*Dezember*/ }
		}

		/**
		 * Befuellt das $_POST bzw. $this->tmp_data_save_array['timesheet'] Array mit Zeitdaten aus der Datenbank
		 *
		 * @return void
		 */
		private function create_time_array()
		{
			$cdb = new MySql;
			$log = new Logbook;

			//Tage abarbeiten
			for($i = 0; $i < count($this->setup_array['days']); $i++)
			{
				$day = intval(substr($this->setup_array['days'][$i]['day'], -2));

				//Taggeseinstellungen auslesen
				//Wochenende
				if ($this->is_holiday($this->setup_array['days'][$i]['day'], 'weekend') === true)
				{
					$this->setup_day_array[$day] = array('feiertag'			=> false,
														'wochenende'		=> true,
														'halbtags'			=> false,
														'reg_tag'			=> false,
														'min_sollstunden'	=> 0,
														'max_sollstunden'	=> 0,
														'urlaubstag'		=> 0);
				}
				//Feiertag
				elseif ($this->is_holiday($this->setup_array['days'][$i]['day'], 'holiday') === true)
				{
					$this->setup_day_array[$day] = array('feiertag'			=> true,
														'wochenende'		=> false,
														'halbtags'			=> false,
														'reg_tag'			=> false,
														'min_sollstunden'	=> 0,
														'max_sollstunden'	=> 0,
														'urlaubstag'		=> 0);
				}
				//Halbtags
				elseif ($this->is_holiday($this->setup_array['days'][$i]['day'], 'halbtags') === true)
				{
					$this->setup_day_array[$day] = array('feiertag'			=> false,
														'wochenende'		=> false,
														'halbtags'			=> true,
														'reg_tag'			=> false,
														'min_sollstunden'	=> round($this->setup_array['min_weektime'] / $this->setup_array['arbeitstage'] / 2, 2),
														'max_sollstunden'	=> round($this->setup_array['max_weektime'] / $this->setup_array['arbeitstage'] / 2, 2),
														'urlaubstag'		=> 0.5);
				}
				//Normaler Arbeitstag
				else
				{
					$this->setup_day_array[$day] = array('feiertag'			=> false,
														'wochenende'		=> false,
														'halbtags'			=> false,
														'reg_tag'			=> true,
														'min_sollstunden'	=> round($this->setup_array['min_weektime'] / $this->setup_array['arbeitstage'], 2),
														'max_sollstunden'	=> round($this->setup_array['max_weektime'] / $this->setup_array['arbeitstage'], 2),
														'urlaubstag'		=> 1);
				}

				//Monatssollstunden
				$this->setup_array['month_sollstunden'] += $this->setup_day_array[$day]['min_sollstunden'];



				$sql = $cdb->select("SELECT t1.kommen, t1.pause, t1.gehen, t1.ist_zeit, t1.absence_id, t1.ueberstunden, t2.working_time AS absence_time FROM hdb_stz t1
									LEFT JOIN hdb_absence_type t2 ON t1.absence_id=t2.absence_type_id
									WHERE t1.user_id=".$this->setup_array['user_id']." AND t1.datum='".$this->setup_array['days'][$i]['day']."'");
				$result = $cdb->fetch_array($sql);

				//pruefen ob Arbeitszeiten vorhanden sind
				if (!empty($result['kommen']) or !empty($result['gehen']))
				{
					$_POST['kommen'][$day]	= $result['kommen'];
					$_POST['pause'][$day]	= $result['pause'];
					$_POST['gehen'][$day]	= $result['gehen'];
				}

				//pruefen ob eine ist_zeit vorhanden ist
				if (!empty($result['ist_zeit']))
				{
					$this->tmp_data_save_array['timesheet']['istzeit'][$day] = $result['ist_zeit'];
					//Monatsiststunden
					$this->setup_array['month_iststunden'] += $result['ist_zeit'];
				}
				//Abwesenheit
				if (!empty($result['absence_id']))
				{
					$_POST['abwesenheit'][$day] = $result['absence_id'];

					if ($this->absence_array[$result['absence_id']]['working_time'] == 1)
					{
						if ($this->absence_array[$result['absence_id']]['total_day'] != 0)
						{
							//Bezahlte Abwesenheitszeit fuer den Monat Addieren
							$this->setup_array['month_absence_time'] += $this->setup_day_array[$day]['min_sollstunden'];
							$this->setup_array['paid_absence_typ'][$day] = true;
						}
						elseif ($this->absence_array[$result['absence_id']]['total_day'] == 0)
						{
							$this->setup_array['month_absence_time'] += $this->setup_day_array[$day]['min_sollstunden'] - $result['ist_zeit'];;
						}
					}

					if ($this->absence_array[$result['absence_id']]['total_day'] == 0)
					{
						//$this->setup_array['month_iststunden'] += $this->setup_day_array[$day]['min_sollstunden'] - $result['ist_zeit'];
					}

					//Urlaubstage zaehlen
					//ACHTUNG die id kann sich aendern
					if ($result['absence_id'] == 2)
					{
						if ($this->absence_array[$result['absence_id']]['total_day'] == 0 and $result['ist_zeit'] > 0)
						{
							$this->setup_array['month_urlaub_count'] += round(($this->setup_day_array[$day]['min_sollstunden'] - $result['ist_zeit']) / $this->setup_day_array[$day]['min_sollstunden'] * $this->setup_day_array[$day]['urlaubstag'], 2);
							$this->tmp_data_save_array['timesheet']['urlaub'][$day] = round(($this->setup_day_array[$day]['min_sollstunden'] - $result['ist_zeit']) / $this->setup_day_array[$day]['min_sollstunden'] * $this->setup_day_array[$day]['urlaubstag'], 2);
						}
						else
						{
							$this->setup_array['month_urlaub_count'] += $this->setup_day_array[$day]['urlaubstag'];
							$this->tmp_data_save_array['timesheet']['urlaub'][$day] = $this->setup_day_array[$day]['urlaubstag'];
						}
					}
					else
					{
						$this->tmp_data_save_array['timesheet']['urlaub'][$day] = 0;
					}
				}
				else
				{
					$this->tmp_data_save_array['timesheet']['urlaub'][$day] = 0;
				}
				//ueberstunden
				if (!empty($result['ueberstunden']))
				{
					if ($result['absence_id'] > 0)
					{
						if ($this->absence_array[$result['absence_id']]['total_day'] != 0)
						{
							$this->tmp_data_save_array['timesheet']['zeit'][$day] = $result['ueberstunden'];
						}
					}
					else
					{
						$this->tmp_data_save_array['timesheet']['zeit'][$day] = $result['ueberstunden'];
					}
					//Monatsueberstunden
					$this->setup_array['month_ueberstunden'] += $result['ueberstunden'];
				}
				else
				{
					if ($result['absence_id'] > 0 and $result['absence_time'] == 0 and $this->setup_day_array[$day]['min_sollstunden'] > 0)
					{
						$this->tmp_data_save_array['timesheet']['zeit'][$day] = 0 - $this->setup_day_array[$day]['min_sollstunden'];
						//Monatsueberstunden
						$this->setup_array['month_ueberstunden'] += $this->tmp_data_save_array['timesheet']['zeit'][$day];
					}
				}

				//Tageskommentare
				$sql = $cdb->select("SELECT kommentar FROM hdb_stz_kommentar WHERE projekt_id=0 AND user_id=".$this->setup_array['user_id']." AND datum='".$this->setup_array['days'][$i]['day']."'");
				$comment_result = $cdb->fetch_array($sql);
				if (isset($comment_result['kommentar']) and !empty($comment_result['kommentar']))
				{
					$_POST['kommentar'][$day] = $comment_result['kommentar'];
				}

				//Projekt
				$tmp_project_time_float = 0;
				for($j = 0; $j < count($this->project_array); $j++)
				{
					$sql = $cdb->select("SELECT stunden, kommen, pause, gehen FROM hdb_stz_projekt WHERE user_id=".$this->setup_array['user_id']." AND projekt_id=".$this->project_array[$j]['id']." AND datum='".$this->setup_array['days'][$i]['day']."'");
					$project_result = $cdb->fetch_array($sql);

					if (isset($project_result['stunden']) and !empty($project_result['stunden']))
					{
						$_POST['projektzeit'][$day][$this->project_array[$j]['id']]['stunden'] = $project_result['stunden'];
						$tmp_project_time_float += $project_result['stunden'];

						//Monats Iststunden fuer ein Projekt zaehlen
						if (isset($this->setup_array['month_project_iststunden'][$this->project_array[$j]['id']]))
						{
							$this->setup_array['month_project_iststunden'][$this->project_array[$j]['id']] += $project_result['stunden'];
						}
						else
						{
							$this->setup_array['month_project_iststunden'][$this->project_array[$j]['id']] = $project_result['stunden'];
						}
						//pruefen Projektzeiten vorhanden sind
						if (isset($project_result['kommen']) and !empty($project_result['kommen']))
						{
							$_POST['projektzeit'][$day][$this->project_array[$j]['id']]['kommen'] = $project_result['kommen'];
							$_POST['projektzeit'][$day][$this->project_array[$j]['id']]['pause'] = $project_result['pause'];
							$_POST['projektzeit'][$day][$this->project_array[$j]['id']]['gehen'] = $project_result['gehen'];
						}
					}

					$sql = $cdb->select("SELECT kommentar FROM hdb_stz_kommentar WHERE projekt_id=".$this->project_array[$j]['id']." AND user_id=".$this->setup_array['user_id']." AND datum='".$this->setup_array['days'][$i]['day']."'");
					$projekt_comment_result = $cdb->fetch_array($sql);
					if (isset($projekt_comment_result['kommentar']) and !empty($projekt_comment_result['kommentar']))
					{
						$_POST['projektkommentar'][$day][$this->project_array[$j]['id']] = $projekt_comment_result['kommentar'];
					}
				}

				//NZS
				if ($result['ist_zeit'] - $tmp_project_time_float != 0)
				{
					$this->tmp_data_save_array['timesheet']['nzs'][$day] = $result['ist_zeit'] - $tmp_project_time_float;
					//Monats NZSstunden
					$this->setup_array['month_nzsstunden'] += $this->tmp_data_save_array['timesheet']['nzs'][$day];
				}
			}

			//Logbucheintrag erstellen wenn der Stundenzettel bearbeitet werden soll
			if (isset($this->setup_array['return']) and $this->setup_array['return'] == 'edit')
			{
				$log->insert_log_stz($this->setup_array, $_POST);
			}
		}
	}
?>