<?php
	/**
	 * verification of timesheet data
	 *
	 * @author Soeren Pieper <soeren.pieper@docemos.de> / Martin Ufer <martin.ufer@docemos.de>
	 * @copyright Copyright (C) 2009  docemos GmbH
	 * @package time-o-meter
	 *
	 * This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 3 of the License, or (at your option) any later version.
	 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
	 * You should have received a copy of the GNU General Public License along with this program; if not, see http://www.gnu.org/licenses/.
	 */

	/**
	 * verification of timesheet data
	 * @package time-o-meter
	 * @subpackage classes
	 */
	class TimeSheetPlausibility
	{
		//Speichert grundeinstellungen
		public $setup_array = array();
		//Speichert alle aufgetrettenen Fehlermeldungen
		public $error_array = array();

		public function __construct()
		{
			$kal = new Kalender;

			//erste tag des aktuellen monats minus 1 monat
			$tmp_lastmonth = $kal->date_calculator(date('Y-m').'-01', 0, -1, 0);

			$this->setup_array['last_date']				= $tmp_lastmonth[0];//erste tag des letzten monats z.B. 2008-12-01
			$this->setup_array['last_year']				= $tmp_lastmonth[2]['year'];//jahr des letzten monats z.B. 2008
			$this->setup_array['last_month']			= $tmp_lastmonth[2]['month'];//monat des letzten monats z.B. 09
			$this->setup_array['last_year_month']		= $tmp_lastmonth[2]['year'].'-'.$tmp_lastmonth[2]['month'];//jahr und monat des letzten monats z.B. 2008-10
			$this->setup_array['current_date']			= date('Y-m').'-01';//erste tag des aktuellen monats z.B. 2008-12-01
			$this->setup_array['current_year']			= date('Y');//Jahr des aktuellen monats z.B. 2008
			$this->setup_array['current_month']			= date('m');//monat des aktuellen monats z.B. 09
			$this->setup_array['current_year_month']	= date('Y-m');//Jahr und monat des aktuellen monats z.B. 2008-10
			$this->setup_array['weekendlist']			= array();//Speichert tage an den Wochenende ist.
		}

		/**
		 * Erstellt ein array mit allen gefundenen moeglichen Fehlern in den Stundenzetteln des letzten Monats
		 *
		 * @param int $user_id
		 * @return array
		 */
		public function chk_timesheet($user_id = 0)
		{
			$cdb = new MySql;

			if ($user_id > 0)
			{
				$where = "user_id=$user_id AND";
			}
			else
			{
				$where = '';
			}

			//alle stundenzettel des vergangenen monats
			$sql = $cdb->select("SELECT id, user_id, datum, uebertrag_stunden, uebertrag_urlaub, abgeschlossen FROM hdb_stz_monat WHERE $where datum='".$this->setup_array['last_date']."'");
			while ($result = $cdb->fetch_array($sql))
			{
				//pruefen ob fuer den Aktuellen Monat bereits ein Stundenzettel existiert
				$sql_c = $cdb->select("SELECT id FROM hdb_stz_monat WHERE user_id=".$result['user_id']." AND datum='".$this->setup_array['current_date']."'");
				$result_c = $cdb->fetch_array($sql_c);

				//fuer den Aktuellen Monat existiert ein Stundenzettel
				if (isset($result_c['id']) and $result_c['id'] > 0)
				{
					//uebertraege pruefen
					$this->chk_balance($result);
				}
				//Der Stundenzettel des letzten Monats ist abgeschlossen und kann geprueft werden
				if ($result['abgeschlossen'] == 'j')
				{
					//Tage pruefen
					$this->chk_day_time($result);
					//Projektzeiten pruefen
					$this->chk_project_time($result);
				}
			}

			return $this->error_array;
		}

		/**
		 * prueft ob die Projektzeiten mit den Tageszeiten uebereinstimmen
		 *
		 * @param array $data
		 * @return void
		 */
		private function chk_project_time($data)
		{
			$cdb = new MySql;

			//pruefen ob dem Mitarbeiter im letzten Monat Projekte zugeordnet waren
			$sql= $cdb->select("SELECT COUNT(t1.user_projekt_id) AS count_id FROM hdb_user_projekt t1
								LEFT JOIN hdb_projekte t2 ON t1.projekt_id=t2.projekt_id
								WHERE t1.user_id=".$data['user_id']." AND (t1.anzeigen='j' OR LEFT(t1.del_time, 6) >=".$this->setup_array['last_year'].$this->setup_array['last_month'].") AND LEFT(t2.start_datum,7)<='".$this->setup_array['last_year_month']."' AND LEFT(t2.end_datum, 7)>='".$this->setup_array['last_year_month']."'");
			$result = $cdb->fetch_array($sql);

			if (isset($result['count_id']) and $result['count_id'] > 0)
			{
				$sql = $cdb->select('SELECT datum, ist_zeit, absence_id FROM hdb_stz WHERE user_id='.$data['user_id']." AND LEFT(datum, 7)='".$this->setup_array['last_year_month']."' ORDER BY datum ASC");
				while ($result = $cdb->fetch_array($sql))
				{
					//es ist kein Abwesenheitsgrund vorhanden
					if (empty($result['absence_id']))
					{
						$sql_p = $cdb->select('SELECT SUM(stunden) AS ist_zeit FROM hdb_stz_projekt WHERE user_id='.$data['user_id']." AND datum='".$result['datum']."'");
						$result_p = $cdb->fetch_array($sql_p);

						//es ist kein Wochenende
						if (!empty($result['ist_zeit']) or !empty($result_p['ist_zeit']))
						{
							if ($result['ist_zeit'] != $result_p['ist_zeit'])
							{
								$this->error_array[$data['user_id']]['project_time'][] = $result['datum'];
							}
						}
					}
				}
			}
		}

		/**
		 * prueft ob zu den einzelnen Tagen Zeiten bzw. ein Abwesenheitsgrund vorhanden ist.
		 *
		 * @param array $data
		 * @return void
		 */
		private function chk_day_time($data)
		{
			$cdb = new MySql;
			$kal = new Kalender;

			//Alle Tage des Monats auflisten an den weder eine Arbeitszeit noch ein Abwesenheitsgrund vorhanden ist
			//Das sollten wenn alles ok ist nur Wochenendtage sein
			$sql = $cdb->select('SELECT datum FROM hdb_stz WHERE user_id='.$data['user_id']." AND LEFT(datum, 7)='".$this->setup_array['last_year_month']."' AND (ist_zeit IS NULL OR ist_zeit=0) AND (absence_id IS NULL OR absence_id=0) ORDER BY datum ASC");
			while ($result = $cdb->fetch_array($sql))
			{
				if (!in_array($result['datum'], $this->setup_array['weekendlist']))
				{
					if ($kal->is_weekend($result['datum']))
					{
						//ist nur um ein erneutes Aufrufen der Kalenderklasse zu ersparen
						$this->setup_array['weekendlist'][] = $result['datum'];
					}
					else
					{
						//Fehler Speichern
						$this->error_array[$data['user_id']]['day_time'][] = $result['datum'];
					}
				}
			}
		}

		/**
		 * prueft die uebertrage der Stundenzettel
		 * @param array $data
		 * @return void
		 */
		private function chk_balance($data)
		{
			$cdb = new MySql;

			$sql = $cdb->select("SELECT startwert_stunden, startwert_urlaub FROM hdb_stz_monat WHERE user_id=".$data['user_id']." AND datum='".$this->setup_array['current_date']."'");
			$result = $cdb->fetch_array($sql);

			//uebertrag und Startwerte sind verschieden
			if ($data['uebertrag_stunden'] != $result['startwert_stunden'])
			{
				//uebertrag suchen
				$sql_u = $cdb->select("SELECT stunden FROM hdb_stz_uebertrag WHERE user_id=".$data['user_id']." AND datum='".$this->setup_array['current_date']."'");
				$result_u = $cdb->fetch_array($sql_u);

				if (isset($result_u['stunden']) and $result_u['stunden'] != 0)
				{
					//Fehler Speichern
					if ($data['uebertrag_stunden'] + $result_u['stunden'] != $result['startwert_stunden'])
					{
						$this->error_array[$data['user_id']]['carry_hour'] = true;
					}
				}
				//Fehler Speichern
				else
				{
					$this->error_array[$data['user_id']]['carry_hour'] = true;
				}
			}
			//uebertrag und Startwerte sind verschieden
			if ($data['uebertrag_urlaub'] != $result['startwert_urlaub'])
			{
				//uebertrag suchen
				$sql_u = $cdb->select("SELECT urlaub FROM hdb_stz_uebertrag WHERE user_id=".$data['user_id']." AND datum='".$this->setup_array['current_date']."'");
				$result_u = $cdb->fetch_array($sql_u);

				if (isset($result_u['urlaub']) and $result_u['urlaub'] != 0)
				{
					//Fehler Speichern
					if ($data['uebertrag_urlaub'] + $result_u['urlaub'] != $result['startwert_urlaub'])
					{
						$this->error_array[$data['user_id']]['carry_holiday'] = true;
					}
				}
				//Fehler Speichern
				else
				{
					$sql_u = $cdb->select('SELECT urlaubstage FROM hdb_user WHERE user_id='.$data['user_id']);
					$result_u = $cdb->fetch_array($sql_u);

					//pruefen ob am jahresanfang der Jahresurlaubsanspruch draufgerechnet wurde
					if ($data['uebertrag_urlaub'] + $result_u['urlaubstage'] != $result['startwert_urlaub'])
					{
						$this->error_array[$data['user_id']]['carry_holiday'] = true;
					}
				}
			}
		}

		/**
		 * Erstellt aus dem error array einen Fehlermeldungsstring fuer das versenden per html mail
		 *
		 * @param array $data
		 * @return string
		 */
		public function get_html_mail($data)
		{
			$cdb = new MySql;
			$kal = new Kalender;

			$user_array = array();

			foreach ($data as $user_id => $data_array)
			{
				$sql = $cdb->select('SELECT vorname, nachname FROM hdb_user WHERE user_id='.$user_id);
				$result = $cdb->fetch_array($sql);

				$user_array[$user_id] = htmlentities($result['vorname'].' '.$result['nachname'], ENT_QUOTES, 'UTF-8', false);
			}
			//User nach Alphabet sortieren
			asort($user_array);
			$return_string = '';

			$return_string .= htmlentities(get_text(310, 'return', 'decode_on'), ENT_QUOTES, 'UTF-8', false).'<br />'."\r\n";//Folgende Probleme sind in den Stundenzetteln der Mitarbeiter aufgetreten:
			$return_string .= '<ul>'."\r\n";

			foreach ($user_array as $user_id => $user_name)
			{
				$return_string .= '<li><strong>'.$user_name.'</strong><br />'."\r\n";
				$return_string .= '<ul>'."\r\n";
				//uebertragsstunden
				if (isset($data[$user_id]['carry_hour']))
				{
					$return_string .= '<li>'.htmlentities(get_text(311, 'return', 'decode_on'), ENT_QUOTES, 'UTF-8', false).'</li>'."\r\n";//Im Aktuellen Stundenzettel stimmen die Startstunden nicht mit den uebertragsstunden des Vormonats ueberein.
				}
				//Urlaubsuebertrag
				if (isset($data[$user_id]['carry_holiday']))
				{
					$return_string .= '<li>'.htmlentities(get_text(312, 'return', 'decode_on'), ENT_QUOTES, 'UTF-8', false).'</li>'."\r\n";//Im Aktuellen Stundenzettel stimmen der Urlaubsuebertrag nicht mit dem uebertrag des Vormonats ueberein.
				}
				//Tageszeit fehlt
				if (isset($data[$user_id]['day_time']) and count($data[$user_id]['day_time']) > 0)
				{
					$return_string .= '<li>'.htmlentities(get_text(313, 'return', 'decode_on'), ENT_QUOTES, 'UTF-8', false)."\r\n";//fuer nachfolgende Tage ist keine Arbeitszeit bzw. kein Abwesenheitsgrund vorhanden:
					$return_string .= '<ul>'."\r\n";

					for ($i = 0; $i < count($data[$user_id]['day_time']); $i++)
					{
						$return_string .= '<li>'.$kal->iso_to_free($data[$user_id]['day_time'][$i]).'</li>'."\r\n";
					}
					$return_string .= '</ul></li>'."\r\n";
					
				}
				//Projektzeit
				if (isset($data[$user_id]['project_time']) and count($data[$user_id]['project_time']) > 0)
				{
					$return_string .= '<li>'.htmlentities(get_text(314, 'return', 'decode_on'), ENT_QUOTES, 'UTF-8', false)."\r\n";//Bei den folgenden Tagen stimmt die Summe der geleisteten Arbeitszeit nicht mit den eigetragenen Projektarbeitszeiten &uuml;berein:
					$return_string .= '<ul>'."\r\n";

					for ($i = 0; $i < count($data[$user_id]['project_time']); $i++)
					{
						$return_string .= '<li>'.$kal->iso_to_free($data[$user_id]['project_time'][$i]).'</li>'."\r\n";
					}
					$return_string .= '</ul>'."\r\n";
					$return_string .= '</li>'."\r\n";
				}
				$return_string .= '</ul>'."\r\n";
				$return_string .= '</li>'."\r\n";
			}
			$return_string .= '<ul>'."\r\n";

			return $return_string;
		}
	}
?>