<?php
	/**
	 * create timesheet reports
	 *
	 * @author Soeren Pieper <soeren.pieper@docemos.de> / Martin Ufer <martin.ufer@docemos.de>
	 * @copyright Copyright (C) 2009  docemos GmbH
	 * @package time-o-meter
	 *
	 * This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 3 of the License, or (at your option) any later version.
	 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
	 * You should have received a copy of the GNU General Public License along with this program; if not, see http://www.gnu.org/licenses/.
	 */

	/**
	 * create timesheet reports
	 * @package time-o-meter
	 * @subpackage classes
	 */
	class TimeSheetReport
	{
		private $setup_array = array();

		/**
		 * Erstellt eine HTML uebersicht der Projektstunden oder ein Datenarray mit den informationen dazu.
		 *
		 * @param string $time_string, YYYY oder YYYY-MM
		 * @param int $project_int
		 * @param string $return_typ, 'html' oder 'array'
		 */
		public function report_project_hours($time_string, $project_int = 0, $return_typ = 'html')
		{
			$cdb = new MySql;

			//Speichert alle Daten
			$data_array = array();
			//Jahresauswahl ohne Projekteingrenzung wird standardmaessig NZS Projekt gewaehlt
			if (strlen($time_string) == 4 and $project_int == 0)
			{
				$project_int = -1;
			}

			//Jahresuebersicht fuer ein Projekt
			if (strlen($time_string) == 4)
			{
				$data_array += $this->get_project_month_array($project_int, $time_string);
			}
			//Monatsdarstellung fuer NZS
			elseif ($project_int == -1 and strlen($time_string) == 7)
			{
				$data_array += $this->get_project_month_array(-1, $time_string);
			}
			//Monatsdarstellung fuer ein Projekte
			elseif ($project_int > 0 and strlen($time_string) == 7)
			{
				$data_array += $this->get_project_month_array($project_int, $time_string);
			}
			//Monatsdarstellung fuer alle Projekte
			elseif ($project_int == 0 and strlen($time_string) == 7)
			{
				//NZS
				$data_array += $this->get_project_month_array(-1, $time_string);

				$sql = $cdb->select("SELECT projekt_id, name, kurzname, start_datum, end_datum FROM hdb_projekte WHERE LEFT(start_datum, 7)<='$time_string' AND LEFT(end_datum, 7)>='$time_string' ORDER BY kurzname ASC, name ASC");
				while ($result = $cdb->fetch_array($sql))
				{
					//Alle Projekte wo auch eine Userzuordnung bestand
					$sql_sub = $cdb->select("SELECT user_projekt_id FROM hdb_user_projekt WHERE projekt_id=".$result['projekt_id']." AND (anzeigen='j' OR LEFT(del_time,6)>='".str_replace('-', '', $time_string)."')");
					$sub_result = $cdb->fetch_array($sql_sub);

					if (isset($sub_result['user_projekt_id']) and !empty($sub_result['user_projekt_id']))
					{
						$data_array += $this->get_project_month_array($result['projekt_id'], $time_string);
					}
				}
			}


			//HTML Ausgabe
			if ($return_typ == 'html')
			{
				return $this->get_html_string($data_array);
			}
			else
			{
				return $data_array;
			}
		}

		/**
		 * Erstellt einen HTML Ausgabe fuer den Report Projektstunden
		 *
		 * @param array $data
		 * @return string
		 */
		private function get_html_string($data)
		{
			$cdb = new MySql;
			$return_string = '';

			foreach ($data as $data_array)
			{
				$array_count = count($data_array);

				if ($array_count > 0)
				{
					$calculate_total_hour_type = '';
					//Sollstunden ermitteln
					$sql = $cdb->select('SELECT ps_monat, ps_gesamt FROM hdb_projekte WHERE projekt_id='.$data_array[0]['pid']);
					$result = $cdb->fetch_array($sql);

					$hour_count = 0;
					//Kopfzeile
					$return_string .= '<br />
										<table cellpadding="2" cellspacing="0" border="0" width="100%" class="border">
											<tr class="table_tr_bg_2">
												<td class="table_header_left" width="25%">'.get_text('', 'return', 'decode_on', 'projekt') /*Projekt*/.'</td>
												<td class="table_header" width="8%">'.get_text('', 'return', 'decode_on', 'datum') /*Datum*/.'</td>
												<td class="table_header" width="20%">'.get_text('', 'return', 'decode_on', 'mitarbeiter') /*Mitarbeiter*/.'</td>
												<td class="table_header" width="18" align="right">'.get_text(359, 'return', 'decode_on') /*Geleistete Stunden*/.'</td>';
					if (!empty($result['ps_monat']))
					{
						$return_string .= '<td class="table_header" width="18%" align="right">'.get_text('', 'return', 'decode_on', 'sollstunden') /*Sollstunden*/.'/'.get_text('', 'return', 'decode_on', 'monat') /*Monat*/.'</td>
												<td class="table_header" width="11%" align="center">'.get_text('', 'return', 'decode_on', 'status') /*Status*/.'</td>';
						$calculate_total_hour_type = 'month';
					}
					elseif (!empty($result['ps_gesamt']))
					{
						$return_string .= '<td class="table_header" width="18%" align="right">'.get_text('', 'return', 'decode_on', 'sollstunden') /*Sollstunden*/.'/'.get_text('', 'return', 'decode_on', 'gesamt_gr') /*Gesamt*/.'</td>
											<td class="table_header" width="11%" align="center">'.get_text('', 'return', 'decode_on', 'status') /*Status*/.'</td>';
						$calculate_total_hour_type = 'all';
					}
					else
					{
						$return_string .= '<td class="table_header" width="18%" align="right">&nbsp;</td>
											<td class="table_header" width="11%">&nbsp;</td>';
					}

					$return_string .= '</tr>';

					//Zeilen je user
					$bg = 1;
					for ($i = 0; $i < $array_count; $i++)
					{
						$return_string .= '<tr class="table_tr_bg_'.$bg.'">';
						//erste zeile Projektname ausgeben
						if ($i == 0)
						{
							$return_string .= '<td>'.$data_array[$i]['p_name'].'</td>';
						}
						else
						{
							$return_string .= '<td>&nbsp;</td>';
						}
						//Datum konvertieren
						$return_string .= '<td>'.str_replace('-', ' / ', $data_array[$i]['month']).'</td>';
						//Username
						$return_string .= '<td>'.$data_array[$i]['user_name'].'</td>';
						//User- Projektstunden
						if (empty($data_array[$i]['hour']))
						{
							$return_string .= '<td align="right">0 h</td>';
						}
						else
						{
							$return_string .= '<td align="right">'.$data_array[$i]['hour'].' h</td>';
							$hour_count += $data_array[$i]['hour'];
						}
						$return_string .= '<td>&nbsp;</td>';
						$return_string .= '<td>&nbsp;</td>';
						$return_string .= '</tr>';

						if ($bg == 1)
						{
							$bg++;
						}
						else
						{
							$bg = 1;
						}
					}
					//Gesamtzeile fuer das Projekt
					$return_string .= '<tr class="table_tr_bg_'.$bg.'">';
					$return_string .= '<td colspan="3" align="right">'.get_text('', 'return', 'decode_on', 'gesamt_gr') /*Gesamt*/.':</td>';
					$return_string .= '<td align="right">'.$hour_count.' h</td>';

					//Keine Projektbenachrichtigung
					if (empty($calculate_total_hour_type))
					{
						$return_string .= '<td>&nbsp;</td>';
						$return_string .= '<td>&nbsp;</td>';
					}
					//Monatsbenachrichtigung
					elseif ($calculate_total_hour_type == 'month')
					{
						$overall_result = $this->get_project_result($data_array[0]['pid'], $data_array[0]['month'], $hour_count, 'month');
						$return_string .= '<td align="right">'.$overall_result['target'].' h</td>';
						$return_string .= '<td align="center">'.$overall_result['status'].'</td>';
					}
					//gesamtbenachrichtigung
					else
					{
						$overall_result = $this->get_project_result($data_array[0]['pid'], $data_array[0]['month'], $hour_count, 'all');
						$return_string .= '<td align="right">'.$overall_result['target'].' h</td>';
						$return_string .= '<td align="center">'.$overall_result['status'].'</td>';
					}
					$return_string .= '</tr>';
					$return_string .= 	'</table>';
				}
			}
			return $return_string;
		}

		/**
		 * Erstellt ein Array mit den Projektsollstunden und den Prozentualen Verbrauch der Stunden
		 *
		 * @param int $project_int
		 * @param string $month
		 * @param float $hour
		 * @param string $type
		 * @return array
		 */
		private function get_project_result($project_int, $month, $hour, $type)
		{
			$cdb = new MySql;

			$return_array = array();

			if ($type == 'month')
			{
				//Sollstunden ermitteln
				$sql = $cdb->select('SELECT ps_monat FROM hdb_projekte WHERE projekt_id='.$project_int);
				$result = $cdb->fetch_array($sql);

				if ($hour > 0 and $result['ps_monat'] > 0)
				{
					$status = round($hour / ($result['ps_monat'] / 100), 2);
					$return_array = array('target' => $result['ps_monat'], 'status' => $status.' %');
				}
				else
				{
					$return_array = array('target' => $result['ps_monat'], 'status' => '0 %');
				}

				return $return_array;
			}
			elseif ($type == 'all')
			{
				//Sollstunden ermitteln
				$sql = $cdb->select('SELECT ps_gesamt FROM hdb_projekte WHERE projekt_id='.$project_int);
				$result = $cdb->fetch_array($sql);

				//Iststunden
				$sql = $cdb->select("SELECT SUM(stunden) AS hour FROM hdb_stz_projekt WHERE projekt_id=$project_int AND LEFT(datum, 7)<='$month'");
				$hour_result = $cdb->fetch_array($sql);

				if ($hour_result['hour'] > 0 and $result['ps_gesamt'] > 0)
				{
					$status = round($hour_result['hour'] / ($result['ps_gesamt'] / 100), 2);
					$return_array = array('target' => $result['ps_gesamt'], 'status' => $status.' %');
				}
				else
				{
					$return_array = array('target' => $result['ps_gesamt'], 'status' => '0 %');
				}

				return $return_array;
			}
		}

		/**
		 * Erstellt fuer ein Projekt ein Array fuer einen Monat oder Jahr
		 *
		 * @param int $project_int
		 * @param string $time_string, YYYY oder YYYY-MM
		 * @return array
		 */
		private function get_project_month_array($project_int, $time_string)
		{
			//Speichert alle daten
			$return_array = array();
			//Daten fuer ein Jahr zusammen stellen
			if (strlen($time_string) == 4)
			{
				for ($i = 1; $i <= 12; $i++)
				{
					if ($i < 10)
					{
						$month = '0'.$i;
					}
					else
					{
						$month = $i;
					}
					//Projekt ID pruefen
					if ($project_int > 0)
					{
						$return_array[$time_string.'-'.$month.'_'.$project_int] = $this->create_project_month_array($project_int, $time_string.'-'.$month);
					}
					else
					{
						$return_array[$time_string.'-'.$month.'_nzs'] = $this->create_nzs_month_array($time_string.'-'.$month);
					}
				}
			}
			//Daten fuer einen Monat zusammen stellen
			else
			{
				//Projekt ID pruefen
				if ($project_int > 0)
				{
					$return_array[$time_string.'_'.$project_int] = $this->create_project_month_array($project_int, $time_string);
				}
				else
				{
					$return_array[$time_string.'_nzs'] = $this->create_nzs_month_array($time_string);
				}
			}
			return $return_array;
		}

		/**
		 * Erstellt ein Array mit den Nichtzugeordneten Zeiten fuer einen Monat
		 *
		 * @param string $month_string, YYYY-MM
		 * @return array
		 */
		private function create_nzs_month_array($month_string)
		{
			$cdb = new MySql;

			//Speichert alle Daten die zurueckgegeben werden
			$return_data_array = array();

			$sql = $cdb->select("SELECT t1.summe_ist_zeit, t2.user_id, t2.vorname, t2.nachname FROM hdb_stz_monat t1
								LEFT JOIN hdb_user t2 ON t1.user_id=t2.user_id
								WHERE LEFT(t1.datum, 7)='$month_string' ORDER BY t2.sort_name ASC");
			while ($result = $cdb->fetch_array($sql))
			{
				$sql_sum = $cdb->select("SELECT SUM(stunden) AS hour_count FROM hdb_stz_projekt WHERE user_id=".$result['user_id']." AND LEFT(datum, 7)='$month_string'");
				$sum_result = $cdb->fetch_array($sql_sum);

				if ($sum_result['hour_count'] < $result['summe_ist_zeit'])
				{
					$return_data_array[] = array('pid' => -1,
												'p_name' => get_text('', 'return', 'decode_on', 'nzs_kurz') /*NZS*/.' - '.get_text('', 'return', 'decode_on', 'nzs') /*Nicht zugeordnete Stunden*/,
												'month' => $month_string,
												'user_id' => $result['user_id'],
												'user_name' => $result['nachname'].', '.$result['vorname'],
												'hour' => round($result['summe_ist_zeit'] - $sum_result['hour_count'], 2));
				}
			}
			return $return_data_array;
		}

		/**
		 * Erstellt ein Array mit den Arbeitszeiten fuer ein Projekt fuer einen Monat
		 *
		 * @param int $project_int
		 * @param string $month_string, YYYY-MM
		 * @return array
		 */
		private function create_project_month_array($project_int, $month_string)
		{
			$cdb = new MySql;

			//Speichert alle Daten die zurueckgegeben werden
			$return_data_array = array();

			$sql = $cdb->select("SELECT t1.projekt_id, t1.user_id, t2.name, t2.kurzname, t3.vorname, t3.nachname FROM hdb_user_projekt t1
								LEFT JOIN hdb_projekte t2 ON t1.projekt_id=t2.projekt_id
								LEFT JOIN hdb_user t3 ON t1.user_id=t3.user_id
								WHERE t1.projekt_id=$project_int AND (t1.anzeigen='j' OR LEFT(t1.del_time,6)>='".str_replace('-', '', $month_string)."') AND LEFT(t2.start_datum, 7)<='$month_string' AND LEFT(t2.end_datum, 7)>='$month_string' ORDER BY t3.sort_name ASC");
			while ($result = $cdb->fetch_array($sql))
			{
				$sql_sum = $cdb->select("SELECT SUM(stunden) AS hour_count FROM hdb_stz_projekt WHERE user_id=".$result['user_id']." AND projekt_id=$project_int AND LEFT(datum, 7)='$month_string'");
				$sum_result = $cdb->fetch_array($sql_sum);

				$return_data_array[] = array('pid' => $result['projekt_id'],
											'p_name' => $result['kurzname'].' - '.$result['name'],
											'month' => $month_string,
											'user_id' => $result['user_id'],
											'user_name' => $result['nachname'].', '.$result['vorname'],
											'hour' => round($sum_result['hour_count'], 2));
			}

			return $return_data_array;
		}

		/**
		 * Erstellt ein Array mit Projekten zu denen eine Projektbenachrichtigung versendet werden soll.
		 * @return array
		 */
		public function cj_projectinformation()
		{
			$cdb = new MySql;
			$ps_data_array = array();

			$current_date = date('Y-m-d');
			$current_month = date('Y-m');

			//Alle AKtiven Projekte zu denen eine Projektbenachrichtigung vorhanden ist
			$sql = $cdb->select("SELECT projekt_id, name, kurzname, ps_monat, ps_gesamt, ps_nachricht FROM hdb_projekte WHERE anzeigen='j' AND start_datum<='$current_date' AND end_datum>='$current_date' AND ps_nachricht>0");
			while($result = $cdb->fetch_array($sql))
			{
				//Monatsstunden
				if (!empty($result['ps_monat']))
				{
					$tmp_ps_type = 'month';
				}
				//Gesamtstunden
				elseif (!empty($result['ps_gesamt']))
				{
					$tmp_ps_type = 'all';
				}
				//Stundentyp vorhanden
				if (isset($tmp_ps_type) and !empty($tmp_ps_type))
				{
					//User- Projektstunden auslesen
					$tmp_ps_data_array = $this->report_project_hours($current_month , $result['projekt_id'], 'array');

					//Daten vorhanden
					if (is_array($tmp_ps_data_array))
					{
						//Monatsindex
						foreach ($tmp_ps_data_array as $data)
						{
							//Userindex
							if (count($data) > 0)
							{
								$hour_count = 0;
								for ($i = 0; $i < count($data); $i++)
								{
									if (!empty($data[$i]['hour']))
									{
										$hour_count += $data[$i]['hour'];
									}
								}

								//Projektgesamtwerte berechnen
								$tmp_projekt_result = $this->get_project_result($result['projekt_id'], $current_month, $hour_count, $tmp_ps_type);

								//es ist ein projektprozentwert vorhanden
								if (isset($tmp_projekt_result['status']))
								{
									//der wert hat am ende ein % fuer die html ausgabe, diesen entfernen
									$tmp_projekt_result['status'] = floatval($tmp_projekt_result['status']);

									//pruefen ob eine benachrichtigung gesendet werden muss
									if ($tmp_projekt_result['status'] >= $result['ps_nachricht'])
									{
										$ps_data_array[] = array('p_id'		=> $result['projekt_id'],
																'name'		=> $result['name'],
																'kurzname'	=> $result['kurzname'],
																'type'		=> $tmp_ps_type,
																'ist'		=> $tmp_projekt_result['status'],
																'soll'		=> $result['ps_nachricht']);
									}
								}
							}
						}
					}
				}
			}

			return $ps_data_array;
		}

		/**
		 * #########################
		 * Projektstundenreport ENDE
		 * #########################
		 */

		/**
		 * Erstellt ein Array mit informationen fuer die PDF Erstellung
		 * @param boole $is_projectreport
		 * @return mixed
		 */
		public function report_pdf_stz($is_projectreport = false, $project_comment = false)
		{
			if (isset($_POST) and count($_POST) > 0)
			{
				$cdb = new MySql;

				$return_array = array();

				//UserID
				if (isset($_POST['user_id_int']) and is_numeric($_POST['user_id_int']))
				{
					$return_array['user_id'] = $_POST['user_id_int'];
				}
				else
				{
					return false;
				}
				//Monat
				if (isset($_POST['month_string']) and !empty($_POST['month_string']))
				{
					$return_array['month_string'] = $_POST['month_string'];
					$return_array['month_int'] = intval($_POST['month_string']);
				}
				else
				{
					return false;
				}
				//Jahr
				if (isset($_POST['year_int']) and is_numeric($_POST['year_int']))
				{
					$return_array['year_int'] = $_POST['year_int'];
				}
				else
				{
					return false;
				}
				//Spalten
				if (isset($_POST['pdf']) and count($_POST['pdf']) > 0)
				{
					$tmp_page_width = 0;
					$tmp_max_page_width = 277;
					$tmp_column_count = 0;

					if (isset($_POST['pdf']['datum']) and $_POST['pdf']['datum'] == true)
					{
						$return_array['column'][] = array('type'	=> 'datum',
														'width'		=> 15,
														'name'		=> get_text('', 'return', 'decode_pdf', 'datum'));
						$tmp_page_width += 15;
						$tmp_column_count++;
					}
					if (isset($_POST['pdf']['kommen']) and $_POST['pdf']['kommen'] == true)
					{
						$return_array['column'][] = array('type'	=> 'kommen',
														'width'		=> 15,
														'name'		=> get_text('', 'return', 'decode_pdf', 'kommen'));
						$tmp_page_width += 15;
						$tmp_column_count++;
					}
					if (isset($_POST['pdf']['pause']) and $_POST['pdf']['pause'] == true)
					{
						$return_array['column'][] = array('type'	=> 'pause',
														'width'		=> 15,
														'name'		=> get_text('', 'return', 'decode_pdf', 'pause'));
						$tmp_page_width += 15;
						$tmp_column_count++;
					}
					if (isset($_POST['pdf']['gehen']) and $_POST['pdf']['gehen'] == true)
					{
						$return_array['column'][] = array('type'	=> 'gehen',
														'width'		=> 15,
														'name'		=> get_text('', 'return', 'decode_pdf', 'gehen'));
						$tmp_page_width += 15;
						$tmp_column_count++;
					}
					if (isset($_POST['pdf']['istzeit']) and $_POST['pdf']['istzeit'] == true)
					{
						$return_array['column'][] = array('type'	=> 'istzeit',
														'width'		=> 15,
														'name'		=> get_text('', 'return', 'decode_pdf', 'ist_zeit'));
						$tmp_page_width += 15;
						$tmp_column_count++;
					}
					if (isset($_POST['pdf']['abwesenheit']) and $_POST['pdf']['abwesenheit'] == true)
					{
						$return_array['column'][] = array('type'	=> 'abwesenheit',
														'width'		=> 20,
														'name'		=> get_text(602, 'return', 'decode_pdf'));
						$tmp_page_width += 20;
						$tmp_column_count++;
					}
					if (isset($_POST['pdf']['zeit']) and $_POST['pdf']['zeit'] == true and $is_projectreport === false)
					{
						$return_array['column'][] = array('type'	=> 'zeit',
														'width'		=> 15,
														'name'		=> get_text('', 'return', 'decode_pdf', 'zeit_plus_minus'));
						$tmp_page_width += 15;
						$tmp_column_count++;
					}
					if ((isset($_POST['pdf']['kommentar']) and $_POST['pdf']['kommentar'] == true and $is_projectreport === false) or $project_comment === true)
					{
						$return_array['column'][] = array('type'	=> 'kommentar',
														'width'		=> 25,
														'name'		=> get_text('', 'return', 'decode_pdf', 'kommentar'));
						$tmp_page_width += 25;
						$tmp_column_count++;
					}
					if (isset($_POST['pdf']['nzs']) and $_POST['pdf']['nzs'] == true and $is_projectreport === false)
					{
						$return_array['column'][] = array('type'	=> 'nzs',
														'width'		=> 15,
														'name'		=> get_text('', 'return', 'decode_pdf', 'nzs_kurz'));
						$tmp_page_width += 15;
						$tmp_column_count++;
					}

					//Projektspalten
					if (isset($_POST['pdf']['projekt']) and count($_POST['pdf']['projekt']) > 0 and $is_projectreport === false)
					{
						foreach ($_POST['pdf']['projekt'] as $pid => $boole)
						{
							if ($boole == true)
							{
								$sql = $cdb->select('SELECT kurzname FROM hdb_projekte WHERE projekt_id='.$pid);
								$p_result = $cdb->fetch_array($sql);

								//Projektspalte mit Projektkommentar
								if (isset($_POST['pdf']['projektkommentar'][$pid]) and $_POST['pdf']['projektkommentar'][$pid] == true)
								{
									//Max Seitenbreite pruefen
									if ($tmp_max_page_width >= $tmp_page_width + 40)
									{
										$return_array['column'][] = array('type'	=> 'project',
																		'width' 	=> 15,
																		'p_id'		=> $pid,
																		'comment'	=> true,
																		'name'		=> $p_result['kurzname']);

										$return_array['column'][] = array('type'	=> 'project_comment',
																		'width' 	=> 25,
																		'p_id'		=> $pid,
																		'name'		=> $p_result['kurzname']);
										$tmp_page_width += 15;
										$tmp_page_width += 25;
										$tmp_column_count++;
										$tmp_column_count++;
									}
								}
								//Projektspalte ohne Projektkommentar
								else
								{
									//Max Seitenbreite pruefen
									if ($tmp_max_page_width >= $tmp_page_width + 15)
									{
										$return_array['column'][] = array('type'	=> 'project',
																		'width' 	=> 15,
																		'p_id'		=> $pid,
																		'comment'	=> false,
																		'name'		=> $p_result['kurzname']);
										$tmp_page_width += 15;
										$tmp_column_count++;
									}
								}
							}
						}
					}
					$return_array['page_width'] = $tmp_page_width;
					//Hochformat
					if ($tmp_page_width <= 190)
					{
						$return_array['page_format'] = 'P';
					}
					//Querformat
					else
					{
						$return_array['page_format'] = 'L';
					}
					$return_array['column_count'] = $tmp_column_count;

					return $this->report_pdf_resize_column($return_array);
				}
				else
				{
					return false;
				}
			}
			else
			{
				return false;
			}
		}

		/**
		 * Passt die Spaltenbreite an die Seitenbreite an.
		 *
		 * @param array $data
		 * @return array
		 */
		public function report_pdf_resize_column($data)
		{
			if ($data['page_format'] == 'L')
			{
				$page_width = 277;
			}
			else
			{
				$page_width = 190;
			}

			//Es ist noch platz auf der Seite
			if ($data['page_width'] < $page_width)
			{
				$diff_page_width = $page_width - $data['page_width'];

				$page_width_count = array();
				$page_width_count[15] = 0;
				$page_width_count[20] = 0;
				$page_width_count[25] = 0;
				$column_index = array();

				for ($i = 0; $i < count($data['column']); $i++)
				{
					if ($data['column'][$i]['width'] == 15)
					{
						$page_width_count[15]++;
						$column_index[15][] = $i;
					}
					elseif ($data['column'][$i]['width'] == 20)
					{
						$page_width_count[20]++;
						$column_index[20][] = $i;
					}
					elseif ($data['column'][$i]['width'] == 25)
					{
						$page_width_count[25]++;
						$column_index[25][] = $i;
					}
				}

				//20 Spalten verbreitern
				if ($page_width_count[20] > 0)
				{
					//20 Spalten um 5mm verbreitern
					if (5 * $page_width_count[20] < $diff_page_width)
					{
						for ($i = 0; $i < count($column_index[20]); $i++)
						{
							$diff_page_width -= 5;

							$data['column'][$column_index[20][$i]]['width'] = 25;
						}
					}
					//weniger als 5mm verwenden
					else
					{
						$tmp_add_column_width = round($diff_page_width / $page_width_count[20], 2);
						$diff_page_width = 0;

						for ($i = 0; $i < count($column_index[20]); $i++)
						{
							$data['column'][$column_index[20][$i]]['width'] += $tmp_add_column_width;
						}
					}
				}
				//25 Spalten verbreitern
				if ($page_width_count[25] > 0)
				{
					//20 Spalten um 10mm verbreitern
					if (10 * $page_width_count[25] < $diff_page_width)
					{
						for ($i = 0; $i < count($column_index[25]); $i++)
						{
							$diff_page_width -= 10;

							$data['column'][$column_index[25][$i]]['width'] = 35;
						}
					}
					//weniger als 10mm verwenden
					else
					{
						$tmp_add_column_width = round($diff_page_width / $page_width_count[25], 2);
						$diff_page_width = 0;

						for ($i = 0; $i < count($column_index[25]); $i++)
						{
							$data['column'][$column_index[25][$i]]['width'] += $tmp_add_column_width;
						}
					}
				}
				//15 Spalten verbreitern
				if ($page_width_count[15] > 0)
				{
					//15 Spalten um 5mm verbreitern
					if (5 * $page_width_count[15] < $diff_page_width)
					{
						for ($i = 0; $i < count($column_index[15]); $i++)
						{
							$diff_page_width -= 5;

							$data['column'][$column_index[15][$i]]['width'] = 20;
						}
					}
					//weniger als 5mm verwenden
					else
					{
						$tmp_add_column_width = round($diff_page_width / $page_width_count[15], 2);
						$diff_page_width = 0;

						for ($i = 0; $i < count($column_index[15]); $i++)
						{
							$data['column'][$column_index[15][$i]]['width'] += $tmp_add_column_width;
						}
					}
				}

				//restbreite der Seite verteilen
				if ($diff_page_width > 0)
				{
					//25 Spalten verbreitern
					if ($page_width_count[25] > 0)
					{
						$tmp_add_column_width = round($diff_page_width / $page_width_count[25], 2);

						for ($i = 0; $i < count($column_index[25]); $i++)
						{
							$data['column'][$column_index[25][$i]]['width'] += $tmp_add_column_width;
						}
					}
					//15 Spalten verbreitern
					elseif ($page_width_count[15] > 0)
					{
						$tmp_add_column_width = round($diff_page_width / $page_width_count[15], 2);

						for ($i = 0; $i < count($column_index[15]); $i++)
						{
							$data['column'][$column_index[15][$i]]['width'] += $tmp_add_column_width;
						}
					}
					//20 Spalten verbreitern
					elseif ($page_width_count[20] > 0)
					{
						$tmp_add_column_width = round($diff_page_width / $page_width_count[20], 2);

						for ($i = 0; $i < count($column_index[20]); $i++)
						{
							$data['column'][$column_index[20][$i]]['width'] += $tmp_add_column_width;
						}
					}
				}
				return $data;
			}
			//keine aenderung erforderlich
			else
			{
				return $data;
			}
		}

		/**
		 * Moduliert das array von $this->report_project_hours() fuer eine Diagrammdarastellung
		 * @param array $data_array
		 * @param string $type
		 * @return array
		 */
		public function get_pdf_pie_chart_data($data_array, $type)
		{
			$kal = new Kalender();

			$tmp_data_array = array();

			if ($type == 'month_all')
			{
				$all_hour = 0;
				$project_hour = 0;
				$project_count = 0;

				foreach ($data_array as $month_index => $month_data)
				{
					if (is_array($month_data) and count($month_data) > 0)
					{
						$project_hour = 0;
						for ($i = 0; $i < count($month_data); $i++)
						{
							if ($i == 0)
							{
								$tmp_data_array[$project_count]['name'] = mb_convert_encoding(html_entity_decode($month_data[$i]['p_name'], ENT_QUOTES), 'UTF-8');
								$tmp_data_array[$project_count]['project_id'] = $month_data[$i]['pid'];
								$tmp_data_array[$project_count]['project_name'] = $month_data[$i]['p_name'];
							}

							if (!empty($month_data[$i]['hour']))
							{
								$project_hour += $month_data[$i]['hour'];
								$all_hour += $month_data[$i]['hour'];
							}
						}

						$tmp_data_array[$project_count]['hour'] = $project_hour;
						$project_count++;
					}
				}
			}
			elseif ($type == 'year_project' or $type == 'year_nzs')
			{
				$month_count = 1;
				$all_hour = 0;

				foreach ($data_array as $month_index => $month_data)
				{
					$tmp_data_array[$month_count]['name'] = mb_convert_encoding(html_entity_decode($kal->get_kalender_text(intval(substr($month_index, 5, 2)), 'month_name'), ENT_QUOTES), 'UTF-8');

					if (is_array($month_data) and count($month_data) > 0)
					{
						$tmp_month_hour_count = 0;
						for ($i = 0; $i < count($month_data); $i++)
						{
							$tmp_month_hour_count += $month_data[$i]['hour'];
							$all_hour += $month_data[$i]['hour'];

							if ($type == 'year_project')
							{
								$tmp_data_array[$month_count]['project_id'] = $month_data[$i]['pid'];
								$tmp_data_array[$month_count]['project_name'] = $month_data[$i]['p_name'];
							}
						}

						$tmp_data_array[$month_count]['hour'] = round($tmp_month_hour_count, 2);
					}
					else
					{
						$tmp_data_array[$month_count]['hour'] = 0;
					}
					$month_count++;
				}
			}
			elseif ($type == 'month_project' or $type == 'month_nzs')
			{
				$all_hour = 0;

				foreach ($data_array as $month_index => $month_data)
				{
					if (is_array($month_data) and count($month_data) > 0)
					{
						for ($i = 0; $i < count($month_data); $i++)
						{
							$tmp_data_array[$i]['name'] = mb_convert_encoding(html_entity_decode($month_data[$i]['user_name'], ENT_QUOTES), 'UTF-8');
							if (!empty($month_data[$i]['hour']))
							{
								$tmp_data_array[$i]['hour'] = $month_data[$i]['hour'];
								$all_hour += $month_data[$i]['hour'];
							}
							else
							{
								$tmp_data_array[$i]['hour'] = 0;
							}

							if ($type == 'month_project')
							{
								$tmp_data_array[$i]['project_id'] = $month_data[$i]['pid'];
								$tmp_data_array[$i]['project_name'] = $month_data[$i]['p_name'];
							}
						}
					}
				}
			}

			if (isset($tmp_data_array) and is_array($tmp_data_array) and count($tmp_data_array) > 0)
			{
				$one_percent = $all_hour / 100;

				$return_array = array();
				$array_index = 0;
				foreach ($tmp_data_array as $data)
				{
					if ($data['hour'] > 0)
					{
						$return_array[$array_index] = array('name'			=> $data['name'],
															'hour'			=> $data['hour'],
															'percent'		=> round($data['hour'] / $one_percent , 2),
															'degree'		=> round(360 / 100 * round($data['hour'] / $one_percent , 2), 2));

						if (isset($data['project_id']) and isset($data['project_name']))
						{
							$return_array[$array_index]['project_id'] = $data['project_id'];
							$return_array[$array_index]['project_name'] = $data['project_name'];
						}
						$array_index++;
					}
				}
				return $return_array;
			}
			else
			{
				return array();
			}
		}

		/**
		 * Gibt die Hintergrundfarben fuer die Giagramme zurueck
		 * @return array
		 */
		public function get_pdf_pie_bg_color()
		{
			if (!isset($this->setup_array['pie_bg_index']))
			{
				$this->setup_array['pie_bg_index'] = 0;
			}
			else
			{
				$this->setup_array['pie_bg_index']++;
			}

			$bg_color_array = array(array(145, 54, 151),
									array(36, 97, 171),
									array(244, 185, 55),
									array(200, 200, 200),
									array(150, 200, 200),
									array(150, 150, 200),
									array(150, 150, 150),
									array(150, 200, 150),
									array(200, 200, 150),
									array(150, 100, 100),
									array(150, 150, 100),
									array(100, 100, 150),
									array(100, 150, 150),
									array(100, 100, 100),
									array(100, 100, 50),
									array(100, 50, 50),
									array(50, 50, 100),
									array(50, 100, 100),
									array(50, 50, 50));

			if (isset($bg_color_array[$this->setup_array['pie_bg_index']]))
			{
				return $bg_color_array[$this->setup_array['pie_bg_index']];
			}
			else
			{
				$this->setup_array['pie_bg_index'] = 0;
				return $bg_color_array[$this->setup_array['pie_bg_index']];
			}
		}

		/**
		 * Moduliert das array von $this->report_project_hours() fuer eine Tabellendarastellung
		 * @param array $data_array
		 * @return array
		 */
		public function get_pdf_table_data($data_array)
		{
			$cdb = new MySql;

			//Projekt ID
			$project_id = 0;
			//Projekt Sollstunden pro monat
			$project_hour_per_month = 0;
			//Projekt Sollstunden gesamt
			$project_hour_per_overall = 0;
			//Projekt Sollstunden aller Monate
			$project_hour_all_month = 0;
			//Projekt Iststunden aller Monate
			$project_actual_hour = 0;
			//Projekt iststunden vor der ausgabe
			$project_actual_hour_first_month = 0;
			//Stunden min max array
			$hour_min_max_array = array(0);
			//pruent min max array
			$percent_min_max_array = array(0);
			//Projektname
			$projectname = '';
			//letzter ausgegebener Monat
			$last_month = '';
			//erster ausgegebener Monat
			$first_month = '';
			//return array
			$return_array = array();

			$foreach_count = 0;
			foreach ($data_array as $month_index => $month_array)
			{
				$last_month = substr($month_index, 0, 7);

				//Projektdaten auslesen
				if ($foreach_count == 0 or empty($projectname))
				{
					if ($foreach_count == 0)
					{
						$first_month = substr($month_index, 0, 7);
					}

					if (isset($month_array[0]['pid']))
					{
						$project_id = $month_array[0]['pid'];
						//Sollstunden ermitteln
						$sql = $cdb->select('SELECT ps_monat, ps_gesamt FROM hdb_projekte WHERE projekt_id='.$project_id);
						$result = $cdb->fetch_array($sql);

						if (!empty($result['ps_monat']))
						{
							$project_hour_per_month = $result['ps_monat'];
							//$hour_min_max_array[] = $project_hour_per_month;
						}
						elseif (!empty($result['ps_gesamt']))
						{
							$project_hour_per_overall = $result['ps_gesamt'];
							$project_hour_all_month = $result['ps_gesamt'];
							//$hour_min_max_array[] = $result['ps_gesamt'];
						}

						$projectname = $month_array[0]['p_name'];
					}
				}

				//alle Monatssollwerte summieren
				if (!empty($project_hour_per_month))
				{
					$project_hour_all_month += $project_hour_per_month;
				}

				//monatsistwerte summieren
				$return_array['month_data'][$last_month]['month_hour'] = 0;
				if (is_array($month_array) and count($month_array) > 0)
				{
					for ($i = 0; $i < count($month_array); $i++)
					{
						$return_array['month_data'][$last_month]['month_hour'] += $month_array[$i]['hour'];
						$project_actual_hour += $month_array[$i]['hour'];
					}
					$hour_min_max_array[] = $return_array['month_data'][$last_month]['month_hour'];
				}
				$foreach_count++;
			}

			//Projektiststunden bis zum zuletzt verwendeten monat
			//wird nur benoetigt wenn die Projektsollstunden fuer das gesamte projekt gelten
			if (empty($project_hour_per_month) and !empty($project_hour_per_overall))
			{
				//iststunden des letzten monats
				$sql = $cdb->select("SELECT SUM(stunden) AS sum_ist FROM hdb_stz_projekt WHERE projekt_id=$project_id AND LEFT(datum, 7)<='$last_month'");
				$result = $cdb->fetch_array($sql);
				$project_actual_hour = $result['sum_ist'];

				//iststunden vor den monaten die ausgegeben werden
				$sql = $cdb->select("SELECT SUM(stunden) AS sum_ist FROM hdb_stz_projekt WHERE projekt_id=$project_id AND LEFT(datum, 7)<'$first_month'");
				$result = $cdb->fetch_array($sql);
				$project_actual_hour_first_month = $result['sum_ist'];

			}

			$return_array['general']['project_id'] = $project_id;
			$return_array['general']['projectname'] = $projectname;
			if (!empty($project_hour_per_month))
			{
				$return_array['general']['project_hour_per_month'] = $project_hour_per_month;
			}
			if (!empty($project_hour_per_overall))
			{
				$return_array['general']['project_hour_per_overall'] = $project_hour_per_overall;
			}
			$return_array['general']['project_hour_all_month'] = $project_hour_all_month;
			$return_array['general']['project_actual_hour'] = $project_actual_hour;

			//Projektsollstunden vorhanden
			if (isset($return_array['general']['project_hour_per_month']) or isset($return_array['general']['project_hour_per_overall']))
			{
				if (isset($return_array['general']['project_hour_per_month']))
				{
					$one_percent = $return_array['general']['project_hour_per_month'] / 100;
				}
				else
				{
					$one_percent = $return_array['general']['project_hour_per_overall'] / 100;
					$tmp_hour = $project_actual_hour_first_month;
				}

				foreach ($return_array['month_data'] as $month_index => $hour)
				{
					if ($hour['month_hour'] > 0)
					{
						if (isset($return_array['general']['project_hour_per_month']))
						{
							$return_array['month_data'][$month_index]['percent'] = round($hour['month_hour'] / $one_percent, 2);
							$percent_min_max_array[] = $return_array['month_data'][$month_index]['percent'];
						}
						else
						{
							$tmp_hour += $hour['month_hour'];
							$return_array['month_data'][$month_index]['percent'] = round($tmp_hour / $one_percent, 2);
							$percent_min_max_array[] = $return_array['month_data'][$month_index]['percent'];
						}
					}
					else
					{
						$return_array['month_data'][$month_index]['percent'] = 0;
					}
				}
			}

			$return_array['general']['hour_max'] = max($hour_min_max_array);
			$return_array['general']['hour_min'] = min($hour_min_max_array);
			$return_array['general']['percent_max'] = max($percent_min_max_array);
			$return_array['general']['percent_min'] = min($percent_min_max_array);

			return $return_array;
		}
	}
?>