<?php
	/**
	 * stores timesheet data into the database
	 *
	 * @author Soeren Pieper <soeren.pieper@docemos.de> / Martin Ufer <martin.ufer@docemos.de>
	 * @copyright Copyright (C) 2009  docemos GmbH
	 * @package time-o-meter
	 *
	 * This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 3 of the License, or (at your option) any later version.
	 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
	 * You should have received a copy of the GNU General Public License along with this program; if not, see http://www.gnu.org/licenses/.
	 */

	/**
	 * stores timesheet data into the database
	 * @package time-o-meter
	 * @subpackage classes
	 */
	class TimeSheetSave
	{
		/**
		 * Speichert allgemeine einstellungen
		 *
		 * @var unknown_type
		 */
		private $setup_array = array();

		/**
		 * Speichert Tageseinstellungen
		 *
		 * @var unknown_type
		 */
		private $setup_day_array = array();

		/**
		 * Speichert alle Abwesenheitsgruende
		 *
		 * @var array
		 */
		private $absence_array = array();

		/**
		 * Array mit allen Daten die in die DB eingetragen werden
		 *
		 * @var unknown_type
		 */
		private $insert_day_array = array();

		/**
		 * Speichert die Stundenzetteldaten bzw. Startet alle Funktionen die dazu benoetigt werden
		 * @return mixed
		 */
		public function save()
		{
			//enthaelt alle erfolgs- nzw. Fehlermeldungen
			$return_array = array();
			if ($this->chk_reload() === true)
			{
				if ($this->chk_checksum() === true)
				{
					if ($this->ts_exists() === true)
					{
						if (count($_POST['kommen']) > 0)
						{
							if ($this->create_setup() === true)
							{
								$insert_array = array();
								$tmp_mixed = array();
								//uebergebene Tage abarbeiten
								foreach ($_POST['kommen'] as $day => $data)
								{
									$tmp_mixed = $this->create_insert_array($day);
									if ($tmp_mixed === false)
									{
										 //Fehler bei der Verarbeitung der Zeiten
										$return_array['error'] = 6;
										return $return_array;
									}
									elseif (is_array($tmp_mixed))
									{
										$insert_array += $tmp_mixed;
									}
									$tmp_array = array();
								}

								//keine Fehler eingetretten Date in DB eintragen
								if ($this->insert_data($insert_array) === true)
								{
									//Alles io
									if ($this->update_month_data() === true)
									{
										return true;
									}
									else
									{
										//Fehler beim Monatsgesamtwerte berechnung
										$return_array['error'] = 8;
									}
								}
								else
								{
									//Fehler beim Eintragen
									$return_array['error'] = 7;
								}
								return $insert_array;
							}
							else
							{
								//keine Einstellungen
								$return_array['error'] = 5;
							}
						}
						else
						{
							//Keine Daten vorhanden
							$return_array['error'] = 4;
						}
					}
					else
					{
						//Kein steundenzettel vorhanden
						$return_array['error'] = 3;
					}
				}
				else
				{
					//Checksummenpruefung
					$return_array['error'] = 2;
				}
			}
			else
			{
				//Reloadsperre
				$return_array['error'] = 1;
			}
			return $return_array;
		}

		/**
		 * Fuehrt eine neuberechnung der gesamtwert (Uebertraege) eines stz durch
		 * @param int $user_id
		 * @param int $year
		 * @param string $month
		 * @return boole
		 */
		public function recalculation_month_data($user_id, $year, $month)
		{
			if ($this->create_setup($user_id, $year, $month) === true)
			{
				if ($this->update_month_data())
				{
					return true;
				}
				else
				{
					return false;
				}
			}
			else
			{
				return false;
			}
		}

		/**
		 * Aktualisiert die Monatsgesammtwerte
		 *
		 * @return boole
		 */
		private function update_month_data()
		{
			$cdb = new MySql;

			$date = $this->setup_array['year'].'-'.$this->setup_array['month'];

			$sql = $cdb->select("SELECT id, startwert_stunden, startwert_urlaub FROM hdb_stz_monat WHERE user_id=".$this->setup_array['user_id']." AND LEFT(datum, 7)='".$date."'");
			$month_result = $cdb->fetch_array($sql);

			//istzeiten zusammenrechnen
			$sql = $cdb->select("SELECT SUM(ist_zeit) AS ist_zeit FROM hdb_stz WHERE user_id=".$this->setup_array['user_id']." AND LEFT(datum, 7)='".$date."'");
			$result = $cdb->fetch_array($sql);

			//summer der istzeit stunden
			if (isset($result['ist_zeit']) && !empty($result['ist_zeit']))
			{
				$sum_ist_zeit = $result['ist_zeit'];
			}
			else
			{
				$sum_ist_zeit = 0;
			}

			$count_krank 		= 0;
			$count_urlaub 		= 0;
			$sum_absence_zeiten = 0;

			//alle tage mit einem Abwesenheitsgrund
			$sql = $cdb->select("SELECT datum, kommen, gehen, ist_zeit, absence_id FROM hdb_stz WHERE user_id=".$this->setup_array['user_id']." AND LEFT(datum, 7)='".$date."' AND absence_id > 0");
			while ($result = $cdb->fetch_array($sql))
			{
				//tag als int
				$day = intval(substr($result['datum'], -2));

				//anzahl der krankheitstage
				if ($result['absence_id'] == 1)
				{
					$count_krank++;
				}
				//anzahl der urlaubstage
				if ($result['absence_id'] == 2)
				{
					//Abwesenheitsgrund als vollen tag verwenden
					if (isset($this->absence_array['full_day'][$result['absence_id']]) and $this->absence_array['full_day'][$result['absence_id']] == true)
					{
						$count_urlaub += $this->setup_day_array[$day]['urlaubstag'];
					}
					else
					{
						//Urlaub nur rechnen wenn auch weniger gearbeitet wurde als verlangt
						if ($result['ist_zeit'] < $this->setup_day_array[$day]['min_sollstunden'])
						{
							$count_urlaub += round(($this->setup_day_array[$day]['min_sollstunden'] - $result['ist_zeit']) / $this->setup_day_array[$day]['min_sollstunden'] * $this->setup_day_array[$day]['urlaubstag'], 2);
						}
					}
				}
				//anzahl der Stunden der bezahlten abwesenheitsgruende
				if ($this->absence_array[$result['absence_id']] === true)
				{
					//Abwesenheitsgrund als vollen tag verwenden
					if (isset($this->absence_array['full_day'][$result['absence_id']]) and $this->absence_array['full_day'][$result['absence_id']] == true)
					{
						$sum_absence_zeiten += $this->setup_day_array[$day]['min_sollstunden'];
					}
					else
					{
						//Urlaub nur rechnen wenn auch weniger gearbeitet wurde als verlangt
						if ($result['ist_zeit'] < $this->setup_day_array[$day]['min_sollstunden'])
						{
							$sum_absence_zeiten += $this->setup_day_array[$day]['min_sollstunden'] - $result['ist_zeit'];
						}
					}
				}
			}

			//Summe der Bezahlten stunden inkl. bezahlter Abwesenheitsgruende
			$bezahlte_stunden = $sum_ist_zeit + $sum_absence_zeiten;

			if ($bezahlte_stunden < $this->setup_array['month_sollstunden'])
			{
				$ueberstunden = $bezahlte_stunden - $this->setup_array['month_sollstunden'];
			}
			elseif ($bezahlte_stunden > $this->setup_array['month_maxsollstunden'])
			{
				$ueberstunden = $bezahlte_stunden - $this->setup_array['month_maxsollstunden'];
			}
			else
			{
				$ueberstunden = 0;
			}

			//neuer stundenuebertrag
			$stundenuebertrag = $month_result['startwert_stunden'] + $ueberstunden;
			//neuer Urlaubsuebertrag
			$urlaubsuebertrag = $month_result['startwert_urlaub'] - $count_urlaub;

			//uebertraege aktualisieren
			if ($cdb->update("UPDATE hdb_stz_monat SET sollstunden=".$this->setup_array['month_sollstunden'].", maxstunden=".$this->setup_array['month_maxsollstunden'].", summe_ist_zeit=$sum_ist_zeit, summe_bezahlt_zeit=$bezahlte_stunden, summe_zeit =$ueberstunden, uebertrag_stunden=$stundenuebertrag, summe_urlaub=$count_urlaub, uebertrag_urlaub=$urlaubsuebertrag, summe_krank=$count_krank, report_vor6h='".$this->setup_array['report']['vor']."', report_nach22h='".$this->setup_array['report']['nach']."', report_ueber10h='".$this->setup_array['report']['mehr']."' WHERE id=".$month_result['id']))
			{
				return true;
			}
			else
			{
				return false;
			}
		}

		private function chk_worktime_report($kommen, $gehen, $zeit)
		{
			//zu fruehe arbeitzeitbeginn
			if (intval(substr($kommen, 0, 2)) < $this->setup_array['report_vor'])
			{
				$this->setup_array['report']['vor'] = 'j';
			}
			//zu spaetes arbeitszeitende
			if (intval(substr($gehen, 0, 2)) >= $this->setup_array['report_nach'])
			{
				$this->setup_array['report']['nach'] = 'j';
			}
			//zu lange Arbeitszeit
			if ($zeit > $this->setup_array['report_mehr'])
			{
				$this->setup_array['report']['mehr'] = 'j';
			}
		}

		/**
		 * Erstellt ein Array fuer einen Tag fuer ein Spaeteres INSERT bzw. UPDATE
		 *
		 * @param int $day
		 * @return mixed
		 */
		private function create_insert_array($day)
		{
			$cdb = new MySql;

			//FIXME: bugfix - da man an wochenenden keinen abwesenheitsgrund auswaehlen kann, aber die eingabe von arbeitszeiten moeglich sein muss
			//ist nur eine notloesung,gibt sicher einen besseren weg
			if (!isset($_POST['abwesenheit'][$day]))
			{
				$_POST['abwesenheit'][$day] = '';
			}

			//Nur tage abarbeiten an denen auch Daten vorhanden sind
			if (isset($_POST['kommen'][$day]) and isset($_POST['pause'][$day]) and isset($_POST['gehen'][$day]) and isset($_POST['abwesenheit'][$day]))
			{
				$get_comment_boole = false;
				$claculate_work_time = true;

				//abwesenheitsgrund wurde gewaehlt
				if (!empty($_POST['abwesenheit'][$day]))
				{
					//uebergebener abwesenheitsgrund existiert in der DB
					if (isset($this->absence_array[$_POST['abwesenheit'][$day]]))
					{
						$insert_day_array[$day]['absence'] = $_POST['abwesenheit'][$day];
						//Gibt die Kommentare zurueck
						$get_comment_boole = true;
					}

					//Abwesenheitsgrund gilt fuer den gesamten tag
					//keine zusaetzliche zeitangabe
					if (isset($this->absence_array['full_day'][$_POST['abwesenheit'][$day]]) and $this->absence_array['full_day'][$_POST['abwesenheit'][$day]] == true)
					{
						$claculate_work_time = false;
					}
				}

				//Tageszeiten berechnen
				//Derangegebene Abwesenheitsgrund ist zuzueglich zur Arbeitszeit
				if ($claculate_work_time === true)
				{
					//arbeitszeitberechnung
					if (!empty($_POST['kommen'][$day]) and !empty($_POST['gehen'][$day]))
					{
						//Tagesarbeitszeit berechnen
						$day_time_array = $this->calculate_worktime($_POST['kommen'][$day], $_POST['pause'][$day], $_POST['gehen'][$day]);
						//Arbeitszeiten wurden korrekt berechnet
						if (count($day_time_array) == 4)
						{
							$insert_day_array[$day]['kommen']	= $day_time_array['kommen'];
							$insert_day_array[$day]['pause']	= $day_time_array['pause'];
							$insert_day_array[$day]['gehen']	= $day_time_array['gehen'];
							$insert_day_array[$day]['istzeit']	= $day_time_array['zeit'];

							//Arbeitszeitreport
							$this->chk_worktime_report($insert_day_array[$day]['kommen'], $insert_day_array[$day]['gehen'], $insert_day_array[$day]['istzeit']);
							//ueberstunden
							if ($day_time_array['zeit'] < $this->setup_day_array[$day]['min_sollstunden'])
							{
								$insert_day_array[$day]['zeit']	= $day_time_array['zeit'] - $this->setup_day_array[$day]['min_sollstunden'];
							}
							elseif ($day_time_array['zeit'] > $this->setup_day_array[$day]['max_sollstunden'])
							{
								$insert_day_array[$day]['zeit']	= $day_time_array['zeit'] - $this->setup_day_array[$day]['max_sollstunden'];
							}
							else
							{
								$insert_day_array[$day]['zeit']	= 0;
							}

							//Projektzeiten berechnen
							$project_time_array = $this->calculate_projectime($day);
							if (!empty($project_time_array))
							{
								$insert_day_array[$day]['projektzeiten'] = $project_time_array;
							}

							//Gibt die Kommentare zurueck
							$get_comment_boole = true;
						}
						else
						{
							return false;
						}
					}
					//es wurde nur eine start bzw. endzeit eingetragen
					elseif (!empty($_POST['kommen'][$day]) or !empty($_POST['gehen'][$day]))
					{
						//kommenzeit
						if (!empty($_POST['kommen'][$day]))
						{
							//wert in das format HH:MM wandeln
							$tmp_kommen = $this->timeval($_POST['kommen'][$day]);
							if (!empty($tmp_kommen))
							{
								$insert_day_array[$day]['kommen'] = $tmp_kommen;
								$insert_day_array[$day]['gehen'] = '';
							}
							else
							{
								$insert_day_array[$day]['kommen'] = '';
								$insert_day_array[$day]['gehen'] = '';
							}
						}
						elseif (!empty($_POST['gehen'][$day]))
						{
							//wert in das format HH:MM wandeln
							$tmp_gehen = $this->timeval($_POST['gehen'][$day]);
							if (!empty($tmp_gehen))
							{
								$insert_day_array[$day]['gehen'] = $tmp_gehen;
								$insert_day_array[$day]['kommen'] = '';
							}
							else
							{
								$insert_day_array[$day]['gehen'] = '';
								$insert_day_array[$day]['kommen'] = '';
							}
						}
						else
						{
							$insert_day_array[$day]['kommen'] = '';
							$insert_day_array[$day]['gehen'] = '';
						}

						$sql = $cdb->select('SELECT user_id, min_weektime, arbeitstage, startdatum FROM hdb_user WHERE user_id='.$this->setup_array['user_id']);
						$user_result = $cdb->fetch_array($sql);

						if (isset($user_result['user_id']) and !empty($user_result['user_id']) and (!empty($insert_day_array[$day]['kommen']) or !empty($insert_day_array[$day]['gehen'])))
						{
							$insert_day_array[$day]['pause']	= intval($_POST['pause'][$day]);
							//$insert_day_array[$day]['zeit']		= 0 - round($user_result['min_weektime'] / $user_result['arbeitstage'], 2);
							$insert_day_array[$day]['zeit']		= 0;
							$insert_day_array[$day]['istzeit']	= 0;

							//Gibt die Kommentare zurueck
							$get_comment_boole = true;

							//Projektzeiten berechnen
							$project_time_array = $this->calculate_projectime($day);
							if (!empty($project_time_array))
							{
								$insert_day_array[$day]['projektzeiten'] = $project_time_array;
							}
						}
						//Error kein user gefunden sollte eigendlich nie passieren
						else
						{
							$insert_day_array[$day]['kommen'] = '';
						}
					}
					//keine Arbeitszeiten angegeben oder geleert
					elseif (empty($_POST['kommen'][$day]) and empty($_POST['gehen'][$day]))
					{
						//Gibt die Kommentare zurueck
						$get_comment_boole = true;
						$insert_day_array[$day]['kommen'] = '';

						//Projektzeiten berechnen
						$project_time_array = $this->calculate_projectime($day);
						if (!empty($project_time_array))
						{
							$insert_day_array[$day]['projektzeiten'] = $project_time_array;
						}
					}
				}

				if ($get_comment_boole === true)
				{
					$comment_array = $this->get_comment($day, true);
					if (!empty($comment_array))
					{
						if (isset($insert_day_array[$day]))
						{
							$insert_day_array[$day] += $comment_array;
						}
						else
						{
							$insert_day_array[$day] = $comment_array;
						}
					}
				}

				if (isset($insert_day_array))
				{
					return $insert_day_array;
				}
				else
				{
					return false;
				}
			}
		}

		/**
		 * Speichet alle Daten aus $data in der DB
		 *
		 * @param array $data
		 * @return boole
		 */
		private function insert_data($data)
		{
			$cdb = new MySql;

			//Fehler Zaehlen
			$count_error_int = 0;
			$day_edit_array = array();
			foreach ($data as $day => $day_data)
			{
				$day_edit_array[] = $day;
				//Isodatum fuer den aktuellen Tag
				if ($day < 10)
				{
					$day_string = "0$day";
				}
				else
				{
					$day_string = $day;
				}
				$date = $this->setup_array['year'].'-'.$this->setup_array['month'].'-'.$day_string;

				//keine Fehler
				//Tagesarbeitszeiten
				if ($count_error_int == 0)
				{
					//Abwesenheitsgrund vorhanden Es werden keine Arbeitszeiten eingetragen
					//FIXME Zukuenftig sollen neben Abwesenheitsgruenden auch noch arbeitszeiten eingetragen werden koennen
					if (isset($day_data['absence']))
					{
						$tmp_sql_absence_id = '';
						if (!$cdb->update("UPDATE hdb_stz SET kommen=NULL, pause=NULL, gehen=NULL, ist_zeit=NULL, absence_id=".$day_data['absence'].", ueberstunden=NULL WHERE user_id=".$this->setup_array['user_id']." AND datum='".$date."' LIMIT 1"))
						{
							$count_error_int++;
						}
					}
					else
					{
						$tmp_sql_absence_id = 'absence_id=0,';
					}
					//Komplette Arbeitszeit vorhanden
					if (!empty($day_data['kommen']) and !empty($day_data['gehen']))
					{
						if (!$cdb->update("UPDATE hdb_stz SET kommen='".$day_data['kommen']."', pause='".$day_data['pause']."', gehen='".$day_data['gehen']."', ist_zeit='".$day_data['istzeit']."', $tmp_sql_absence_id ueberstunden='".$day_data['zeit']."' WHERE user_id=".$this->setup_array['user_id']." AND datum='".$date."' LIMIT 1"))
						{
							$count_error_int++;
						}
					}
					//unvollstaendige arbeitszeit vorhanden
					elseif (!empty($day_data['kommen']) or !empty($day_data['gehen']))
					{
						//nur die kommenzeit vorhanden
						if (!empty($day_data['kommen']))
						{
							if (!$cdb->update("UPDATE hdb_stz SET kommen='".$day_data['kommen']."', pause='".$day_data['pause']."', gehen=NULL, ist_zeit='".$day_data['istzeit']."', $tmp_sql_absence_id ueberstunden='".$day_data['zeit']."' WHERE user_id=".$this->setup_array['user_id']." AND datum='".$date."' LIMIT 1"))
							{
								$count_error_int++;
							}
						}
						//nur die gehen zeit vorhanden
						else
						{
							if (!$cdb->update("UPDATE hdb_stz SET kommen=NULL, pause='".$day_data['pause']."', gehen='".$day_data['gehen']."', ist_zeit='".$day_data['istzeit']."', $tmp_sql_absence_id ueberstunden='".$day_data['zeit']."' WHERE user_id=".$this->setup_array['user_id']." AND datum='".$date."' LIMIT 1"))
							{
								$count_error_int++;
							}
						}
					}
					//keine arbeitszeit vorhanden
					elseif (empty($day_data['kommen']) and empty($day_data['gehen']))
					{
						if (!$cdb->update("UPDATE hdb_stz SET kommen=NULL, pause=NULL, gehen=NULL, ist_zeit=NULL, $tmp_sql_absence_id ueberstunden=NULL WHERE user_id=".$this->setup_array['user_id']." AND datum='".$date."' LIMIT 1"))
						{
							$count_error_int++;
						}
						if (!$cdb->update("DELETE FROM hdb_stz_projekt WHERE user_id=".$this->setup_array['user_id']." AND datum='".$date."'"))
						{
							$count_error_int++;
						}
						/*
						 * Mit Version 0.9.6 werden kommentare zu tagen ohne arbeitszeiten erlaubt
						//Kommentare entfernen diese werden gegebenenfalls neu eingetragen per insert
						if (!$cdb->update("DELETE FROM hdb_stz_kommentar WHERE user_id=".$this->setup_array['user_id']." AND datum='".$date."'"))
						{
							$count_error_int++;
						}*/
					}
				}

				//keine Fehler
				//Projektzeiten
				if ($count_error_int == 0 and isset($day_data['projektzeiten']) and is_array($day_data['projektzeiten']))
				{
					foreach ($day_data['projektzeiten'] as $projekt_id => $projekt_zeit)
					{
						//pruefen ob bereits ein eintrag vorhanden ist
						$sql = $cdb->select("SELECT stz_pro_id, stunden FROM hdb_stz_projekt WHERE user_id=".$this->setup_array['user_id']." AND projekt_id=".$projekt_id." AND datum='".$date."'");
						$pro_result = $cdb->fetch_array($sql);

						//Genau Projektzeitenangabe
						if (isset($projekt_zeit['kommen']))
						{
							//Update
							if (isset($pro_result['stz_pro_id']) and $pro_result['stz_pro_id'] > 0)
							{
								if (!$cdb->update("UPDATE hdb_stz_projekt SET stunden=".$projekt_zeit['stunden'].", kommen='".$projekt_zeit['kommen']."', pause='".$projekt_zeit['pause']."', gehen='".$projekt_zeit['gehen']."' WHERE stz_pro_id=".$pro_result['stz_pro_id']))
								{
									$count_error_int++;
								}
							}
							else//Neueintrag
							{
								if (!$cdb->insert("INSERT INTO hdb_stz_projekt (user_id, projekt_id, stunden, datum, kommen, pause, gehen) VALUES (".$this->setup_array['user_id'].", ".$projekt_id.", '".$projekt_zeit['stunden']."', '$date', '".$projekt_zeit['kommen']."', '".$projekt_zeit['pause']."', '".$projekt_zeit['gehen']."')"))
								{
									$count_error_int++;
								}
							}
						}
						else//einfach Projektzeitenangabe
						{
							//Update
							if (isset($pro_result['stz_pro_id']) and $pro_result['stz_pro_id'] > 0)
							{
								//alte stunden und neue sind unterschiedlich eventuell bereits eingetragene Projektzeiten entfernen
								if ($projekt_zeit['stunden'] != $pro_result['stunden'])
								{
									if (!$cdb->update("UPDATE hdb_stz_projekt SET stunden=".$projekt_zeit['stunden'].", kommen=NULL, pause=NULL, gehen=NULL WHERE stz_pro_id=".$pro_result['stz_pro_id']))
									{
										$count_error_int++;
									}
								}
								else
								{
									if (!$cdb->update("UPDATE hdb_stz_projekt SET stunden=".$projekt_zeit['stunden']." WHERE stz_pro_id=".$pro_result['stz_pro_id']))
									{
										$count_error_int++;
									}
								}
							}
							else//Neueintrag
							{
								if (!$cdb->insert("INSERT INTO hdb_stz_projekt (user_id, projekt_id, stunden, datum) VALUES (".$this->setup_array['user_id'].", ".$projekt_id.", '".$projekt_zeit['stunden']."', '$date')"))
								{
									$count_error_int++;
								}
							}
						}
						//Abbruch der foreachschleife bei Fehler
						if ($count_error_int > 0)
						{
							break;
						}
					}
				}

				//keine Fehler bis her
				//Tageskommentar
				if ($count_error_int == 0 and isset($day_data['tageskommentar']))
				{
					//pruefen ob bereits ein Kommentar vorhanden ist
					$sql = $cdb->select("SELECT kommentar_id FROM hdb_stz_kommentar WHERE projekt_id=0 AND user_id=".$this->setup_array['user_id']." AND datum='".$date."'");
					$kom_result = $cdb->fetch_array($sql);

					if (isset($kom_result['kommentar_id']) and $kom_result['kommentar_id'] > 0)
					{
						if (!$cdb->update("UPDATE hdb_stz_kommentar SET kommentar='".$day_data['tageskommentar']."' WHERE kommentar_id=".$kom_result['kommentar_id']))
						{
							$count_error_int++;
						}
					}
					else//Neieintrag
					{
						if (!$cdb->insert("INSERT INTO hdb_stz_kommentar (projekt_id, user_id, datum, kommentar) VALUES (0, ".$this->setup_array['user_id'].", '".$date."', '".$day_data['tageskommentar']."')"))
						{
							$count_error_int++;
						}
					}
				}

				//keine Fehler bis her
				//Projektkommentare
				if ($count_error_int == 0 and isset($day_data['projektkommentar']) and is_array($day_data['projektkommentar']))
				{
					foreach ($day_data['projektkommentar'] as $projekt_id => $projekt_kommentar)
					{
						//pruefen ob bereits ein Kommentar vorhanden ist
						$sql = $cdb->select("SELECT kommentar_id FROM hdb_stz_kommentar WHERE projekt_id=".$projekt_id." AND user_id=".$this->setup_array['user_id']." AND datum='".$date."'");
						$kom_result = $cdb->fetch_array($sql);

						if (isset($kom_result['kommentar_id']) and $kom_result['kommentar_id'] > 0)
						{
							if (!$cdb->update("UPDATE hdb_stz_kommentar SET kommentar='$projekt_kommentar' WHERE kommentar_id=".$kom_result['kommentar_id']))
							{
								$count_error_int++;
							}
						}
						else//Neieintrag
						{
							if (!$cdb->insert("INSERT INTO hdb_stz_kommentar (projekt_id, user_id, datum, kommentar) VALUES (".$projekt_id.", ".$this->setup_array['user_id'].", '".$date."', '$projekt_kommentar')"))
							{
								$count_error_int++;
							}
						}
					}
				}

				//Fehler abbruch der foreachschleife
				if ($count_error_int > 0)
				{
					break;
				}
			}

			/*
			//alle nichtbearbeiteten Tage leeren
			for ($i = 1; $i <= $this->setup_array['month_days_count']; $i++)
			{
				//nur Tage leeren zu denen keine bearbeitung erfolgte
				if (!in_array($i, $day_edit_array))
				{
					if ($i < 10)
					{
						$day_string = "0$i";
					}
					else
					{
						$day_string = $i;
					}

					$date = $this->setup_array['year'].'-'.$this->setup_array['month'].'-'.$day_string;
					//Tages Werte leeren
					if (!$cdb->update("UPDATE hdb_stz SET kommen=NULL, pause=NULL, gehen=NULL, ist_zeit=NULL, absence_id=NULL, ueberstunden=NULL WHERE user_id=".$this->setup_array['user_id']." AND datum='".$date."' LIMIT 1"))
					{
						$count_error_int++;
					}
					//Projektzeiten entfernen diese werden gegebenenfalls neu eingetragen per insert
					if (!$cdb->update("DELETE FROM hdb_stz_projekt WHERE user_id=".$this->setup_array['user_id']." AND datum='".$date."'"))
					{
						$count_error_int++;
					}
					//Kommentare entfernen diese werden gegebenenfalls neu eingetragen per insert
					if (!$cdb->update("DELETE FROM hdb_stz_kommentar WHERE user_id=".$this->setup_array['user_id']." AND datum='".$date."'"))
					{
						$count_error_int++;
					}
				}
			}
	*/

			//keine Fehler
			if ($count_error_int == 0)
			{
				return true;
			}
			else
			{
				return false;
			}
		}

		/**
		 * Liest die Kommentare fuer einen Tag aus
		 *
		 * @param int $day
		 * @param boole $get_project
		 * @return array
		 */
		private function get_comment($day, $get_project)
		{
			$return_array = array();
			//Tageskommentar
			if (isset($_POST['kommentar'][$day]))
			{
				$return_array['tageskommentar'] = str_replace('[br]', "\n", $_POST['kommentar'][$day]);
			}
			//Projektkommentar
			if ($get_project === true and isset($_POST['projektkommentar'][$day]) and is_array($_POST['projektkommentar'][$day]) and count($_POST['projektkommentar'][$day]) > 0)
			{
				foreach ($_POST['projektkommentar'][$day] as $project_id => $kommentar)
				{
					$return_array['projektkommentar'][$project_id] = str_replace('[br]', "\n", $kommentar);
				}
			}
			return $return_array;
		}

		/**
		 * prueft bzw. Berechnet die Projektzeiten
		 *
		 * @param int $day
		 * @return array
		 */
		private function calculate_projectime($day)
		{
			//pruefen ob Prjektarray existiert und inhalt hat
			if (isset($_POST['projektzeit'][$day]) and is_array($_POST['projektzeit'][$day]) and count($_POST['projektzeit'][$day]) > 0)
			{
				$return_array = array();
				//Projektzeiten fuer den tag abarbeiten
				foreach ($_POST['projektzeit'][$day] as $project_id => $data)
				{
					$project_calculate_boole = true;
					$project_time = array();
					//Projektzeit ist vorhanden
					if (isset($data['stunden']))
					{
						if (empty($data['stunden']))
						{
							if ($this->is_projecttime_exists($day, $project_id))
							{
								$data['stunden'] = 0;
							}
							else
							{
								$project_calculate_boole = false;
							}
						}
						if ($project_calculate_boole === true)
						{
							//Genaue Projektzeit ist vorhanden
							if (isset($data['kommen']) and isset($data['pause']) and isset($data['gehen']) and !empty($data['kommen']) and !empty($data['gehen']))
							{
								$project_time = $this->calculate_worktime($data['kommen'], $data['pause'], $data['gehen']);
							}
							//Genau Projektzeitangaben vorhanden
							if (count($project_time) > 0)
							{
								//Sollte nur ungleich sein wenn genaue Projektzeiten angegebene werden und die Stunden nachtraeglich nochmal veraendert werden
								if ($data['stunden'] == $project_time['zeit'])
								{
									$return_array[$project_id]['kommen']	= $project_time['kommen'];
									$return_array[$project_id]['pause']		= $project_time['pause'];
									$return_array[$project_id]['gehen']		= $project_time['gehen'];
									$return_array[$project_id]['stunden']	= $project_time['zeit'];
								}
								else//genaue Zeitangaben verwerfen und nur Stunden verwenden
								{
									$return_array[$project_id]['stunden'] = $data['stunden'];
								}
							}
							else
							{
								$return_array[$project_id]['stunden'] = $data['stunden'];
							}
						}
					}
				}
				return $return_array;
			}
			else
			{
				return array();
			}
		}

		/**
		 * prueft ob zu einem Tag bereits eine Projektzeitangabe existiert
		 *
		 * @param int $day
		 * @param int $project_id
		 * @return boole
		 */
		private function is_projecttime_exists($day, $project_id)
		{
			$cdb = new MySql;

			if ($day < 10)
			{
				$date_string = $this->setup_array['year'].'-'.$this->setup_array['month'].'-0'.$day;
			}
			else
			{
				$date_string = $this->setup_array['year'].'-'.$this->setup_array['month'].'-'.$day;
			}

			$sql = $cdb->select("SELECT stz_pro_id FROM hdb_stz_projekt WHERE user_id=".$this->setup_array['user_id']." AND projekt_id=$project_id AND datum='$date_string'");
			$result = $cdb->fetch_array($sql);

			if (isset($result['stz_pro_id']) and $result['stz_pro_id'] > 0)
			{
				return true;
			}
			else
			{
				return false;
			}
		}

		/**
		 * Berechnet die Arbeitszeit
		 *
		 * @param string $kommen
		 * @param int $pause
		 * @param string $gehen
		 * @return array
		 */
		public function calculate_worktime($kommen, $pause, $gehen)
		{
			$kommen = $this->timeval($kommen);
			$pause = intval($pause);
			$gehen = $this->timeval($gehen);

			if (!empty($kommen) and !empty($gehen))
			{
				//Zeiten in stunde und minute aufsplitten
				$kommen_array = explode(':', $kommen);
				$gehen_array = explode(':', $gehen);

				//kommenzeit
				$kommen_stunde_int = intval($kommen_array[0]);
				$kommen_minute_int = intval($kommen_array[1]);

				//gehenzeit
				$gehen_stunde_int = intval($gehen_array[0]);
				$gehen_minute_int = intval($gehen_array[1]);

				//gesamtwerte
				$kommen_time_int = 0;
				$gehen_time_int = 0;
				$total_time_float = 0;

				//kommenzeit bis 24 Uhr berechnen
				if ($kommen_minute_int != 0)
				{
					$kommen_time_int = 60 - $kommen_minute_int;
					$kommen_stunde_int++;
				}
				$kommen_time_int += (24 - $kommen_stunde_int) * 60;

				//gehenzeit bis 24 uhr berechnen
				if ($gehen_minute_int != 0)
				{
					$gehen_time_int = 60 - $gehen_minute_int;
					$gehen_stunde_int++;
				}
				$gehen_time_int += (24 - $gehen_stunde_int) * 60;

				//Arbeitszeit berechnen wenn diese am selben tag ist
				//if ($gehen_stunde_int > $kommen_stunde_int)
				if ($kommen_time_int > $gehen_time_int)
				{
					$total_time_float = $kommen_time_int - $gehen_time_int;
				}
				else//arbeitszeit geht ueber zwei tage
				{
					$total_time_float = $kommen_time_int + 1440 - $gehen_time_int;
				}

				//pause sollte keinen negativwert verursachen
				if ($total_time_float > $pause)
				{
					$total_time_float -= $pause;
					$total_time_float = round($total_time_float / 60, 2);

					return array('kommen' => $kommen, 'pause' => $pause, 'gehen' => $gehen, 'zeit' => $total_time_float);
				}
				else
				{
					return array();
				}
			}
			else
			{
				return array();
			}
		}

		/**
		 * Laed alle erforderlichen Grundeinstellungen aus der DB
		 *
		 * @return boole
		 */
		private function create_setup($user_id = 0, $year_int = 0, $month_string = '')
		{
			$cdb = new MySql;

			if (!isset($_POST['user']) and !empty($user_id))
			{
				$_POST['user'] = $user_id;
			}

			if (!isset($_POST['year']) and !empty($year_int))
			{
				$_POST['year'] = $year_int;
			}

			if (!isset($_POST['month']) and !empty($month_string))
			{
				$_POST['month'] = $month_string;
			}

			$sql = $cdb->select('SELECT user_id, min_weektime, max_weektime, arbeitstage FROM hdb_user WHERE user_id='.$_POST['user']);
			$user_result = $cdb->fetch_array($sql);

			//pruefen ob der angegebene User existiert
			if (isset($user_result['user_id']) and $user_result['user_id'] > 0)
			{
				// ACHTUNG: aenderungen des Jahresurlaubsanspruches im Laufenden Jahr werden nicht beruecksichtigt.
				// Diese koennen nur mit einer uebertragsaenderung beruecksitigt werden
				// Ansonsten wird der neue Urlaubsanspruch erst im naechsten Jahr beruecksichtigt

				$sql = $cdb->select("SELECT id, startwert_stunden, startwert_urlaub, wochenarbeitszeit, max_wochenarbeitszeit, arbeitstage, pausenzeit, abgeschlossen FROM hdb_stz_monat WHERE user_id=".$_POST['user']." AND datum='".$_POST['year'].'-'.$_POST['month'].'-'."01'");
				$time_result = $cdb->fetch_array($sql);

				//pruefen ob Stundenzettel existiert
				if (isset($time_result['id']) and $time_result['id'] > 0)
				{
					$this->setup_array['user_id']					= $user_result['user_id'];
					$this->setup_array['startwert_urlaub']			= $time_result['startwert_urlaub'];
					$this->setup_array['startwert_stunden']			= $time_result['startwert_stunden'];
					$this->setup_array['month']						= $_POST['month'];
					$this->setup_array['year']						= $_POST['year'];
					$this->setup_array['month_days_count']			= date('t', mktime(0, 0, 0, $_POST['month'], 1, $_POST['year']));

					if (isset($GLOBALS['setup_array']['stz_round_time']) and $GLOBALS['setup_array']['stz_round_time'] == 'j')
					{
						$this->setup_array['round_time']				= true;
					}
					else
					{
						$this->setup_array['round_time']				= false;
					}

					$this->setup_array['month_urlaub_count']		= 0;
					$this->setup_array['month_sollstunden']			= 0;//Speichert die Sollstunden fuer einen Monat
					$this->setup_array['month_maxsollstunden']		= 0;//Speichert die Maxsollstunden fuer einen Monat
					$this->setup_array['month_iststunden']			= 0;//Speichert die Iststunden fuer einen Monat
					$this->setup_array['month_ueberstunden']		= 0;//Speichert die ueberstunden fuer einen Monat
					$this->setup_array['month_urlaub_count']		= 0;//Speichert die anzahl der Urlaubstage fuer einen Monat
					$this->setup_array['report_vor']				= intval($GLOBALS['setup_array']['stz_report_vor']);//Speichert die Stunde vor der nicht gearbeitet werden sollte
					$this->setup_array['report_nach']				= intval($GLOBALS['setup_array']['stz_report_nach']);//Speichert die Stunde nach der nicht gearbeitet werden sollte
					$this->setup_array['report_mehr']				= intval($GLOBALS['setup_array']['stz_report_mehr_als']);//Speichert die Stundenanzahl die pro tag nicht ueberschritten werden sollte
					$this->setup_array['report']					= array('vor' => 'n', 'nach' => 'n', 'mehr' => 'n');
					//Aktueller Monat wird bearbeitet
					//fuer den aktuellen Monat koennen aenderungen in den Arbeitszeiten vorgenommen werden, diese werden bei der aktuellen berechnung sofort beruecksichtigt.
					if ($_POST['month'] == date('m') and $_POST['year'] == date('Y'))
					{
						$this->setup_array['min_weektime']	= $user_result['min_weektime'];
						$this->setup_array['max_weektime']	= $user_result['max_weektime'];
						$this->setup_array['arbeitstage']	= $user_result['arbeitstage'];
						// FIXME: Pausenzeit sollte aus der usertabelle geholt werden, nicht wie derzeit aus der setup-tabelle
						$this->setup_array['pausenzeit'] = intval($GLOBALS['setup_array']['stz_pausenminuten']);//Speichert die Voreinstellung der taegl. Pausenminuten
					}
					//es wird ein aelterer Stundenzettel bearbeitet
					//Eventuelle aenderungen an den Arbeitszeiten werden nicht beruecksichtigt
					else
					{
						$this->setup_array['min_weektime']	= $time_result['wochenarbeitszeit'];
						$this->setup_array['max_weektime']	= $time_result['max_wochenarbeitszeit'];
						$this->setup_array['arbeitstage']	= $time_result['arbeitstage'];
						$this->setup_array['pausenzeit']	= $time_result['pausenzeit'];
					}
					// Min und Max Arbeitszeit fuer einen tag
					// ACHTUNG: nur eine einfach genauigkeit. Der Wert wird auf zwei Stellen nach dem Komma gerundet
					$this->setup_array['min_daytime']		= round($this->setup_array['min_weektime'] / $this->setup_array['arbeitstage'], 2);
					$this->setup_array['max_daytime']		= round($this->setup_array['max_weektime'] / $this->setup_array['arbeitstage'], 2);

					//Tageseinstellungen auslesen
					for ($i = 1; $i <= $this->setup_array['month_days_count']; $i++)
					{
						if ($i < 10)
						{
							$date = $_POST['year'].'-'.$_POST['month'].'-0'.$i;
						}
						else
						{
							$date = $_POST['year'].'-'.$_POST['month'].'-'.$i;
						}

						//Taggeseinstellungen auslesen
						//Wochenende
						if ($this->is_holiday($date, 'weekend') === true)
						{
							$this->setup_day_array[$i] = array('feiertag'			=> false,
																'wochenende'		=> true,
																'halbtags'			=> false,
																'reg_tag'			=> false,
																'min_sollstunden'	=> 0,
																'max_sollstunden'	=> 0,
																'urlaubstag'		=> 0,
																'date'				=> $date);
						}
						//Feiertag
						elseif ($this->is_holiday($date, 'holiday') === true)
						{
							$this->setup_day_array[$i] = array('feiertag'			=> true,
																'wochenende'		=> false,
																'halbtags'			=> false,
																'reg_tag'			=> false,
																'min_sollstunden'	=> 0,
																'max_sollstunden'	=> 0,
																'urlaubstag'		=> 0,
																'date'				=> $date);
						}
						//Halbtags
						elseif ($this->is_holiday($date, 'halbtags') === true)
						{
							$this->setup_day_array[$i] = array('feiertag'			=> false,
																'wochenende'		=> false,
																'halbtags'			=> true,
																'reg_tag'			=> false,
																'min_sollstunden'	=> round($this->setup_array['min_weektime'] / $this->setup_array['arbeitstage'] / 2, 2),
																'max_sollstunden'	=> round($this->setup_array['max_weektime'] / $this->setup_array['arbeitstage'] / 2, 2),
																'urlaubstag'		=> 0.5,
																'date'				=> $date);
						}
						//Normaler Arbeitstag
						else
						{
							$this->setup_day_array[$i] = array('feiertag'			=> false,
																'wochenende'		=> false,
																'halbtags'			=> false,
																'reg_tag'			=> true,
																'min_sollstunden'	=> round($this->setup_array['min_weektime'] / $this->setup_array['arbeitstage'], 2),
																'max_sollstunden'	=> round($this->setup_array['max_weektime'] / $this->setup_array['arbeitstage'], 2),
																'urlaubstag'		=> 1,
																'date'				=> $date);
						}
						//Sollstunden fuer den Monat
						$this->setup_array['month_sollstunden']		+= $this->setup_day_array[$i]['min_sollstunden'];
						//Maxsollstunden fuer den Monat
						$this->setup_array['month_maxsollstunden']	+= $this->setup_day_array[$i]['max_sollstunden'];
					}
					//Abwesenheitsgruende auslesen
					$sql = $cdb->select('SELECT absence_type_id, working_time, total_day FROM hdb_absence_type ORDER BY absence_type_id ASC');
					while ($result = $cdb->fetch_array($sql))
					{
						if ($result['working_time'] == 1)
						{
							$this->absence_array[$result['absence_type_id']] = true;
						}
						else
						{
							$this->absence_array[$result['absence_type_id']] = false;
						}

						if ($result['total_day'] == 1)
						{
							$this->absence_array['full_day'][$result['absence_type_id']] = true;
						}
						else
						{
							$this->absence_array['full_day'][$result['absence_type_id']] = false;
						}
					}
					return true;
				}
				else
				{
					return false;
				}
			}
			else
			{
				return false;
			}
		}

		/**
		 * prueft einen Sting auf ein gueltiges Zeitformat HH:MM
		 *
		 * @param string $time
		 * @return string
		 */
		public function timeval($time)
		{
			//ersetzt alle nicht nummerischen zeichen durch :
			$time = preg_replace('@\D+@', ':', $time);

			if (substr_count($time, ':') == 1)
			{
				$ex_time_array = explode(':', $time);
				$ex_time_array[0] = intval($ex_time_array[0]);
				$ex_time_array[1] = intval($ex_time_array[1]);

				if ($ex_time_array[0] >= 0 and $ex_time_array[0] <= 23 and $ex_time_array[1] >= 0 and $ex_time_array[1] <= 59)
				{
					//Zeitangaben runden
					//ACHTUNG $this->setup_array['round_time'] existiert nur nach dem aufruf von $this->create_setup()
					if (isset($this->setup_array['round_time']) and $this->setup_array['round_time'] === true)
					{
						return $this->round_time($time);
					}
					////Zeitangaben runden
					//Das selbe wie oben halt nur fuer den fall das $this->setup_array['round_time'] nicht definiert wurde
					elseif (isset($GLOBALS['setup_array']['stz_round_time']) and $GLOBALS['setup_array']['stz_round_time'] == 'j')
					{
						return $this->round_time($time);
					}
					else
					{
						return $time;
					}
				}
				else
				{
					return '';
				}
			}
			else
			{
				return '';
			}
		}

		/**
		 * Rundet Zeitangaben auf volle viertel stunden auf bzw. ab
		 *
		 * @param string $time
		 * @return string
		 */
		public function round_time($time)
		{
			$time_array = explode(':', $time);
			$minute = intval($time_array[1]);
			$stunde = intval($time_array[0]);

			if ($minute > 0 and $minute <= 7)
			{
				$minute = 0;
			}
			elseif ($minute >= 8 and $minute <= 22)
			{
				$minute = 15;
			}
			elseif ($minute >= 23 and $minute <= 37)
			{
				$minute = 30;
			}
			elseif ($minute >= 38 and $minute <= 52)
			{
				$minute = 45;
			}
			elseif ($minute >= 53)
			{
				$minute = 0;
				$stunde++;
				if ($stunde == 24)
				{
					$stunde = 0;
				}
			}

			if ($stunde < 10)
			{
				$stunde = '0'.$stunde;
			}

			if ($minute < 10)
			{
				$minute = '0'.$minute;
			}
			return $stunde.':'.$minute;
		}

		/**
		 * pruefen ob das uebergebene Datum ein Freier Tag ist
		 *
		 * @param string $date
		 * @param string $typ
		 * @param int $user_id
		 * @return boole
		 */
		public function is_holiday($date, $typ, $user_id = 0)
		{
			$cdb = new MySql;
			$kal = new Kalender;

			if (empty($user_id))
			{
				$user_id = $this->setup_array['user_id'];
			}

			//Feiertage finden
			if ($typ == 'holiday')
			{
				$sql = $cdb->select("SELECT absence_id FROM hdb_stz WHERE user_id=$user_id AND datum='$date'");
				$result = $cdb->fetch_array($sql);

				//ACHTUNG: die 3 als ID kann sich aendern
				if (isset($result['absence_id']) and $result['absence_id'] == 3)
				{
					return true;
				}
				else
				{
					return false;
				}
			}
			//Wochenende finden
			elseif ($typ == 'weekend')
			{
				if ($kal->is_weekend($date))
				{
					return true;
				}
				else
				{
					return false;
				}
			}
			//Arbeitstage mit verringerter Sollstundenzahl
			elseif ($typ == 'halbtags')
			{
				if ($kal->is_half_holiday($date))
				{
					return true;
				}
				else
				{
					return false;
				}
			}
		}

		/**
		 * prueft ob zu den uebergebenen Daten ein Stundenzettel vorhanden ist.
		 *
		 * @return boole
		 */
		private function ts_exists()
		{
			$cdb = new MySql;

			$sql = $cdb->select("SELECT id FROM hdb_stz_monat WHERE user_id=".$_POST['user'] ." AND datum='".$_POST['year'].'-'.$_POST['month'].'-01'."'");
			$result = $cdb->fetch_array($sql);

			if (isset($result['id']) and $result['id'] > 0)
			{
				$this->setup_array['stz_month_id'] = $result['id'];
				return true;
			}
			else
			{
				return false;
			}
		}

		/**
		 * Reload pruefung
		 *
		 * @return boole
		 */
		private function chk_reload()
		{
			//pruefen ob Reload und Checksumme vorhanden sind
			if (isset($_POST['reload_sperre_string']))
			{
				$re = new Reload;

				//pruefen ob Checksumme und Reloadsperre ok sind
				if ($re->check($_POST['reload_sperre_string']) == true)
				{
					return true;
				}
			}
			return false;
		}

		/**
		 * Checksummen pruefung
		 *
		 * @return boole
		 */
		private function chk_checksum()
		{
			//pruefen ob Reload und Checksumme vorhanden sind
			if (isset($_POST['VCS']))
			{
				$chk = new Checksum;

				//pruefen ob Checksumme und Reloadsperre ok sind
				if ($chk->Chk($_POST['VCS']) === true)
				{
					return true;
				}
			}
			return false;
		}

		/**
		 * Traegt einen genehmigten Abwesenheitsgrund in den Stundenzettel ein
		 * @param int $as_id
		 * @return boole
		 */
		public function insert_absence_request($as_id)
		{
			$cdb = new MySql();
			$kal = new Kalender();

			$return = false;
			$sql = $cdb->select('SELECT t1.absence_type_id, t1.user_id, t1.date_from, t1.date_to, t1.status, t2.total_day FROM hdb_absence_list t1
								LEFT JOIN hdb_absence_type t2 ON t1.absence_type_id=t2.absence_type_id
								WHERE t1.as_id='.$as_id);
			$result = $cdb->fetch_array($sql);

			if ($result['status'] == 'approved')
			{
				$day_array = array();
				if ($result['date_from'] == $result['date_to'])
				{
					$day_array[]['iso'] = $result['date_from'];
				}
				else
				{
					$day_array = $kal->get_difference($result['date_from'], $result['date_to']);
				}

				$month_stz_is_writable_array = array();
				for ($i = 0; $i < count($day_array); $i++)
				{
					$tmp_date = $day_array[$i]['iso'];

					//nicht an wochenenden eintragen
					if (!$kal->is_weekend($tmp_date))
					{
						$tmp_year_month = substr($tmp_date, 0, 7);
						if (!isset($month_stz_is_writable_array[$tmp_year_month]))
						{
							$sub_sql = $cdb->select('SELECT id FROM hdb_stz_monat WHERE user_id='.$result['user_id']." AND LEFT(datum, 7)='$tmp_year_month' AND abgeschlossen='n'");
							$sub_result = $cdb->fetch_array($sub_sql);

							if (isset($sub_result['id']) and !empty($sub_result['id']))
							{
								$month_stz_is_writable_array[$tmp_year_month] = true;
							}
							else
							{
								$month_stz_is_writable_array[$tmp_year_month] = false;;
							}
						}

						if ($month_stz_is_writable_array[$tmp_year_month] == true)
						{
							$sub_sql = $cdb->select('SELECT * FROM hdb_stz WHERE user_id='.$result['user_id']." AND datum='$tmp_date'");
							$sub_result = $cdb->fetch_array($sub_sql);

							if (isset($sub_result['stz_id']) and !empty($sub_result['stz_id']))
							{
								$insert_absence = false;
								//keine zeiteintragung oder abwesenheitsgruende vorhanden
								if (empty($sub_result['kommen']) and empty($sub_result['gehen']) and empty($sub_result['absence_id']))
								{
									$insert_absence = true;
								}
								elseif ($result['total_day'] == 0 and empty($sub_result['absence_id']))
								{
									$insert_absence = true;
								}

								if ($insert_absence == true)
								{
									$cdb->update('UPDATE hdb_stz SET absence_id='.$result['absence_type_id'].' WHERE user_id='.$result['user_id']." AND datum='$tmp_date'");
									$return = true;
								}
							}
						}
					}
				}
			}

			return $return;
		}
	}
?>