<?php
	/**
	 * management of file- and image-uploads
	 *
	 * @author Soeren Pieper <soeren.pieper@docemos.de> / Martin Ufer <martin.ufer@docemos.de>
	 * @copyright Copyright (C) 2009  docemos GmbH
	 * @package time-o-meter
	 *
	 * This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 3 of the License, or (at your option) any later version.
	 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
	 * You should have received a copy of the GNU General Public License along with this program; if not, see http://www.gnu.org/licenses/.
	 */

	/**
	 * management of file- and image-uploads
	 * @package time-o-meter
	 * @subpackage classes
	 */
	class Upload
	{
		/**
		* Bild-Upload
		* @param array $image, Array von $_FILES['name']
		* @param string $new_imagename, Gegebenenfalls neuer Dateiname. Wenn kein Name angegeben ist wird der Original Dateiname verwendet. Achtung wenn Sonderzeichen im Dateinamen enthalten sind werden diese entfernt.
		* @param string $pfad, Absoluter Pfad zum Ordner in dem das Bild gespeichert werden soll. Wenn der Angegebene Ornder nicht exsistiert wird versucht diesen anzulegen.
		* @param int $max_size, Max. Dateigroesse in Byte.
		* @param array $max_image_size, $max_image_size[0] = Max Bildbreite in px. $max_image_size[1] = Max Bildhoehe in px. $max_image_size[2] j = Wenn das Bild zu Gross ist, ist eine verkleinerung ist zwingend erforderlich (eine verkleinerung ist nur bei jpg, gif und png moeglich bei allen anderen Formaten wird der Upload abgebrochen). n = Eine verkleinerung ist nicht zwingend erfprderlich. Die Verkleinerung wird nur bei jpg, gif und png Bildern durchgefuehrt. Andere Bildformate z.B. BMP werden gespeichert auch wenn Sie zu gross sind. Alle Angaben in $max_image_size sind keine Pflichtangaben, jedoch muss bei einer Angabe der Index eingehalten werden.
		* @param array $image_typ, enthaelt alle zulaessigen Dateiendungen. Alle Angaben in kleinschrift.
		* @param string $chmod, CHMOD Wert den das Bild nach einem Upload erhalten soll.
		* @return array array('name'=>'Bildname unter dem die Datei gespeichert wurde','typ'=>'MIME Typ','size'=>'Dateigroesse in Byte','error'=>array('1=Fehler beim Upload','2=Falscher Dateityp','3=Datei ist zu Gross (Byte)','4=Dateipfad konnt nicht gefunden bzw nicht angelegt werden','5=Das BIld ist in seinen abmassen zu gross (PX)','6=Datei konnte nicht gespeichert werden','7=Die CHMOD`s konnten nicht geaendert werden','8=Das Bild konnte nicht verkleinert werden'))
		*/
		public function image_upload($image, $new_imagename, $pfad, $max_size, $max_image_size, $image_typ, $chmod = 0644)
		{
			//Array fuer die return meldungen
			$return = array();

			//pruefen ob der Upload ohne Fehler erfolgte
			if ($image['error'] == 0)
			{
				//pruefen ob die Datei den richtigen Dateityp hat
				if ($this->chk_filetyp($image['name'], $image_typ))
				{
					//filesize pruefen
					if ($this->chk_filesize($image['size'], $max_size))
					{
						//Dateipfad pruefen
						$pfad = $this->chk_file_pfad($pfad);
						if ($pfad)
						{
							//Bildgroesse (px) pruefen und pruefen ob das Bild verkleinert werden muss
							$size_chk = $this->chk_image_size($max_image_size, $image['tmp_name']);

							if ($size_chk[0])
							{
								//Dateinamen pruefen
								$new_imagename = $this->chk_file_name($image['name'], $new_imagename);

								//Temp Dateinamen vergeben
								if ($size_chk[1])
								{
									$new_imagename2 = $new_imagename;
									$new_imagename = 'tmp_'.$new_imagename;
								}
								else
								{
									$new_imagename2 = $new_imagename;
								}
								//Bild Speichern
								if (@move_uploaded_file($image['tmp_name'], $pfad.$new_imagename))
								{
									//pruefen ob Bild verkleinert werden muss
									if ($size_chk[1])
									{
										//pruefen ob bild ein GIF ist
										if ($size_chk[2] == 1)
										{
											$result = $this->image_edit_gif($pfad, $new_imagename, $new_imagename2, $max_image_size);
											if ($result[0])
											{
												$return['name'] = $new_imagename2;
												$return['typ'] = $image['type'];
												$return['size'] = $result[1];
											}
											else
											{
												//Das Bild konnte nicht verkleinert werden
												$return['error'][] = 8;
												//Im Fehlerfall erstelltes Bild loeschen
												$this->del_temp_image($pfad.$new_imagename2);
											}
										}
										//pruefen ob bild ein JPG ist
										elseif ($size_chk[2] == 2)
										{
											$result = $this->image_edit_jpg($pfad, $new_imagename, $new_imagename2, $max_image_size);
											if ($result[0])
											{
												$return['name'] = $new_imagename2;
												$return['typ'] = $image['type'];
												$return['size'] = $result[1];
											}
											else
											{
												//Das Bild konnte nicht verkleinert werden
												$return['error'][] = 8;
												//Im Fehlerfall erstelltes Bild loeschen
												$this->del_temp_image($pfad.$new_imagename2);
											}
										}
										//pruefen ob bild ein PNG ist
										elseif ($size_chk[2] == 3)
										{
											$result = $this->image_edit_png($pfad, $new_imagename, $new_imagename2, $max_image_size);
											if ($result[0])
											{
												$return['name'] = $new_imagename2;
												$return['typ'] = $image['type'];
												$return['size'] = $result[1];
											}
											else
											{
												//Das Bild konnte nicht verkleinert werden
												$return['error'][] = 8;
												//Im Fehlerfall erstelltes Bild loeschen
												$this->del_temp_image($pfad.$new_imagename2);
											}
										}

										//Das Originalbild loeschen
										$this->del_temp_image($pfad.$new_imagename);
									}
									else
									{
										//Return Daten wenn das Bild nicht verkleinert werden musste
										$return['name'] = $new_imagename;
										$return['typ'] = $image['type'];
										$return['size'] = $image['size'];
									}

									if (!$this->chk_chmod($pfad.$new_imagename2,$chmod))
									{
										//Die CHMOD`s konnten nicht geaendert werden
										$return['error'][] = 7;
									}
								}
								else
								{
									//Datei konnte nicht gespeichert werden
									$return['error'][] = 6;
								}
							}
							else
							{
								//Das BIld ist in seinen abmassen zu gross
								$return['error'][] = 5;
							}
						}
						else
						{
							//Dateipfad konnt nicht gefunden bzw nicht angelegt werden
							$return['error'][] = 4;
						}
					}
					else
					{
						//Datei ist zu gross
						$return['error'][] = 3;
					}
				}
				else
				{
					//falscher dateityp
					$return['error'][] = 2;
				}
			}
			else
			{
				//fehler beim upload
				$return['error'][] = 1;
			}
			return $return;
		}

		/**
		* Datei Upload
		* @param array $file, Array von $_FILES['name']
		* @param string $new_filename, Gegebenenfalls neuer Dateiname. Wenn kein Name angegeben ist wird der Original Dateiname verwendet. Achtung wenn Sonderzeichen im Dateinamen enthalten sind werden diese entfernt.
		* @param string $pfad, Absoluter Pfad zum Ordner in dem das Bild gespeichert werden soll. Wenn der Angegebene Ornder nicht exsistiert wird versucht diesen anzulegen.
		* @param int $max_size, Max. Dateigroesse in Byte.
		* @param array $file_typ, enthaelt alle zulaessigen Dateiendungen. Alle Angaben in kleinschrift.
		* @param string $chmod, CHMOD Wert den das Bild nach einem Upload erhalten soll.
		* @return array array('name'=>'Dateiname unter dem die Datei gespeichert wurde','typ'=>'MIME Typ','size'=>'Dateigroesse in Byte','error'=>array('1=Fehler beim Upload','2=Falscher Dateityp','3=Datei ist zu Gross (Byte)','4=Dateipfad konnt nicht gefunden bzw nicht angelegt werden','5=Datei konnte nicht gespeichert werden','6=Die CHMOD`s konnten nicht geaendert werden'))
		*/
		public function file_upload($file, $new_filename, $pfad, $max_size, $file_typ, $chmod = 0644)
		{
			//array fuer returnmeldungen
			$return = array();

			//pruefen ob der Upload ohne Fehler erfolgte
			if ($file['error'] == 0)
			{
				//prueft dateityp
				if ($this->chk_filetyp($file['name'], $file_typ))
				{
					//filesize pruefen
					if ($this->chk_filesize($file['size'], $max_size))
					{
						//Dateipfad pruefen
						$pfad = $this->chk_file_pfad($pfad);
						if ($pfad)
						{
							//dateiname vergeben
							$new_filename = $this->chk_file_name($file['name'], $new_filename);
							//datei speichern
							if (@move_uploaded_file($file['tmp_name'], $pfad.$new_filename))
							{
								$return['name'] = $new_filename;
								$return['typ'] = $file['type'];
								$return['size'] = $file['size'];

								//chmods setzten
								if (!$this->chk_chmod($pfad.$new_filename, $chmod))
								{
									//Die CHMOD`s konnten nicht geaendert werden
									$return['error'][] = 6;
								}
							}
							else
							{
								//Datei konnte nicht gespeichert werden
								$return['error'][] = 5;
							}
						}
						else
						{
							//Dateipfad konnt nicht gefunden bzw nicht angelegt werden
							$return['error'][] = 4;
						}
					}
					else
					{
						//Datei ist zu gross
						$return['error'][] = 3;
					}
				}
				else
				{
					//falscher dateityp
					$return['error'][] = 2;
				}
			}
			else
			{
				//fehler beim upload
				$return['error'][] = 1;
			}
			return $return;
		}

		/**
		* GIF Bild verkleinern
		* @param string $pfad, Absoluter Pfad zum Ordner in dem das Bild gespeichert werden soll.
		* @param string $tmp_imagename, Temp Bildname.
		* @param string $new_imagename, Neuer Bildname.
		* @param array $max_image_size, $max_image_size[0] = Max Bildbreite in px. $max_image_size[1] = Max Bildhoehe in px. $max_image_size[2] j = Wenn das Bild zu Gross ist, ist eine verkleinerung ist zwingend erforderlich (eine verkleinerung ist nur bei jpg, gif und png moeglich bei allen anderen Formaten wird der Upload abgebrochen). n = Eine verkleinerung ist nicht zwingend erfprderlich. Die Verkleinerung wird nur bei jpg, gif und png Bildern durchgefuehrt. Andere Bildformate z.B. BMP werden gespeichert auch wenn Sie zu gross sind. Alle Angaben in $max_image_size sind keine Pflichtangaben, jedoch muss bei einer Angabe der Index eingehalten werden.
		* @return array array('0'=>'bool','1'=>'Dateigroesse in Byte')
		*/
		private function image_edit_gif($pfad, $tmp_imagename, $new_imagename, $max_image_size)
		{
			//prueft ob die temp Datei vorhanden ist welche verkleinert werden soll
			if (file_exists($pfad.$tmp_imagename))
			{
				if (function_exists("imagegif"))
				{
					$tmp_image = @imagecreatefromgif($pfad.$tmp_imagename);

					//erstellt die neuen Bildmasse
					$result = $this->new_image_size($tmp_image, $max_image_size);
					$new_x = $result[0];
					$new_y = $result[1];
					$tmp_x = $result[2];
					$tmp_y = $result[3];

					//erstellt das neue bild und speichert es ab
					$new_image = @imagecreatetruecolor($new_x, $new_y);
					@imagecopyresized($new_image, $tmp_image, 0, 0, 0, 0, $new_x, $new_y, $tmp_x, $tmp_y);
					@imagegif($new_image, $pfad.$new_imagename);

					//leert den Speicher
					@imagedestroy($new_image);

					$return[0] = TRUE;
					$return[1] = @filesize($pfad.$new_imagename);
					return $return;
				}
				else
				{
					return $return[0] = FALSE;
				}
			}
			else
			{
				return $return[0] = FALSE;
			}
		}

		/**
		* JPG Bild verkleinern
		* @param string $pfad, Absoluter Pfad zum Ordner in dem das Bild gespeichert werden soll.
		* @param string $tmp_imagename, Temp Bildname.
		* @param string $new_imagename, Neuer Bildname.
		* @param array $max_image_size, $max_image_size[0] = Max Bildbreite in px. $max_image_size[1] = Max Bildhoehe in px. $max_image_size[2] j = Wenn das Bild zu Gross ist, ist eine verkleinerung ist zwingend erforderlich (eine verkleinerung ist nur bei jpg, gif und png moeglich bei allen anderen Formaten wird der Upload abgebrochen). n = Eine verkleinerung ist nicht zwingend erfprderlich. Die Verkleinerung wird nur bei jpg, gif und png Bildern durchgefuehrt. Andere Bildformate z.B. BMP werden gespeichert auch wenn Sie zu gross sind. Alle Angaben in $max_image_size sind keine Pflichtangaben, jedoch muss bei einer Angabe der Index eingehalten werden.
		* @return array array('0'=>'bool','1'=>'Dateigroesse in Byte')
		*/
		private function image_edit_jpg($pfad, $tmp_imagename, $new_imagename, $max_image_size)
		{
			//prueft ob die temp Datei vorhanden ist welche verkleinert werden soll
			if (file_exists($pfad.$tmp_imagename))
			{
				if (function_exists("imagejpeg"))
				{
					$tmp_image = @imagecreatefromjpeg($pfad.$tmp_imagename);

					//erstellt die neuen Bildmasse
					$result = $this->new_image_size($tmp_image, $max_image_size);
					$new_x = $result[0];
					$new_y = $result[1];
					$tmp_x = $result[2];
					$tmp_y = $result[3];

					//erstellt das neue bild und speichert es ab
					$new_image = @imagecreatetruecolor($new_x, $new_y);
					@imagecopyresized($new_image, $tmp_image, 0, 0, 0, 0, $new_x, $new_y, $tmp_x, $tmp_y);
					@imagejpeg($new_image, $pfad.$new_imagename);

					//leert den Speicher
					@imagedestroy($new_image);

					$return[0] = TRUE;
					$return[1] = @filesize($pfad.$new_imagename);
					return $return;
				}
				else
				{
					return $return[0] = FALSE;
				}
			}
			else
			{
				return $return[0] = FALSE;
			}
		}

		/**
		* PNG Bild verkleinern
		* @param string $pfad, Absoluter Pfad zum Ordner in dem das Bild gespeichert werden soll.
		* @param string $tmp_imagename, Temp Bildname.
		* @param string $new_imagename, Neuer Bildname.
		* @param array $max_image_size, $max_image_size[0] = Max Bildbreite in px. $max_image_size[1] = Max Bildhoehe in px. $max_image_size[2] j = Wenn das Bild zu Gross ist, ist eine verkleinerung ist zwingend erforderlich (eine verkleinerung ist nur bei jpg, gif und png moeglich bei allen anderen Formaten wird der Upload abgebrochen). n = Eine verkleinerung ist nicht zwingend erfprderlich. Die Verkleinerung wird nur bei jpg, gif und png Bildern durchgefuehrt. Andere Bildformate z.B. BMP werden gespeichert auch wenn Sie zu gross sind. Alle Angaben in $max_image_size sind keine Pflichtangaben, jedoch muss bei einer Angabe der Index eingehalten werden.
		* @return array array('0'=>'bool','1'=>'Dateigroesse in Byte')
		*/
		private function image_edit_png($pfad, $tmp_imagename, $new_imagename, $max_image_size)
		{
			//prueft ob die temp Datei vorhanden ist welche verkleinert werden soll
			if (file_exists($pfad.$tmp_imagename))
			{
				if (function_exists("imagepng"))
				{
					$tmp_image = @imagecreatefrompng($pfad.$tmp_imagename);

					//erstellt die neuen Bildmasse
					$result = $this->new_image_size($tmp_image, $max_image_size);
					$new_x = $result[0];
					$new_y = $result[1];
					$tmp_x = $result[2];
					$tmp_y = $result[3];

					//erstellt das neue bild und speichert es ab
					$new_image = @imagecreatetruecolor($new_x, $new_y);
					@imagecopyresized($new_image, $tmp_image, 0, 0, 0, 0, $new_x, $new_y, $tmp_x, $tmp_y);
					@imagepng($new_image, $pfad.$new_imagename);

					//leert den Speicher
					@imagedestroy($new_image);

					$return[0] = TRUE;
					$return[1] = @filesize($pfad.$new_imagename);
					return $return;
				}
				else
				{
					return $return[0] = FALSE;
				}
			}
			else
			{
				return $return[0] = FALSE;
			}
		}

		/**
		* Erstellt die Bildmasse des zu verkleinerten Bildes
		* @param image $tmp_image, Temp Bild
		* @param array $max_image_size, $max_image_size[0] = Max Bildbreite in px. $max_image_size[1] = Max Bildhoehe in px. $max_image_size[2] j = Wenn das Bild zu Gross ist, ist eine verkleinerung ist zwingend erforderlich (eine verkleinerung ist nur bei jpg, gif und png moeglich bei allen anderen Formaten wird der Upload abgebrochen). n = Eine verkleinerung ist nicht zwingend erfprderlich. Die Verkleinerung wird nur bei jpg, gif und png Bildern durchgefuehrt. Andere Bildformate z.B. BMP werden gespeichert auch wenn Sie zu gross sind. Alle Angaben in $max_image_size sind keine Pflichtangaben, jedoch muss bei einer Angabe der Index eingehalten werden.
		* @return array array('0'=>'neue Breite','1'=>'neue Hoehe','2'=>'Temp Breite','3'=>'Temp Hoehe')
		*/
		private function new_image_size($tmp_image, $max_image_size)
		{
			//bildgroesse des temp bildes
			$tmp_y = @imagesy($tmp_image);
			$tmp_x = @imagesx($tmp_image);

			//prueft ob eine max breite vorgegeben ist wenn nicht wird die breite des temp bildes gewaehlt
			if (!isset($max_image_size[0]) or empty($max_image_size[0]))
			{
				$max_image_size[0] = $tmp_x;
			}
			//prueft ob eine max hoehe vorgegeben ist wenn nicht wird die hoehe des temp bildes gewaehlt
			if (!isset($max_image_size[1]) or empty($max_image_size[1]))
			{
				$max_image_size[1] = $tmp_x;
			}
			//prueft ob das temp bild zu breit ist
			if ($tmp_x > $max_image_size[0])
			{
				$new_x = $max_image_size[0];
				$new_y = intval($tmp_y * $new_x / $tmp_x);
			}
			else
			{
				$new_x = $tmp_x;
				$new_y = $tmp_y;
			}
			//prueft ob das temp bild zu hoch ist
			if ($new_y > $max_image_size[1])
			{
				$new_y = $max_image_size[1];
				$new_x = intval($tmp_x * $new_y / $tmp_y);
			}
			$return[0] = $new_x;
			$return[1] = $new_y;
			$return[2] = $tmp_x;
			$return[3] = $tmp_y;
			return $return;
		}

		/**
		* prueft ob ein Bild bearbeitet werden muss.
		* @param array $maximagesize, $max_image_size[0] = Max Bildbreite in px. $max_image_size[1] = Max Bildhoehe in px. $max_image_size[2] j = Wenn das Bild zu Gross ist, ist eine verkleinerung ist zwingend erforderlich (eine verkleinerung ist nur bei jpg, gif und png moeglich bei allen anderen Formaten wird der Upload abgebrochen). n = Eine verkleinerung ist nicht zwingend erfprderlich. Die Verkleinerung wird nur bei jpg, gif und png Bildern durchgefuehrt. Andere Bildformate z.B. BMP werden gespeichert auch wenn Sie zu gross sind. Alle Angaben in $max_image_size sind keine Pflichtangaben, jedoch muss bei einer Angabe der Index eingehalten werden.
		* @param array $image, Array von $_FILES['tmp_name']
		* @return array array('0'=>'neue Breite','1'=>'neue Hoehe','2'=>'Temp Breite','3'=>'Temp Hoehe')
		*/
		private function chk_image_size($maximagesize, $image)
		{
			//prueft ob eine aenderung Pflicht ist.
			if (isset($maximagesize[2]) and $maximagesize[2] == 'j')
			{
				$aenderung_erforderlich = 'j';
			}
			else
			{
				$aenderung_erforderlich = 'n';
			}

			//ermittelt die Bildmasse
			$imagesize = @getimagesize($image);

			//max Bildmasse setzten wenn keine vorgegeben
			if (!isset($maximagesize[0]) or empty($maximagesize[0]))
			{
				$maximagesize[0] = $imagesize[0] + 1;
			}
			if (!isset($maximagesize[1]) or empty($maximagesize[1]))
			{
				$maximagesize[1] = $imagesize[1] + 1;
			}

			//$return[0] = True = Upload kann erfolgen, False = Kein Upload
			//$return[1] = TRUE = Bild muss bearbeitet werden, False = Bild muss nicht bearbeitet werden
			//$return[2] = 1= GIF, 2 = JPG, 3 = PNG

			//pruefen ob Bild zu Gross ist
			if ($imagesize[0] > $maximagesize[0] or $imagesize[1] > $maximagesize[1])
			{
				//pruefen ob Bild bearbeitet werden kann (GIF, JPG, PNG)
				if ($imagesize[2] == 1 or $imagesize[2] == 2 or $imagesize[2] == 3)
				{
					$return[0] = TRUE;
					$return[1] = TRUE;
					$return[2] = $imagesize[2];
				}
				//Upload Kann erfolgen keine Bildaenderung
				elseif ($aenderung_erforderlich == 'n')
				{
					$return[0] = TRUE;
					$return[1] = FALSE;
				}
				//keine Upload keine Bildaenderung
				else
				{
					$return[0] = FALSE;
					$return[1] = FALSE;
				}
			}
			//Upload kann erfolgen keine Bildaenderung
			else
			{
				$return[0] = TRUE;
				$return[1] = FALSE;
			}
			return $return;
		}

		/**
		* Loescht Bilder.
		* @param string $tmp_image, Absoluter Pfad mit Bildname
		* @return void
		*/
		private function del_temp_image($tmp_image)
		{
			if (file_exists($tmp_image))
			{
				@unlink($tmp_image);
			}
		}

		/**
		* aendert CHMOD
		* @param string $file, Absoluter Pfad mit Bildname
		* @param string $chmod, CHMOD Wert
		* @return bool
		*/
		private function chk_chmod($file, $chmod)
		{
			$system = trim(strtoupper(PHP_OS));
			if (substr($system, 0, 5) == 'LINUX')
			{
				//aendert die chmod aud Octal wert (4stellig)
				if (@chmod($file, $chmod))
				{
					return TRUE;
				}
				else
				{
					return FALSE;
				}
			}
			else
			{
				return TRUE;
			}
		}

		/**
		* Vergibt und aendert gegebenfalls den Dateinamen
		* @param string $filename, Original Dateiname
		* @param string $new_filename, Neuer Dateiname ohne Dateiendung
		* @return string Neuer Dateiname
		*/
		private function chk_file_name($filename, $new_filename='')
		{
			//pruefen on neuer Dateiname angegeben ist
			if (empty($new_filename))
			{
				//Neuen Dateinamen vergeben
				$new_filename = $filename;
			}
			else
			{
				//Dateiendung des Originalnamens finden
				$file_endung = explode(".", $filename);
				$file_endung = trim(strtolower($file_endung[count($file_endung)-1]));

				//Dateiendung an den neuen Dateinamen anhaengen, falls notwendig
				$temp_strlen = strlen($file_endung) +1;
				if (substr($new_filename, -$temp_strlen) != '.'.$file_endung)
				{
					$new_filename = $new_filename.'.'.$file_endung;
				}
			}
			//Sonderzeichen die im Dateinaemen ersetzt werden
			$search_array = array(' ','²','³','ü','Ü','ö','Ö','ä','Ä','ß','!','"','$','%','&','/','{','(','[',')',']','=','}','?','\\','`','´','*','+','~','\'','#',';',',',':','>','<','|','-','^','°','@');
			$replace_array = array('_','2','3','ue','UE','oe','OE','ae','AE','sz','_','_','_','_','_','_','_','_','_','_','_','_','_','_','_','_','_','_','_','_','_','_','_','_','_','_','_','_','_','_','_','_');

			//Sonderzeichen ersetzten
			$new_filename = str_replace($search_array, $replace_array, $new_filename);

			//doppelte _ entfernen
			$while_count = 0;
			while ($while_count == 0)
			{
				$new_filename = str_replace('__', '_', $new_filename);

				if (substr_count($new_filename, '__') == 0)
				{
					$while_count = 1;
				}
			}
			return $new_filename;
		}

		/**
		* prueft ob der angegebene Pfad existiert und legt ihn gegebenenfalls an.
		* @param string $pfad
		* @return mixed Pfad oder FALSE
		*/
		private function chk_file_pfad($pfad)
		{
			//wenn Pfad leer wird die Datei auf Hauptebene gespeichert
			if (!empty($pfad))
			{
				//pruefen ob Pfad mit / endet
				if ($pfad{strlen($pfad)-1} != '/')
				{
					$pfad = $pfad.'/';
				}
				//pruefen ob Pfad existiert
				if (file_exists($pfad))
				{
					return $pfad;
				}
				else
				{
					//Ordner anlegen
					if (@mkdir($pfad))
					{
						return $pfad;
					}
					else
					{
						return FALSE;
					}
				}
			}
			else
			{
				return './';
			}
		}

		/**
		* prueft den Dateityp.
		* @param string $filename
		* @param array $filetyp, Dateiendungen in einem array alle Dateiendungen klein schreiben
		* @return bool
		*/
		private function chk_filetyp($filename, $filetyp)
		{
			//prueft ob es eine Dateitypbeschraenkung gibt
			if (is_array($filetyp))
			{
				$file_endung = explode(".", $filename);
				$file_endung = trim(strtolower($file_endung[count($file_endung)-1]));
				//Prpft ob Dateiendung im array vorhanden ist
				if (in_array($file_endung, $filetyp))
				{
					return TRUE;
				}
				else
				{
					return FALSE;
				}
			}
			else
			{
				return TRUE;
			}
		}

		/**
		* prueft ob die Datei zu Gross ist (Byte).
		* @param int $filesize, Dateigroesse in Byte
		* @param int $maxfilesize, Max. Dateigroesse in Byte
		* @return bool
		*/
		private function chk_filesize($filesize, $maxfilesize)
		{
			//prueft ob eine max. Dateigroessen Beschraenkung vorhanden ist
			if ($maxfilesize > 0)
			{
				//prueft ob die Datei zu Gross ist
				if ($filesize > $maxfilesize)
				{
					return FALSE;
				}
				else
				{
					return TRUE;
				}
			}
			else
			{
				return TRUE;
			}
		}
	}
?>