<?php
	/**
	 * provides general functions for the version-management
	 * @author spieper
	 * @package time-o-meter
	 *
	 */

	/**
	 * provides general functions for the version-management
	 * @package time-o-meter 
	 * @subpackage classes
	 */
	class VersionControl
	{
		public $setup_array = array();

		/**
		 * Allgemeine Grundeinstellungen laden
		 * @return void
		 */
		public function __construct()
		{
			//Timeout herabsetzen default ist 60
			ini_set('default_socket_timeout', 5);
			//Pfad wur wsdl Datei
			$this->setup_array['wsdl_pfad']			= 'http://www.version.time-o-meter.com/tom_version.wsdl';
			//Aktuelle Version
			$this->setup_array['current_version']	= TOM_VERSION;
		}

		/**
		 * Prueft ob Updates vorhanden sind
		 * @return mixed. true = Updates vorhanden. false = keine Updates vorhanden. 0 = kein Kontakt zum Updateservice.
		 */
		public function chk_updates()
		{
			try
			{
				// Soap-Client initialisieren
				$client = @new SoapClient($this->setup_array['wsdl_pfad']);
				$result = $client->chk_updates($this->setup_array['current_version']);

				//Updates vorhanden
				if ($result == true)
				{
					return true;
				}
				//keine Updates vorhanden
				else
				{
					return false;
				}
			}
			catch(Exception $e)
			{
				return 0;
			}
		}

		/**
		 * Prueft ob zwischen zwei Versionen Aenderungen an der Datenbank erfolgt sind. Wird fuer das wiederherstellen eines Backups verwendet.
		 * @param string $db_version
		 * @return mixed. true = keine Aenderung. false = Aenderungen vorhanden. 0 = kein Kontakt zum Updateservice.
		 */
		public function chk_backup_version($db_version)
		{
			try
			{
				// Soap-Client initialisieren
				$client = @new SoapClient($this->setup_array['wsdl_pfad']);
				$result = $client->chk_backup_version($this->setup_array['current_version'], $db_version);

				//keine Aenderung der Datenbank in den beiden Version. Backup kann verwendet werden.
				if ($result == true)
				{
					return true;
				}
				//Aenderung der Datenbank zwischen den beiden Versionen. Backup nicht verwenden.
				else
				{
					return false;
				}
			}
			catch(Exception $e)
			{
				return 0;
			}
		}
	}
?>