<?php
	/**
	 * error handler / Anwendungsspezifische Fehlermeldung
	 * Die Datei beinhaltet Funktionen die zum anwendungsspezifischen Fehlerlog verwendet werden koennen.
	 * @author Soeren Pieper <soeren.pieper@docemos.de>
	 * @copyright 2007 docemos
	 * @package time-o-meter
	 * @subpackage inc
	 */

	/**
	 * Konstante fuer das Errorlevel
	 */
	define('NOTICE', E_USER_NOTICE);

	/**
	 * Konstante fuer das Errorlevel
	 */
	define('WARNING', E_USER_WARNING);

	/**
	 * Konstante fuer das Errorlevel
	 */
	define('ERROR', E_USER_ERROR);

	$GLOBALS['error_line'] = 0;

	/**
	 * Funktion gibt den Dateinamen (Speicherintervall) fuer Logdateien zurueck udn den Speicherpfad.
	 * @param string $typ
	 * @return string
	 */
	function getErrorFileSetup($typ = '')
	{
		if ($typ == 'pfad')
		{
			//pfad zur logdatei
			return TOM_ABS_PFAD.'files/log/error/';
		}
		else
		{
			//Dateiname fuer die Logdatei. Der String wirde in der Funktion gmdate() verwendet.
			//return 'Ymd';//Taeglich eine neue Datei erstellen.
			//return 'YW';//Pro KW eine neue Datei erstellen.
			return 'Ym';//Pro Monat eine neue Datei erstellen.
			//return 'Y';//Pro Jahr eine neue Datei erstellen.
		}
	}

	/**
	 * Schreibt den Endtag einer Logdatei und setzt diese auf Leserechte.
	 * @return void
	 */
	function closeErrorFiles()
	{
		//Pfad zu den Logdateien
		$pfad_logfolder_string = getErrorFileSetup('pfad');
		//Namenszusammensetzung der Aktuellen Logdateien
		$logfile_current_string = gmdate(getErrorFileSetup()).'.log';

		if ($folder_handle = opendir($pfad_logfolder_string))
		{
			//Alle Logdateien auslesen
			$filenames_array = array();
			while (false !== ($file = readdir($folder_handle)))
			{
				if (is_file($pfad_logfolder_string.$file) and is_writable($pfad_logfolder_string.$file) and $file != $logfile_current_string and substr($file,-3) == 'log')
				{
					$filenames_array[] = $file;
				}
			}
			//pruefen ob Logdateien vorhanden sind
			if (count($filenames_array) > 0)
			{
				for ($i = 0; $i <= count($filenames_array); $i++)
				{
					if ($h = @fopen($pfad_logfolder_string.$filenames_array[$i],'a+'))
					{
						fseek($h,-11,SEEK_END);
						$end_tag_string = fgets($h,1024);

						//Datei wurde bereits abgeschlossen
						if ($end_tag_string == '</errorlog>')
						{
							@fclose($h);
							if (PHP_OS == 'LINUX')
							{
								@chmod($pfad_logfolder_string.$filenames_array[$i],444);
							}
						}
						else
						{
							fseek($h,0,SEEK_END);
							@fwrite($h,'</errorlog>');
							@fclose($h);
							if (PHP_OS == 'LINUX')
							{
								@chmod($pfad_logfolder_string.$filenames_array[$i],444);
							}
						}
					}
				}
			}
		}
	}
	/**
	 * Errorbehandlung
	 * Behandlung von eventuell auftrettenden Fehlern
	 * @param string $error_typ
	 * @param string $error_message
	 * @param string $error_file
	 * @param int $error_line
	 * @return void
	 * @TODO mail() Funktion sollte je nach Projekt angepasst werden
	 */
	function errorHandler($error_typ = NOTICE, $error_message, $error_file, $error_line)
	{
		//Errortypen die Verarbeitet werden koennen
		$errortyp_array = array(NOTICE	=> 'Mitteilung Skript wurde nicht abgebrochen.',
								WARNING	=> 'Warnung Skript wurde nicht abgebrochen.',
								ERROR	=> 'Error Skript wurde abgebrochen.');

		//Fehlertypen die Gespeichert werden sollen
		//$save_errors_array = array(WARNING, ERROR);
		//sollte nur waerend der entwicklung einer Anwendung verwendet werden
		$save_errors_array = array(NOTICE, WARNING, ERROR);

		//Fehlermeldungen im XML Format erfassen
		$error_report_string = "\t<errorreport>\r\n";
		$error_report_string .= "\t\t<time>".utf8_encode(gmdate('YmdHis'))."</time>\r\n";
		$error_report_string .= "\t\t<server>".utf8_encode($_SERVER['SERVER_ADDR'])."</server>\r\n";
		$error_report_string .= "\t\t<phpversion>".PHP_VERSION."</phpversion>\r\n";
		$error_report_string .= "\t\t<os>".PHP_OS."</os>\r\n";
		$error_report_string .= "\t\t<type>".utf8_encode($errortyp_array[$error_typ])."</type>\r\n";
		if (!empty($error_message))
		{
			$error_report_string .= "\t\t<message>".utf8_encode($error_message)."</message>\r\n";
		}
		else
		{
			$error_report_string .= "\t\t<message />\r\n";
		}
		$error_report_string .= "\t\t<file>".utf8_encode($_SERVER['SCRIPT_FILENAME'])."</file>\r\n";
		$error_report_string .= "\t\t<line>".utf8_encode($GLOBALS['error_line'])."</line>\r\n";
		$error_report_string .= "\t</errorreport>\r\n";

		if (in_array($error_typ,$save_errors_array))
		{
			$pfad_logfolder_string = getErrorFileSetup('pfad');
			$logfile_string = getErrorFileSetup();

			$pfad_logfile_string = $pfad_logfolder_string.gmdate($logfile_string).'.log';

			//die angegebene Log Datei existiert noch nicht
			if (!file_exists($pfad_logfile_string))
			{
				if ($log_file_handle = @fopen($pfad_logfile_string,'w'))
				{
					//Achtung der Schliessende Haupttag </logroot> fehlt in den Logdateien er wird erst duch die Function closeErrorFiles() durch einen Cronjob erzeugt
					@fwrite($log_file_handle,'<?xml version="1.0" encoding="utf-8"?>'."\r\n<errorlog>\r\n");
				}
			}
			else
			{
				$log_file_handle = @fopen($pfad_logfile_string,'a');
			}
			if (isset($log_file_handle))
			{
				@fwrite($log_file_handle,$error_report_string);
				@fclose($log_file_handle);
			}

			//gegebenenfalls Fehlermeldung per E-Mail versenden
			$send_mail_bool = true;
			//$send_mail_bool = false;
			if ($send_mail_bool)
			{
				//Bei welchen Fehlertypen soll eine E-Mail versendet werden.
				$send_errors_array = array(ERROR);
				if (in_array($error_typ,$send_errors_array))
				{
					//E-Mail versendet
					//FIXME mail() sollte durch eine Mailklasse ersetzt werden
					$to = 'recipient@domain.de';
					$subject = 'Errormessage';
					//mail($to, $subject, $error_report_string);
				}
			}
		}
	}
	/**
	 * Funktion uebergibt die jeweilige Fehlermeldung an errorHandler()
	 * @param string $error_message
	 * @param int $error_typ
	 * @param int $error_line
	 * @return string
	 */
	function setError($error_message, $error_typ = NOTICE, $error_line)
	{
		$GLOBALS['error_line'] = $error_line;
		//trigger_error($error_message, $error_typ);
		errorHandler($error_typ, $error_message, '', $error_line);
		return $error_message;
	}
	/**
	 * Definition des neuen ErrorHandler
	 */
	//$old_error_handler = set_error_handler('errorHandler');
?>