<?php
	/**
	 * this file contains all includes for the project. it loads required classes and creates instances.
	 * @package time-o-meter
	 * @subpackage inc
	 */

	/**
	 * einbinden der config-datei
	 */
	if (file_exists('config/config.php'))
	{
		require_once('config/config.php');
	}
	elseif (file_exists('../config/config.php'))
	{
		require_once('../config/config.php');
	}
	elseif (file_exists('../../config/config.php'))
	{
		require_once('../../config/config.php');
	}
	elseif (file_exists('../../../config/config.php'))
	{
		require_once('../../../config/config.php');
	}

	define('TOM_GLOBAL_VAR_NAME', substr(strtoupper(md5(TOM_SESSION_NAME)), 0, 8));

	//Errorprotokolle schreiben
	require_once(TOM_ABS_PFAD.'inc/error_handler.php');

	/**
	 * Included Die Klassendateien
	 *
	 * @param string $class
	 */
	function __autoload($class)
	{
		//Mailklasse
		if ($class == 'PHPMailer' and file_exists(TOM_ABS_PFAD.'inc/classes/PHPMailer/class.phpmailer.php'))
		{
			require_once(TOM_ABS_PFAD.'inc/classes/PHPMailer/class.phpmailer.php');
		}
		//Zipklasse
		elseif ($class == 'PclZip' and file_exists(TOM_ABS_PFAD.'inc/classes/zip/pclzip.lib.php'))
		{
			require_once(TOM_ABS_PFAD.'inc/classes/zip/pclzip.lib.php');
		}
		//TCPDF
		elseif ($class == 'TCPDF' and file_exists(TOM_ABS_PFAD.'inc/classes/tcpdf/tcpdf.php'))
		{
			require_once(TOM_ABS_PFAD.'inc/classes/tcpdf/tcpdf.php');
		}
		elseif (file_exists(TOM_ABS_PFAD.'inc/classes/'.$class.'.php'))
		{
			require_once(TOM_ABS_PFAD.'inc/classes/'.$class.'.php');
		}
	}
	//Standardklassen laden
	$cdb = new MySql;

	//Achtung die Klasse GlobalVars() darf mit aussnahme der loginseite und loginklasse nur hier definiert werden
	$gv = new GlobalVars();

	$setup = new Setup;

	require_once(TOM_ABS_PFAD.'inc/get_setup.php');

	//Variablenpruefsummen
	$chk_sum = new Checksum;
	//Globale Variablen pruefen
	$chk_var = new ChkVariable;
	//prueft Automatisch Alle $_POST, $_GET und $_COOKIE Variablen.
	$chk_var->chkGlobals();
	//Kalenderklasse
	$kal = new Kalender;
	//Loginpruefung
	$login = new Login();
	//Reloadklasse
	$reload = new Reload;
	//Zugriffsrechte
	$access = new Access;
	//Filteroptionen Speichern
	$sf = new SaveFilter;

	//Filterdaten aus SESSION entfernen
	if	(isset($_GET['clear_sf']) and $_GET['clear_sf'] == 1)
	{
		$sf->clear();
	}

	/**
	 * Loginpruefung
	 */
	//Nicht beim Cronjob und Loginseite pruefen
	if (!defined('PAGE_AREA') or (PAGE_AREA != 'CJ' and PAGE_AREA != 'LOGIN'))
	{
		//Erster aufruf nach Login
		//Die Action darf nur einmal durchgefuehrt werden dafuer dient $GLOBALS['FOM_VAR']['FOM_COOKIE_CHK']
		if (isset($_GET['action']) and $_GET['action'] == 'login')
		{
			if (isset($GLOBALS['TOM_VAR']['TOM_COOKIE_CHK']) and $GLOBALS['TOM_VAR']['TOM_COOKIE_CHK'] == 1)
			{
				$GLOBALS['TOM_VAR']['TOM_COOKIE_CHK'] = 0;

				//Pruefen ob ein Cookie vorhanden ist
				//Pruefen ob das Cookie den selben inhalt wie in der DB hat
				if (isset($_COOKIE[TOM_SESSION_NAME]) and isset($GLOBALS['TOM_VAR']['TOM_SESSION_COOKIE']) and $_COOKIE[TOM_SESSION_NAME] == $GLOBALS['TOM_VAR']['TOM_SESSION_COOKIE'])
				{
					//ACHTUNG diese Variable darf nicht mehr veraendert werden
					$GLOBALS['TOM_VAR']['TOM_COOKIE_EXISTS'] = 1;
				}
			}
		}

		//Logindaten Pruefen
		if (!$login->chk_login_key($gv->get_key()))
		{
			header('Location: '.TOM_ABS_URL.'index.php');
			exit();
		}
		else
		{
			// Benutzergruppen-ID auslesen
			$sql_groupid = $cdb->select('SELECT group_id FROM hdb_user WHERE user_id='.USER_ID);
			$result_groupid = $cdb->fetch_array($sql_groupid);
			define('USER_GROUP_ID', $result_groupid['group_id']);
		}

		//Logout
		if (isset($_GET['action']) and $_GET['action'] == 'logout')
		{
			$login->logout($gv->get_key());
			header('Location: '.TOM_ABS_URL.'login/index.php?action=logout');
			exit();
		}

		//Wartungsmodus (nur Admins duerfen eingeloggt bleiben)
		if ($setup_array['maintenance_mode'] == 'j' and (!defined('USER_GROUP_ID') or USER_GROUP_ID != 1))
		{
			$login->logout($gv->get_key());
			header("Location: ".TOM_ABS_URL."login/index.php?action=logout");
			exit();
		}
	}


	/**
	* pruefen, ob user-sprache gesetzt ist - sonst default-sprache der db verwenden
	*/
	if (defined('USER_ID'))
	{
		$sql_usersprache = $cdb->select('SELECT sprache_id FROM hdb_user WHERE user_id='.USER_ID);
		$result_usersprache = $cdb->fetch_array($sql_usersprache);

		if (isset($result_usersprache['sprache_id']) && $result_usersprache['sprache_id'] > 0)
		{
			$GLOBALS['user_sprache'] = $result_usersprache['sprache_id'];
		}
		else
		{
			$GLOBALS['user_sprache'] = $setup_array['default_language'];
		}
	}

	/**
	* Included benoetigte job dateien
	*/
	if (isset($_POST['job_string']))
	{
		$meldung_array = array();
		$jobpfad_string = getcwd();
		if (file_exists($jobpfad_string.'/job.php'))
		{
			require_once($jobpfad_string.'/job.php');
		}
		else
		{
			// Die ausfuehrende Job-Datei wurde nicht gefunden!
			$meldung_array['error'][] = setError(get_text('', 'return', 'decode_on', 'fehler') /*Es ist ein Fehler aufgetreten.*/, WARNING, __LINE__);
		}
	}

	require_once(TOM_ABS_PFAD.'inc/function.php');
	require_once(TOM_ABS_PFAD.'inc/menu.php');
?>