<?php
	/**
	 * Webservice für die Authentifizierung von Webservice anfragen
	 * @package time-o-meter
	 * @subpackage webservice
	 */

	define('PAGE_AREA','LOGIN');
	// WSDL Cache abschalten
	ini_set('soap.wsdl_cache_enabled', false);
	ini_set('soap.wsdl_cache_ttl', 10);
	ini_set('soap.wsdl_cache_limit', 1);

	require_once('../inc/include.php');

	/**
	 * Prüft ob zu den Übergebenen Benutzerdaten ein Aktiver Benutzeraccount vorhanden ist
	 *
	 * @param string $pw
	 * @param string $user
	 * @return string
	 */
	function ws_login($pw, $user)
	{
		$sl = new Login();

		return $sl->webservice_login($pw, $user);
	}

	/**
	 * Prüft den Uebergebenen WS Key
	 *
	 * @param string $ws_key
	 * @return string
	 */
	function chk_ws_key($ws_key)
	{
		$sl = new Login();

		return $sl->webservice_key($ws_key);
	}

	//Prüfen ob SoapServer installiert ist
	//siehe http://de3.php.net/manual/de/book.soap.php
	if (class_exists('SoapServer'))
	{
		//Pruefen ob in den grundeinstellungen die Webservice aktiviert sind
		if (isset($setup_array) and isset($setup_array['webservice_tom']) and isset($setup_array['webservice_tom']['aktiv']) and $setup_array['webservice_tom']['aktiv'] == 1)
		{
			//Pruefen das min. eine Funktion aktiviert ist
			if (isset($setup_array['webservice_tom']['functions']) and is_array($setup_array['webservice_tom']['functions']) and count($setup_array['webservice_tom']['functions']) > 0)
			{
				$ip_access = true;

				//IP Adressbeschraenkungen vorhanden
				//IP Adresse Pruefen
				if (isset($setup_array['webservice_tom']['ip']) and is_array($setup_array['webservice_tom']['ip']) and count($setup_array['webservice_tom']['ip']) > 0)
				{
					//keine remote ip
					//kann eigendlich nur der fall sein wenn der uafruf lokal über die konsole erfolgt
					if (isset($_SERVER['REMOTE_ADDR']) and !empty($_SERVER['REMOTE_ADDR']))
					{
						$ip_access = chk_request_ip($_SERVER['REMOTE_ADDR'], $setup_array['webservice_tom']['ip']);
					}
				}

				if ($ip_access === true)
				{
					$function_array = array();
					foreach ($setup_array['webservice_tom']['functions'] as $function => $function_aktiv)
					{
						if ($function_aktiv == 1)
						{
							$function_array[] = $function;
						}
					}

					if (count($function_array) > 0)
					{
						$server = new SoapServer(TOM_ABS_URL.'web_services/access.wsdl');
						$server->addFunction(array('ws_login', 'chk_ws_key'));
						$server->handle();
					}
					else
					{
						setError('Keine Webservicefunctionen in den Einstellungen vorhanden', WARNING, __LINE__);
					}
				}
				else
				{
					setError('Die Remote IP ist nicht in den Webserviceeinstellungen vorhanden', WARNING, __LINE__);
				}
			}
			else
			{
				setError('Keine Webservicefunctionen in den Einstellungen vorhanden', WARNING, __LINE__);
			}
		}
		else
		{
			setError('Keine Webserviceeinstellungen vorhanden', WARNING, __LINE__);
		}
	}
	else
	{
		setError('SOAP Class existiert nicht', WARNING, __LINE__);
	}
?>