<?php
	/**
	 * Webserviceaufruf für die Datenabfrage
	 * @package time-o-meter
	 * @subpackage webservice
	 */
	//FIXME: ist nur fuer den Pfad zur WSDL
	require_once ('../../config/config.php');

	//Klasse fuer Webservice Login
	require_once('client_access.php');

	//Liest Dateien per Webservice aus dem DMS
	class WebServiceTom
	{
		private $setup_array = array();
		private $error;

		public function __construct()
		{
			//ACHTUNG: TOM_ABS_URL kommt aus der config.php
			$this->setup_array['wsdl_pfad'] = TOM_ABS_URL.'web_services/tom.wsdl';
		}

		private function set_error($e)
		{
			$this->error = $e;
		}

		public function get_error()
		{
			return $this->error;
		}

		/**
		 * Gibt einen oder mehrere User aus TOM zurueck
		 * @param int $user_id
		 * @param string $return_type 'array', 'json' oder 'xml'
		 */
		public function get_user($user_id = 0, $return_type = 'xml')
		{
			//Webservice Login
			//ACHTUNG benoetigt client_access.php
			$wsa = new WebServiceLogin();
			$ws_key = $wsa->get_ws_key();
			if ($ws_key !== false)
			{
				try
				{
					// Soap-Client initialisieren
					$client = @new SoapClient($this->setup_array['wsdl_pfad']);
					$result = $client->get_user($ws_key, $user_id, $return_type);
					if (!empty ($result))
					{
						if ($return_type == 'array')
						{
							return @unserialize($result);
						}
						//xml oder json string
						else
						{
							return $result;
						}
					}
					else
					{
						$this->set_error('no result');
						return false;
					}
				}
				catch(Exception $e)
				{
					$this->set_error($e);
					return false;
				}
			}
			else
			{
				$this->set_error('no access');
				return false;
			}
		}

		/**
		 * Gibt ein oder mehrere Projekte zurueck
		 * @param int $project_id
		 * @param string $return_type 'array', 'json' oder 'xml'
		 * @return string
		 */
		public function get_project($project_id = 0, $return_type = 'xml')
		{
			//Webservice Login
			//ACHTUNG benoetigt client_access.php
			$wsa = new WebServiceLogin();
			$ws_key = $wsa->get_ws_key();

			if ($ws_key !== false)
			{
				try
				{
					// Soap-Client initialisieren
					$client = @new SoapClient($this->setup_array['wsdl_pfad']);
					$result = $client->get_project($ws_key, $project_id, $return_type);

					if (!empty ($result))
					{
						if ($return_type == 'array')
						{
							return @unserialize($result);
						}
						//xml oder json string
						else
						{
							return $result;
						}
					}
					else
					{
						$this->set_error('no result');
						return false;
					}
				}
				catch(Exception $e)
				{
					$this->set_error($e);
					return false;
				}
			}
			else
			{
				$this->set_error('no access');
				return false;
			}
		}

		/**
		 * Gibt die Stunden fuer ein oder mehrere Projekte fuer einen Monat oder mehrere Monate zurueck
		 * @param $date im Format YYYY oder YYYY-MM
		 * @param int $project_id
		 * @param string $return_type 'array', 'json' oder 'xml'
		 * @return string
		 */
		public function get_project_hours($date, $project_id = 0, $return_type = 'xml')
		{
			//Webservice Login
			//ACHTUNG benoetigt client_access.php
			$wsa = new WebServiceLogin();
			$ws_key = $wsa->get_ws_key();

			if ($ws_key !== false)
			{
				try
				{
					// Soap-Client initialisieren
					$client = @new SoapClient($this->setup_array['wsdl_pfad']);
					$result = $client->get_project_hours($ws_key, $date, $project_id, $return_type);

					if (!empty ($result))
					{
						if ($return_type == 'array')
						{
							return @unserialize($result);
						}
						//xml oder json string
						else
						{
							return $result;
						}
					}
					else
					{
						$this->set_error('no result');
						return false;
					}
				}
				catch(Exception $e)
				{
					$this->set_error($e);
					return false;
				}
			}
			else
			{
				$this->set_error('no access');
				return false;
			}
		}

		/**
		 * Traegt fuer einen user die Arbeitszeiten für einen Tag ein
		 * @param mixed $user. int oder string. int = user_id aus hdb_user oder username aus hdb_user (Achtung gross kleinschreibung beachten)
		 * @param string $date. Datum im Format YYYY-MM-DD. Wenn der String leer ist wird das aktuelle Datum verwendet
		 * @param string $time. $time kann kommen, pause und gehen enthalten oder nur eine teit und das system entscheidet selbst ob es kommen oder gehen ist. Alle Zeitangaben im Format HH:MM und pause als int Wert. Uebergabemöglichkeiten HH:MM|int|HH:MM oder HH:MM
		 * @return boole
		 */
		public function insert_user_time($user, $date, $time)
		{
			//Webservice Login
			//ACHTUNG benoetigt client_access.php
			$wsa = new WebServiceLogin();
			$ws_key = $wsa->get_ws_key();

			if ($ws_key !== false)
			{
				try
				{
					// Soap-Client initialisieren
					$client = @new SoapClient($this->setup_array['wsdl_pfad']);
					$result = $client->insert_user_time($ws_key, $user, $date, $time);

					if ($result == 'true')
					{
						return true;
					}
					else
					{
						$this->set_error('insert error');
						return false;
					}
				}
				catch(Exception $e)
				{
					$this->set_error($e);
					return false;
				}
			}
			else
			{
				$this->set_error('no access');
				return false;
			}
		}

		public function add_user($title, $first_name, $last_name, $street, $house_number, $zip, $city, $phone, $fax, $mobile_phone, $email, $phone_private, $fax_private, $mobile_phone_private, $email_private, $date_of_birth, $personnel_number, $leave_days, $regular_working_hours_per_week, $maximum_working_hours_per_week, $entry_date, $separation_date, $start_date, $employee_status, $group_id, $username, $pw, $language)
		{
			//Webservice Login
			//ACHTUNG benoetigt client_access.php
			$wsa = new WebServiceLogin();
			$ws_key = $wsa->get_ws_key();

			if ($ws_key !== false)
			{
				try
				{
					// Soap-Client initialisieren
					$client = @new SoapClient($this->setup_array['wsdl_pfad']);
					$result = $client->add_user($ws_key, $title, $first_name, $last_name, $street, $house_number, $zip, $city, $phone, $fax, $mobile_phone, $email, $phone_private, $fax_private, $mobile_phone_private, $email_private, $date_of_birth, $personnel_number, $leave_days, $regular_working_hours_per_week, $maximum_working_hours_per_week, $entry_date, $separation_date, $start_date, $employee_status, $group_id, $username, $pw, $language);

					if ($result > 0)
					{
						return $result;
					}
					else
					{
						$this->set_error('insert error');
						return false;
					}
				}
				catch(Exception $e)
				{
					$this->set_error($e);
					return false;
				}
			}
			else
			{
				$this->set_error('no access');
				return false;
			}
		}

		public function get_language($language_id, $return_type)
		{
			//Webservice Login
			//ACHTUNG benoetigt client_access.php
			$wsa = new WebServiceLogin();
			$ws_key = $wsa->get_ws_key();

			if ($ws_key !== false)
			{
				try
				{
					// Soap-Client initialisieren
					$client = @new SoapClient($this->setup_array['wsdl_pfad']);
					$result = $client->get_language($ws_key, $language_id, $return_type);

					if (!empty ($result))
					{
						if ($return_type == 'array')
						{
							return @unserialize($result);
						}
						//xml oder json string
						else
						{
							return $result;
						}
					}
					else
					{
						$this->set_error('no result');
						return false;
					}
				}
				catch(Exception $e)
				{
					$this->set_error($e);
					return false;
				}
			}
			else
			{
				$this->set_error('no access');
				return false;
			}
		}

		public function get_usergroups($user_group_id, $return_type)
		{
			//Webservice Login
			//ACHTUNG benoetigt client_access.php
			$wsa = new WebServiceLogin();
			$ws_key = $wsa->get_ws_key();

			if ($ws_key !== false)
			{
				try
				{
					// Soap-Client initialisieren
					$client = @new SoapClient($this->setup_array['wsdl_pfad']);
					$result = $client->get_usergroups($ws_key, $user_group_id, $return_type);

					if (!empty ($result))
					{
						if ($return_type == 'array')
						{
							return @unserialize($result);
						}
						//xml oder json string
						else
						{
							return $result;
						}
					}
					else
					{
						$this->set_error('no result');
						return false;
					}
				}
				catch(Exception $e)
				{
					$this->set_error($e);
					return false;
				}
			}
			else
			{
				$this->set_error('no access');
				return false;
			}
		}

		public function get_employee_status($employee_status_id, $return_type)
		{
			//Webservice Login
			//ACHTUNG benoetigt client_access.php
			$wsa = new WebServiceLogin();
			$ws_key = $wsa->get_ws_key();

			if ($ws_key !== false)
			{
				try
				{
					// Soap-Client initialisieren
					$client = @new SoapClient($this->setup_array['wsdl_pfad']);
					$result = $client->get_employee_status($ws_key, $employee_status_id, $return_type);

					if (!empty ($result))
					{
						if ($return_type == 'array')
						{
							return @unserialize($result);
						}
						//xml oder json string
						else
						{
							return $result;
						}
					}
					else
					{
						$this->set_error('no result');
						return false;
					}
				}
				catch(Exception $e)
				{
					$this->set_error($e);
					return false;
				}
			}
			else
			{
				$this->set_error('no access');
				return false;
			}
		}

		public function add_project($project_name, $project_shortname, $client_project_number, $internal_project_number, $order_number, $start_date, $end_date, $project_hours_month, $project_hours_all, $project_message)
		{
			//Webservice Login
			//ACHTUNG benoetigt client_access.php
			$wsa = new WebServiceLogin();
			$ws_key = $wsa->get_ws_key();

			if ($ws_key !== false)
			{
				try
				{
					// Soap-Client initialisieren
					$client = @new SoapClient($this->setup_array['wsdl_pfad']);
					$result = $client->add_project($ws_key, $project_name, $project_shortname, $client_project_number, $internal_project_number, $order_number, $start_date, $end_date, $project_hours_month, $project_hours_all, $project_message);

					if ($result > 0)
					{
						return $result;
					}
					else
					{
						$this->set_error('insert error');
						return false;
					}
				}
				catch(Exception $e)
				{
					$this->set_error($e);
					return false;
				}
			}
			else
			{
				$this->set_error('no access');
				return false;
			}
		}

		public function insert_project_time($user, $project_id, $project_time, $date)
		{
			//Webservice Login
			//ACHTUNG benoetigt client_access.php
			$wsa = new WebServiceLogin();
			$ws_key = $wsa->get_ws_key();

			if ($ws_key !== false)
			{
				try
				{
					// Soap-Client initialisieren
					$client = @new SoapClient($this->setup_array['wsdl_pfad']);
					$result = $client->insert_project_time($ws_key, $user, $project_id, $project_time, $date);

					if ($result == 'true')
					{
						return true;
					}
					else
					{
						$this->set_error('insert error');
						return false;
					}
				}
				catch(Exception $e)
				{
					$this->set_error($e);
					return false;
				}
			}
			else
			{
				$this->set_error('no access');
				return false;
			}
		}

		public function close_timesheet($user, $date)
		{
			//Webservice Login
			//ACHTUNG benoetigt client_access.php
			$wsa = new WebServiceLogin();
			$ws_key = $wsa->get_ws_key();

			if ($ws_key !== false)
			{
				try
				{
					// Soap-Client initialisieren
					$client = @new SoapClient($this->setup_array['wsdl_pfad']);
					$result = $client->close_timesheet($ws_key, $user, $date);

					if ($result == 'true')
					{
						return true;
					}
					else
					{
						$this->set_error('insert error');
						return false;
					}
				}
				catch(Exception $e)
				{
					$this->set_error($e);
					return false;
				}
			}
			else
			{
				$this->set_error('no access');
				return false;
			}
		}

		public function get_user_project_assignment($user, $project_id, $date, $return_type)
		{
			//Webservice Login
			//ACHTUNG benoetigt client_access.php
			$wsa = new WebServiceLogin();
			$ws_key = $wsa->get_ws_key();

			if ($ws_key !== false)
			{
				try
				{
					// Soap-Client initialisieren
					$client = @new SoapClient($this->setup_array['wsdl_pfad']);
					$result = $client->get_user_project_assignment($ws_key, $user, $project_id, $date, $return_type);

					if (!empty ($result))
					{
						if ($return_type == 'array')
						{
							return @unserialize($result);
						}
						//xml oder json string
						else
						{
							return $result;
						}
					}
					else
					{
						$this->set_error('no result');
						return false;
					}
				}
				catch(Exception $e)
				{
					$this->set_error($e);
					return false;
				}
			}
			else
			{
				$this->set_error('no access');
				return false;
			}
		}

		public function add_user_project_assignment($user, $project_id)
		{
			//Webservice Login
			//ACHTUNG benoetigt client_access.php
			$wsa = new WebServiceLogin();
			$ws_key = $wsa->get_ws_key();

			if ($ws_key !== false)
			{
				try
				{
					// Soap-Client initialisieren
					$client = @new SoapClient($this->setup_array['wsdl_pfad']);
					$result = $client->add_user_project_assignment($ws_key, $user, $project_id);

					if ($result > 0)
					{
						return $result;
					}
					else
					{
						$this->set_error('insert error');
						return false;
					}
				}
				catch(Exception $e)
				{
					$this->set_error($e);
					return false;
				}
			}
			else
			{
				$this->set_error('no access');
				return false;
			}
		}
	}

	//$wst = new WebServiceTom();

	/*
	if ($upa_result = $wst->add_user_project_assignment(32, 136))
	{
		echo '<pre>';
			print_r($upa_result);
		echo '</pre>';
	}
	else
	{
		echo '<pre>';
			print_r($wst->get_error());
		echo '</pre>';
	}
	*/

	/*
	//if ($upa_result = $wst->get_user_project_assignment(32, 0, '2010-07-01', 'array'))
	//if ($upa_result = $wst->get_user_project_assignment(32, 0, '', 'array'))
	//if ($upa_result = $wst->get_user_project_assignment(0, 106, '', 'array'))
	if ($upa_result = $wst->get_user_project_assignment(32, 106, '', 'array'))
	{
		echo '<pre>';
			print_r($upa_result);
		echo '</pre>';
	}
	else
	{
		echo '<pre>';
			print_r($wst->get_error());
		echo '</pre>';
	}
	*/

	/*
	if ($ct_result = $wst->close_timesheet(33, '2010-07-01'))
	{
		echo '<pre>';
			print_r($ct_result);
		echo '</pre>';
	}
	else
	{
		echo '<pre>';
			print_r($wst->get_error());
		echo '</pre>';
	}
	*/

	/*
	//if ($pt_result = $wst->insert_project_time(32, 106, 4, '2010-07-01'))
	if ($pt_result = $wst->insert_project_time(32, 106, '06:00-30-10:30', '2010-07-01'))
	{
		echo '<pre>';
			print_r($pt_result);
		echo '</pre>';
	}
	else
	{
		echo '<pre>';
			print_r($wst->get_error());
		echo '</pre>';
	}
	*/

	/*
	if ($p_result = $wst->add_project('WSDL Projekt', 'WSDL', 'c1', 'i1', 'o1', '2010-07-01', '2010-12-31', '50', '0', '70'))
	{
		echo '<pre>';
			print_r($p_result);
		echo '</pre>';
	}
	else
	{
		echo '<pre>';
			print_r($wst->get_error());
		echo '</pre>';
	}
	*/

	/*
	if ($es_result = $wst->get_employee_status(0, 'array'))
	{
		echo '<pre>';
			print_r($es_result);
		echo '</pre>';
	}
	else
	{
		echo '<pre>';
			print_r($wst->get_error());
		echo '</pre>';
	}
	*/

	/*
	if ($ug_result = $wst->get_usergroups(0, 'array'))
	{
		echo '<pre>';
			print_r($ug_result);
		echo '</pre>';
	}
	else
	{
		echo '<pre>';
			print_r($wst->get_error());
		echo '</pre>';
	}
	*/

	/*
	if ($lng_result = $wst->get_language(0, 'array'))
	{
		echo '<pre>';
			print_r($lng_result);
		echo '</pre>';
	}
	else
	{
		echo '<pre>';
			print_r($wst->get_error());
		echo '</pre>';
	}
	*/

	/*
	if ($user_result = $wst->add_user(1, 'firstname', 'lastname', 'street', '5', '01109', 'DD', '555 - 555', '555 - 556', '555 - 557', 'c@mydomain.com', '555 - 558', '555 - 559', '555 - 560', 'p@mydomain.com', '1979-02-11', '001', '25', '40', '40', '2010-07-01', '', '2010-07-01', '', '1', 'flastname', 'secret', '1'))
	{
		echo '<pre>';
			print_r($user_result);
		echo '</pre>';
	}
	else
	{
		echo '<pre>';
			print_r($wst->get_error());
		echo '</pre>';
	}
	*/

	/*
	if ($user_result = $wst->get_user(0, 'array'))
	{
		echo '<pre>';
			print_r($user_result);
		echo '</pre>';
	}
	else
	{
		echo '<pre>';
			print_r($wst->get_error());
		echo '</pre>';
	}
	*/

	/*
	if ($pro_result = $wst->get_project(0, 'array'))
	{
		echo '<pre>';
			print_r($pro_result);
		echo '</pre>';
	}
	else
	{
		echo '<pre>';
			print_r($wst->get_error());
		echo '</pre>';
	}
	*/
?>