<?php
	/**
	 * Webservice für die Datenbereitstellung von Webservice anfragen
	 * @package time-o-meter
	 * @subpackage webservice
	 */

	define('PAGE_AREA','LOGIN');
	// WSDL Cache abschalten
	ini_set('soap.wsdl_cache_enabled', false);
	ini_set('soap.wsdl_cache_ttl', 10);
	ini_set('soap.wsdl_cache_limit', 1);

	require_once('../inc/include.php');

	/**
	 * Wandelt ein Array in einen XML String
	 * @param array $data_array
	 * @param string $tag_name
	 * @param boole $first_call
	 * @return string
	 */
	function array_to_xml($data_array, $tag_name, $first_call = true)
	{
		$xml_string = '';

		if ($first_call == false)
		{
			$xml_string .= "<$tag_name>\n";
		}

		foreach ($data_array as $index => $data)
		{
			if (is_array($data))
			{
				$xml_string .= array_to_xml($data, $tag_name, false);
			}
			else
			{
				$xml_string .= "\t<$index>$data</$index>\n";
			}
		}

		if ($first_call == false)
		{
			$xml_string .= "</$tag_name>\n";
		}

		if ($first_call == true)
		{
			$xml_string = '<?xml version="1.0" encoding="utf-8"?><root>'.$xml_string.'</root>';
		}
		return $xml_string;
	}

	/**
	 * Legt einen neuen User im TOM an
	 * @param string 	$ws_key
	 * @param int 		$title,							0==empty, 1==Mr., 2==Mrs.
	 * @param string	$first_name,					* max length 50
	 * @param string	$last_name, 					* max length 50
	 * @param string	$street, 						max length 75
	 * @param string	$house_number, 					max length 5
	 * @param string	$zip, 							max length 5
	 * @param string	$city, 							max length 50
	 * @param string	$phone, 						max length 25
	 * @param string	$fax, 							max length 25
	 * @param string	$mobile_phone, 					max length 25
	 * @param string	$email, 						* max length 75
	 * @param string	$phone_private, 				max length 25
	 * @param string	$fax_private, 					max length 25
	 * @param string	$mobile_phone_private, 			max length 25
	 * @param string	$email_private,					max length 75
	 * @param string	$date_of_birth, 				* date('Y-m-d') 2010-12-31
	 * @param string	$personnel_number, 				max length 50
	 * @param int		$leave_days, 					* max 255
	 * @param float		$regular_working_hours_per_week *
	 * @param float		$maximum_working_hours_per_week *
	 * @param string	$entry_date, 					date('Y-m-d') 2010-12-31
	 * @param string	$separation_date, 				date('Y-m-d') 2010-12-31
	 * @param string	$start_date, 					* date('Y-m-d') 2010-12-31
	 * @param int		$employee_status				see get_employee_status()
	 * @param int		$group_id						* see get_usergroups()
	 * @param string	$username						*
	 * @param string	$pw								*
	 * @param int		$language						* get_language()
	 * @return int										0=error or user_id from hdb_user
	 */
	function add_user($ws_key, $title, $first_name, $last_name, $street, $house_number, $zip, $city, $phone, $fax, $mobile_phone, $email, $phone_private, $fax_private, $mobile_phone_private, $email_private, $date_of_birth, $personnel_number, $leave_days, $regular_working_hours_per_week, $maximum_working_hours_per_week, $entry_date, $separation_date, $start_date, $employee_status, $group_id, $username, $pw, $language)
	{
		if (!empty($ws_key))
		{
			$cdb = new MySql;
			$sl = new Login();

			//ws_key ok
			if ($sl->webservice_key($ws_key) == 'true')
			{
				$access = new Access();
				//Zugriffsrechte
				if ($access->check_group(USER_GROUP_ID, 'admin_user_write'))
				{
					//require values
					if (!empty($first_name) and !empty($last_name) and !empty($email) and !empty($date_of_birth) and !empty($leave_days) and !empty($regular_working_hours_per_week) and !empty($maximum_working_hours_per_week) and !empty($start_date) and !empty($group_id) and !empty($username) and !empty($pw) and !empty($language))
					{
						$kal = new Kalender();

						//check dates
						$date_error = false;
						if ($date_of_birth == '0000-00-00' or $kal->is_iso_date($date_of_birth) == false)
						{
							$date_error = true;
						}

						if ($start_date == '0000-00-00' or $kal->is_iso_date($start_date) == false)
						{
							$date_error = true;
						}

						if (empty($entry_date))
						{
							$entry_date = '0000-00-00';
						}
						elseif ($kal->is_iso_date($entry_date) == false)
						{
							$date_error = true;
						}

						if (empty($separation_date))
						{
							$separation_date = '0000-00-00';
						}
						elseif ($kal->is_iso_date($separation_date) == false)
						{
							$date_error = true;
						}

						//no date errors
						if ($date_error == false)
						{
							$chk_v = new ChkVariable();

							if ($title < 0 or $title > 2)
							{
								$title = 0;
							}

							$first_name = $chk_v->modifyString($first_name);
							$last_name = $chk_v->modifyString($last_name);
							if (!empty($street))
							{
								$street = $chk_v->modifyString($street);
							}
							if (!empty($house_number))
							{
								$house_number = $chk_v->modifyString($house_number);
							}
							if (!empty($zip))
							{
								$zip = $chk_v->modifyString($zip);
							}
							if (!empty($city))
							{
								$city = $chk_v->modifyString($city);
							}
							if (!empty($phone))
							{
								$phone = $chk_v->modifyString($phone);
							}
							if (!empty($fax))
							{
								$fax = $chk_v->modifyString($fax);
							}
							if (!empty($mobile_phone))
							{
								$mobile_phone = $chk_v->modifyString($mobile_phone);
							}
							if (!empty($email))
							{
								$email = $chk_v->modifyString($email);
							}
							if (!empty($phone_private))
							{
								$phone_private = $chk_v->modifyString($phone_private);
							}
							if (!empty($fax_private))
							{
								$fax_private = $chk_v->modifyString($fax_private);
							}
							if (!empty($mobile_phone_private))
							{
								$mobile_phone_private = $chk_v->modifyString($mobile_phone_private);
							}
							if (!empty($email_private))
							{
								$email_private = $chk_v->modifyString($email_private);
							}
							if (!empty($personnel_number))
							{
								$personnel_number = $chk_v->modifyString($personnel_number);
							}
							if (!empty($username))
							{
								$username = $chk_v->modifyString($username);
							}
							if (!empty($pw))
							{
								$pw = $chk_v->modifyString($pw);
							}

							if (!is_numeric($leave_days))
							{
								$leave_days = 0;
							}
							if (!is_numeric($regular_working_hours_per_week))
							{
								$regular_working_hours_per_week = 0;
							}
							else
							{
								$regular_working_hours_per_week = round($regular_working_hours_per_week, 2);
							}
							if (!is_numeric($maximum_working_hours_per_week))
							{
								$maximum_working_hours_per_week = 0;
							}
							else
							{
								$maximum_working_hours_per_week = round($maximum_working_hours_per_week, 2);
							}
							if (!is_numeric($employee_status))
							{
								$employee_status = 0;
							}
							if (!is_numeric($language))
							{
								$language = 1;
							}

							$sort_name_string = $first_name.', '.$last_name;
							$sort_name_string = str_replace('&Auml;', 'A', $sort_name_string);
							$sort_name_string = str_replace('&Ouml;', 'O', $sort_name_string);
							$sort_name_string = str_replace('&Uuml;', 'U', $sort_name_string);
							$sort_name_string = str_replace('&auml;', 'a', $sort_name_string);
							$sort_name_string = str_replace('&uuml;', 'u', $sort_name_string);
							$sort_name_string = str_replace('&ouml;', 'o', $sort_name_string);
							$sort_name_string = str_replace('&szlig;', 'sz', $sort_name_string);

							if (is_numeric($group_id))
							{
								$sql = $cdb->select('SELECT group_id FROM hdb_usergroups WHERE group_id='.$group_id);
								$result = $cdb->fetch_array($sql);

								if (isset($result['group_id']) and $result['group_id'] > 0)
								{
									$sql = $cdb->select("SELECT user_id FROM hdb_user WHERE LOWER(username)='".strtolower($username)."'");
									$result = $cdb->fetch_array($sql);

									if (!isset($result['user_id']) or empty($result['user_id']))
									{
										$cp = new CryptPw();
										$pw = $cp->encode_pw($pw);
										if ($cdb->insert("INSERT INTO hdb_user (anrede, vorname, nachname, sort_name, str, hausnr, plz, ort, tel, fax, handy, email, tel_private, fax_private, handy_private, email_private, geburtsdatum, personalnr, urlaubstage, min_weektime, max_weektime, eintrittsdatum, austrittsdatum, startdatum, mastatus_id, group_id, username, pw, login_aktiv, show_rei_ko, sprache_id) VALUES ($title,'$first_name','$last_name','".$sort_name_string."','$street','$house_number','$zip','$city','$phone','$fax','$mobile_phone','$email','$phone_private','$fax_private','$mobile_phone_private','$email_private','$date_of_birth','$personnel_number','$leave_days','$regular_working_hours_per_week','$maximum_working_hours_per_week','$entry_date','$separation_date','$start_date','$employee_status','$group_id','$username','$pw','j','n','$language')"))
										{
											$last_id = $cdb->get_last_insert_id();
											if ($last_id > 0)
											{
												$ts = new TimeSheet();
												//Leeren Stundenzettel zum Startdatum anlegen
												if ($ts->insert_new_time_sheet($last_id))
												{
													return $last_id;
												}
											}
										}
									}
								}
							}
						}
					}
				}
			}
		}
		return 0;
	}

	/**
	 * Gibt einen oder mehrere User aus TOM zurueck
	 * @param string $ws_key Login Key von access.php
	 * @param int $user_id
	 * @param string $return_type 'json' oder 'xml'
	 */
	function get_user($ws_key, $user_id = 0, $return_type = 'xml')
	{
		if (!empty($ws_key))
		{
			$cdb = new MySql;
			$sl = new Login();

			//ws_key ok
			if ($sl->webservice_key($ws_key) == 'true')
			{
				$access = new Access();
				//Zugriffsrechte
				if ($access->check_group(USER_GROUP_ID, 'admin_user_read'))
				{
					if ($user_id > 0)
					{
						$where = "WHERE user_id=$user_id AND anzeigen='j'";
					}
					else
					{
						$where = "WHERE anzeigen='j'";
					}

					$return_array = array();
					$sql = $cdb->select("SELECT user_id, vorname, nachname, personalnr FROM hdb_user $where ORDER BY sort_name ASC");
					while ($result = $cdb->fetch_array($sql))
					{
						$return_array[] = array('user_id' 			=> mb_convert_encoding($result['user_id'], 'UTF-8'),
												'first_name'		=> mb_convert_encoding(html_entity_decode($result['vorname'], ENT_QUOTES), 'UTF-8'),
												'last_name'			=> mb_convert_encoding(html_entity_decode($result['nachname'], ENT_QUOTES), 'UTF-8'),
												'personnel_number'	=> mb_convert_encoding(html_entity_decode($result['personalnr'], ENT_QUOTES), 'UTF-8'));
					}

					if (count($return_array) > 0)
					{
						if ($return_type == 'array')
						{
							return @serialize($return_array);
						}
						//json string
						elseif ($return_type == 'json')
						{
							return json_encode($return_array);
						}
						//xml string
						else
						{
							return array_to_xml($return_array, 'user');
						}
					}
					else
					{
						return '';
					}
				}
				else
				{
					return '';
				}
			}
			else
			{
				return '';
			}
		}
		else
		{
			return '';
		}
	}

	/**
	 * Gibt eine oder alle Sprachen zurueck
	 * @param string $ws_key
	 * @param int $language_id
	 * @param string $return_type
	 * @return string
	 */
	function get_language($ws_key, $language_id = 0, $return_type = 'xml')
	{
		if (!empty($ws_key))
		{
			$cdb = new MySql;
			$sl = new Login();

			//ws_key ok
			if ($sl->webservice_key($ws_key) == 'true')
			{
				$access = new Access();
				//Zugriffsrechte
				if ($access->check_group(USER_GROUP_ID, 'admin_setup'))
				{
					if ($user_group_id > 0)
					{
						$where = ' WHERE sprache_id='.$language_id." AND anzeigen='j'";
					}
					else
					{
						$where = " WHERE anzeigen='j'";
					}

					$return_array = array();
					$sql = $cdb->select('SELECT sprache_id, bezeichnung FROM hdb_sprachen'.$where );
					while ($result = $cdb->fetch_array($sql))
					{
						$return_array[] = $result;
					}

					if (count($return_array) > 0)
					{
						if ($return_type == 'array')
						{
							return @serialize($return_array);
						}
						//json string
						elseif ($return_type == 'json')
						{
							return json_encode($return_array);
						}
						//xml string
						else
						{
							return array_to_xml($return_array, 'usergroups');
						}
					}
					else
					{
						return '';
					}
				}
				else
				{
					return '';
				}
			}
			else
			{
				return '';
			}
		}
		else
		{
			return '';
		}
	}

	/**
	 * Gibt eine oder alle Usergruppen zurueck
	 * @param string $ws_key
	 * @param int $user_group_id
	 * @param string $return_type
	 * @return string
	 */
	function get_usergroups($ws_key, $user_group_id = 0, $return_type = 'xml')
	{
		if (!empty($ws_key))
		{
			$cdb = new MySql;
			$sl = new Login();

			//ws_key ok
			if ($sl->webservice_key($ws_key) == 'true')
			{
				$access = new Access();
				//Zugriffsrechte
				if ($access->check_group(USER_GROUP_ID, 'admin_setup'))
				{
					$where = '';
					if ($user_group_id > 0)
					{
						$where = ' WHERE group_id='.$user_group_id;
					}

					$return_array = array();
					$sql = $cdb->select('SELECT group_id, group_name FROM hdb_usergroups'.$where );
					while ($result = $cdb->fetch_array($sql))
					{
						$return_array[] = $result;
					}

					if (count($return_array) > 0)
					{
						if ($return_type == 'array')
						{
							return @serialize($return_array);
						}
						//json string
						elseif ($return_type == 'json')
						{
							return json_encode($return_array);
						}
						//xml string
						else
						{
							return array_to_xml($return_array, 'usergroups');
						}
					}
					else
					{
						return '';
					}
				}
				else
				{
					return '';
				}
			}
			else
			{
				return '';
			}
		}
		else
		{
			return '';
		}
	}

	/**
	 * Gibt einen oder alle Mitarbeiterstatus zurueck
	 * @param string $ws_key
	 * @param int $employee_status_id
	 * @param string $return_type
	 * @return string
	 */
	function get_employee_status($ws_key, $employee_status_id = 0, $return_type = 'xml')
	{
		if (!empty($ws_key))
		{
			$cdb = new MySql;
			$sl = new Login();

			//ws_key ok
			if ($sl->webservice_key($ws_key) == 'true')
			{
				$access = new Access();
				//Zugriffsrechte
				if ($access->check_group(USER_GROUP_ID, 'admin_setup'))
				{
					$where = '';
					if ($employee_status_id > 0)
					{
						$where = ' WHERE mastatus_id='.$employee_status_id;
					}

					$return_array = array();
					$sql = $cdb->select('SELECT * FROM hdb_ma_status'.$where );
					while ($result = $cdb->fetch_array($sql))
					{
						$return_array[] = $result;
					}

					if (count($return_array) > 0)
					{
						if ($return_type == 'array')
						{
							return @serialize($return_array);
						}
						//json string
						elseif ($return_type == 'json')
						{
							return json_encode($return_array);
						}
						//xml string
						else
						{
							return array_to_xml($return_array, 'employee');
						}
					}
					else
					{
						return '';
					}
				}
				else
				{
					return '';
				}
			}
			else
			{
				return '';
			}
		}
		else
		{
			return '';
		}
	}

	/**
	 * Erstellt ei neues Projekt
	 * @param string	$ws_key
	 * @param string	$project_name				* max length 75
	 * @param string	$project_shortname			* max length 10
	 * @param string	$client_project_number		max length 25
	 * @param string	$internal_project_number	max length 10
	 * @param string	$order_number				max length 25
	 * @param string	$start_date					* date('Y-m-d') 2010-12-31
	 * @param string	$end_date					* date('Y-m-d') 2010-12-31
	 * @param int		$project_hours_month
	 * @param int		$project_hours_all
	 * @param int		$project_message			0 or 10 or 20 or 30 or 40 or 50 or 60 or 70 or 80 or 90 or 100
	 * @return int		0 == error or project_id from hdb_projekte
	 */
	function add_project($ws_key, $project_name, $project_shortname, $client_project_number, $internal_project_number, $order_number, $start_date, $end_date, $project_hours_month, $project_hours_all, $project_message)
	{
		if (!empty($ws_key))
		{
			$cdb = new MySql;
			$sl = new Login();

			//ws_key ok
			if ($sl->webservice_key($ws_key) == 'true')
			{
				$access = new Access();
				//Zugriffsrechte
				if ($access->check_group(USER_GROUP_ID, 'admin_project_write'))
				{
					if (!empty($project_name) and !empty($project_shortname) and !empty($start_date) and !empty($end_date))
					{
						$kal = new Kalender();

						//check dates
						$date_error = false;
						if ($start_date == '0000-00-00' or $kal->is_iso_date($start_date) == false)
						{
							$date_error = true;
						}
						if ($end_date == '0000-00-00' or $kal->is_iso_date($end_date) == false)
						{
							$date_error = true;
						}

						if ($date_error == false)
						{
							$chk_v = new ChkVariable();

							$project_name = $chk_v->modifyString($project_name);
							$project_shortname = $chk_v->modifyString($project_shortname);
							$client_project_number = $chk_v->modifyString($client_project_number);
							$internal_project_number = $chk_v->modifyString($internal_project_number);
							$order_number = $chk_v->modifyString($order_number);

							if (is_numeric($project_hours_month) and $project_hours_month > 0)
							{
								$project_hours_month = intval($project_hours_month);
								if ($project_hours_month < 0)
								{
									$project_hours_month = 0;
								}
								$project_hours_all = 0;
							}
							elseif (is_numeric($project_hours_all) and $project_hours_all > 0)
							{
								$project_hours_all = intval($project_hours_all);
								if ($project_hours_all < 0)
								{
									$project_hours_all = 0;
								}
								$project_hours_month = 0;
							}
							else
							{
								$project_hours_all = 0;
								$project_hours_month = 0;
							}

							if ($project_hours_all == 0 and $project_hours_month == 0)
							{
								$project_message = 0;
							}
							else
							{
								$true_int_array = array(0, 10, 20, 30, 40, 50, 60, 70, 80, 90, 100);
								if (!in_array($project_message, $true_int_array))
								{
									$project_message = 0;
								}
							}
							if ($cdb->insert("INSERT INTO hdb_projekte (name, kurzname, nr_kunde, nr_intern, nr_bestell, start_datum, end_datum, ps_monat, ps_gesamt, ps_nachricht) VALUES ('$project_name', '$project_shortname', '$client_project_number', '$internal_project_number', '$order_number', '$start_date', '$end_date', '$project_hours_month', '$project_hours_all', '$project_message')"))
							{
								$last_id = $cdb->get_last_insert_id();
								if ($last_id > 0)
								{
									return $last_id;
								}
							}
						}
					}
				}
			}
		}
		return 0;
	}

	/**
	 * Gibt ein oder mehrere Projekte zurueck
	 * @param string $ws_key Login Key von access.php
	 * @param int $project_id
	 * @param string $return_type 'json' oder 'xml'
	 * @return string
	 */
	function get_project($ws_key, $project_id = 0, $return_type = 'xml')
	{
		if (!empty($ws_key))
		{
			$cdb = new MySql;
			$sl = new Login();

			//ws_key ok
			if ($sl->webservice_key($ws_key) == 'true')
			{
				$access = new Access();
				//Zugriffsrechte
				if ($access->check_group(USER_GROUP_ID, 'admin_project_read'))
				{
					if ($project_id > 0)
					{
						$where = "WHERE projekt_id=$project_id AND anzeigen='j'";
					}
					else
					{
						$where = "WHERE anzeigen='j'";
					}

					$return_array = array();
					$sql = $cdb->select("SELECT projekt_id, name, kurzname, nr_kunde, nr_intern, nr_bestell, start_datum, end_datum, ps_monat, ps_gesamt FROM hdb_projekte $where ORDER BY name ASC, kurzname ASC");
					while ($result = $cdb->fetch_array($sql))
					{
						$return_array[] = array('project_id' 			=> mb_convert_encoding($result['projekt_id'], 'UTF-8'),
												'project_name'			=> mb_convert_encoding(html_entity_decode($result['name'], ENT_QUOTES), 'UTF-8'),
												'nickname'				=> mb_convert_encoding(html_entity_decode($result['kurzname'], ENT_QUOTES), 'UTF-8'),
												'customer_number'		=> mb_convert_encoding(html_entity_decode($result['nr_kunde'], ENT_QUOTES), 'UTF-8'),
												'internal_number'		=> mb_convert_encoding(html_entity_decode($result['nr_intern'], ENT_QUOTES), 'UTF-8'),
												'order_number'			=> mb_convert_encoding(html_entity_decode($result['nr_bestell'], ENT_QUOTES), 'UTF-8'),
												'start_date'			=> mb_convert_encoding(html_entity_decode($result['start_datum'], ENT_QUOTES), 'UTF-8'),
												'end_date'				=> mb_convert_encoding(html_entity_decode($result['end_datum'], ENT_QUOTES), 'UTF-8'),
												'project_hours_month'	=> mb_convert_encoding(html_entity_decode($result['ps_monat'], ENT_QUOTES), 'UTF-8'),
												'project_hours_all'		=> mb_convert_encoding(html_entity_decode($result['ps_gesamt'], ENT_QUOTES), 'UTF-8'));
					}

					if (count($return_array) > 0)
					{
						if ($return_type == 'array')
						{
							return @serialize($return_array);
						}
						//json string
						elseif ($return_type == 'json')
						{
							return json_encode($return_array);
						}
						//xml string
						else
						{
							return array_to_xml($return_array, 'project');
						}
					}
					else
					{
						return '';
					}
				}
				else
				{

				}
			}
			else
			{
				return '';
			}
		}
		else
		{
			return '';
		}
	}

	/**
	 * Gibt die Stunden fuer ein oder mehrere Projekte fuer einen Monat oder mehrere Monate zurueck
	 * @param string $ws_key Login Key von access.php
	 * @param $date im Format YYYY oder YYYY-MM
	 * @param int $project_id
	 * @param string $return_type 'json' oder 'xml'
	 * @return string
	 */
	function get_project_hours($ws_key, $date, $project_id = 0, $return_type = 'xml')
	{
		if (!empty($ws_key))
		{
			$cdb = new MySql;
			$sl = new Login();

			//ws_key ok
			if ($sl->webservice_key($ws_key) == 'true')
			{
				$access = new Access();
				//Zugriffsrechte
				if ($access->check_group(USER_GROUP_ID, 'admin_project_read') and $access->check_group(USER_GROUP_ID, 'admin_reports_read'))
				{
					$tsr = new TimeSheetReport;
					$result_array = $tsr->report_project_hours($date, $project_id, 'array');

					$return_array = array();

					foreach ($result_array as $data_array)
					{
						for ($i = 0; $i < count($data_array); $i++)
						{
							$return_array[] = array('project_id'	=> mb_convert_encoding($data_array[$i]['pid'], 'UTF-8'),
													'project_name'	=> mb_convert_encoding(html_entity_decode($data_array[$i]['p_name'], ENT_QUOTES), 'UTF-8'),
													'date'			=> mb_convert_encoding($data_array[$i]['month'], 'UTF-8'),
													'user_id'		=> mb_convert_encoding($data_array[$i]['user_id'], 'UTF-8'),
													'user_name'		=> mb_convert_encoding(html_entity_decode($data_array[$i]['user_name'], ENT_QUOTES), 'UTF-8'),
													'hour'			=> mb_convert_encoding($data_array[$i]['hour'], 'UTF-8'));
						}
					}

					if (count($return_array) > 0)
					{
						if ($return_type == 'array')
						{
							return @serialize($return_array);
						}
						//json string
						elseif ($return_type == 'json')
						{
							return json_encode($return_array);
						}
						//xml string
						else
						{
							return array_to_xml($return_array, 'project_hours');
						}
					}
					else
					{
						return '';
					}
				}
				else
				{
					return '';
				}
			}
			else
			{
				return '';
			}
		}
		else
		{
			return '';
		}
	}

	/**
	 * Traegt fuer einen user die Arbeitszeiten für einen Tag ein
	 * @param string $ws_key. Login Key von access.php
	 * @param mixed $user. int oder string. int = user_id aus hdb_user oder username aus hdb_user (Achtung gross kleinschreibung beachten)
	 * @param string $date. Datum im Format YYYY-MM-DD. Wenn der String leer ist wird das aktuelle Datum verwendet
	 * @param string $time. $time kann kommen, pause und gehen enthalten oder nur eine teit und das system entscheidet selbst ob es kommen oder gehen ist. Alle Zeitangaben im Format HH:MM und pause als int Wert. Uebergabemöglichkeiten HH:MM-int-HH:MM oder HH:MM
	 * @return string
	 */
	function insert_user_time($ws_key, $user, $date, $time)
	{
		if (!empty($ws_key))
		{
			$cdb = new MySql;
			$sl = new Login();
			$kal = new Kalender();
			$tss = new TimeSheetSave();

			$insert_array = array();

			//ws_key ok
			if ($sl->webservice_key($ws_key) == 'true')
			{
				//user finden
				if (is_numeric($user))
				{
					//User anhand der id finden
					$sql = $cdb->select('SELECT user_id, min_weektime, max_weektime, arbeitstage FROM hdb_user WHERE user_id='.$user);
				}
				else
				{
					//user anhand des usernamens finden
					$sql = $cdb->select("SELECT user_id, min_weektime, max_weektime, arbeitstage FROM hdb_user WHERE username='$user'");
				}
				$user_result = $cdb->fetch_array($sql);

				//user gefunden
				if (isset($user_result['user_id']) and $user_result['user_id'] > 0)
				{
					if ($user_result['user_id'] == USER_ID)
					{
						$stz_access = true;
					}
					else
					{
						$stz_access = $access->check_group(USER_GROUP_ID, 'admin_stz');
					}

					if ($stz_access)
					{
						//user id in insertarray speichern
						$insert_array['user_id'] = $user_result['user_id'];

						//kein datum uebergeben. aktuelles nehmen
						if (empty($date))
						{
							$insert_array['date'] = date('Y-m-d');
						}
						//datum uebergeben. pruefen ob dies ein ISO datum ist
						elseif (!empty($date) and $kal->is_iso_date($date) !== false)
						{
							$insert_array['date'] = $date;
						}
						//fehler
						else
						{
							return 'false';
						}

						//Pruefen ob der Monat bereits abgeschlossen ist
						//in diesem Fall wird das eintragen verweigert
						$month_sql = $cdb->select("SELECT id, wochenarbeitszeit, max_wochenarbeitszeit, arbeitstage, abgeschlossen FROM hdb_stz_monat WHERE user_id=".$insert_array['user_id']." AND LEFT(datum, 7)='".substr($insert_array['date'], 0, 7)."'");
						$month_result = $cdb->fetch_array($month_sql);

						if (isset($month_result['id']))
						{
							//Stundenzettel ist bereits abgeschlossen kein bearbeiten mehr moeglich
							if ($month_result['abgeschlossen'] == 'j')
							{
								return 'false';
							}
							//offener stz
							else
							{
								$min_weektime = $month_result['wochenarbeitszeit'];
								$max_weektime = $month_result['max_wochenarbeitszeit'];
								$arbeitstage = $month_result['arbeitstage'];
							}
						}
						//noch keine Monatsstundenzettel vorhanden
						//Sollte nur am anfang eines monats moeglich sein wenn der vormonat noch nicht abgeschlossen ist
						else
						{
							$min_weektime = $user_result['min_weektime'];
							$max_weektime = $user_result['max_weektime'];
							$arbeitstage = $user_result['arbeitstage'];
						}

						$day_setup = array();
						//Taggeseinstellungen auslesen
						//Wochenende
						if ($tss->is_holiday($insert_array['date'], 'weekend', $user_result['user_id']) === true)
						{
							$day_setup = array('feiertag'			=> false,
												'wochenende'		=> true,
												'halbtags'			=> false,
												'reg_tag'			=> false,
												'min_sollstunden'	=> 0,
												'max_sollstunden'	=> 0,
												'urlaubstag'		=> 0,
												'date'				=> $insert_array['date']);
						}
						//Feiertag
						elseif ($tss->is_holiday($insert_array['date'], 'holiday', $user_result['user_id']) === true)
						{
							$day_setup = array('feiertag'			=> true,
												'wochenende'		=> false,
												'halbtags'			=> false,
												'reg_tag'			=> false,
												'min_sollstunden'	=> 0,
												'max_sollstunden'	=> 0,
												'urlaubstag'		=> 0,
												'date'				=> $insert_array['date']);
						}
						//Halbtags
						elseif ($tss->is_holiday($insert_array['date'], 'halbtags', $user_result['user_id']) === true)
						{
							$day_setup = array('feiertag'			=> false,
												'wochenende'		=> false,
												'halbtags'			=> true,
												'reg_tag'			=> false,
												'min_sollstunden'	=> round($min_weektime / $arbeitstage / 2, 2),
												'max_sollstunden'	=> round($max_weektime / $arbeitstage / 2, 2),
												'urlaubstag'		=> 0.5,
												'date'				=> $insert_array['date']);
						}
						//Normaler Arbeitstag
						else
						{
							$day_setup = array('feiertag'			=> false,
												'wochenende'		=> false,
												'halbtags'			=> false,
												'reg_tag'			=> true,
												'min_sollstunden'	=> round($min_weektime / $arbeitstage, 2),
												'max_sollstunden'	=> round($max_weektime / $arbeitstage, 2),
												'urlaubstag'		=> 1,
												'date'				=> $insert_array['date']);
						}

						//gibt an ob ein insert bzw. update durgeführt werden soll
						$sql_insert = true;

						//Pruefen ob fuer den tag ein datensatz vorhanden ist
						$sql = $cdb->select("SELECT stz_id, kommen, pause, gehen FROM hdb_stz WHERE user_id=".$insert_array['user_id']." AND datum='".$insert_array['date']."'");
						$stz_result = $cdb->fetch_array($sql);

						//Datensatz fuer diesen tag bereits vorhanden nur update moeglich
						if (isset($stz_result['stz_id']) and $stz_result['stz_id'] > 0)
						{
							//kein insert sondern update
							$sql_insert = false;
						}


						//Zeiten ist ein string. Der String sollte immer die nachfolgende struktur haben.
						//H:i-i-H:i
						if (strpos($time, '-') !== false)
						{
							$time_array = explode('-', $time);
							//alle drei datensaetze vorhanden
							if (isset($time_array[0]) and isset($time_array[1]) and isset($time_array[2]))
							{
								//kommen
								$time_array[0] = $tss->timeval($time_array[0]);
								//pause
								$time_array[1] = intval($time_array[1]);
								//gehen
								$time_array[2] = $tss->timeval($time_array[2]);

								//min kommen oder gehen vorhanden
								if (!empty($time_array[0]) or !empty($time_array[2]))
								{
									//Zeiten runden
									if (isset($GLOBALS['setup_array']['stz_round_time']) and $GLOBALS['setup_array']['stz_round_time'] == 'j')
									{
										$insert_array['kommen'] = $tss->round_time($time_array[0]);
										$insert_array['pause'] = $time_array[1];
										$insert_array['gehen'] = $tss->round_time($time_array[2]);
									}
									else
									{
										$insert_array['kommen'] = $time_array[0];
										$insert_array['pause'] = $time_array[1];
										$insert_array['gehen'] = $time_array[2];
									}
								}
								else
								{
									return 'false';
								}
							}
							else
							{
								return 'false';
							}
						}
						//Zeit ist ein string
						elseif (!empty($time))
						{
							//datentypen pruefen
							$time = $tss->timeval($time);

							//Zeiten runden
							if (isset($GLOBALS['setup_array']['stz_round_time']) and $GLOBALS['setup_array']['stz_round_time'] == 'j')
							{
								$time = $tss->round_time($time);
							}

							if (!empty($time))
							{
								//Datensatz fuer diesen tag bereits vorhanden nur update moeglich
								if (isset($stz_result['stz_id']) and $stz_result['stz_id'] > 0)
								{
									//noch keine kommen zeit vorhanden
									//der uebergebene zeitwert wird als kommen gewertet
									if (empty($stz_result['kommen']))
									{
										$insert_array['kommen'] = $time;
										$insert_array['pause'] = intval($GLOBALS['setup_array']['stz_pausenminuten']);
										$insert_array['gehen'] = '';
									}
									//kommen bereits vorhanden
									//der uebergebene Wert wird als gehen gewertet
									else
									{
										$insert_array['kommen'] = $stz_result['kommen'];
										$insert_array['pause'] = $stz_result['pause'];
										$insert_array['gehen'] = $time;

									}
								}
								//kein datensatz vorhanden nur insert moeglich
								//der uebergebene zeitwert wird als kommen gewertet
								else
								{
									$insert_array['kommen'] = $time;
									$insert_array['pause'] = intval($GLOBALS['setup_array']['stz_pausenminuten']);
									$insert_array['gehen'] = '';
								}
							}
							else
							{
								return 'false';
							}
						}

						if (!empty($insert_array['gehen']))
						{
							$day_time_array = $tss->calculate_worktime($insert_array['kommen'], $insert_array['pause'], $insert_array['gehen']);
							$insert_array['ist_zeit'] = $day_time_array['zeit'];
							//ueberstunden
							if ($day_time_array['zeit'] < $day_setup['min_sollstunden'])
							{
								$insert_array['zeit'] = $day_time_array['zeit'] - $day_setup['min_sollstunden'];
							}
							elseif ($day_time_array['zeit'] > $day_setup['max_sollstunden'])
							{
								$insert_array['zeit'] = $day_time_array['zeit'] - $day_setup['max_sollstunden'];
							}
							else
							{
								$insert_array['zeit'] = 0;
							}
						}
						else
						{
							$insert_array['ist_zeit'] = '';
							$insert_array['zeit'] = 0;
						}

						//Insert
						//Sollte nur am anfang eines monats moeglich sein wenn der vormonat noch nicht abgeschlossen ist
						if ($sql_insert === true)
						{
							if (empty($insert_array['gehen']))
							{
								//gehen und ist_zeit haben als standard NULL brauchen also nicht eingetragen werden
								if ($cdb->insert("INSERT INTO hdb_stz (user_id, datum, kommen, pause) VALUES (".$insert_array['user_id'].", '".$insert_array['date']."', '".$insert_array['kommen']."', '".$insert_array['pause']."')"))
								{
									return 'true';
								}
								else
								{
									return 'false';
								}
							}
							else
							{
								if ($cdb->insert("INSERT INTO hdb_stz (user_id, datum, kommen, pause, gehen, ist_zeit, ueberstunden) VALUES (".$insert_array['user_id'].", '".$insert_array['date']."', '".$insert_array['kommen']."', '".$insert_array['pause']."', '".$insert_array['gehen']."', '".$insert_array['ist_zeit']."', '".$insert_array['zeit']."')"))
								{
									return 'true';
								}
								else
								{
									return 'false';
								}
							}
						}
						//update
						else
						{
							if (empty($insert_array['gehen']))
							{
								//gehen und ist_zeit haben als standard NULL
								if ($cdb->update("UPDATE hdb_stz SET kommen='".$insert_array['kommen']."',
																	pause='".$insert_array['pause']."',
																	gehen=NULL,
																	ist_zeit=NULL,
																	ueberstunden=NULL
																	WHERE user_id=".$insert_array['user_id']." AND datum='".$insert_array['date']."'"))
								{
									return 'true';
								}
								else
								{
									return 'false';
								}
							}
							else
							{
								if ($cdb->update("UPDATE hdb_stz SET kommen='".$insert_array['kommen']."',
																	pause='".$insert_array['pause']."',
																	gehen='".$insert_array['gehen']."',
																	ist_zeit='".$insert_array['ist_zeit']."',
																	ueberstunden='".$insert_array['zeit']."'
																	WHERE user_id=".$insert_array['user_id']." AND datum='".$insert_array['date']."'"))
								{
									return 'true';
								}
								else
								{
									return 'false';
								}
							}
						}
					}
				}
			}
		}
		return 'false';
	}

	/**
	 * Traegt Projektzeiten in den stz ein
	 * @param string	$ws_key
	 * @param mixed		$user			user_id or username
	 * @param int		$project_id
	 * @param mixed		$project_time	projecthours (float) or projecttime H:i-i-H:i == 08:00-45-09:45 = 1 houre
	 * @param string	$date			date('Y-m-d') 2010-12-31
	 * @return string
	 */
	function insert_project_time($ws_key, $user, $project_id, $project_time, $date)
	{
		if (!empty($ws_key))
		{
			$cdb = new MySql;
			$sl = new Login();
			$kal = new Kalender();
			$tss = new TimeSheetSave();
			$chk_v = new ChkVariable();

			//ws_key ok
			if ($sl->webservice_key($ws_key) == 'true')
			{
				$access = new Access();

				//Userid suchen
				//user finden
				if (is_numeric($user))
				{
					//User anhand der id finden
					$sql = $cdb->select('SELECT user_id FROM hdb_user WHERE user_id='.$user);
				}
				else
				{
					$user = $chk_v->modifyString($user);
					//user anhand des usernamens finden
					$sql = $cdb->select("SELECT user_id FROM hdb_user WHERE username='$user'");
				}
				$user_result = $cdb->fetch_array($sql);

				if (isset($user_result['user_id']) and $user_result['user_id'] > 0)
				{
					if ($user_result['user_id'] == USER_ID)
					{
						$stz_access = true;
					}
					else
					{
						$stz_access = $access->check_group(USER_GROUP_ID, 'admin_stz');
					}

					if ($stz_access)
					{
						//check dates
						if ($date != '0000-00-00' and $kal->is_iso_date($date) !== false)
						{
							if (is_numeric($project_id))
							{
								if (!empty($project_time))
								{
									$year_month = substr($date, 0, 7);
									$year_month_int = str_replace('-', '', $year_month);

									$sql = $cdb->select('SELECT id FROM hdb_stz_monat WHERE user_id='.$user_result['user_id']." AND LEFT(datum, 7)='$year_month' AND abgeschlossen='n'");
									$result = $cdb->fetch_array($sql);

									//Stundenzettel ist noch nicht abgeschlossen
									if ($result['id'] > 0)
									{
										//projektzuordnunh kontrollieren
										$sql= $cdb->select("SELECT t2.projekt_id FROM hdb_user_projekt t1
															LEFT JOIN hdb_projekte t2 ON t1.projekt_id=t2.projekt_id
															WHERE t1.user_id=".$user_result['user_id']." AND t1.projekt_id=$project_id AND (t1.anzeigen='j' OR LEFT(t1.del_time, 6) >=".$year_month_int.") AND LEFT(t2.start_datum,7)<='$year_month' AND LEFT(t2.end_datum, 7)>='$year_month'");
										$result = $cdb->fetch_array($sql);

										if (isset($result['projekt_id']) and $result['projekt_id'] > 0)
										{
											$project_time_error = true;

											//einzelner stundenwert
											if (is_numeric($project_time))
											{
												$project_time = round(floatval($project_time), 2);
												$project_time_error = false;
											}
											//H:i-i-H:i
											elseif (substr_count($project_time, '-') == 2)
											{
												$ex_array = explode('-', $project_time);
												$project_time = 0;

												if (is_array($ex_array) and count($ex_array) == 3)
												{
													$kommen = $tss->timeval($ex_array[0]);
													$pause = intval($ex_array[1]);
													$gehen = $tss->timeval($ex_array[2]);

													//Zeiten runden
													if (isset($GLOBALS['setup_array']['stz_round_time']) and $GLOBALS['setup_array']['stz_round_time'] == 'j')
													{
														$kommen = $tss->round_time($kommen);
														$gehen = $tss->round_time($gehen);
													}

													if (!empty($kommen) and !empty($gehen))
													{
														$project_time_array = $tss->calculate_worktime($kommen, $pause, $gehen);

														if (!empty($project_time_array))
														{
															$kommen =		$project_time_array['kommen'];
															$pause =		$project_time_array['pause'];
															$gehen = 		$project_time_array['gehen'];
															$project_time =	$project_time_array['zeit'];

															if (!empty($kommen) and !empty($gehen) and !empty($project_time))
															{
																$project_time_error = false;
															}
														}
													}
												}
											}

											if ($project_time_error == false)
											{
												$sql = $cdb->select('SELECT stz_pro_id FROM hdb_stz_projekt WHERE user_id='.$user_result['user_id'].' AND projekt_id='.$project_id." AND datum='$date'");
												$result = $cdb->fetch_array($sql);

												//Update
												if (isset($result['stz_pro_id']) and $result['stz_pro_id'] > 0)
												{
													if (isset($kommen))
													{
														$sql_stz = "UPDATE hdb_stz_projekt SET stunden='$project_time', kommen='$kommen', pause='$pause', gehen='$gehen' WHERE stz_pro_id=".$result['stz_pro_id'];
													}
													else
													{
														$sql_stz = "UPDATE hdb_stz_projekt SET stunden='$project_time', kommen is NULL, pause is NULL, gehen is NULL WHERE stz_pro_id=".$result['stz_pro_id'];
													}

												}
												//Insert
												else
												{
													if (isset($kommen))
													{
														$sql_stz = "INSERT INTO hdb_stz_projekt (user_id, projekt_id, stunden, datum, kommen, pause, gehen) VALUES (".$user_result['user_id'].", $project_id, '$project_time', '$date', '$kommen', '$pause', '$gehen')";
													}
													else
													{
														$sql_stz = "INSERT INTO hdb_stz_projekt (user_id, projekt_id, stunden, datum) VALUES (".$user_result['user_id'].", $project_id, '$project_time', '$date')";
													}
												}

												if (isset($sql_stz) and $cdb->query($sql_stz))
												{
													return 'true';
												}
											}
										}
									}
								}
							}
						}
					}
				}
			}
		}
		return 'false';
	}

	/**
	 * stz close
	 * @param string	$ws_key
	 * @param mixed		$user user_id or username
	 * @param string	$date date('Y-m-d') 2010-12-31
	 * @return string
	 */
	function close_timesheet($ws_key, $user, $date)
	{
		if (!empty($ws_key))
		{
			$cdb = new MySql;
			$sl = new Login();
			$kal = new Kalender();
			$tss = new TimeSheetSave();
			$ts = new TimeSheet();
			$chk_v = new ChkVariable();

			//ws_key ok
			if ($sl->webservice_key($ws_key) == 'true')
			{
				$access = new Access();

				//Userid suchen
				//user finden
				if (is_numeric($user))
				{
					//User anhand der id finden
					$sql = $cdb->select('SELECT user_id FROM hdb_user WHERE user_id='.$user);
				}
				else
				{
					$user = $chk_v->modifyString($user);
					//user anhand des usernamens finden
					$sql = $cdb->select("SELECT user_id FROM hdb_user WHERE username='$user'");
				}
				$user_result = $cdb->fetch_array($sql);

				if (isset($user_result['user_id']) and $user_result['user_id'] > 0)
				{
					if ($user_result['user_id'] == USER_ID)
					{
						$stz_access = true;
					}
					else
					{
						$stz_access = $access->check_group(USER_GROUP_ID, 'admin_stz');
					}

					if ($stz_access)
					{
						//check dates
						if ($date != '0000-00-00' and $kal->is_iso_date($date) !== false)
						{
							$year = substr($date, 0, 4);
							$month = substr($date, 5, 2);

							$sql = $cdb->select('SELECT id FROM hdb_stz_monat WHERE user_id='.$user_result['user_id']." AND LEFT(datum, 7)='".$year."-".$month."' AND abgeschlossen='n'");
							$result = $cdb->fetch_array($sql);

							//Stundenzettel ist noch nicht abgeschlossen
							if ($result['id'] > 0)
							{
								if ($tss->recalculation_month_data($user_result['user_id'], $year, $month))
								{
									if ($ts->close_time_sheet($user_result['user_id'], $month, $year) === true)
									{
										return 'true';
									}
								}
							}
						}
					}
				}
			}
		}
		return 'false';
	}

	/**
	 * Gibt die Zuordnungen zwischen user und Projekt zurueck
	 * @param string	$ws_key
	 * @param mixed		$user			user_id or username
	 * @param int		$project_id
	 * @param string	$date			date('Y-m-d') 2010-12-31
	 * @param string	$return_type
	 * @return mixed
	 */
	function get_user_project_assignment($ws_key, $user, $project_id, $date, $return_type = 'xml')
	{
		if (!empty($ws_key))
		{
			$cdb = new MySql;
			$sl = new Login();
			$kal = new Kalender();
			$chk_v = new ChkVariable();

			//ws_key ok
			if ($sl->webservice_key($ws_key) == 'true')
			{
				$access = new Access();

				if ($access->check_group(USER_GROUP_ID, 'admin_project_read'))
				{
					$where_array = array();

					if (is_numeric($project_id) and $project_id > 0)
					{
						$where_array[] = 't1.projekt_id='.$project_id;
					}

					if (!empty($user))
					{
						//Userid suchen
						//user finden
						if (is_numeric($user))
						{
							//User anhand der id finden
							$sql = $cdb->select('SELECT user_id FROM hdb_user WHERE user_id='.$user);
						}
						else
						{
							$user = $chk_v->modifyString($user);
							//user anhand des usernamens finden
							$sql = $cdb->select("SELECT user_id FROM hdb_user WHERE username='$user'");
						}
						$user_result = $cdb->fetch_array($sql);
						if (isset($user_result['user_id']) and $user_result['user_id'] > 0)
						{
							$where_array[] = 't1.user_id='.$user_result['user_id'];
						}
					}

					if (!empty($date) and $date != '0000-00-00' and $kal->is_iso_date($date) !== false)
					{
						$year_month = substr($date, 0, 7);
						$year_month_int = str_replace('-', '', $year_month);

						$where_array[] = "(t1.anzeigen='j' OR LEFT(t1.del_time, 6) >=".$year_month_int.") AND LEFT(t2.start_datum,7)<='$year_month' AND LEFT(t2.end_datum, 7)>='$year_month'";
					}
					else
					{
						$where_array[] = "t1.anzeigen='j'";
					}

					$where = '';
					foreach ($where_array as $where_sql)
					{
						if (empty($where))
						{
							$where = ' WHERE '.$where_sql;
						}
						else
						{
							$where .= ' AND '.$where_sql;
						}
					}

					$return_array = array();
					//projektzuordnunh kontrollieren
					$sql= $cdb->select("SELECT t2.projekt_id, t1.user_id FROM hdb_user_projekt t1
										LEFT JOIN hdb_projekte t2 ON t1.projekt_id=t2.projekt_id
										$where");

					while ($result = $cdb->fetch_array($sql))
					{
						$return_array[] = $result;
					}

					if (count($return_array) > 0)
					{
						if ($return_type == 'array')
						{
							return @serialize($return_array);
						}
						//json string
						elseif ($return_type == 'json')
						{
							return json_encode($return_array);
						}
						//xml string
						else
						{
							return array_to_xml($return_array, 'project_hours');
						}
					}
				}
			}
		}
		return '';
	}

	/**
	 * Traegt eine neue Zuordnung zwischen User und Orpjekt ein
	 * @param string	$ws_key
	 * @param mixed		$user user_id or username
	 * @param int		$project_id
	 * @return int		0 == error or user_projekt_id from hdb_user_projekt
	 */
	function add_user_project_assignment($ws_key, $user, $project_id)
	{
		if (!empty($ws_key))
		{
			$cdb = new MySql;
			$sl = new Login();
			$chk_v = new ChkVariable();

			//ws_key ok
			if ($sl->webservice_key($ws_key) == 'true')
			{
				$access = new Access();

				//Userid suchen
				//user finden
				if (is_numeric($user))
				{
					//User anhand der id finden
					$sql = $cdb->select('SELECT user_id FROM hdb_user WHERE user_id='.$user);
				}
				else
				{
					$user = $chk_v->modifyString($user);
					//user anhand des usernamens finden
					$sql = $cdb->select("SELECT user_id FROM hdb_user WHERE username='$user'");
				}
				$user_result = $cdb->fetch_array($sql);

				if (isset($user_result['user_id']) and $user_result['user_id'] > 0)
				{
					if (is_numeric($project_id))
					{
						if ($access->check_group(USER_GROUP_ID, 'admin_project_read'))
						{
							$sql = $cdb->select('SELECT user_projekt_id FROM hdb_user_projekt WHERE user_id='.$user_result['user_id']." AND projekt_id=$project_id AND anzeigen='j'");
							$result = $cdb->fetch_array($sql);

							if (isset($result['user_projekt_id']) and $result['user_projekt_id'] > 0)
							{
								return $result['user_projekt_id'];
							}
							else
							{
								if ($cdb->insert("INSERT INTO hdb_user_projekt (projekt_id, user_id) VALUES ($project_id, ".$user_result['user_id'].")"))
								{
									$last_id = $cdb->get_last_insert_id();

									if ($last_id > 0)
									{
										return $last_id;
									}
								}
							}
						}
					}
				}
			}
		}
		return 0;
	}

	//Prüfen ob SoapServer installiert ist
	//siehe http://de3.php.net/manual/de/book.soap.php
	if (class_exists('SoapServer'))
	{
		//Pruefen ob in den grundeinstellungen die Webservice aktiviert sind
		if (isset($setup_array) and isset($setup_array['webservice_tom']) and isset($setup_array['webservice_tom']['aktiv']) and $setup_array['webservice_tom']['aktiv'] == 1)
		{
			//Pruefen das min. eine Funktion aktiviert ist
			if (isset($setup_array['webservice_tom']['functions']) and is_array($setup_array['webservice_tom']['functions']) and count($setup_array['webservice_tom']['functions']) > 0)
			{
				$ip_access = true;

				//IP Adressbeschraenkungen vorhanden
				//IP Adresse Pruefen
				if (isset($setup_array['webservice_tom']['ip']) and is_array($setup_array['webservice_tom']['ip']) and count($setup_array['webservice_tom']['ip']) > 0)
				{
					//keine remote ip
					//kann eigendlich nur der fall sein wenn der uafruf lokal über die konsole erfolgt
					if (isset($_SERVER['REMOTE_ADDR']) and !empty($_SERVER['REMOTE_ADDR']))
					{
						$ip_access = chk_request_ip($_SERVER['REMOTE_ADDR'], $setup_array['webservice_tom']['ip']);
					}
				}

				if ($ip_access === true)
				{
					$function_array = array();
					foreach ($setup_array['webservice_tom']['functions'] as $function => $function_aktiv)
					{
						if ($function_aktiv == 1)
						{
							$function_array[] = $function;
						}
					}

					if (count($function_array) > 0)
					{
						$server = new SoapServer(TOM_ABS_URL.'web_services/tom.wsdl');
						//$server->addFunction(array('get_user', 'get_project', 'get_project_hours', 'insert_user_time'));
						$server->addFunction($function_array);
						$server->handle();
					}
					else
					{
						setError('Keine Webservicefunctionen in den Einstellungen vorhanden', WARNING, __LINE__);
					}
				}
				else
				{
					setError($_SERVER['REMOTE_ADDR'].'Die Remote IP ist nicht in den Webserviceeinstellungen vorhanden', WARNING, __LINE__);
				}
			}
			else
			{
				setError('Keine Webservicefunctionen in den Einstellungen vorhanden', WARNING, __LINE__);
			}
		}
		else
		{
			setError('Keine Webserviceeinstellungen vorhanden', WARNING, __LINE__);
		}
	}
	else
	{
		setError('SOAP Class existiert nicht', WARNING, __LINE__);
	}
?>